/*global require*/
/*global cc*/
/*global LumosityUtils*/

require( "src/Utility/LumosityUtils.js" );

var LumosityInverseNode;
(function()
{
    "use strict";

    LumosityInverseNode = cc.Node.extend(
    {
        /** @lends LumosityInverseNode.prototype */

        /**
        *  A LumosityInverseNode is a node that calculates and applies the inverse scale
        *  of all of its parents to itself. This creates a node who is not affected by its
        *  parents scale but can be scaled directly.
        *
        *  Disclaimer: Scaling the parent after the this node has been added will affect the node.
        */
        onEnter: function()
        {
            var hierarchyScale = cc.PointZero();
            var newScale = cc.p( 1, 1 );
            this._super();

            hierarchyScale = LumosityUtils.getNodeHierarchicalScale( this );
            newScale = cc.p( this.getScaleX() * hierarchyScale.x, this.getScaleY() * hierarchyScale.y );
            
            var contentScale = cc.Director.getInstance().getContentScaleFactor();
            if (contentScale < 1) {
                newScale.x /= contentScale;
                newScale.y /= contentScale;
            };
            // TEST
            //newScale.x *= 16, newScale.y *= 16;
            ll.log("newScale " + newScale.x + ", " + newScale.y + " contentscale " + contentScale);
            
            this.setScaleX( newScale.x );
            this.setScaleY( newScale.y );
        },

        onExit: function()
        {
            this._super();

            this.setScaleX( 1 );
            this.setScaleY( 1 );
        },

        setScale: function( scale, scaleY )
        {
            var inverseScale = cc.PointZero();
            var tempScaleX = scale;
            var tempScaleY = (scaleY || scaleY === 0) ? scaleY : scale;

            inverseScale = LumosityUtils.getInverseScale( cc.p( tempScaleX, tempScaleY ) );
            this._super( inverseScale );
        },

        setScaleX: function( newScaleX )
        {
            var inverseScale = cc.PointZero();

            inverseScale = LumosityUtils.getInverseScale( cc.p( newScaleX, 1 ) );
            this._super( inverseScale.x );
        },

        setScaleY: function( newScaleY )
        {
            var inverseScale = cc.PointZero();

            inverseScale = LumosityUtils.getInverseScale( cc.p( 1, newScaleY ) );
            this._super( inverseScale.y );

            //cc.log( this.nodeName + " LumosityLabel::setScaleY inverseScale: " + JSON.stringify( inverseScale ) );
        }
    } );
}() );
