#!/usr/bin/python
import glob, os, sys

# short script to fix up the names from the original mobile version to match the new
# system that eskews the @2x and -1024h conventions.

def fixupImageNames(dirPath):
	if 'iphone' in dirPath:
		fixupIphoneNames(dirPath)
	elif 'ipad_retina' in dirPath:
		fixupIpadRetinaNames(dirPath)
	else:
		fixupIpadNames(dirPath)

def fixupIphoneNames(dirPath):
	files = [path for path in glob.glob(os.path.join(dirPath, '*png')) if path.endswith('@2x.png')]
	for file in files:
		undecorated = file[:-7] + '.png'
		# remove the non-retinal version and replace it with the retinal one
		if os.path.exists(undecorated):
			os.remove(undecorated)
		os.rename(file, undecorated)

def fixupIpadRetinaNames(dirPath):
	files = [path for path in glob.glob(os.path.join(dirPath, '*png')) if path.endswith('-1024h@2x.png')]
	for file in files:
		# remove the non-retinal version and replace it with the retinal one
		shorterVersion = file[:-13] + '-1024h.png'
		assert os.path.exists(shorterVersion)
		os.remove(shorterVersion)
		os.rename(file, file[:-13]+'.png')


def fixupIpadNames(dirPath):
	files = [path for path in glob.glob(os.path.join(dirPath, '*png')) if path.endswith('-1024h@2x.png')]
	for file in files:
		# remove the retinal version and strip the extension from the non-retinal one
		shorterVersion = file[:-13] + '-1024h.png'
		os.remove(file)
		os.rename(shorterVersion, file[:-13]+'.png')

def stripPrefixes(dirPath):
	prefix = 'chalkboardChallenge_'
	files = [file for file in glob.glob(os.path.join(dirPath, '*.png')) if os.path.split(file)[1].startswith(prefix)]
	for file in files:
		d,f = os.path.split(file)
		outpath = os.path.join(d,f[len(prefix):])
		print outpath
		os.rename(file, outpath)


if __name__ == '__main__':
	assert len(sys.argv) == 2
	fixupImageNames(sys.argv[1])
	stripPrefixes(sys.argv[1])

