/*global cc*/

/**
 * An EaseType is an enum of the different eases that can be applied.
 * The enum values directly match what comes from design data.
 *
 * @name EaseType
 * @class
 * @readonly
 * @enum {string}
 */
var EaseType =
{
    None: "none",
    In: "in",
    Out: "out",
    InOut: "inout"
};

var EaseWrapper =
{    
    /**
     * Can be used to apply ease to any action.
     *
     * @param {cc.Action} action This action will be returned, but wrapped within an ease action.
     * @param {EaseType} easeType The type of ease we should wrap the action with.
     */
    wrapActionWithEase:function( action, easeType )
    {
        if ( !easeType )
        {
            return action;
        }
        
        switch ( easeType )
        {
            case EaseType.None:
                return action;
            case EaseType.Out:
                return cc.EaseOut.create( action, 2 );
            case EaseType.In:
                return cc.EaseIn.create( action, 2 );
            case EaseType.InOut:
                return cc.EaseInOut.create( action, 2 );
            default:
                return action;
        }
    }
};