/*global require*/
/*global cc*/
/*global GameComponent*/
/*global GameWidgetHud*/

require( "src/Game/Component/GameComponent.js" );
require( "src/Game/Widget/GameWidgetHud.js" );

var GameComponentScore;
( function()
{
    "use strict";

    GameComponentScore = GameComponent.extend(
    {
        _score: 0,

        /** @lends GameComponentScore.prototype */
        
        /**
         * A GameComponentScore is a subclass of GameComponent that can be used to set up games with
         * score. Provides functionality to manage the game score. 
         *
         * !IMPORTANT!
         *
         * This class expects that a game's settings.json file includes definitions for the following:
         *     scoreType
         *     pointsPerCorrect
         *     pointsPerIncorrect
         *     minScore
         *     lastTrialBonus
         *     
         * @constructs
         */
        ctor:function( controller )
        {
            this._super( controller );

            var model = this._controller.model;
            var settings = this._controller.settings;
            var view = this._controller.view;
            var hudWidget = view._widgets[ GameWidgetHud.prototype.getType() ];

            this._score = settings.minScore;

            // Initialize the model and hud widget score value
            model.score = this._score;
            hudWidget.setScore( this._score );
        },

        /**
        * Returns Component type as a string.
        *
        * @returns {string}
        */
        getType: function()
        {
            return "GameComponentScore";
        },

        setScore: function( score )
        {
            var settings = this._controller.settings;
            var model = this._controller.model;
            var view = this._controller.view;
            var hudWidget = view._widgets[ GameWidgetHud.prototype.getType() ];

            model.prevScore = this._score;
            this._score = Math.max( settings.minScore, score );
            model.score = this._score;
            hudWidget.setScore( this._score );  
        },

        getScore: function()
        {
            return this._score;
        },

        _setBonus: function( bonus )
        {
            this._bonus = Math.max( 0, bonus );
            this._controller.model.bonus = this._bonus;
        },

        onEndGameplay:function()
        {  
            var settings = this._controller.settings;
            var newScore = this._score;
            var bonusScore = settings.lastTrialBonus;

            newScore += bonusScore;

            this._setBonus( bonusScore );
            this.setScore( newScore );
        }
    } );
}() );