/*global cc*/
/*global lumosity*/
/*global require*/
/*global Game*/
/*global GameComponentTimer*/
/*global ColorMatchSettings*/
/*global ColorMatchModel*/
/*global ColorMatchView*/
/*global LumosityUtils*/
/*global ColorMatchTrialType*/
/*global ColorMatchAnimationStatus*/
/*global GameComponentScoreMultiplier*/

var ColorMatch;
(function ()
{
    "use strict";

    require( "src/Game/Base/Game.js" );
    require( "src/Game/ColorMatchSettings.js" );
    require( "src/Game/ColorMatchModel.js" );
    require( "src/Game/ColorMatchView.js" );
    require( "src/Utility/LumosityUtils.js" );

    ColorMatch = Game.extend(
    {

        /** @lends ColorMatch.prototype */

        /**
         *  ColorMatch is the main class for the game, Color Match!
         *
         *  @constructs
         */
        ctor:function()
        {
            this._super();

            this.timeRemaining = this.settings.gameLengthInSeconds;

        },

        onBegin: function()
        {
            this._super();

            this.view.enableButtons( false );

            if (this.model.persistentData.isPracticeMode && !LumosityUtils.isFitTestModeOn())
            {
                delete this._components[(new GameComponentTimer(this)).getType()];
            }
            else if (!this.model.persistentData.isPracticeMode)
            {
                this._components = {};
                this._createComponents();
                this.view._widgets[GameWidgetHud.prototype.getType()].showScore();
                this.view._widgets[GameWidgetHud.prototype.getType()].showTime();
            }

            if (this.model.persistentData.isPracticeMode)
            {
                this.view[ "fitTestPrompt" ].setString( localize( "PracticeTrialCorrect" ) );
            }
            else
            {
                this.view[ "prompt" ].setString( localize( "DefaultPromptLabel" ));
            }

        },

        onStartGameplay:function()
        {
            this.model.shit = 0;

            if (!this.model.tutorialSkipped)
            {
                this._super();
                this.view.setHintVisibility( true );
                this._setHintsShowing( true );
                this.view.enableButtons( true );
                this.model.isBlockingInput = false;

                if (this.model.persistentData.isPracticeMode)
                {
                    this.createNewStimuli("onlyMatch");
                    this.model.prevResultCorrect = true;
                    this.model.animationStatus = ColorMatchAnimationStatus.practicePromptDisplayed;
                    if (LumosityUtils.isFitTestModeOn())
                    {
                        this.view.playAnimation( "practiceProblem" );
                        this.view.playAnimation( "practicePrompt" );
                    }
                    else
                    {
                        this.view.playAnimation("tutorialProblem");
                        this.view.playAnimation("tutorialPrompt");
                    }
                }
                else
                {
                    this.createNewStimuli();
                    this.view.playAnimation( "problem" );
                }
            }
        },


        playStartingAnimation: function()
        {
            if ( this.model.restartToPracticeMode || (this.model.persistentData.isPracticeMode && LumosityUtils.isFitTestModeOn()) )
            {
                this.view.playAnimation( "practiceBegin", this.onStartGameplay );
            }
            else if (this.model.persistentData.isPracticeMode && !this.model.isFitTestMode)
            {
                // WHY DOES THIS NOT WORK???? :-(
                this.view.animationController.play("startTutorial", this.onStartGameplay, this);
            }
            else
            {
                this.view.playAnimation( "gameBegin", this.onStartGameplay );
            }
        },

        playEndAnimation: function ()
        {
            if ( !this.model.persistentData.isPracticeMode )
            {
                var results = this.exportResults();
                var automatedTestReport = this.exportAutomatedTestReport();
                if( automatedTestReport )
                {
                    // Append automated test report if we have one.
                    results.automatedTestReport = automatedTestReport;
                }

                lumosity.game.report( JSON.stringify( results ) );
                this.view.playAnimation( "gameEnd", this.onFinish.bind( this, false ) );
            }
            else if (LumosityUtils.isFitTestModeOn())
            {
                this.view.playAnimation( "practiceEnd", this.onStartFitTest );
            }
        },

        onRestart: function()
        {
            this._super();
            this.model.score = 0;
            this.view.hud.setScore( 0 );
            this._setHintsShowing( false );
            this.view.setHintVisibility( false );
            this.view.enableButtons( false );
        },

        skipTutorialButton_Tapped:function()
        {
            if ( !this.model.tutorialSkipped && this.model.isGameRunning )
            {
                this.view.enableButtons(false);
                this._setHintsShowing( false );
                // More?
                this.onCompleteTutorial(true);
            }
        },

        onEndGameplay:function()
        {
            this._setHintsShowing( false );
            this.view.enableButtons( false );
            this.getMetadataComponent()._trialCount = this.model.totalResponseCount;
            this.model.isGameRunning = false;
            this.model.isGameOver = true;

            this.finalizeMetadata();

            if (this.model.persistentData.isPracticeMode)
            {
                this.tutorialMetadata.finishes += 1;
                if (!LumosityUtils.isFitTestModeOn())
                {
                    this.onCompleteTutorial();
                }
            }

            if (LumosityUtils.isFitTestModeOn())
            {
                this.getMetadataComponent().tutorialMetadata = this.tutorialMetadata;
            }

            LumosityUtils.callFunctionOnCollection(this._components, "onEndGameplay");
            this.playEndAnimation();

            if (this._interactionAutomator)
            {
                this._interactionAutomator.finishRecordingScript();
            }
            if (lumosity.game.gameplayEnded)
            {
                lumosity.game.gameplayEnded();
            }
        },

        checkAnswer:function( yesPressed )
        {
            //
            //NOTE: This early return check doesn't seem to be
            //      of any value when tested....
            //
            if ( this.model.isBlockingInput === true )
            {
                return;
            }
            this.factorResults( yesPressed === this.model.yesIsCorrect );
        },

        factorResults:function( success )
        {
            //increase how many responses we have seen this game
            this.model.totalResponseCount = this.model.totalResponseCount + 1;

            //block input now
            this.model.isBlockingInput = true;

            // Create a new Metadata trialObject
            var trial = {};

            trial.progress = this._components[ GameComponentScoreMultiplier.prototype.getType() ]._meterValue - 1;

            trial.multiplier = this._components[ GameComponentScoreMultiplier.prototype.getType() ]._multiplier;

            this.model.prevResultCorrect = success;

            // correct answer, give some points
            if ( success )
            {

                trial.points = 50 * trial.multiplier;

                // Update the score in the model.
                this.adjustScoreForCorrectAnswer();

                // Increase consecutive total
                this.model.consecutiveCorrectCount = this.model.consecutiveCorrectCount + 1;
                if ( this.model.isUsingTopStimulus )
                {
                    this.model.consecutiveCorrectNumberCount = this.model.consecutiveCorrectNumberCount + 1;
                }
                else
                {
                    this.model.consecutiveCorrectLetterCount = this.model.consecutiveCorrectLetterCount + 1;
                }

                this.model.totalCorrectResponseCount = this.model.totalCorrectResponseCount + 1;

            }
            // incorrect answer, give penalty
            else
            {

                trial.points = 0;

                // update the score
                this.adjustScoreForIncorrectAnswer();

                // update correct count
                this.model.consecutiveCorrectCount = 0;

                // Enable input when in practice mode and answer has been answered incorrectly.
                if ( this.model.persistentData.isPracticeMode )
                {
                    this.model.isBlockingInput = false;
                }
            }


            // Reaction time
            var now = new Date().getTime();
            var reactionTime = now - this.model.lastAnswerTime;
            this.model.averageReactionTime = this.model.averageReactionTime
                                             * ( 1.0 - 1.0 / this.model.totalResponseCount )
                                             + reactionTime * ( 1.0 / this.model.totalResponseCount );
            this.model.lastAnswerTime = now;

            // METADATA
            trial.color = this.settings.colors[this.model.coloredColorIndex].code;

            trial.correct = success ? "T" : "F";

            trial.describe_word = this.settings.colors[this.model.describingWordIndex].code;

            trial.color_word = this.settings.colors[this.model.coloredWordIndex].code;

            trial.type = this.model.currentTrial.type;

            this.endTrial( trial, success, false, this.model.getInhibition(), false, true, true );

            //update the values displayed and in the instance
            this.view.setScore( this.model.score );

            var scoreMultiplierComponent = this._components[ GameComponentScoreMultiplier.prototype.getType() ];

            // ui: feedback, instructions
            if ( success )
            {
                // Play feedback animation
                if ( this.model.isNewMultiplier && !this.model.persistentData.isPracticeMode )
                {
                    this.view.showFeedback( true, scoreMultiplierComponent._multiplier, true );
                }
                else
                {
                    this.view.showFeedback( true, 1, true );
                }

            }
            else
            {
                // play feedback animation
                this.view.showFeedback( false , scoreMultiplierComponent._multiplier, true );
            }

            // End the "Guided Practice" if you've gotten enough correct
            if ( this.model.persistentData.isPracticeMode && this.model.totalCorrectResponseCount === 4 && this.model.isGuided )
            {
                this.model.unGuidedTrialsInit();

                // Uncomment to bring back the mid-game, practice pop-up
                // this.view.enableButtons( false );

                // this.view.createFrontDimLayer();

                // this.view.showPostPracticeUnguidedPopUp();

            }

            // give a new card
            if ( this.model.persistentData.isPracticeMode )
            {
                if ( success )
                {
                    // 2 guided Match Trials
                    if ( this.model.totalCorrectResponseCount < 2 )
                    {
                        this.model.isGuided = true;
                        this.model.practiceType = "onlyMatch";
                        this.createNewStimuli( this.model.practiceType );
                    }
                    // 2 guided No-Match Trials
                    else if ( this.model.totalCorrectResponseCount < 4 )
                    {
                        this.model.isGuided = true;
                        this.model.practiceType = "noMatch";
                        this.createNewStimuli( this.model.practiceType);
                    }
                    // 8 trials: 4 guided No-Match Trials, 4 guided Match Trials
                    else if ( this.model.totalCorrectResponseCount < 12 )
                    {
                        this.model.isGuided = false;
                        this.model.practiceType = this.model.getNewUnguidedTrial();
                        this.createNewStimuli( this.model.practiceType );
                    }
                }
                else
                {
                    // DON'T Change the hint, manually sets to previous elements.
                    this.view.updateStimuli( this.settings.colors[this.model.describingWordIndex].name,
                         this.settings.colors[this.model.coloredWordIndex].name,
                         this.settings.colors[this.model.coloredColorIndex].color,
                         this.model.prevResultCorrect,
                         this.model.practiceType,
                         this.model.isGuided );
                }
            }
            else
            {
                this.createNewStimuli();
                //
                //NOTE: Creating a new stimuli DOES NOT update the view with the new
                //      stimuli information, it just randomizes indexes and such for
                //      the next trial. Actually updating the view is based on the .fla
                //      timeline, which may not get called if the user is answering while
                //      the feedback widget is still playing. Honestly, I am in awe that
                //      this bug didn't cause more damage.
                //
                this.view.playAnimation( "problem", this.problemAnimationFinished );
            }
        },

        // Uncomment to bring back the mid-game, practice pop-up
        // popup_isFinished: function()
        // {
        //     if ( this.model.persistentData.isPracticeMode )
        //     {
        //         this.view.enableButtons( true );

        //         this.view.removeFrontDimLayer();
        //     }
        // },

        getRandomInt: function( min, max )
        {
            return Math.floor(Math.random() * (max - min + 1)) + min;
        },

        feedback_isFinished: function( feedback )
        {
            this.feedbackStillPlaying = false;

            // End the Tutorial if you've gotten enough correct
            if ( this.model.persistentData.isPracticeMode && this.model.totalCorrectResponseCount === 12 && this.model.isPlaying())
            {
                this.onEndGameplay();
            }
        },


        createNewStimuli:function( practiceType )
        {
            // Initialize Metadata
            this.getMetadataComponent().beginMetadataTrial();

            // THIS is where you make your stimuli cards
            var correctColorIndex = this.getRandomInt( 0, this.settings.colors.length - 1 );

            if ( this.model.persistentData.isPracticeMode
                && this.model.previousCorrectColorIndex === correctColorIndex
                && this.shouldBeDifferentFromLast() )
            {
                correctColorIndex = this.getRandomColorIndexExcluding( correctColorIndex );
            }

            this.model.setDescribingWordIndex( correctColorIndex );

            var diffWordDiffColorChance;
            var sameWordDiffColorChance;
            var diffWordSameColorChance;
            var sameWordSameColorChance;

            if ( practiceType )
            {
                switch ( practiceType )
                {
                    case "onlyMatch":
                        diffWordDiffColorChance = -1;
                        sameWordDiffColorChance = -1;
                        diffWordSameColorChance = 50;
                        sameWordSameColorChance = 100;
                        break;
                    case "noMatch":
                        diffWordDiffColorChance = 50;
                        sameWordDiffColorChance = 100;
                        diffWordSameColorChance = -1;
                        sameWordSameColorChance = -1;
                        break;
                }
            }
            else
            {
                //
                //NOTE: These don't appear to change between trials, why not just cache them?
                //
                diffWordDiffColorChance = ( this.settings.differentWordDifferentColorChance * 100 );
                sameWordDiffColorChance = ( this.settings.sameWordDifferentColorChance * 100 ) + diffWordDiffColorChance;
                diffWordSameColorChance = ( this.settings.differentWordSameColorChance * 100 ) + sameWordDiffColorChance;
                sameWordSameColorChance = ( this.settings.sameWordSameColorChance * 100 ) + diffWordSameColorChance;
            }

            var randomTrialValue = this.getRandomInt( 0, 99 );
            var otherColorIndex;

            if ( randomTrialValue < diffWordDiffColorChance )
            {

                otherColorIndex = this.getRandomColorIndexExcluding( correctColorIndex );
                var otherWordIndex = this.getRandomColorIndexExcluding( otherColorIndex );

                this.model.setColoredColorIndex( otherColorIndex );
                this.model.setColoredWordIndex( otherWordIndex );
                this.model.setIsYesCorrect( false );
                this.model.setTrialType( ColorMatchTrialType.ColorMatchTrialTypeDifferentWordDifferentColor );

            }
            else if ( randomTrialValue < sameWordDiffColorChance )
            {

                otherColorIndex = this.getRandomColorIndexExcluding( correctColorIndex );

                if ( this.model.persistentData.isPracticeMode
                     && otherColorIndex === this.model.previousOtherColorIndex
                     && this.shouldBeDifferentFromLast() )
                {
                    // Repeat until you find index that doesn't equal previous trial OR correctColor
                    while ( otherColorIndex === correctColorIndex )
                    {
                        otherColorIndex = this.getRandomColorIndexExcluding( otherColorIndex );
                    }
                }

                this.model.setColoredColorIndex( otherColorIndex );
                this.model.setColoredWordIndex( correctColorIndex );
                this.model.setIsYesCorrect( false );
                this.model.setTrialType( ColorMatchTrialType.ColorMatchTrialTypeSameWordDifferentColor );

            }
            else if ( randomTrialValue < diffWordSameColorChance )
            {

                otherColorIndex = this.getRandomColorIndexExcluding( correctColorIndex );

                if ( this.model.persistentData.isPracticeMode
                    && otherColorIndex === this.model.previousOtherColorIndex
                    && this.shouldBeDifferentFromLast() )
                {
                    // Also can't equal correctColorIndex
                    while ( otherColorIndex === correctColorIndex )
                    {
                        otherColorIndex = this.getRandomColorIndexExcluding( otherColorIndex );
                    }
                }

                this.model.setColoredColorIndex( correctColorIndex );
                this.model.setColoredWordIndex( otherColorIndex );
                this.model.setIsYesCorrect( true );
                this.model.setTrialType( ColorMatchTrialType.ColorMatchTrialTypeDifferentWordSameColor );

            }
            else if ( randomTrialValue < sameWordSameColorChance )
            {
                this.model.setColoredColorIndex( correctColorIndex );
                this.model.setColoredWordIndex( correctColorIndex );
                this.model.setIsYesCorrect( true );
                this.model.setTrialType( ColorMatchTrialType.ColorMatchTrialTypeSameWordSameColor );
            }

            this.model.previousCorrectColorIndex = correctColorIndex;
            this.model.previousOtherColorIndex = otherColorIndex;

        },

        shouldBeDifferentFromLast: function ()
        {
            return ( this.model.totalCorrectResponseCount === 1
                     || this.model.totalCorrectResponseCount === 3 );
        },

        //this is exact code from the C++ version but seems to indicate that color towards the higher end of the array would be more likely
        getRandomColorIndexExcluding: function ( index )
        {
            var randomIndex = this.getRandomInt( 0, this.settings.colors.length - 2 );
            if ( randomIndex >= index )
            {
                randomIndex += 1;
            }
            return randomIndex;
        },

        adjustScoreForCorrectAnswer: function()
        {

            var scoreMultiplierComponent = this._components[ GameComponentScoreMultiplier.prototype.getType() ];

            // increment the multiplier
            this.model.correctAnswerStreak = this.model.correctAnswerStreak + 1;

            //clamp the multiplier index - TODO, the model should probably do this itself
            if ( this.model.scoreMultiplierIndex >= this.settings.getMultiplierCount() )
            {
                this.model.scoreMultiplierIndex = this.settings.getMultiplierCount() - 1;
            }

            this.model.isNewMultiplier = false;

            // Check if the streak is complete
            if ( this.model.correctAnswerStreak === this.settings.getMultiplierStreak( this.model.scoreMultiplierIndex ) )
            {
                this.model.correctAnswerStreak = 0;

                if ( scoreMultiplierComponent._multiplier < 10 )
                {
                   this.model.scoreMultiplierIndex = this.model.scoreMultiplierIndex + 1;
                   this.model.isNewMultiplier = true;
                }
            }

            scoreMultiplierComponent.increaseMultiplierProgress();

        },

        adjustScoreForIncorrectAnswer: function()
        {
            if ( this.model.correctAnswerStreak > 0 )
            {
                this.model.correctAnswerStreak = 0;
            }
            else
            {
                this.model.scoreMultiplierIndex = this.model.scoreMultiplierIndex - 1;

                //clamp the multiplier index
                if ( this.model.scoreMultiplierIndex < 0 )
                {
                    this.model.scoreMultiplierIndex = 0;
                }
            }

            var scoreMultiplierComponent = this._components[ GameComponentScoreMultiplier.prototype.getType() ];

            scoreMultiplierComponent.decreaseMultiplierProgress();

        },

        /** This only adjusts the view, it doesn't calculate an actual new stimulus. */
        cue_showNextStimulus:function()
        {
            this.view.updateStimuli( this.settings.colors[this.model.describingWordIndex].name,
                                     this.settings.colors[this.model.coloredWordIndex].name,
                                     this.settings.colors[this.model.coloredColorIndex].color,
                                     this.model.prevResultCorrect,
                                     this.model.practiceType,
                                     this.model.isGuided );
            this.model.isBlockingInput = false;
        },

        /* Cue from feedback widget, used to show new stimuli. */
        feedback_isLargest:function( feedbackWidget )
        {
            if ( this.model.persistentData.isPracticeMode )
            {
                // Practice Mode has ended
                if (this.model.totalCorrectResponseCount === 12)
                {
                    this.view.playAnimation( "practicePromptOut" );
                }
                else if ( this.model.practiceType === "onlyMatch" )
                {
                    if ( this.model.prevResultCorrect )
                    {
                        this.changeAnimation( "practicePrompt" );
                    }
                    else
                    {
                        this.changeAnimation( "practicePromptYes" );

                    }
                }
                else if ( this.model.practiceType === "noMatch" )
                {
                    if ( this.model.prevResultCorrect )
                    {
                        this.changeAnimation( "practicePrompt" );
                    }
                    else
                    {
                        this.changeAnimation( "practicePromptNo" );
                    }
                }
            }
        },

        changeAnimation: function ( animation )
        {
            switch( animation )
            {
                case "practicePrompt":

                    if ( this.model.animationStatus === ColorMatchAnimationStatus.practicePromptDisplayed )
                    {
                        // Only update the stimuli
                        this.view.playAnimation( "practiceProblem" );
                    }

                    else if ( this.model.animationStatus === ColorMatchAnimationStatus.practicePromptYesDisplayed )
                    {
                        this.view.playAnimation( "practicePromptOut", this.displayPracticePrompt );

                        this.model.animationStatus = ColorMatchAnimationStatus.practicePromptDisplayed;
                    }

                    else if ( this.model.animationStatus === ColorMatchAnimationStatus.practicePromptNoDisplayed )
                    {
                        this.view.playAnimation( "practicePromptOut", this.displayPracticePrompt );

                        this.model.animationStatus = ColorMatchAnimationStatus.practicePromptDisplayed;
                    }
                    break;

                case "practicePromptYes":

                    if ( this.model.animationStatus === ColorMatchAnimationStatus.practicePromptDisplayed )
                    {
                        this.view.playAnimation( "practicePromptOut", this.displayPracticePromptYes );

                        this.model.animationStatus = ColorMatchAnimationStatus.practicePromptYesDisplayed;
                    }

                    else if ( this.model.animationStatus === ColorMatchAnimationStatus.practicePromptYesDisplayed )
                    {
                        // Don't update the stimuli
                    }

                    else if ( this.model.animationStatus === ColorMatchAnimationStatus.practicePromptNoDisplayed )
                    {
                        this.view.playAnimation( "practicePromptOut", this.displayPracticePromptYes );

                        this.model.animationStatus = ColorMatchAnimationStatus.practicePromptYesDisplayed;
                    }
                    break;

                case "practicePromptNo":
                    if ( this.model.animationStatus === ColorMatchAnimationStatus.practicePromptDisplayed )
                    {
                        this.view.playAnimation( "practicePromptOut", this.displayPracticePromptNo );

                        this.model.animationStatus = ColorMatchAnimationStatus.practicePromptNoDisplayed;
                    }

                    else if ( this.model.animationStatus === ColorMatchAnimationStatus.practicePromptYesDisplayed )
                    {
                        this.view.playAnimation( "practicePromptOut", this.displayPracticePromptNo );

                        this.model.animationStatus = ColorMatchAnimationStatus.practicePromptNoDisplayed;
                    }

                    else if ( this.model.animationStatus === ColorMatchAnimationStatus.practicePromptNoDisplayed )
                    {
                        // Don't update the stimuli
                    }
                    break;
            }
        },

        displayPracticePrompt: function ()
        {
            this.view[ "fitTestPrompt" ].setString( localize( "PracticeTrialCorrect" ) );
            this.view.playAnimation( "practicePrompt" );
            this.view.playAnimation( "practiceProblem" );
        },

        displayPracticePromptYes: function ()
        {
            this.view[ "fitTestPrompt" ].setString( localize( "PracticeMatchingTrialIncorrect" ) );
            this.view.playAnimation( "practicePrompt" );
        },

        displayPracticePromptNo: function ()
        {
            this.view[ "fitTestPrompt" ].setString( localize( "PracticeNonMatchingTrialIncorrect" ) );
            this.view.playAnimation( "practicePrompt" );
        },


        cue_showCountdown: function()
        {
            this.view.showCountdown();
        },

        _setHintsShowing: function( showing )
        {
            if ( showing )
            {
                this.view.showHints( true );
            }
            else
            {
                this.view.hideHints( true );

            }
        },

        _createSettings:function()
        {
            return new ColorMatchSettings( "settings.json" );
        },

        _createModel:function()
        {
            return new ColorMatchModel();
        },

        _createView:function( screenSize )
        {
            return new ColorMatchView( screenSize, this.settings, this.model, this, true, this.settings.colors[3].color );
        },

        _buttonPressed:function( buttonName )
        {
            this._super( buttonName );

            if ( this.model.isPlaying() )
            {
                if ( buttonName === "yes" )
                {
                    this.checkAnswer( true );

                }
                else if ( buttonName === "no" )
                {
                    this.checkAnswer( false );
                }
            }
        }
    } );
}() );
