/*global cc*/
/*global require*/
/*global GameView*/
/*global GameWidgetFeedback*/
/*global GameWidgetFlipbook*/
/*global GameWidgetHint*/
/*global LayoutLoader*/
/*global HintOrientation*/
/*global localize*/
/*global GameModel*/
/*global LumosityUtils*/

var ColorMatchTrialType =
{
    ColorMatchTrialTypeDifferentWordDifferentColor: 0,
    ColorMatchTrialTypeSameWordDifferentColor: 1,
    ColorMatchTrialTypeSameWordSameColor: 2,
    ColorMatchTrialTypeDifferentWordSameColor: 3,
    ColorMatchTrialTypeCount: 4
};

var ColorMatchAnimationStatus =
{
    practicePromptDisplayed: 0,
    practicePromptYesDisplayed: 1,
    practicePromptNoDisplayed: 2
};

var ColorMatchModel;
(function ()
{

    "use strict";

    require( "src/Game/Base/GameModel.js" );
    require( "src/Utility/LumosityUtils.js" );

    ColorMatchModel = GameModel.extend(
    {
        /** @lends ColorMatchModel.prototype */
        /**
         * The Brain Shift model. Mostly adds stuff dealing with top and bottom stimuli, vowels, and consonants.
         * @constructs
         */
        ctor:function()
        {
            this._super();
        },

        startTime: 0,

        /* Metadata Trial Object */
        currentTrial: {
            color: "",          // Color of bottom word [R=red, Y=yellow, B=blue, K=black]
            correct: null,      // Was the trial correctly answered? [T/F]
            multiplier: 0,      // Multiplier applied to the current score increment
            response_time: 0,   // Response time in milliseconds
            time_offset: 0,     // At start of trial, number of milliseconds since beginning of game
            describe_word: "",  // Top word [R=red, Y=yellow, B=blue, K=black]
            color_word: "",     // Bottom word [R=red, Y=yellow, B=blue, K=black]
            type: 0             // Type of trial [0 - diff word diff color, 1 - diff word same color, 2 - same word diff color, 3 - same word same color -- 1 and 2 are response inhibition trials]
        },

        /* True iff previous Practice Mode trial was correct. */
        prevResultCorrect: true,

        practiceType: "onlyMatch",

        /* True iff the current Practice Mode trial is "guided" */
        isGuided: true,

        /* True iff the multiplier has changed from the previous trial.
         * Useful for not playing the same feedback animation in a row. */
        isNewMulitplier: false,

        describingWordIndex: 0,
        coloredWordIndex: 0,
        coloredColorIndex: 0,
        yesIsCorrect: false,
        _trialType: ColorMatchTrialType.ColorMatchTrialTypeDifferentWordDifferentColor,

        consecutiveCorrectNumberCount: 0,
        consecutiveCorrectLetterCount: 0,

        isUsingTopStimulus: true,
        topValue: "0x",
        bottomValue: "0x",
        bottomColor: "0",

        unGuidedTrialsInit: function ()
        {
            this.unGuidedTrials = ["onlyMatch", "onlyMatch", "onlyMatch", "onlyMatch",
                                   "noMatch", "noMatch", "noMatch", "noMatch"];
        },

        /* Gets a trial from the UnGuidedTrials list, then removes that trial
         * to ensure all items are reached in the course of a tutorial. */
        getNewUnguidedTrial: function()
        {
            var randomIndex = LumosityUtils.getRandomInt( 0, this.unGuidedTrials.length - 1);
            
            var retval = this.unGuidedTrials[randomIndex];
            
            this.unGuidedTrials.splice( randomIndex, 1 );
            
            return retval;
        },

        increaseMultiplierStreak: function()
        {
            this.prevMultiplier = this.scoreMultiplierIndex;
            this.scoreMultiplierIndex += 1;
        },

        /* Getters and Setters */

        setDescribingWordIndex: function( index )
        {
            this.describingWordIndex = index;
        },

        setColoredColorIndex: function( index )
        {
            this.coloredColorIndex = index;
        },

        setColoredWordIndex: function( index )
        {
            this.coloredWordIndex = index;
        },

        setIsYesCorrect: function( correct )
        {
            this.yesIsCorrect = correct;
        },

        setTrialType: function( type )
        {
            this.trialType = type;
            this.currentTrial.type = type;
        },

        getInhibition: function() {
            return (this.currentTrial.type === ColorMatchTrialType.ColorMatchTrialTypeSameWordDifferentColor
                    || this.currentTrial.type === ColorMatchTrialType.ColorMatchTrialTypeSameWordSameColor);
        },

        exportResults:function( stats, bestStatKey )
        {
            var stats = {};
            stats[ this.StatsKeys.CardStat ] = this.totalCorrectResponseCount; 
            return this._super( stats, this.StatsKeys.CardStat );
        },

        setTopStimulus: function( stringValue )
        {
            this.topValue = stringValue;

            this.currentTrial.describe_word = stringValue;
        },

        setBottomStimulus: function( stringValue, stringColor )
        {
            this.bottomValue = stringValue;
            this.bottomColor = stringColor;

            this.currentTrial.color_word = stringValue;
            this.currentTrial.color = stringColor;

        }
        
    } );
}() );