/*global cc*/
/*global require*/
/*global GameView*/
/*global GameWidgetFeedback*/
/*global GameWidgetFlipbook*/
/*global GameWidgetHint*/
/*global LayoutLoader*/
/*global HintOrientation*/
/*global localize*/
/*global GameSettings*/

var ColorMatchSettings;
( function()
{
    "use strict";

    require( "src/Game/Base/GameSettings.js" );

    /**
     * The Brain Shift settings. Adds convenience functions for determining if things exist in the vowel and consonant lists.
     * @class
     * @name ColorMatchSettings
     */
     ColorMatchSettings = GameSettings.extend(
     {
        responseAccuracyThreshold: 0.70,
        pointsPerCorrect: 50,
        pointsPerIncorrect: 0,
        removeInstructionsConsecutiveCountRequired: -1,
        differentWordDifferentColorChance: 0.25,
        sameWordDifferentColorChance: 0.25,
        sameWordSameColorChance: 0.25,
        differentWordSameColorChance: 0.25,
        streakRequiredForBonus: 5,
        pointMultiplierIncreasePerBonus: 1,
        maximumPointMultiplier: 4,
        addedSecondsPerBonus: 7,
        bonusTimeLowCutoff: 15,
        bonusTimeLowLostPerIncorrect: 5,
        bonusTimeHighMultiplePerIncorrect: 0.5,
        colors: [
                   {
                       name: "redLocalizedKey",
                       code: "R",
                       color: 
                        {
                           Red: 233,
                           Green: 51,
                           Blue: 51,
                           Alpha: 255
                        }
                   },
                   {
                       name: "yellowLocalizedKey",
                       code: "Y",
                       color: 
                        {
                           Red: 253,
                           Green: 204,
                           Blue: 19,
                           Alpha: 255
                        }
                   },
                   {
                       name: "blueLocalizedKey",
                       code: "B",
                       color: 
                        {
                            Red: 37,
                            Green: 143,
                            Blue: 211,
                            Alpha: 255
                        }
                   },
                   {
                       name: "blackLocalizedKey",
                       code: "K",
                       color: 
                        {
                           Red: 51,
                           Green: 51,
                           Blue: 51,
                           Alpha: 255
                        }
                   }
                ],

        /** @lends ColorMatchSettings.prototype */
        
        /*
         * Returns whether or not the even numbers in the settings contain the passed number.
         *
         * @param {string} number The number to check against (as a string).
         * @returns {boolean} Whether or not the passed number is even.
         */
        doEvensContain: function( number )
        {
            var i;
            for ( i = 0; i < this.evenCharacterArray.length; i++ )
            {
                if ( this.evenCharacterArray[ i ] === number )
                {
                    return true;
                }
            }
            return false;
        },

        /**
         * Returns whether or not the vowels in the settings contain the passed letter.
         *
         * @param {string} letter The letter to check against.
         * @returns {boolean} Whether or not the passed letter is a vowel.
         */
        doVowelsContain: function( letter )
        {
            var i;
            for ( i = 0; i < this.vowelCharacterArray.length; i++ )
            {
                if ( this.vowelCharacterArray[ i ] === letter )
                {
                    return true;
                }
            }
            return false;
        }
    } );
}() );