/*global cc*/
/*global lumosity*/
/*global require*/
/*global GameView*/
/*global GameWidgetFeedback*/
/*global GameWidgetFlipbook*/
/*global GameWidgetHint*/
/*global LayoutLoader*/
/*global HintOrientation*/
/*global localize*/
/*global lu*/

var ColorMatchView;
(function ()
{
    "use strict";

    require( "src/Game/Base/GameView.js" );
    require( "src/Game/Widget/GameWidgetFeedback.js" );
    require( "src/Game/Widget/GameWidgetFlipbook.js" );
    require( "src/Game/Widget/GameWidgetHint.js" );
    require( "src/Animation/LayoutLoader.js" );

    ColorMatchView = GameView.extend(
    {

        _blueTextColor: new cc.c3b( 48, 191, 235 ),
        _buttonFont: "fonts/MuseoSans-500.ttf",
        _buttonTextSize: 48,
        _buttonTextDownColor: cc.WHITE,
        _pt_text_size: 100,
        _non_pt_text_size: 120,
        hintColor: null,
        hintFont: null,

        ctor:function( screenSize, settings, model, interactionResponder, preload, blackColor )
        {
            this._super( screenSize, settings, model, interactionResponder, preload );

            //TODO - move this into a HUD widget
            this.hud.setScore( 0 );
            this[ "top stimuli" ].setColor( cc.c4b( blackColor.Red, blackColor.Green, blackColor.Blue, blackColor.Alpha ) );
        },

        //****************TODO MOVE TO CARD WIDGET****************//
        cue_cardBounceIn:function()
        {
            this.flipbooks.cardBounceIn.play( this["top card"] );
            this.flipbooks.cardBounceIn.play( this["bottom card"] );
        },

        cue_cardBounceOut:function()
        {
            this.flipbooks.cardBounceIn.play( this["top card"], true );
            this.flipbooks.cardBounceIn.play( this["bottom card"], true );

        },

        //****************TODO MOVE TO CARD WIDGET****************//
        enableButtons:function( isEnabled )
        {
            this.no.setEnabled( isEnabled );
            this.yes.setEnabled( isEnabled );
        },

        /**
         * Updates the stimuli based on a passsed in value. Either the top or the bottom is shown,
         * and the other is given an emptystring.
         *
         * @param {string} value The value to show.
         * @param {boolean} isTop If true, we show the value in the top, and hide the bottom. If false, the opposite.
         */
        updateStimuli:function( topString, bottomString, bottomColor, correct, practiceType, isGuided )
        {
            if ( !this.model.persistentData.isPracticeMode )
            {
                this[ "prompt" ].adjustsFontSizeToFitWidth = false;
                this[ "prompt" ].setString( localize( "DefaultPromptLabel" ));
            }

            if (lumosity.game.locale == "pt")  // Portuguese will have a smaller font; thanks a lot, vermelho
            {
                this[ "top stimuli" ].setInitialFontSize(this._pt_text_size);
                this[ "bottom stimuli" ].setInitialFontSize(this._pt_text_size);
            } else {
                this[ "top stimuli" ].setInitialFontSize(this._non_pt_text_size);
                this[ "bottom stimuli" ].setInitialFontSize(this._non_pt_text_size);
            }

            this[ "top stimuli" ].setDimensions( cc.size(0, 0) );
            this[ "bottom stimuli" ].setDimensions( cc.size(0, 0) );
            this[ "top stimuli" ].setString( localize( topString, "Default" ) );
            this[ "bottom stimuli" ].setString( localize( bottomString, "Default" ) );
            this[ "bottom stimuli" ].setColor( cc.c4b( bottomColor.Red, bottomColor.Green, bottomColor.Blue, bottomColor.Alpha ) );
        },

        enableButtonsToTrue: function()
        {
            this.enableButtons( true );
        },

        //@Override
        preload:function()
        {
            this._super();
            this.flipbooks.cardBounceIn = new GameWidgetFlipbook( "cardBounceIn", "card/cardAnimIn{0}.png", 11 );
            this.flipbooks[ "resetCards" ] = new GameWidgetFlipbook( "resetCards", "card/cardAnimIn{0}.png", 1 );
        },


        onRestart: function()
        {
            this._super();

            this.flipbooks[ "resetCards" ].play( this["top card"] );
            this.flipbooks[ "resetCards" ].play( this["bottom card"] );
        },


        //@Override
        _createViews:function()
        {

        },

        setHintVisibility:function ( visibile )
        {
            this.letterHint.setVisible( visibile );
            this.numberHint.setVisible( visibile );
        },

        showHints:function ( animated )
        {
            this.letterHint.show( animated );
            this.numberHint.show( animated );
        },

        hideHints:function ( animated )
        {
            this.letterHint.hide( animated );
            this.numberHint.hide( animated );
        },

        //@Override
        layout_readingData: function( nodeName, nodeData )
        {
            var node = this._super( nodeName, nodeData );
            if ( node )
            {
                return node;
            }

            switch( nodeName )
            {
                case "hintTextColor":
                    this.hintColor = nodeData.textData.color;
                    this.hintFont = lu.getFontLocation( nodeData.textData.face );
                    return false;
            }

        },

        //@Override
        layout_creatingNode: function( nodeName, nodeData )
        {
            var node = this._super( nodeName, nodeData );
            if ( node )
            {
                return node;
            }

            switch( nodeName )
            {
                case "top card":
                    return this._createSprite( "card/cardAnimIn000.png" );
                case "bottom card":
                    return this._createSprite( "card/cardAnimIn000.png" );
                case "yes":
                    return this._createButton( "buttons/game_buttonInputUp.png", "buttons/game_buttonInputDown.png", "yes", localize ( "Yes_Label" ),
                                                this._buttonFont, this._buttonTextSize, this._blueTextColor, this._buttonTextDownColor,
                                                nodeData[ LayoutLoader.sizeKey ] );
                case "no":
                    return this._createButton( "buttons/game_buttonInputUp.png", "buttons/game_buttonInputDown.png", "no",  localize ( "No_Label" ),
                                                this._buttonFont, this._buttonTextSize, this._blueTextColor, this._buttonTextDownColor,
                                                nodeData[ LayoutLoader.sizeKey ] );
                case "hudBackgroundColor":
                    this.hud.setBackgroundColor( nodeData.textData.color );
                    return false;
                case "hudTextColor":
                    this.hud.setTextColor( nodeData.textData.color );
                    return false;
                case "letterHint":
                    return new GameWidgetHint( HintOrientation.Up, localize( "BottomHint" ), this.hintColor, this.hintFont );
                case "numberHint":
                    return new GameWidgetHint( HintOrientation.Down, localize( "TopHint" ), this.hintColor, this.hintFont );
            }

            return null;
        }

    } );
}() );
