/*global cc*/
/*global ll*/
/*global require*/


var GameWidget;
(function()
{
    "use strict";

    require( "src/Utility/LumosityLogger.js" );

    GameWidget = cc.Node.extend(
    {
        /** @lends GameWidget */

        /** An object that should respond to 'callback' calls. */
        _delegate: null,

        /** Array of buttons in this widget. */
        _buttons: [],

        /**
         *  A GameWidget is a view that contains functionality we may want to use across multiple game views.
         *  Since it is a view, it should have no controller or model information.
         *
         *  @constructs
         */
        ctor:function()
        {
            this._super();
            this.init();

            this.setAnchorPoint( cc.p( 0.5, 0.5 ) );

            this._buttons = [];

            this.nodeName = this.getType();
            this.nodeType = this.nodeName;
        },

        preload:function()
        {
            //implement in subclasses
        },

        /**
         *  Most things will use this type to decide where and how to store the widget.
         *  As such, the type should always be unique. By default, this will just use
         *  the name of the actual class.
         *
         *  @returns {string} The type identifier of this widget.
         */
        getType:function()
        {
            return "GameWidget";
        },

        /**
         * Sets new delegate for the widget.
         *
         * @param delegate
         */
        setDelegate: function( delegate )
        {
            this._delegate = delegate;
        },

        /**
         * Returns array of buttons in this widget.
         *
         * @returns {*}
         */
        getButtons: function()
        {
            return this._buttons;
        },


        /**
         * Called whenever game is paused.
         */
        onPause: function()
        {
            ll.verbose( this.getType(), "onPause" );
        },

        /**
         * Called whenever game is unpaused (resumed).
         */
        onResume: function()
        {
            ll.verbose( this.getType(), "onResume" );
        },

        /**
         * Called whenever game is restarted.
         */
        onRestart: function()
        {
            ll.verbose( this.getType(), "onRestart" );
        }
    });

}());

