/*global cc*/
/*global require*/
/*global GameWidget*/
/*global _*/
/*global LumosityUtils*/

require( "src/Game/Widget/GameWidget.js" );
require( "src/Utility/underscore.js" );
require( "src/Utility/LumosityUtils.js" );

//TODO - needs to be much more modular
//TODO - remove all the hardcoded values
var GameWidgetFlipbook = GameWidget.extend(
{
    /** @lends GameWidgetFlipbook.prototype */
    
    /** The name of the cc.AnimationCache that we will be storing all our frame data in. */
    cacheName: "",


    /**
     * Returns Widget type as a string.
     *
     * @returns {string}
     */
    getType: function()
    {
        return "GameWidgetFlipbook";
    },

    
    /**
     *  A GameWidgetFlipbook is a essentially a wrapper for cc.AnimationCache that knows how to read
     *  a flipbook from our animation data. You can pass in an image location and count to automatically
     *  use index-based searching to pull the entire flipbook.
     *
     *  @constructs
     *  @param {string} name This is a unique name that is used with cc.AnimationCache to decide how to store and retrieve images.
     *  @param {string} fileNameFormat The format for the image files. Will be used to finding the images. TODO - document better
     *  @param {number} pngCount How many images we're going to load into the flipbook.
     */
    ctor:function( name, fileNameFormat, pngCount )
    {
        this._super();

        this._buttons = [];
        
        this.cacheName = name;
        
        var frames = _.map( _.range( 0, pngCount ), function( fileNameFormat, n ) { return fileNameFormat.format( LumosityUtils.padWithLeadingZeroes( n, 3 ) ); }.bind( this, fileNameFormat ) );
        var anim = cc.Animation.create( _.map( frames, this.createAnimationFrame ), 0.033, 1 );
        cc.AnimationCache.getInstance().addAnimation( anim, this.cacheName );
    },

    getAnimationCache:function()
    {
        return cc.AnimationCache.getInstance().getAnimation( this.cacheName );
    },

    play:function( sprite, reverse, repeatNumber )
    {
        if ( sprite )
        {
            var action = cc.Animate.create( this.getAnimationCache() );
            if ( reverse )
            {
                action = action.reverse();
            }
            if ( repeatNumber )
            {
                action = cc.Repeat.create(action, repeatNumber);
            }
            sprite.runAction( action );
        }
    },

    stopAnimation: function( sprite )
    {
        sprite.stopAllActions();

        var animation = this.getAnimationCache();
        if( !animation )
        {
            return;
        }

        var frames = animation.getFrames();
        var framesLength = frames.length;
        if( framesLength <= 0 )
        {
            return;
        }

        var spriteFrame = frames[ framesLength - 1 ].getSpriteFrame();
        sprite.setDisplayFrame( spriteFrame );
    },
    
    createAnimationFrame:function( pngName )
    {
        var texture = cc.TextureCache.getInstance().addImage( pngName );
        var textureSize = texture.getContentSize();
        var spriteFrame = cc.SpriteFrame.createWithTexture( texture, cc.rect( 0, 0, textureSize.width, textureSize.height ) );
        var animFrame = new cc.AnimationFrame();
        animFrame.initWithSpriteFrame( spriteFrame, 1, null );
        return animFrame;
    },
    
    destroy:function()
    {
        cc.AnimationCache.getInstance().getAnimation( this.cacheName );
    }
});