/*global cc*/
/*global require*/
/*global lu*/
/*global ll*/
/*global audio*/
/*global localize*/
/*global GameWidget*/
/*global LumositySprite*/
/*global LumositySwipeView*/
/*global LumosityButton*/
/*global LumosityButtonState*/
/*global LayoutLoader*/
/*global AnimationController*/
/*global LumosityAspectNode*/

require( "src/Game/Widget/GameWidget.js" );
require( "src/View/LumositySprite.js" );
require( "src/View/LumositySwipeView.js" );
require( "src/View/LumosityButton.js" );
require( "src/View/LumosityAspectNode.js" );
require( "src/Animation/LayoutLoader.js" );

var GameWidgetHowToPlay = GameWidget.extend(
{
    /** @lends HowToPlayWidget.prototype */

    /** Game Model */
    _model: null,

    /** Container for all of the How To Play pages */
    _cardView: null,

    /** Buttons **/
    _rightButtonContainer: null,
    _rightButton: null,
    _leftButtonContainer: null,
    _leftButton: null,
    _bottomButtonContainer: null,
    _bottomButtonsAnimatedContainer: null,
    _bottomButtonsAspectNode: null,
    _playButton: null,
    _backButton: null,

    _buttonParent: null,
    _buttonsVisible: false,
    _leftButtonVisible: false,
    _rightButtonVisible: false,

    _zOrder: 100,
    _sideButtonSize: cc.size ( 40, 120 ),
    _bottomButtonsContainerSize: cc.size ( 640, 120 ),
    _playButtonSize: cc.size ( 510, 120 ),
    _backButtonSize: cc.size ( 120, 120 ),


    /**
     *  Most things will use this type to decide where and how to store the widget.
     *  As such, the type should always be unique. By default, this will just use
     *  the name of the actual class.
     *
     *  @returns {string} The type identifier of this widget.
     */
    getType:function()
    {
        return "GameWidgetHowToPlay";
    },

    
    ctor: function( model, settings, delegate, buttonParent, preload )
    {
        this._super();

        this._model = model;

        this._delegate = delegate;

        this._buttonParent = buttonParent;
        this._buttonsVisible = false;
        this._leftButtonVisible = false;
        this._rightButtonVisible = false;
        this._buttons = [];

        // Get screensize
        var screenSize = cc.Director.getInstance().getWinSize();

        this._designSize = screenSize;
        this.setContentSize( this._designSize );
        
        // XXX this.settings is undefined and LumositySwipeView does not use settings.
        this._cardView = new LumositySwipeView( this.settings, this, false );
        this._cardView.nodeName = "HowToPlay:CardView";
        this.addChild( this._cardView );

        // Add buttons
        var i;
        var buttons = this._cardView.getButtons();
        for ( i = 0; i < buttons.length; i++ )
        {
            this._buttons.push( buttons[i] );
        }

        // Create buttons
        this._createButtons();

        // Setup layout
        this._loadLayout();

        // Create pages
        this._createPages( settings );

        // Setup animation controller
        this._createAnimationController();

        // Fix play button width
        this._fixPlayButtonWidth();
    },

    /**
     * Creates buttons that can be used to navigate between pages.
     *
     * @private
     */
    _createButtons: function()
    {
        // Image names
        var buttonUpImageName = null;
        var buttonDownImageName = null;
        var buttonDisabledImageName = null;
        // Image objects
        var buttonUpImage = null;
        var buttonDownImage = null;
        var buttonDisabledImage = null;

        // Settings for right button
        buttonUpImageName = "howToPlays/howToPlayButtonRightUp.png";
        buttonDownImageName = "howToPlays/howToPlayButtonRightDown.png";
        buttonDisabledImageName = "howToPlays/howToPlayButtonRightUp.png";

        buttonUpImage = cc.Sprite.create( buttonUpImageName );
        buttonDownImage = cc.Sprite.create( buttonDownImageName );
        buttonDisabledImage = cc.Sprite.create( buttonDisabledImageName );

        // Right Button Container
        this._rightButtonContainer = cc.Node.create();
        this._rightButtonContainer.nodeName = "HowToPlay:RightButtonContainer";
        this._rightButtonContainer.setContentSize( this._sideButtonSize );
        this._rightButtonContainer.setAnchorPoint( cc.p( 1, 0.5 ) );
        this._rightButtonContainer.setPosition( cc.p( this._designSize.width / 2, 0 ) );
        if ( this._buttonParent )
        {
            this._buttonParent.addChild( this._rightButtonContainer, this._zOrder );
        }
        else
        {
            this.addChild( this._rightButtonContainer, this._zOrder );
        }

        // Create Right button
        this._rightButton = new LumosityButton();
        this._rightButton.setBackgroundImageForState( buttonUpImage, LumosityButtonState.ButtonStateNormal );
        this._rightButton.setBackgroundImageForState( buttonDownImage, LumosityButtonState.ButtonStateSelected );
        this._rightButton.setBackgroundImageForState( buttonDisabledImage, LumosityButtonState.ButtonStateDisabled );
        this._rightButton.setTarget( this );
        this._rightButton.setReleasedCallback( this._onRightButtonTapped );
        this._rightButton.setAnchorPoint( cc.p( 1, 0.5 ) );
        this._rightButton.setPosition( cc.p( this._sideButtonSize.width, this._sideButtonSize.height / 2 ) );
        // buttons are very thin, so expand invisible hit region a little toward the center
        this._rightButton.setHitRegion(
            [
                cc.p( this._sideButtonSize.width * -1,  this._sideButtonSize.height * 0 ),
                cc.p( this._sideButtonSize.width * -1,  this._sideButtonSize.height * 1 ),
                cc.p( this._sideButtonSize.width * 1,   this._sideButtonSize.height * 1 ),
                cc.p( this._sideButtonSize.width * 1,   this._sideButtonSize.height * 0 )
            ]
        );
        this._rightButton.nodeName = "HowToPlay:RightButton";
        this._rightButtonContainer.addChild( this._rightButton );
        this._buttons.push( this._rightButton );



        // Settings for left button
        buttonUpImageName = "howToPlays/howToPlayButtonLeftUp.png";
        buttonDownImageName = "howToPlays/howToPlayButtonLeftDown.png";
        buttonDisabledImageName = "howToPlays/howToPlayButtonLeftUp.png";

        buttonUpImage = cc.Sprite.create( buttonUpImageName );
        buttonDownImage = cc.Sprite.create( buttonDownImageName );
        buttonDisabledImage = cc.Sprite.create( buttonDisabledImageName );

        // Left Button Container
        this._leftButtonContainer = cc.Node.create();
        this._leftButtonContainer.nodeName = "HowToPlay:LeftButtonContainer";
        this._leftButtonContainer.setContentSize( this._sideButtonSize );
        this._leftButtonContainer.setAnchorPoint( cc.p( 0, 0.5 ) );
        this._leftButtonContainer.setPosition( cc.p( 0, this._designSize.height / 2 ) );
        if ( this._buttonParent )
        {
            this._buttonParent.addChild( this._leftButtonContainer, this._zOrder );
        }
        else
        {
            this.addChild( this._leftButtonContainer, this._zOrder );
        }

        // Create left button
        this._leftButton = new LumosityButton();
        this._leftButton.setBackgroundImageForState( buttonUpImage, LumosityButtonState.ButtonStateNormal );
        this._leftButton.setBackgroundImageForState( buttonDownImage, LumosityButtonState.ButtonStateSelected );
        this._leftButton.setBackgroundImageForState( buttonDisabledImage, LumosityButtonState.ButtonStateDisabled );
        this._leftButton.setTarget( this );
        this._leftButton.setReleasedCallback( this._onLeftButtonTapped );
        this._leftButton.setAnchorPoint( cc.p( 0.5, 0.5 ) );
        this._leftButton.setPosition( cc.p( this._sideButtonSize.width / 2, this._sideButtonSize.height / 2 ) );
        // buttons are very thin, so expand invisible hit region a little toward the center
        this._leftButton.setHitRegion(
            [
                cc.p( this._sideButtonSize.width * 0,   this._sideButtonSize.height * 0 ),
                cc.p( this._sideButtonSize.width * 0,   this._sideButtonSize.height * 1 ),
                cc.p( this._sideButtonSize.width * 2,   this._sideButtonSize.height * 1 ),
                cc.p( this._sideButtonSize.width * 2,   this._sideButtonSize.height * 0 )
            ]
        );
        this._leftButton.nodeName = "HowToPlay:LeftButton";
        this._leftButtonContainer.addChild( this._leftButton );
        this._buttons.push( this._leftButton );


        // Settings for play button
        var font = "fonts/MuseoSans-500.ttf";
        var fontSize = 48;
        var fontColor = cc.c3b( 70, 191, 235 );
        var fontDownColor = cc.c3b( 255, 255, 255 );

        buttonUpImage = cc.Sprite.create( "buttons/game_howToPlay_playUp.png" );
        buttonDownImage = cc.Sprite.create( "buttons/game_howToPlay_playDown.png" );
        buttonDisabledImage = cc.Sprite.create( "buttons/game_howToPlay_playUp.png" );

        // Play Button Container
        this._bottomButtonContainer = cc.Node.create();
        this._bottomButtonContainer.nodeName = "HowToPlay:PlayNowButtonContainer";
        this._bottomButtonContainer.setContentSize( this._bottomButtonsContainerSize );
        this._bottomButtonContainer.setAnchorPoint( cc.p( 0.5, 0 ) );
        this._bottomButtonContainer.setPosition( cc.p( this._designSize.width / 2, 0 ) );
        if ( this._buttonParent )
        {
            this._buttonParent.addChild( this._bottomButtonContainer, this._zOrder );
        }
        else
        {
            this.addChild( this._bottomButtonContainer, this._zOrder );
        }

        // Bottom Buttons Animated Container
        this._bottomButtonsAnimatedContainer = cc.Node.create();
        this._bottomButtonsAnimatedContainer.nodeName = "HowToPlay:PlayNowButtonContainer";
        this._bottomButtonsAnimatedContainer.setContentSize( this._bottomButtonsContainerSize );
        this._bottomButtonsAnimatedContainer.setAnchorPoint( cc.p( 0.5, 1 ) );
        this._bottomButtonsAnimatedContainer.setPosition( cc.p( this._bottomButtonsContainerSize.width / 2, this._bottomButtonsContainerSize.height ) );
        this._bottomButtonContainer.addChild( this._bottomButtonsAnimatedContainer );


        // Bottom Buttons Animated Container
        this._bottomButtonsAspectNode = new LumosityAspectNode ( true );
        this._bottomButtonsAspectNode.nodeName = "HowToPlay:BottomButtonsAspectNode";
        this._bottomButtonsAspectNode.setAnchorPoint( cc.p( 0, 0 ) );
        this._bottomButtonsAspectNode.setPosition( cc.p( 0, 0 ) );
        this._bottomButtonsAnimatedContainer.addChild( this._bottomButtonsAspectNode );


        var playButtonLocalizedText = localize( "HowToPlay_PlayButtonText", "Play" );

        // Play Button
        this._playButton = new LumosityButton();
        this._playButton.setBackgroundImageForState( buttonUpImage, LumosityButtonState.ButtonStateNormal );
        this._playButton.setBackgroundImageForState( buttonDownImage, LumosityButtonState.ButtonStateSelected );
        this._playButton.setBackgroundImageForState( buttonDisabledImage, LumosityButtonState.ButtonStateDisabled );
        this._playButton.setTitleForState( playButtonLocalizedText, LumosityButtonState.ButtonStateNormal );
        this._playButton.setTitleColorForState( fontColor, LumosityButtonState.ButtonStateNormal );
        this._playButton.setTitleColorForState( fontDownColor, LumosityButtonState.ButtonStateSelected );
        this._playButton.setTitleColorForState( fontColor, LumosityButtonState.ButtonStateHighlighted );
        this._playButton.setTitleColorForState( fontColor, LumosityButtonState.ButtonStateDisabled );
        this._playButton.titleLabel.setFontName( font );
        this._playButton.titleLabel.setInitialFontSize( fontSize );
        this._playButton.titleLabel.setPosition( cc.p( this._playButtonSize.width / 2, this._playButtonSize.height / 2 ) );
        this._playButton.nodeName = "HowToPlay:PlayNowButton";
        this._playButton.setTarget( this );
        this._playButton.setReleasedCallback( this._playNowButtonTapped );
        this._playButton.setAnchorPoint( cc.p( 1, 0 ) );
        this._playButton.setPosition( cc.p( this._bottomButtonsContainerSize.width, 0 ) );
        this._playButton.setContentSize( this._playButtonSize );
        this._bottomButtonsAspectNode.addChild( this._playButton );
        this._buttons.push( this._playButton );



        // Settings for back buttons
        buttonUpImage = cc.Sprite.create( "buttons/game_howToPlay_backUp.png" );
        buttonDownImage = cc.Sprite.create( "buttons/game_howToPlay_backDown.png" );
        buttonDisabledImage = cc.Sprite.create( "buttons/game_howToPlay_backUp.png" );

        // Back Button
        this._backButton = new LumosityButton();
        this._backButton.setBackgroundImageForState( buttonUpImage, LumosityButtonState.ButtonStateNormal );
        this._backButton.setBackgroundImageForState( buttonDownImage, LumosityButtonState.ButtonStateSelected );
        this._backButton.setBackgroundImageForState( buttonDisabledImage, LumosityButtonState.ButtonStateDisabled );
        this._backButton.nodeName = "HowToPlay:BackButton";
        this._backButton.setTarget( this );
        this._backButton.setReleasedCallback( this._backNowButtonTapped );
        this._backButton.setAnchorPoint( cc.p( 0, 0 ) );
        this._backButton.setPosition( cc.p( 0, 0 ) );
        this._backButton.setContentSize( this._backButtonSize );
        this._bottomButtonsAspectNode.addChild( this._backButton );
        this._buttons.push( this._backButton );

        // Disable buttons on start
        this._setButtonsEnabled( false );
    },

    _playNowButtonTapped: function()
    {
        if ( this._delegate && this._delegate.howToPlay_playNowTapped )
        {
            this._delegate.howToPlay_playNowTapped();
        }
    },

    _onRightButtonTapped: function()
    {
        this.nextPage();
    },

    _onLeftButtonTapped: function()
    {
        this.previousPage();
    },

    _backNowButtonTapped: function()
    {
        if ( this._delegate && this._delegate.howToPlay_backNowButtonTapped )
        {
            this._delegate.howToPlay_backNowButtonTapped();
        }
    },



    _fixPlayButtonWidth: function()
    {
        var newButtonPositionX = 130;

        // Calculate play button width
        var playButtonContentSize = this._playButton.getContentSize();
        var totalDeviceWidth = this._bottomButtonContainer.getContentSize().width * this._bottomButtonContainer.getScaleX();
        var bottomContainerScaleY = Math.min( this._bottomButtonContainer.getScaleX(), this._bottomButtonContainer.getScaleY() );
        playButtonContentSize.width = totalDeviceWidth / bottomContainerScaleY - newButtonPositionX;

        this._playButton.setContentSize( playButtonContentSize );
        this._playButton.setAnchorPoint( cc.p( 0, 0 ) );
        this._playButton.setPosition( cc.p ( newButtonPositionX, 0 ) );

        var buttonUpImage = cc.Sprite.create( "buttons/game_howToPlay_playUp.png" );
        var buttonDownImage = cc.Sprite.create( "buttons/game_howToPlay_playDown.png" );
        var buttonDisabledImage = cc.Sprite.create( "buttons/game_howToPlay_playUp.png" );

        // Play Button
        this._playButton.setBackgroundImageForState( buttonUpImage, LumosityButtonState.ButtonStateNormal );
        this._playButton.setBackgroundImageForState( buttonDownImage, LumosityButtonState.ButtonStateSelected );
        this._playButton.setBackgroundImageForState( buttonDisabledImage, LumosityButtonState.ButtonStateDisabled );
        this._playButton.setContentSize( playButtonContentSize );

        this._playButton.titleLabel.setPosition( cc.p( playButtonContentSize.width / 2, playButtonContentSize.height / 2 ) );
    },



    _setButtonsEnabled: function( isEnabled )
    {
        var i;
        var button = null;
        for ( i = 0; i < this._buttons.length; i++ )
        {
            button = this._buttons[ i ];
            button.setEnabled( isEnabled );
        }
    },

    /**
     * Moves swipe view to the next card, hides next button if last card.
     */
    nextPage: function()
    {
        if ( this._cardView )
        {
            if( this._cardView.isTransitioning() === true ) { return; }

            this._cardView.nextPage();

            if( this._cardView._transitioning === true )
            {
                this._setButtonsEnabled( false );
            }

            if ( this._cardView.showLeftButton() && !this._leftButtonVisible )
            {
                this.animationController.play( "leftButtonIn", (function() { this._leftButtonVisible = true; }).bind( this ), this );
            }

            if ( !this._cardView.showRightButton() && this._rightButtonVisible )
            {
                this.animationController.play( "rightButtonOut", (function() { this._rightButtonVisible = false; }).bind( this ), this );
            }
        }
    },

    /**
     * Moves swipe view to the previous card, hides previous button if last card.
     */
    previousPage: function()
    {
        if ( this._cardView )
        {
            if( this._cardView.isTransitioning() === true ) { return; }

            this._cardView.previousPage();

            if( this._cardView._transitioning === true )
            {
                this._setButtonsEnabled( false );
            }

            if ( this._cardView.showRightButton() && !this._rightButtonVisible )
            {
                this.animationController.play( "rightButtonIn", (function() { this._rightButtonVisible = true; }).bind( this ), this );
            }

            if ( !this._cardView.showLeftButton() && this._leftButtonVisible )
            {
                this.animationController.play( "leftButtonOut", (function() { this._leftButtonVisible = false; }).bind( this ), this );
            }
        }
    },

    getCurrentPageNumber: function()
    {
        return this._cardView.getCurrentPageNumber();
    },

    getPageCount: function()
    {
        return this._cardView.getPageCount();
    },

    getPageText: function()
    {
        var currentPageNumber = this.getCurrentPageNumber();
        var cardView = this._cardView;
        var page = null;

        if( !cardView || !cardView._pages )
        {
            return "";
        }

        page = cardView.getCurrentPage();
        if( !page || !page.instructions )
        {
            return "";
        }

        return page.instructions.getString();
    },

    /**
     * Callback from SwipeView for when a page gets centered.
     */
    swipeView_onCenterPage: function()
    {
        if ( this._cardView.isTransitioning() === true ) { return; }

        this._setButtonsEnabled( true );
    },

    /**
     * Callback from SwipeView for when left side button should be visible or not.
     * @param show
     */
    swipeView_showLeftButton: function ( show )
    {

    },

    /**
     * Callback from SwipeView for when right side button should be visible or not.
     * @param show
     */
    swipeView_showRightButton: function ( show )
    {

    },

    
    _loadLayout: function()
    {
        LayoutLoader.loadLayoutFile( "howToPlay_layout.json", this, this, this );
    },

    layout_creatingNode: function( nodeName, nodeData )
    {
        switch( nodeName )
        {
            case "bottomButtonsContainer": return this._bottomButtonContainer;
            case "bottomButtonsAnimatedContainer": return this._bottomButtonsAnimatedContainer;

            //case "backButtonContainer": return this._backButtonContainer;
            //case "backButton": return this._backButton;

            case "leftContainer": return this._leftButtonContainer;
            case "leftButton": return this._leftButton;

            case "rightContainer": return this._rightButtonContainer;
            case "rightButton": return this._rightButton;

            case "swipeViewContainer": return null;
            case "swipeView": return this._cardView;
        }


        return false;
    },


    layout_addingNodeToParent: function( nodeName, nodeData, node, parentNode )
    {
        if ( nodeName === "bottomButtonsContainer" || nodeName === "bottomButtonsAnimatedContainer" || nodeName === "playNowButton" || nodeName === "backButtonContainer" || nodeName === "backButton"
            || nodeName === "leftContainer" || nodeName === "leftButton" || nodeName === "rightContainer" || nodeName === "rightButton" )
        {
            return false;
        }

        return true;
    },


    /**
     *  Creates this view's AnimationController from the game's settings file. That way of doing it
     *  is temporary, later we will want to pass the animation file name into this function. The
     *  controller is also automatically created with the shared game animation data.
     */
    _createAnimationController:function()
    {
        this.animationController = new AnimationController( "howToPlay_animations.json", this, this, [ this ] );

        this[ "bottomButtonsAnimatedContainer" ] = this._bottomButtonsAnimatedContainer;
        this[ "leftButton" ] = this._leftButton;
        this[ "rightButton" ] = this._rightButton;
        //this[ "backButton" ] = this._backButton;
    },

    _createPages: function( settings )
    {
        var howToPlayLayouts = settings.howToPlayLayouts;
        if ( howToPlayLayouts && howToPlayLayouts.length > 0 )
        {
            var layoutFile;
            var i;
            for ( i = 0; i < howToPlayLayouts.length; i++ )
            {
                layoutFile = howToPlayLayouts[ i ];
                this._createPage( layoutFile );
            }
        }
        else
        {
            ll.error( "GameWidgetHowToPlay", "_createAnimationController", "Error! No How To Play layouts found." );
        }
    },

    _createTemplatePage: function()
    {
        // Create new page
        var pageSize = cc.size( 530, 740 );
        var newPage = cc.Node.create();
        newPage.nodeName = "Page";
        newPage.setAnchorPoint( cc.p( 0.5, 0.5 ) );
        newPage.setContentSize( pageSize );
        this._cardView.addPage( newPage );

        // Might be useful for debugging
        /*
        var color = cc.c3b( LumosityUtils.getRandomInt( 0, 255 ), LumosityUtils.getRandomInt(0, 255 ), LumosityUtils.getRandomInt(0, 255 ) );
        var coloredBackground = LumosityUtils.createRectangularSprite( pageSize, color );
        coloredBackground.nodeName = "coloredBackground";
        coloredBackground.setAnchorPoint( cc.p( 0.5, 0.5 ) );
        coloredBackground.setPosition( cc.p( pageSize.width / 2, pageSize.height / 2 ) );
        newPage.addChild( coloredBackground );
        */

        // Load template layout
        LayoutLoader.loadLayoutFile( "howToPlay_page_layout.json", newPage, newPage, this._templatePageLayoutCreatingNode.bind( this ) );

        // Set page number (not an index, starts at 1)
        newPage.pageNumber = this._cardView.getPageCount();
        if ( newPage.number )
        {
            newPage.number.setString( newPage.pageNumber + "." );
        }

        return newPage;
    },

    _templatePageLayoutCreatingNode:function( nodeName, nodeData )
    {
        if ( nodeName === "slideDotSpacing" )
        {
            if ( this._cardView )
            {
                this._cardView.setDotSpacing( nodeData.size.width, nodeData.size.height );
            }
        }

        return null;
    },

    _createPage: function( layoutFile )
    {
        this._currentPage = undefined;

        var newPage = this._createTemplatePage();
        if ( newPage )
        {
            // Set proper text
            // if ( newPage.instructions )
            // {
            //     if ( newPage.instructions.textBoxSize )
            //     {
            //         // Find the scale for the parent
            //         var scaleX = this._cardView.getScaleX();
            // 
            //         // Get instructions current box size
            //         var dimesions = newPage.instructions.textBoxSize;
            //         // Scale width by the scale of the container
            //         dimesions.width = dimesions.width * scaleX;
            //         // We don't care about height limit, so use 0
            //         dimesions.height = 0;
            // 
            //         // Set new dimensions
            //         newPage.instructions.setDimensions( dimesions );
            //     }
            // }

            // Layout loader needs to know which page it is modifying.
            this._currentPage = newPage;

            // Load template layout
            LayoutLoader.loadLayoutFile( LumosityGlobals.getLocalizedPath( layoutFile ), newPage.screenshot, newPage.screenshot, this._pageLayoutCreatingNode.bind( this ) );
        }

        this._currentPage = undefined;
    },

    _pageLayoutCreatingNode: function( nodeName, nodeData )
    {
        var localizedText = null;

        if ( nodeName === "instructionText" )
        {
            if ( this._currentPage && this._currentPage.instructions )
            {
                localizedText = localize( nodeData[ LayoutLoader.textDataKey ][ LayoutLoader.textKeyKey ], nodeData[ LayoutLoader.textDataKey ][ LayoutLoader.textStringKey ] );
                cc.log( "instructionText: " + this._currentPage + " " + localizedText );
                this._currentPage.instructions.setString( localizedText );
            }
            return false;
        }
        return null;
    },

    slideButtons: function( inToView )
    {
        if ( inToView )
        {
            //
            // Reset the view to the initial card.
            //
            this.setPageNumber( 0, true );
            //
            //
            //
            if ( !this._buttonsVisible )
            {
                if ( this.animationController )
                {
                    this.animationController.play( "playButtonIn", this._playButtonInFinished, this );

                    if ( this._cardView.showLeftButton() )
                    {
                        this.animationController.play( "leftButtonIn", (function() { this._leftButtonVisible = true; }).bind( this ), this );
                    }
                    if ( this._cardView.showRightButton() )
                    {
                        this.animationController.play( "rightButtonIn",  (function() { this._rightButtonVisible = true; }).bind( this ), this );
                    }
                }
            }
        }
        else
        {
            if ( this._buttonsVisible )
            {
                if ( this.animationController )
                {
                    // Disable buttons during transitions & while offscreen
                    this._setButtonsEnabled ( false );

                    this.animationController.play( "playButtonOut", this._playButtonOutFinished, this );

                    if ( this._cardView.showLeftButton() )
                    {
                        this.animationController.play( "leftButtonOut", (function() { this._leftButtonVisible = false; }).bind( this ), this );
                    }
                    if ( this._cardView.showRightButton() )
                    {
                        this.animationController.play( "rightButtonOut",  (function() { this._rightButtonVisible = false; }).bind( this ), this );
                    }
                }
            }
        }
    },

    _playButtonInFinished: function()
    {
        this._buttonsVisible = true;
        this._setButtonsEnabled ( true );
    },

    _playButtonOutFinished: function()
    {
        this._buttonsVisible = false;
    },

    setPageNumber: function( pageNumber, instant )
    {
        this._cardView.setPage( pageNumber, instant );
    }

});
