/*global cc*/
/*global Game*/
/*global require*/
/*global LumosityUtils*/
/*global lu*/
/*global EbbAndFlowView*/
/*global EbbAndFlowModel*/
/*global LumosityGlobals*/
/*global InteractionType*/
/*global EbbAndFlowSettings*/
/*global CardinalDirections*/
/*global GameComponentScoreMultiplier*/
/*global metadataAggregateKeys*/
/*global metadataTrialKeys*/
/*global localize*/

var EbbAndFlow;
(function()
{
    "use strict";

    require( "src/Game/Base/Game.js" );
    require( "src/Game/EbbAndFlowView.js" );
    require( "src/Game/EbbAndFlowModel.js" );
    require( "src/Utility/LumosityUtils.js" );
    require( "src/Game/EbbAndFlowSettings.js" );
    require( "src/Game/Component/GameComponentScoreMultiplier.js" );

    var switchProbabilityIncrement = 0.05;
    var minimumBeforeSwitch        = 1;

    EbbAndFlow = Game.extend(
    {
        //SHED'S TODO - this stuff should all be in your model, to avoid stuff like MOBGAMES-1436
        _switchProbability: null,
        _consecutiveTurns:  null,
        _gameModePointing:  null,
        _pointing:          null,
        _moving:            null,
        _score:             null,
        _ignoreSwipes:      null,
        _gameModeSwitched:  true,
        _lastInteractionWasCorrect: false,

        //
        // Scripted experience for tutorial (practice) mode.
        //
        _tutorialIndex: 0,

        _numTriesPointing:   0,
        _numCorrectPointing: 0,
        _numTriesMoving:     0,
        _numCorrectMoving:   0,

        _movingResponseTimes: [],
        _pointingResponseTimes: [],

        ctor:function()
        {
            this._super();
            this._init();
        },

        _init: function()
        {
            this._initGameData();
            this._createRandomTutorialTrials();
        },

        _initGameData: function()
        {
            this._score             = 0;
            this._switchProbability = 0;
            this._consecutiveTurns  = 0;
            this._tutorialIndex     = 0;
            this._gameModePointing  = true;
            this._gameModeSwitched  = true;
            this._lastInteractionWasCorrect = false;

            this.view.setViewCallbackFunc( function()
            {
                var startingFitTestSession = !this.model.isGameRunning && this._fitTestModeOn && this._practicing && !this._performingFitTest;
                if( startingFitTestSession === true || ( this._practicing && !this._fitTestModeOn ) )
                {
                    this.view.animateLeavesFlying( this._gameModePointing );
                    this.view.setModeIndicator( ( this._gameModePointing ) ? 0 : 1 );
                    this.setIgnoreSwipes( false );
                    // this.onStartGameplay();
                }
            }.bind( this ) );

            /*
                This function is unnecessary (at the moment). If we find we need to do something
                in between tutorial and fit test modes this would be the appropriate place to put it.
             */
            this.view.setLeafCallbackFunc( function()
            {
                if( this.model.persistentData.isPracticeMode === true )
                {
                    //
                    //TODO: Special fit test stuff here (if you need).
                    //
                }
            }.bind( this ) );
        },

        _createRandomTutorialTrials: function()
        {
            //these start with what is in the settings, then we add to them
            this._pointingScript = lu.duplicateObject( this.settings.pointingScript );
            this._movingScript = lu.duplicateObject( this.settings.movingScript );
            this._modeScript = lu.duplicateObject( this.settings.modeScript );
            
            var index = 0;
            var limit = 6;
            var savedProbability = switchProbabilityIncrement;
            switchProbabilityIncrement = 0.07;
            for( index = 0; index < limit; index += 1 )
            {
                this.setupRegularTrial();
                this._pointingScript.push( this._pointing );
                this._movingScript.push( this._moving );
                this._modeScript.push( this._gameModePointing );
            }
            switchProbabilityIncrement = savedProbability;
        },

        onBegin: function()
        {
            this._super();
            this._fitTestModeOn     = LumosityUtils.isFitTestModeOn();
            this._performingFitTest = this.model.isFitTestMode;
            this._practicing        = this.model.persistentData.isPracticeMode;

            if ( this.model.persistentData.isPracticeMode && !LumosityUtils.isFitTestModeOn() )
            {
                // this.view._widgets[ GameWidgetHud.prototype.getType() ]._hideAll();

                delete this._components[ ( new GameComponentTimer( this ) ).getType() ];
            }
            else
            {
                if ( !this.model.persistentData.isPracticeMode )
                {
                    this._components = {};
                    this._createComponents();
                    this.view._widgets[ GameWidgetHud.prototype.getType() ].showScore();
                    this.view._widgets[ GameWidgetHud.prototype.getType() ].showTime();
                }
            }

            this._gameSetup();
        },

        _gameSetup: function()
        {
            this._initGameData();

            this.randomizeTrial();

            this.view.pointLeavesInDirection( this._pointing );
            this.view.moveLeavesInDirection( this._moving );

            this.timeRemaining = this.settings.gameLengthInSeconds;
            this.setIgnoreSwipes( true );
            this.model.correctTrials = 0;

            this._movingResponseTimes = [];
            this._pointingResponseTimes = [];
            this._movingResponseTimes.push( 0 );
            this._pointingResponseTimes.push( 0 );
        },

        randomizeTrial: function()
        {
            var endGuidedTutorial = Boolean(this._tutorialIndex > this._pointingScript.length - 1);
            var randomizedTutorial = Boolean(this._tutorialIndex > 4);
            if( this._fitTestModeOn === true || ( this._fitTestModeOn === false && this._practicing === true ) )
            {
                if( this._practicing === true && this._performingFitTest === false )
                {
                    this.setupTutorialTrial();

                    if( endGuidedTutorial === true && this._lastInteractionWasCorrect === true )
                    {
                        this.view.hidePromptLabel();
                        //
                        //NOTE: This ensures onGameplayEnd() gets called, but also ensures
                        //      I can minimize state-wrangling.
                        //
                        this.onComponentTimerEnd();
                    }
                    else if( randomizedTutorial === true )
                    {
                        if( this._lastInteractionWasCorrect === false )
                        {
                            this.view.setPromptLabelText( ( this._gameModePointing === true ) ? localize( "PointingPrompt" ) : localize( "MovingPrompt" ) );
                            this.view.showPromptLabel();
                        }
                        else
                        {
                            this.view.hidePromptLabel();
                        }
                    }
                    else if( this._gameModeSwitched === true )
                    {
                        this.view._animateTextChange( ( this._gameModePointing === true ) ? localize( "PointingPrompt" ) : localize( "MovingPrompt" ) );
                    }
                    return;
                }
                else if( this._performingFitTest === true && this._practicing === false )
                {
                    this.view.hidePromptLabel();
                    this.setupRegularTrial();
                }
            }
            else if( this._fitTestModeOn === false )
            {
                this.setupRegularTrial();
            }
        },

        setupTutorialTrial: function()
        {
            var previousGameMode = this._gameModePointing;
            if( this._tutorialIndex > 0 && this._lastInteractionWasCorrect === false )
            {
                return;
            }
            this._gameModePointing = this._modeScript[ this._tutorialIndex ];
            
            this.model.currentTrial[ metadataTrialKeys.trialSwitch ] = ( previousGameMode === this._gameModePointing ) ? "T" : "F";

            this._pointing = this._pointingScript[ this._tutorialIndex ];
            this._moving = this._movingScript[ this._tutorialIndex ];

            this._gameModeSwitched = ( previousGameMode === this._gameModePointing ) ? false : true;
            this._tutorialIndex += 1;
        },

        setupRegularTrial: function()
        {
            this._updateGameMode();
            this._pointing = CardinalDirections.getValues().getRandomElement();
            this._moving   = LumosityUtils.randomBool() ? this._pointing : LumosityUtils.otherElement( CardinalDirections.getValues(), [ this._pointing ] );
        },

        _updateGameMode: function()
        {
            var previousGameMode = this._gameModePointing;
            this._switchProbability += this._consecutiveTurns >= minimumBeforeSwitch ? switchProbabilityIncrement : 0;
            if( LumosityUtils.randomBool( this._switchProbability ) )
            {
                this._gameModePointing  = !this._gameModePointing;
                this._gameModeSwitched  = true;
                this._consecutiveTurns  = 1;
                this._switchProbability = 0;
            }
            else
            {
                this._gameModeSwitched  = false;
                this._consecutiveTurns += 1;
            }
            this.model.currentTrial[ metadataTrialKeys.trialSwitch ] = ( previousGameMode === this._gameModePointing ) ? "T" : "F";
        },

        onStartGameplay: function()
        {
            if ( !this.model.tutorialSkipped )
            {
                this._super();
                //
                //TODO: Ask Aaron about metadata reporting here and what specifically they want
                //      reported.
                //
                if( this._fitTestModeOn === false || this._practicing === false )
                {
                    this.model.metadata[ metadataAggregateKeys.boardLayout ] = this.view.getBoardLayout();
                    this.beginTrial();
                }
            }
        },
    
        playStartingAnimation: function()
        {
            if ( ( this.model.restartToPracticeMode || ( this.model.persistentData.isPracticeMode && this.model.isFitTestMode ) ) && this.view.hasAnimation( "practiceBegin" ) )
            {
                this.view.playAnimation( "practiceBegin", this.onStartGameplay );
            }
            else if ( this.model.persistentData.isPracticeMode && !this.model.isFitTestMode ) 
            {
                this.view.playAnimation( "startTutorial", this.onStartGameplay );
            }
            else
            {
                this.view.playAnimation( "gameBegin", this.onStartGameplay );
            }
        },

        playEndAnimation: function ()
        {
            if ( !this.model.persistentData.isPracticeMode )
            {
                var results = this.exportResults();
                var automatedTestReport = this.exportAutomatedTestReport();
                if( automatedTestReport )
                {
                    // Append automated test report if we have one.
                    results.automatedTestReport = automatedTestReport;
                }

                lumosity.game.report( JSON.stringify( results ) );
                this.view.playAnimation( "gameEnd", this.onFinish.bind( this, false ) );
            }
            else if ( LumosityUtils.isFitTestModeOn() )
            {
                this.view.playAnimation( "gameEnd", this.onStartFitTest );
            }
        },
        
        onRestart: function()
        {
            this._init();
            this._gameSetup();
            this._super();
        },

        setupNextTrial: function()
        {
            this.randomizeTrial();
            if( this.model.isGameRunning === true )
            {
                this.view.changeDirections( this._gameModePointing, this._pointing, this._moving );
            }
        },

        /**
         * Overrides super method to handle optional swipe interactions.
         * 
         * @param  {Interaction} interaction  The interaction that took place (expects Interaction.type === Swipe).
         * @return {null}
         */
        interaction_received:function( interaction )
        {
            this._super( interaction, true );

            if( this._ignoreSwipes === true || this.model.isGameRunning === false || this.model.isGamePaused === true ){ return; } 

            var scoringComponent = null;
            var isCorrect = null; 

            if ( interaction.getType() === InteractionType.Swipe )
            {
                scoringComponent = this.getScoringComponent();
                isCorrect = this._interactionIsCorrect( this._gameModePointing, interaction ) || LumosityUtils.isDebugModeOn() ;

                this._lastInteractionWasCorrect = isCorrect;

                this.model.totalResponseCount = this.model.totalResponseCount + 1;

                // 
                //WATERY EFFECTS TESTING
                //
                // var effect = cc.Liquid.create( 3, cc.size(16,12), 4, 10); 
                // this.view._background.runAction( effect );
                // 
                //WATERY EFFECTS TESTING
                //  

                this.model.currentTrial[ metadataTrialKeys.correct ]           = ( isCorrect === true ) ? "T" : "F";
                this.model.currentTrial[ metadataTrialKeys.responseDirection ] = this.formatDirectionForMetadata( interaction.getSwipeDirection() );
                this.model.currentTrial[ metadataTrialKeys.multiplier ]        = this._components[ GameComponentScoreMultiplier.prototype.getType() ]._multiplier;
                this.model.currentTrial[ metadataTrialKeys.progress ]          = this._components[ GameComponentScoreMultiplier.prototype.getType() ].getProgressMetadata();
                this.model.currentTrial[ metadataTrialKeys.movementDirection ] = this.formatDirectionForMetadata( this._moving );
                this.model.currentTrial[ metadataTrialKeys.pointingDirection ] = this.formatDirectionForMetadata( this._pointing );
                this.model.currentTrial[ metadataTrialKeys.responseType ]      = ( this._gameModePointing === true ) ? "P" : "M";
                
                if( isCorrect === true )
                {
                    scoringComponent.increaseMultiplierProgress();
                    this.model.correctTrials += 1;

                    this.model.totalCorrectResponseCount = this.model.totalCorrectResponseCount + 1;
                    //
                    // Update metadata.
                    //
                    this.getMetadataComponent()._incongruentStimuliCorrect += Number( !this._hasCongruentStimuli() );
                }
                else
                {
                    this.model.incorrectTrials += 1;
                    scoringComponent.decreaseMultiplierProgress();
                }
                this.view.showFeedback( isCorrect );

                this.endTrial( this.model.currentTrial, isCorrect, this._gameModeSwitched, this._hasCongruentStimuli() );
                this.beginTrial();
            }
        },

        /**
         * Checks the current swipe interaction against the current game mode to
         * determine "correctness".
         * 
         * @param  {Boolean} gameModePointing Whether we are in pointing mode or not.
         * @param  {Interaction} interaction  The interaction that took place (expects Interaction.type === Swipe).
         * @return {Boolean}
         */
        _interactionIsCorrect: function( gameModePointing, interaction )
        {
            if( gameModePointing === true )
            {
                return Boolean( this._pointing === interaction.getSwipeDirection() );
            }
            return Boolean( this._moving === interaction.getSwipeDirection() );
        }, 

        usesInteractionType:function( type )
        {
            return ( type === InteractionType.Swipe ) || this._super( type );
        },

        cue_showCountdown: function()
        {
            this.view.showCountdown();
        },

        countdown_onOne: function()
        {
            this.view.animateLeavesFlying( this._gameModePointing );
        },

        countdown_onFinished: function()
        {
            this.view.setModeIndicator( ( this._gameModePointing ) ? 0 : 1 );
            this.setIgnoreSwipes( false );
        },

        onPause: function()
        {
            this._super.apply( this, arguments );
            this.view.pauseAnimations();
        },

        onResume: function()
        {
            this._super();
            this.view.resumeAnimations();
        },

        skipTutorialButton_Tapped: function()
        {
            if ( !this.model.tutorialSkipped )
            {
                this.playEndAnimation();

                this.setIgnoreSwipes( true );
                this.view.playLeafSinkAnimation();
                this.view.hidePromptLabel();

                this.tutorialMetadata.finishes += 1;
                this.getMetadataComponent().tutorialMetadata = this.tutorialMetadata; 
                LumosityUtils.callFunctionOnCollection( this._components, "onEndGameplay" );

                this.onCompleteTutorial( true );
            }
        },

        onEndGameplay:function()
        {
            var index = 0;
            var averageMovingResponseTime = 0;
            var averagePointingResponseTime = 0;
            var practiceModeOn = this.model.persistentData.isPracticeMode;

            if( practiceModeOn === false )
            {
                this.model.metadata[ metadataAggregateKeys.numTriesPointing ]   = this._numTriesPointing; 
                this.model.metadata[ metadataAggregateKeys.numCorrectPointing ] = this._numCorrectPointing;
                this.model.metadata[ metadataAggregateKeys.numTriesMoving ]   = this._numTriesMoving; 
                this.model.metadata[ metadataAggregateKeys.numCorrectMoving ] = this._numCorrectMoving;

                //
                // Calculate the average response times.
                //
                for( index = 0; index < this._pointingResponseTimes.length; index += 1 )
                {
                    averagePointingResponseTime += this._pointingResponseTimes[ index ];
                }
                averagePointingResponseTime = averagePointingResponseTime / this._pointingResponseTimes.length;
                this.model.metadata[ metadataAggregateKeys.responseTimePointing ] = Math.round( LumosityUtils.convertSecondsToMilliseconds( averagePointingResponseTime ) );

                for( index = 0; index < this._movingResponseTimes.length; index += 1 )
                {
                    averageMovingResponseTime += this._movingResponseTimes[ index ];
                }
                averageMovingResponseTime = averageMovingResponseTime / this._movingResponseTimes.length;
                this.model.metadata[ metadataAggregateKeys.responseTimeMoving ] = Math.round( LumosityUtils.convertSecondsToMilliseconds( averageMovingResponseTime ) );
            }

            // METADATA FIX 10/Feb/2017: This was one too high before.
            this.getMetadataComponent()._trialCount = this._numTriesMoving + this._numTriesPointing;

            this.model.isGameRunning = false;
            this.model.isGameOver = true;

            this.finalizeMetadata();

            if ( this.model.persistentData.isPracticeMode )
            {
                this.tutorialMetadata.finishes += 1;

                if ( !LumosityUtils.isFitTestModeOn() )
                {
                    this.onCompleteTutorial( false );
                }
            }

            if ( LumosityUtils.isFitTestModeOn() || ( !LumosityUtils.isFitTestModeOn() && this.model.persistentData.isPracticeMode ) )
            {
                this.getMetadataComponent().tutorialMetadata = this.tutorialMetadata; 
            }

            LumosityUtils.callFunctionOnCollection( this._components, "onEndGameplay" );

            this.playEndAnimation();

            //finish the interaction script in case there is one (if you want it absolutely tied to the session)
            //this is harmless to call if nothing is being recorded.
            if( this._interactionAutomator )
            {
                this._interactionAutomator.finishRecordingScript();
            }

            //for automation support, you can hook into this to end automation
            if ( lumosity.game.gameplayEnded )
            {
                lumosity.game.gameplayEnded();
            }

            this.setIgnoreSwipes( true );
            this.view.playLeafSinkAnimation();
        },

        _sinkAnimationFinished: function()
        {
            if( this.model.persistentData.isPracticeMode === true )
            {
                this.onStartFitTest();
            }
        },

        _createSettings:function()
        {
            return new EbbAndFlowSettings( "settings.json" );
        },

        _createModel:function()
        {
            return new EbbAndFlowModel();
        },

        _createView:function( screenSize )
        {
            return new EbbAndFlowView( screenSize, this.settings, this.model, this );
        },

        setIgnoreSwipes: function( ignoreSwipes )
        {
            this._ignoreSwipes = ignoreSwipes;
        },

        beginTrial: function()
        {
            this._super();
            var metadata = this.getMetadataComponent();
            metadata._incongruentStimuliCount += Number( !this._hasCongruentStimuli() );
            this.model.currentTrial = {};

        },

        endTrial: function( trialObject, trialCorrect, switchedTrial, congruentStimuli )
        {
            if( this._gameModePointing === true )
            {
                this._numTriesPointing += 1;
                if( trialCorrect === true )
                {
                    this._numCorrectPointing += 1;
                }
                this._pointingResponseTimes.push( this.getMetadataComponent()._responseTime );
            }
            else if( this._gameModePointing === false )
            {
                this._numTriesMoving += 1;
                if( trialCorrect === true )
                {
                    this._numCorrectMoving += 1;
                }
                this._movingResponseTimes.push( this.getMetadataComponent()._responseTime );
            }

            this.setupNextTrial();
            this.model.currentTrial[ metadataTrialKeys.boardOffset ] = this.view.getOffsetBoardLayout();
            // METADATA FIX, 10/Feb/2017: switchedTrial and congruentStimuli were corrected.
            // Previously, switchedTrial was what congruentStimuli should have been, and congruentStimuli was always false.
            // WTF.
            this._super( trialObject, trialCorrect, switchedTrial, congruentStimuli, true, false );
        },

        _hasCongruentStimuli: function()
        {
            return ( this._pointing === this._moving );
        },

        configureMetaDataKeys: function()
        {
            this._super();
            //
            // Customized trial data
            //
            metadataTrialKeys.movementDirection = "movement_direction"; // Direction of movement (U, D, L, R).
            metadataTrialKeys.pointingDirection = "pointing_direction"; // Direction of pointing (U, D, L, R).
            metadataTrialKeys.responseDirection = "response_direction"; // Direction input by the user (U, D, L, R).
            metadataTrialKeys.responseType      = "response_type";      // Movement/Pointing (M/P).
            metadataTrialKeys.trialSwitch       = "switch";             // Was the response type different from the previous trial? (T/F)
            metadataTrialKeys.correct           = "correct";            // Was the trial correctly answered? (T/F)
            metadataTrialKeys.multiplier        = "multiplier";         // Multiplier applied to the current score increment.
            metadataTrialKeys.boardOffset       = "board_offset";       // x,y offset from initial position, at start of trial.
            metadataTrialKeys.progress          = "progress";           // Describes the number of "pips" a player has accrued on this trial.
            //
            // More aggregate data is needed
            //
            metadataAggregateKeys.numTriesPointing = "num_tries_pointing"; // Number of attempts when the leaves are pointing.
            metadataAggregateKeys.numCorrectPointing = "num_correct_pointing"; // Number of correct trials when leaves are pointing.
            metadataAggregateKeys.responseTimePointing = "response_time_pointing"; // Average response times for when leaves are pointing.
            metadataAggregateKeys.numTriesMoving = "num_tries_moving"; // Number of attempts when the leaves are moving.
            metadataAggregateKeys.numCorrectMoving = "num_correct_moving"; // Number of correct trials when leaves are moving.
            metadataAggregateKeys.responseTimeMoving = "response_time_moving"; // Average response times for when leaves are moving.
        },

        /*
            NOTE: This method will always return 'U' in out of bounds cases.
         */
        formatDirectionForMetadata: function( cardinalDirection )
        {
            var codedDirection = "U";
            if( cardinalDirection === 1 )
            {
                codedDirection = "R";
            }
            else if( cardinalDirection === 2 )
            {
                codedDirection = "D";
            }
            else if( cardinalDirection === 3 )
            {
                codedDirection = "L";
            }
            return codedDirection; 
        }
    });

}());
