/*global cc*/
/*global require*/
/*global GameView*/
/*global GameWidgetFeedback*/
/*global GameWidgetFlipbook*/
/*global GameWidgetHud*/
/*global Flipbook*/
/*global LeafCollection*/
/*global LumosityUtils*/
/*global Leaf*/
/*global LayoutLoader*/
/*global AnimationChanges*/
/*global audio*/
/*global LumosityLabel*/
/*global lumosity*/

require( "src/Game/Base/GameView.js" );
require( "src/Game/Widget/GameWidgetFeedback.js" );
require( "src/Game/Widget/GameWidgetFlipbook.js" );
require( "src/Game/Widget/GameWidgetHud.js" );
require( "src/Game/Widget/GameWidgetFlipbook.js" );
require( "src/Game/LeafCollection.js" );
require( "src/View/LumosityLabel.js" );

//
//NOTE: Put this in it's own file.
//
var CardinalDirections =
{
    North: 0,
    East:  1,
    South: 2,
    West:  3,

    getKeys: function()
    {
        "use strict";
        return [ "North", "East", "South", "West" ];
    },

    getValues: function()
    {
        "use strict";
        return [ 0, 1, 2, 3 ];
    }
};

var EbbAndFlowView = GameView.extend(
{
    /** The speed at which leaves move (in seconds). */
    leafSpeed: 210,

    _leaf_collection: null,
    _move_direction: CardinalDirections.North,
    _pointing_direction: CardinalDirections.North,
    _feedback_sprite: null,
    _modeIndicatorImages: [ "modeIndicatorPointing.png", "modeIndicatorMoving.png" ],
    _feedback_images: [ "game_feedbackIncorrect.png", "game_feedbackCorrect.png"],
    _horizontalPadding: 0,
    _verticalPadding: 0,
    _promptLabel: null,
    _consecutiveTurns: null,
    _contentSize: null,
    _viewActionCompleted: null,

    ctor:function( screenSize, settings, model, interactionResponder, preload )
    {
        "use strict";
        //
        //NOTE: Hackey shit ensues: I have to modify the settings before
        //      passing them along. Removing the compressed image field because we
        //      don't actually have one for *most* resolutions. Only the device with
        //      a screen width of 640 uses the compressed image. We NEED a proper
        //      resources manager.
        //
        var useCompressedImage = ( screenSize.width > 400 && screenSize.width <= 640 );
        if( !useCompressedImage )
        {
            delete settings.compressedBackgroundImage;
        }
        this._contentSize = screenSize;
        this._super( screenSize, settings, model, interactionResponder, preload );
        this._calculatePlayableAreaPadding();

        // modify text size of moving/pointing text during gameplay
        var targetFontSize = 30;
        if( lumosity.game.locale === "fr" ) 
        {
            targetFontSize = 22;
        }
        else if( lumosity.game.locale === "de" || lumosity.game.locale === "es" || lumosity.game.locale === "pt" || lumosity.game.locale === "ko" )
        {
            targetFontSize = 24;
        }
        else // en, ja
        {
            targetFontSize = 30;
        }
        this[ "pointingLabel" ].setInitialFontSize( targetFontSize );
        this[ "movingLabel" ].setInitialFontSize( targetFontSize );    

        //
        //NOTE: This is a hack that should be fixed through flash export tool.
        //     
        this.animationController.setNodeIgnoredChanges( "pointingLabel", AnimationChanges.Scale );
        this.animationController.setNodeIgnoredChanges( "movingLabel", AnimationChanges.Scale );
    },

    _calculatePlayableAreaPadding: function()
    {
        "use strict";
        var winSize = cc.Director.getInstance().getWinSize();
        var aspect = winSize.height / winSize.width;

        if( aspect > 1.5 )
        {
            this._verticalPadding = Math.abs( winSize.width - winSize.height ) * 0.07;
        }
        else if( aspect < 1.5 )
        {
            this._horizontalPadding = Math.abs( winSize.width - winSize.height ) * 0.07;
        }
        else
        {
            this._horizontalPadding = 0;
            this._verticalPadding = 0;
        }
    },

    changeDirections:function( is_pointing, point_direction, move_direction )
    {
        "use strict";
        var index = Number( !is_pointing );
        this.interactionResponder.setIgnoreSwipes( true );
        this._leaf_collection.stopActionsOnLeaves();
        this.setModeIndicator( index );
        this.pointLeavesInDirection( point_direction );
        this.moveLeavesInDirection( move_direction );
        this.playFlipAnimation( is_pointing );
        this.playIdleAnimation();
        this.scrollLeaves( move_direction );
    },

    playFlipAnimation:function( is_pointing )
    {
        "use strict";
        this._leaf_collection.animateLeavesFlipping( is_pointing,
                                                    cc.CallFunc.create( this._leafFlipCompleted, this ) );
    },

    _leafFlipCompleted:function( nodeExecutingAction )
    {
        "use strict";
        var leaves = this._leaf_collection._leaves,
            last_leaf = leaves[ leaves.length - 1 ]._leaf_sprite;
        if( nodeExecutingAction === last_leaf )
        {
            Leaf._play_flip_sound = true;
        }
    },

    flipAnimationEnded: function()
    {
        "use strict";
        this.playRippleAnimation();
    },

    setModeIndicator: function( index )
    {
        "use strict";
        var frameName = this._modeIndicatorImages[ index ];
        var displayFrame = cc.SpriteFrameCache.getInstance().getSpriteFrame( frameName );
        this.modeIndicator.setDisplayFrame( displayFrame );
        this.setModeIndicatorTextColor( Boolean( index ) );
    },

    setModeIndicatorTextColor: function( isPointing )
    {
        "use strict";
        var fontColors = [ cc.c3b( 255, 255, 255 ), cc.c3b( 0, 0, 0 ) ];
        this.pointingLabel.setColor( fontColors[ Number( isPointing ) ] );
        this.movingLabel.setColor( fontColors[ Number( !isPointing ) ] );
    },

    pointLeavesInDirection:function( target_point_direction )
    {
        "use strict";
        this._leaf_collection.pointLeavesInDirection( target_point_direction );
    },

    moveLeavesInDirection:function( move_direction )
    {
        "use strict";
        this._move_direction = move_direction;
    },

    playRippleAnimation:function()
    {
        "use strict";
        this._leaf_collection.animateWaterRipple();
    },

    playIdleAnimation:function()
    {
        "use strict";
        var degrees = 2.5,
            rotational_speed = 0.4,
            rotate_action = cc.Sequence.create( 
                cc.RotateBy.create( rotational_speed, degrees ),
                cc.RotateBy.create( rotational_speed, -degrees ),
                cc.RotateBy.create( rotational_speed, -degrees ),
                cc.RotateBy.create( rotational_speed, degrees )
                ),
            idle_animation = cc.RepeatForever.create( rotate_action );
        this._leaf_collection.runActionOnLeaves( idle_animation );
    },

    scrollLeaves:function( move_direction )
    {
        "use strict";
        var index = 0,
            leaves = this._leaf_collection._leaves;
        for( index = 0; index < leaves.length; index += 1 )
        {
            this._scrollLeaf( leaves[index]._leaf_sprite, move_direction );
        }
    },

    _scrollLeaf:function( leaf, move_direction )
    {
        "use strict";
        var offscreen_point = this.getOffscreenPoint( move_direction, leaf ),
            move_time = this.getMoveDuration( move_direction, offscreen_point, leaf ),
            move_action = cc.Sequence.create(
                cc.MoveTo.create( move_time, offscreen_point ),
                cc.CallFunc.create( this._scrollLeavesFinished, this )
            );
        leaf.runAction( move_action );
    },

    getOffscreenPoint:function( move_direction, sprite )
    {
        "use strict";
        var winSize = cc.rect( -this._horizontalPadding, -this._verticalPadding, this.gameContent.getContentSize().width + this._horizontalPadding, this.gameContent.getContentSize().height + this._verticalPadding ),
            half_sprite = sprite.getContentSize().height/2,
            offscreen_point = cc.p( 0, 0);
        if( move_direction === CardinalDirections.North )
        {
            offscreen_point = cc.p( sprite.getPosition().x, ( winSize.height + half_sprite + this._verticalPadding ) ); 
        }
        else if( move_direction === CardinalDirections.South )
        {
            offscreen_point = cc.p( sprite.getPosition().x, -( half_sprite + this._verticalPadding ) );
        }
        else if( move_direction === CardinalDirections.East )
        {
            offscreen_point = cc.p( ( winSize.width + half_sprite + this._horizontalPadding ), sprite.getPosition().y );
        }
        else if( move_direction === CardinalDirections.West )
        {
            offscreen_point = cc.p( -( half_sprite + this._horizontalPadding ), sprite.getPosition().y );
        } 
        return offscreen_point;
    },

    getMoveDuration:function( move_direction, offscreen_point, sprite )
    {
        "use strict";
        var leaf_distance = 0;
        if( move_direction === CardinalDirections.North || move_direction === CardinalDirections.South )
        {
            leaf_distance = Math.abs( offscreen_point.y - sprite.getPosition().y );
        }
        else if( move_direction === CardinalDirections.East || move_direction === CardinalDirections.West )
        {
            leaf_distance = Math.abs( offscreen_point.x - sprite.getPosition().x );
        }
        return Math.abs( leaf_distance ) / this.leafSpeed;
    },

    _scrollLeavesFinished:function( nodeExecutingAction )
    {
        "use strict";
        this.wrapAroundScreen( this._move_direction, nodeExecutingAction );
    },

    wrapAroundScreen:function( move_direction, sprite )
    {
        "use strict";
        var offscreen_point = this.getOffscreenPoint( this._getOppositeDirection(), sprite );
        sprite.setPosition( offscreen_point );
        this._scrollLeaf( sprite, move_direction );
    },

    _createViews:function()
    {
        "use strict";
        this._leaf_collection = new LeafCollection( this.gameContent, this.interactionResponder );
    },

    animateLeavesFlying: function( mode )
    {
        "use strict";
        var index = 0,
            leaves = this._leaf_collection._leaves,
            leaf = null;
        for( index = 0; index < leaves.length; index += 1 )
        {
            leaf = leaves[index]._leaf_sprite;
            leaf.setPosition( this.getOffscreenPoint( this._getOppositeDirection(), leaf ) );
        }
        this._leaf_collection.animateFlyIn( mode,
                                            cc.CallFunc.create( this.enterLeavesFinishedCallback, this ),
                                            cc.CallFunc.create( this.enterLeavesRotationFinishedCallback, this ) );
        audio.playEffect( "sounds/windBlow.mp3" );
        if( LumosityUtils.isFitTestModeOn() === true || this.model.persistentData.isPracticeMode === true )
        {
            this.showPromptLabel();
        }
    },

    //
    //NOTE: Put into the CardinalDirections class.
    //
    _getOppositeDirection:function()
    {
        "use strict";
        var reversed_directions = [ 2, 3, 0, 1 ];
        return reversed_directions[ this._move_direction ];
    },

    enterLeavesFinishedCallback:function( nodeExecutingAction )
    {
        "use strict";
        this._scrollLeaf( nodeExecutingAction, this._move_direction );
    },

    enterLeavesRotationFinishedCallback:function()
    {
        "use strict";
        this.playRippleAnimation();
        this.playIdleAnimation();
    },

    playLeafSinkAnimation:function()
    {
        "use strict";
        var pointingMode = this._getGameDisplayMode();
        this._leaf_collection.animateLeavesSinking( !pointingMode, this._move_direction );
    },

    _getGameDisplayMode: function()
    {
        "use strict";
        var frameCache = cc.SpriteFrameCache.getInstance(),
            movingFrame = frameCache.getSpriteFrame( this._modeIndicatorImages[ 1 ] );
        return this.modeIndicator.isFrameDisplayed( movingFrame );
    },

    pauseAnimations: function()
    {
        "use strict";
        this._leaf_collection.pauseLeafAnimations();
    },

    resumeAnimations: function()
    {
        "use strict";
        this._leaf_collection.resumeLeafAnimations();
    },

    _layoutCreatingNode: function( nodeName, nodeData )
    {
        "use strict";
        if( nodeName === "modeIndicator" )
        {
            nodeData[ LayoutLoader.imageKey ] = "modeIndicatorDefault.png";

            if( lumosity.device.id === "html5" || cc.Application.getInstance().getTargetPlatform() === cc.TARGET_PLATFORM.ANDROID )
            {
                nodeData[ LayoutLoader.spriteSheetNameKey ] = "UI/defaultModeIndicator";
                nodeData[ LayoutLoader.spritesheetFormatKey ] = ".png";
            }
            else
            {
                nodeData[ LayoutLoader.spriteSheetNameKey ] = "UI/modeIndicator";
                nodeData[ LayoutLoader.spritesheetFormatKey ] = ".pvr.ccz";
            }
            return null;
        }
        return this._super( nodeName, nodeData );
    },

    onRestart: function()
    {
        "use strict";
        this._super();

        var blackColor = cc.c3b( 0, 0, 0 );
        var frameCache = cc.SpriteFrameCache.getInstance();
        var defaultModeFrame = frameCache.getSpriteFrame( "modeIndicatorDefault.png" );

        this.modeIndicator.setDisplayFrame( defaultModeFrame );
        this.pointingLabel.setColor( blackColor );
        this.movingLabel.setColor( blackColor );

        this._promptVisible = false;

        this.resetLeafCollection();
    },

    resetLeafCollection: function()
    {
        "use strict";
        this._leaf_collection.resetLeaves( this );
    },

    getBoardLayout: function()
    {
        "use strict";
        return this._leaf_collection.getInitialLeafPositions();
    },

    getOffsetBoardLayout: function()
    {
        "use strict";
        return this._leaf_collection.getOffsetLeafPositions();
    },

    getLeafScale: function()
    {
        "use strict";
        return this._leaf_collection.getLeafScale();
    },
    /*
     ===============================================
        
                FIT-TEST EXCELLENCE

     ===============================================
     */
    showPromptLabel: function()
    {
        "use strict";
        this._animatePromptIn();
    },

    hidePromptLabel: function()
    {
        "use strict";
        this._animatePromptOut();
    },

    setPromptLabelText: function( text )
    {
        "use strict";
        if( this.fitTestPrompt )
        {
            this.fitTestPrompt.setString( text );
        }
    },

    _animateTextChange: function( newText )
    {
        "use strict";
        this.animationController.play( "practicePromptOut",
            function()
            {
                this.setPromptLabelText( newText );
                this.animationController.play( "practicePromptIn", null, null );
            }, null );
    },

    _animatePromptIn: function()
    {
        "use strict";
        if( this._promptVisible === true ){ return; }
        this.animationController.play( "dimAreaIn",
            function()
            {
                this.animationController.play( "practicePromptIn", null, null );
            },
            null );

        this._promptVisible = true;
    },

    _animatePromptOut: function()
    {
        "use strict";
        if( this._promptVisible === true )
        {
            this.animationController.play( "practicePromptOut",
                function()
                {
                    this.animationController.play( "dimAreaOut", null, null );
                },
                null );
            this._promptVisible = false;
        }
    },

    createDimLayer: function()
    {
        "use strict";
        //
        //NOTE: We do not want to create the dim layer.
        //
    },

    removeDimLayer: function()
    {
        "use strict";
        //
        //NOTE: We do not want to remove the dim layer.
        //
    },

    setViewCallbackFunc: function( func )
    {
        "use strict";
        if( typeof func === 'function' )
        {
            this._viewActionCompleted = func;
        }
    },

    setLeafCallbackFunc: function( func )
    {
        "use strict";
        if( typeof func === 'function' )
        {
            this._leaf_collection.setSinkingCallback( func );
        }
    },

    cue_startCountdown: function()
    {
        "use strict";
        if( this.model.persistentData.isPracticeMode === true )
        {
            this._viewActionCompleted();
        }
        else
        {
            this._super();
        }
    }
});
            
