/*global cc*/
/*global LumosityUtils*/
/*global CardinalDirections*/
/*global audio*/

var Leaf = cc.Class.extend(
{
	_starting_pos: null,
	_leaf_sprite: null,
	_ripple_sprite: null,
	_pointing: null,
    _leafSinkAnimation: null,

    //TEMPORARY IPAD TEST!!
    _winSize: null,
    _leafColor: "green",

	ctor:function()
	{
		"use strict";
        this._winSize = cc.Director.getInstance().getWinSize();
        this._leaf_sprite = cc.Sprite.createWithSpriteFrameName( "greenLeafFlipAnim000.png" );
        this._leaf_sprite.setVisible( false );

        this._createRipple( this._leaf_sprite.getContentSize() );
        this._leaf_sprite.addChild( this._ripple_sprite );

        this._pointing = CardinalDirections.North;
        Leaf._play_land_sound = true;
        Leaf._play_flip_sound = true;
	},

    pauseAnimations: function()
    {
        "use strict";
        this._leaf_sprite.pauseSchedulerAndActions();
        this._ripple_sprite.pauseSchedulerAndActions();
    },

    resumeAnimations: function()
    {
        "use strict";
        this._leaf_sprite.resumeSchedulerAndActions();
        this._ripple_sprite.resumeSchedulerAndActions();
    },

    stopAnimations: function()
    {
        "use strict";
        this._leaf_sprite.stopAllActions();
        this._ripple_sprite.stopAllActions();
    },

	_createRipple:function( parent_size )
	{
		"use strict";
        this._ripple_sprite = cc.Sprite.create( "rippleAnimation006.png" );
        this._ripple_sprite.setPosition( cc.p( parent_size.width/2, parent_size.height/2 ) );
        this._ripple_sprite.setOpacity( 0 );
        this._ripple_sprite.setVisible( false );
	},

	playRippleAnimation:function()
	{
        "use strict";
        var total_animation_time = 7,
            fade_action = cc.Sequence.create( 
                cc.FadeTo.create( total_animation_time * 0.03, 220 ),
                cc.FadeTo.create( total_animation_time * 0.06, 0),
                cc.CallFunc.create( this._rippleAnimationFinished, this )
            ),
            scale_action = cc.ScaleTo.create( total_animation_time, 5 );

        this._ripple_sprite.setScale( 1 );
        this._ripple_sprite.setVisible( true );
        this._ripple_sprite.setPosition( cc.p( this._leaf_sprite.getContentSize().width/2, 
        										this._leaf_sprite.getContentSize().height/2 ) );
        this._ripple_sprite.runAction( fade_action );
        this._ripple_sprite.runAction( scale_action );
	},

    setVisible: function( visible )
    {
        "use strict";
        this._leaf_sprite.setVisible( visible );
        this._ripple_sprite.setVisible( visible );
    }, 

	_rippleAnimationFinished:function( nodeExecutingAction )
    {
        "use strict";
        nodeExecutingAction.setVisible( false );
    },

	playIdleAnimation:function()
	{
        "use strict";
        var degrees = 2.5,
            rotational_speed = 0.4,
            rotate_action = cc.Sequence.create( 
                cc.RotateBy.create( rotational_speed, degrees ),
                cc.RotateBy.create( rotational_speed, -degrees ),
                cc.RotateBy.create( rotational_speed, -degrees ),
                cc.RotateBy.create( rotational_speed, degrees )
                ),
            idle_animation = cc.RepeatForever.create( rotate_action );
        this._leaf_sprite.runAction( idle_animation );
	},

    playFlyInAnimation:function( mode, move_finished_callback, rotation_finished_callback )
    {
    	"use strict";
        var move_action = cc.Sequence.create( 
        		cc.MoveTo.create( 0.7, this._starting_pos ),
        		move_finished_callback
			),
        	rotate_action = cc.Sequence.create( 
	            cc.RotateBy.create( 0.8, 20 ),
	            rotation_finished_callback
            ),
            land_sound_action = cc.Sequence.create( 
            	cc.DelayTime.create( 0.6 ),
            	cc.CallFunc.create( this.playLandSound, this ) ),
        	move_ease_out = cc.EaseInOut.create( move_action.clone(), 2.0 );

        this.setLeafColor( mode );
        this._leaf_sprite.setDisplayFrame( cc.SpriteFrameCache.getInstance().getSpriteFrame( this.getLeafColor() + "LeafFlipAnim000.png" ) );
    	this._leaf_sprite.setVisible( true );
        this._leaf_sprite.setRotation( this._leaf_sprite.getRotation() - 20 );  //<----- The leaves appear to rotate in slightly, depending on which direction they're pointing.

        this._leaf_sprite.runAction( rotate_action );
        this._leaf_sprite.runAction( move_ease_out );
        this._leaf_sprite.runAction( land_sound_action );
    },

    playLandSound: function()
    {
    	"use strict";
    	if( Leaf._play_land_sound )
    	{
	    	Leaf._play_land_sound = false;
	    	audio.playEffect( "sounds/leavesLand.mp3" );
	    }
    },

    playFlipAnimation: function( mode, updateLeafColorCallback, callbackAction )
    {
    	"use strict";
        var flipToGreen = Boolean( mode );
        var flipInAnimation = cc.Animation.create( this._getFlipInFrames(), 0.03 ),
            flipOutAnimation = cc.Animation.create( this._getFlipOutFrames( flipToGreen ), 0.03 ),
            playIn_action = cc.Animate.create( flipInAnimation ),
            playOut_action = cc.Animate.create( flipOutAnimation ),
            sequence = cc.Sequence.create( playIn_action, updateLeafColorCallback, playOut_action, callbackAction ),
            sound_sequence = cc.Sequence.create( cc.DelayTime.create( 0.02 ),
                                                cc.CallFunc.create( this.playFlipSound, this ) );
    	this._leaf_sprite.runAction( sequence );
    	this._leaf_sprite.runAction( sound_sequence );
    	this.playRippleAnimation();
    },

    _getFlipInFrames: function()
    {
        "use strict";
        var frameName = "", frameCount = 7, index = 0, frames = [], cache = cc.SpriteFrameCache.getInstance();
        for( index = 0; index < frameCount; index += 1 )
        {
            frameName = this.getLeafColor() + "LeafFlipAnim00" + index.toString() + ".png";
            frames.push( cache.getSpriteFrame( frameName ) );
        }
        return frames;
    },

    _getFlipOutFrames: function( mode )
    {
        "use strict";
        var frameName = "", frameCount = 15, index = 7, frames = [], cache = cc.SpriteFrameCache.getInstance();
        for( index = 7; index < frameCount; index += 1 )
        {
            this.setLeafColor( mode );
            frameName = this.getLeafColor() + "LeafFlipAnim0" + ( index > 9 ? "" : "0" ) + index.toString() + ".png";
            frames.push( cache.getSpriteFrame( frameName ) );
        }
        return frames;
    },

    playFlipSound: function()
    {
        "use strict";
        if( Leaf._play_flip_sound )
        {
            Leaf._play_flip_sound = false;
            audio.playEffect( "sounds/leavesFlip.mp3");
        }
    },

    playSinkAnimation:function( mode, moving, callbackFunc )
    {
    	"use strict";
        var rotatedIndex = 4 + ( moving - this._pointing );
        var animationName = ( mode ? 1 : 0 ).toString() + ( rotatedIndex >= 4 ? 0 : rotatedIndex ).toString();
        var animation = cc.AnimationCache.getInstance().getAnimation( animationName );
        this._leaf_sprite.stopAllActions();
        this._leaf_sprite.runAction( cc.Sequence.create( cc.Animate.create( animation ), callbackFunc ) );
        this.playRippleAnimation();
    },

    pointInDirection: function( target_point_direction )
    {
        "use strict";
        var current_rotation = 90 * this._pointing,
            rotate_by = 90 * ( target_point_direction - this._pointing );
        this._leaf_sprite.setRotation( rotate_by + current_rotation );
        this._pointing = target_point_direction;
    },

    setLeafColor: function( mode )
    {
        "use strict";
        this._leafColor = ( mode === true ? "green" : "orange" );
    },

    getLeafColor: function()
    {
        "use strict";
        return this._leafColor;
    }
});
