/*global cc*/
/*global LumosityUtils*/
/*global require*/
/*global Leaf*/
/*global CardinalDirections*/
/*global audio*/
/*global lumosity*/

require( "src/Game/Leaf.js" );

var LeafCollection = cc.Class.extend( //<--- Why should I extend from Class?
{
    /** A member array that stores the cc.Sprite leaves. */
    _leaves: [],

    /** The amount by which leaves should be offset from each other on the x-axis (horizontally). */
    _x_offset: 0,

    /** The amount by which leaves should be offset from each other on the y-axis (vertically). */
    _y_offset: 0,

    _sinkingLeafCount: 0,

    _sprite_buffer: null,

    _sinkActionCompleted: null,

    /** Represents the starting positions of all leaves. */
    _startingPositions: [],

    interactionResponder: null,


    /**
     * Constructor that takes the number of leaves you would like to have.
     * @param  {int} leaf_count The number of leaves to create.
     */
    ctor:function( display, responder )
    {
        "use strict";
        if ( lumosity.device.id !== "html5" )
        {
            cc.SpriteFrameCache.getInstance().addSpriteFrames( "leaves/leaves0.plist", "leaves/leaves0.pvr.ccz" );
            cc.SpriteFrameCache.getInstance().addSpriteFrames( "leaves/leaves1.plist", "leaves/leaves1.pvr.ccz" );
        }
        //
        //NOTE: Terrible hackiness going on here. If making a sprite fails, then we fall back on loading PNG
        //      data. What would be preferable is to get the device capabilities sent to us from the app side,
        //      and take the appropriate action based on that.
        //
        var temp = cc.Sprite.createWithSpriteFrameName( "greenLeafFlipAnim000.png" );
        if( !temp || ( temp.textureLoaded && !temp.textureLoaded() ) )
        {
            cc.SpriteFrameCache.getInstance().removeSpriteFramesFromFile( "leaves/leaves0.plist" );
            cc.SpriteFrameCache.getInstance().removeSpriteFramesFromFile( "leaves/leaves1.plist" );
            cc.SpriteFrameCache.getInstance().addSpriteFrames( "leaves/defaultleaves0.plist", "leaves/defaultleaves0.png" );
            cc.SpriteFrameCache.getInstance().addSpriteFrames( "leaves/defaultleaves1.plist", "leaves/defaultleaves1.png" );
        }

        this._createLeaves( this._calculateLeafCount( display.getContentSize() ) );
        this._createSinkAnimations();
        this._addLeavesRandomly( display, true );
        this.interactionResponder = responder;
    },

    _calculateLeafCount: function( display_size )
    {
        "use strict";
        var temp_leaf = cc.Sprite.createWithSpriteFrameName( "greenLeafFlipAnim000.png" );
        this._sprite_buffer = temp_leaf.getContentSize().height/2;
        return Math.ceil( display_size.height / 124 );//temp_leaf.getContentSize().width );
    },

    _createLeaves:function( leaf_count )
    {
        "use strict";
        var index = 0;
        for( index = 0; index < leaf_count; index += 1 )
        {
            this._leaves[index] = new Leaf(); 
        }
        cc.log("MJDE CODE " + leaf_count + " leaves allocated!");
    },

    _addLeavesRandomly:function( display, addLeaves )
    {
        "use strict";
        var x_coords = [],
            y_coords = [];
        this._calculateLeafSpace( display.getContentSize() );
        this._setupLeafPositions( x_coords, y_coords );
        this.randomizeLeafPositions( y_coords, this._y_offset );
        this._applyRandomLeafPositions( display, x_coords, y_coords, addLeaves );
    },

    _calculateLeafSpace:function( display_size )
    {
        "use strict";
        if( this._x_offset > 0 || this._y_offset > 0 ){ return; }
        this._x_offset = this._leafSpaceForDimension( display_size.width );
        this._y_offset = this._leafSpaceForDimension( display_size.height );
    },

    _leafSpaceForDimension:function( width_or_height )
    {
        "use strict";
        return ( width_or_height + this._sprite_buffer ) / this._leaves.length - 1;
    },

    _setupLeafPositions:function( x_coords, y_coords )
    {
        "use strict";
        var index = 0;
        for( index = 0; index < this._leaves.length; index += 1 )
        {
            x_coords.push( this._x_offset * index );
            y_coords.push( this._y_offset * index );
        }
    },

    randomizeLeafPositions:function( coords, offset )
    {
        "use strict";
        do
        {
            coords = LumosityUtils.shuffle( coords );
        } while( this._leavesTooClose( coords, offset ) );
    },

    _leavesTooClose:function( array, unit )
    {
        "use strict";
        var index = 0;
        for( index = 0; index < array.length; index += 1 )
        {
            if( Math.abs( array[index] - array[index - 1] ) <= unit )
            {
                return true;
            }
        }
        return Math.abs( array[0] - array[array.length - 1] ) <= unit;
    },

    _applyRandomLeafPositions:function( display, x_coords, y_coords, addLeaf )
    {
        "use strict";
        var index = 0,
            ref_leaf = null;
        for( index = 0; index < this._leaves.length; index += 1 )
        {
            ref_leaf = this._leaves[index]._leaf_sprite;
            ref_leaf.setPosition( x_coords.pop(), y_coords.pop() );
            this._leaves[ index ]._starting_pos = ref_leaf.getPosition();

            if( addLeaf === true )
            {
                display.addChild( ref_leaf );
            }
        }
    },

    runActionOnLeaves:function( action )
    {
        "use strict";
        var index = 0;
        for( index = 0; index < this._leaves.length; index += 1 )
        {
            this._leaves[ index ]._leaf_sprite.runAction( action.clone() );
        }
    },

    stopActionsOnLeaves:function()
    {
        "use strict";
        var index = 0;
        for( index = 0; index < this._leaves.length; index += 1 )
        {
            this._leaves[ index ]._leaf_sprite.stopAllActions();
        }
    },

    animateWaterRipple:function()
    {
        "use strict";
        var index = 0;
        for( index = 0; index < this._leaves.length; index += 1 )
        {
            this._leaves[ index ].playRippleAnimation();
        }
    },

    animateFlyIn: function( mode, move_finished_callback, rotation_finished_callback )
    {
        "use strict";
        var index = 0;
        for( index = 0; index < this._leaves.length; index += 1 )
        {
            this._leaves[ index ].playFlyInAnimation( mode, move_finished_callback, rotation_finished_callback );
        }
    },

    animateLeavesSinking:function( mode, moving )
    {
        "use strict";
        var index = 0;
        for( index = 0; index < this._leaves.length; index += 1 )
        {
            this._leaves[ index ].playSinkAnimation( mode, moving, cc.CallFunc.create( this.leafFinishedSinking, this ) );
        }
    },

    leafFinishedSinking: function( leaf )
    {
        "use strict";
        this._sinkingLeafCount += 1;
        if( this._sinkingLeafCount >= this._leaves.length )
        {
            //
            // Signifies the last leaf has finished animating it's sink.
            //
            this._sinkActionCompleted();
        }
    },

    setSinkingCallback: function( func )
    {
        "use strict";
        if( typeof func === 'function' )
        {
            this._sinkActionCompleted = func;
        }
    },

    animateLeavesFlipping: function( mode, callback_action )
    {
        "use strict";
        var index = 0,
            allowInteractionCallback = cc.CallFunc.create( this._allowInteraction, this );
        for( index = 0; index < this._leaves.length; index += 1 )
        {
            this._leaves[ index ].playFlipAnimation( mode, allowInteractionCallback, callback_action );
        }
    },

    _allowInteraction: function()
    {
        "use strict"
        this.interactionResponder.setIgnoreSwipes( false );
    },

    pointLeavesInDirection: function( target_point_direction )
    {
        "use strict";
        var index = 0,
            leaf = null;
        for( index = 0; index < this._leaves.length; index +=1 )
        {
            leaf = this._leaves[ index ];
            leaf.pointInDirection( target_point_direction );
        }
    },

    leavesArePointingVertically: function()
    {
        "use strict";
        var direction = this._leaves[0]._pointing;
        return ( direction === CardinalDirections.North || direction === CardinalDirections.South );
    },

    pauseLeafAnimations: function()
    {
        "use strict";
        LumosityUtils.callFunctionOnCollection( this._leaves, "pauseAnimations" );
    },

    resumeLeafAnimations: function()
    {
        "use strict";
        LumosityUtils.callFunctionOnCollection( this._leaves, "resumeAnimations" );
    },

    stopLeafAnimations: function()
    {
        "use strict";
        LumosityUtils.callFunctionOnCollection( this._leaves, "stopAnimations");
    },

    setVisibleOnLeaves: function( visible )
    {
        "use strict";
        LumosityUtils.callFunctionOnCollection( this._leaves, "setVisible", [ visible ] );
    },

    resetLeaves: function( display )
    {
        "use strict";
        this.setVisibleOnLeaves( false );
        this.resumeLeafAnimations();
        this.stopLeafAnimations();
        this._addLeavesRandomly( display, false );
    },

    getInitialLeafPositions: function()
    {
        "use strict";
        var index = 0;
        var leafPosition = null;
        var positionsString = "";
        for( index = 0; index < this._leaves.length; index += 1 )
        {
            leafPosition = this._leaves[ index ]._leaf_sprite.getPosition();
            this._startingPositions.push( leafPosition );
            positionsString = positionsString.concat( leafPosition.x + "," + leafPosition.y + "|" );
        }
        return positionsString.slice( 0, positionsString.length - 1 );
    },

    getOffsetLeafPositions: function()
    {
        "use strict";
        var leafPosition = this._leaves[0]._leaf_sprite.getPosition();
        var startingLeafPosition = this._leaves[0]._starting_pos;
        var positionsString = "";
        positionsString = positionsString.concat( Math.round(startingLeafPosition.x - leafPosition.x) + ";" + Math.round(startingLeafPosition.y - leafPosition.y) );
        return positionsString;
    },

    getLeafScale: function()
    {
        "use strict";
        return this._leaves[ 0 ]._leaf_sprite.getScale();
    },

    _createSinkAnimations:function()
    {
        "use strict";
        var directions = CardinalDirections.getValues();
        var frames;
        var index = 0;
        var modeCounter = 0;
        var mode = 0;
        var totalModeCount = 2;
        var sharedAnimationCache = cc.AnimationCache.getInstance();
        var animationName;
        for( modeCounter = 0; modeCounter < totalModeCount; modeCounter += 1 )
        {
            for( index = 0; index < directions.length; index += 1 )
            {
                frames = this._getSinkAnimationFrames( mode, directions[ index ] );
                animationName = mode.toString() + directions[ index ].toString();
                sharedAnimationCache.addAnimation( cc.Animation.create( frames, 0.05 ),
                                                   animationName );
            }
            mode += 1;
        }
    },

    _getSinkAnimationFrames:function( mode, moving )
    {
        "use strict";
        var index = 0;
        var name = "";
        var frames = [];
        var frame_count = 60;
        var cache = cc.SpriteFrameCache.getInstance();
        var base_name = this._getBaseName( mode, moving );
        for( index = 0; index < frame_count; index += 1 )
        {
            name = String( base_name ) + ( index > 9 ? "" : "0" ) + index.toString() + ".png";
            frames.push( cache.getSpriteFrame( name ) );
        }
        return frames;
    },

    _getBaseName: function( mode, moving )
    {
        "use strict";
        return ( mode ? "green" : "orange" ) + "Leaf" + this._getSinkDirection( moving );
    },

    _getSinkDirection: function( moving )
    {
        "use strict";
        var directions = [ "SinkUpAnim0", "SinkRightAnim0", "SinkDownAnim0", "SinkLeftAnim0" ];
        return directions[ Number( moving ) ];
    } 

});
