/*global cc*/
/*global require*/

require( "src/Animation/PropertyAnimator.js" );
/*global PropertyAnimator*/
/*global LumosityUtils*/
/*global EaseWrapper*/

var AlphaAnimator;
(function()
{
    "use strict";
    
    AlphaAnimator = PropertyAnimator.extend(
    {
        /** @lends AlphaAnimator.prototype */        
        
        /**
         * An AlphaAnimator will animate alpha changes on a node. Because this can only be applied to
         * Sprite's, this is actually by far the most complicated of the PropertyAnimator's.
         *
         * @constructs
         */
        ctor: function()
        {
            this._super( "FadeTo" );
        },
        
        /**
         * The AlphaAnimator must override this because it does wonky shit related to cascading alpha
         * calls from a node down to all its sprite children, etc.
         */
        addAnimateProperty:function( prevSequence, node, endValue, duration, easeType, timeStarted )
        {
            endValue = this._convertDesignValueToGameValue( endValue, node );
            
            var actions = LumosityUtils.getFadeToForNode( node, duration, endValue );
        
            if ( actions.length > 0 )
            {
                //the node was a node, so just apply the action directly to it
                if ( actions[ 0 ].node === node )
                {
                    var action = actions[ 0 ].action;
                    action = EaseWrapper.wrapActionWithEase( action, easeType );
                    return this._addAction( prevSequence, action );
                }
                //otherwise, we had to apply the action to its children, so we need to loop through them all
                else
                {
                    var childIndex;
                    for ( childIndex = 0; childIndex < actions.length; childIndex++ )
                    {
                        var fadeAction = actions[ childIndex ].action; //this._wrapWithEaseAction( actions[ childIndex ].action, easeType );
                        var sequenceAction = cc.Sequence.create( cc.DelayTime.create( timeStarted ), fadeAction );
                    
                        //the action must be fired on that node separately, since the sequence is for the node from this function
                        actions[ childIndex ].node.runAction( sequenceAction );
                        //__animationDebugLog( "   fading a child instead of a parent, delaying for " + timeStarted + " seconds." );
                    }
                }
            }
            return prevSequence;
        },
        
        _convertDesignValueToGameValue: function( designValue )
        {
            return Math.floor( designValue * 255 );
        },
        
        _setFunction: function( node, value )
        {
            if ( node.setOpacity && !isNaN( value ) )
            {
                node.setOpacity( value );
            }
        }
    });
}() );