/*global cc*/

var GameComponent;
( function()
{
    "use strict";

    GameComponent = cc.Class.extend(
    {
        /** @lends GameComponent.prototype */
        
        /** The controller that we're linked to. */
        _controller:null,
        
        /**
         *  A GameComponent is a controller class that does some subset of functionality
         *  that we want to be reusable across multiple Games. They often will
         *  directly manipulate the GameModel and may manipulate the visuals in
         *  matching GameWidgets or in the GameView.
         *  
         *  A GameComponent can implement any of these functions in order to intercept
         *  those function calls from its Game.
         *  onUpdate( delta )
         *  onUpdateEachSecond()
         *  onBegin()
         *  onStartGameplay()
         *  onEndGameplay()
         *  onFinish()
         *
         *  @constructs
         *  @param {object} object The object who will receive delegate method calls to.t.
         */ 
         
        ctor:function( controller )
        {
            this._controller = controller;
        },
        
        /**
        * Returns Component type as a string.
        *
        * @returns {string}
        */
        getType:function()
        {
            return "GameComponent";
        },

        /**
         *  Returns a value stored in this GameComponent's model. You should always use
         *  this or a convenience method that uses this to access members this GameComponent
         *  stores in the model.
         *
         *  @param {string} key The key of the value that you want.
         *  @returns The value for the passed key. Can be any type, or undefined.
         */
        getModelValue:function( key )
        {
            return this._controller.model[ key ];
        },
        
        /**
         *  Sets a value to be stored in this GameComponent's model. You should always use
         *  this or a convenience method that uses this to set members in this
         *  component's model.
         *
         *  @param {string} key The key of the value that we want to set.
         *  @param value The value that we want to set at that key. Can be any type, or undefined.
         */
        setModelValue:function( key, value )
        {
            this._controller.model[ key ] = value;
        }
    });
}() );