var cc = cc;
var require = require;
var audio = cc.AudioEngine.getInstance();

require( "src/Game/Widget/GameWidget.js" );
var GameWidget = GameWidget;

require( "src/Game/Widget/GameWidgetFlipbook.js" );
var GameWidgetFlipbook = GameWidgetFlipbook;

require( "src/Animation/AnimationController.js" );
var AnimationController = AnimationController;

require( "src/Utility/LumosityUtils.js" );
var LumosityUtils = LumosityUtils;

//TODO - load the animation and use that
var GameWidgetFeedback = GameWidget.extend(
{
    /** @lends GameWidgetFeedback.prototype */

    /** A core object that gets drawn. */
    sprite: null,

    /** A pulse object that gets drawn behind the core sprite. */
    pulseSprite: null,

    /** A container for the feeback sprites. */
    feedbackContainer: null,

    /** A container for all flipbooks that exist in this view. */
    flipbooks:{},
    
    /** The full scale of this feedback widget. Can be increased to change the maximum size. */
    fullScale: 1.0,

    /** Flag that is set to true while feedback is being displayed. */
    _isPlaying: false,

    /** Flag that is set to true while feedback 'in' animation is playing. */
    _isPlayingIntro: false,

    /** A model object that we can grab things like score, multiplier, and meterValue from. */
    _model: null,

    /**
     * Returns Widget type as a string.
     *
     * @returns {string}
     */
    getType: function()
    {
        return "GameWidgetFeedback";
    },
    
    /**
     *  A GameWidgetFeedback is a widget that can be used to display the feedback used across most
     *  games. This includes a green checkmark or red X, multiple multipliers, and also playing
     *  sounds corresponding to the images displayed. Currently,this implementation is unfinished and broken.
     *  
     * @param  {GameModel} model The current game model.
     * @param  {Object} delegate An object to call feedback-related functions on.
     * @return {null} 
     */
    ctor:function( model, delegate )
    {
        this._super();

        this.nodeName = "Feedback::Root";
        
        this._model = model;        
        this._delegate = delegate;

        this.fullScale = this.getScale();

        this._buttons = [];

        this.feedbackContainer = cc.Node.create();
        this.addChild( this.feedbackContainer );

        // Create pulse sprite
        this.pulseSprite = cc.Sprite.create( "feedback/game_feedbackPulse2Anim000.png" );
        this.pulseSprite.setAnchorPoint( cc.p( 0.5, 0.5 ) );
        this.pulseSprite.setVisible( false );
        this.pulseSprite.nodeName = "Feedback::PulseSprite";
        this.feedbackContainer.addChild( this.pulseSprite );

        // Create sprite
        this.sprite = cc.Sprite.create( "feedback/game_feedbackCorrect.png" );
        this.sprite.setAnchorPoint( cc.p( 0.5, 0.5 ) );
        this.sprite.setVisible( false );
        this.sprite.nodeName = "Feedback::Sprite";
        this.feedbackContainer.addChild( this.sprite );

        // Setup animation controller
        this._createAnimationController();
    },

    /**
     * Preloads images
     *
     * @param settings Game settings that provides
     */
    preload:function( settings )
    {
        var baseFeedbackPath = "feedback/";

        cc.TextureCache.getInstance().addImage( baseFeedbackPath + "game_feedbackCorrect.png" );
        cc.TextureCache.getInstance().addImage( baseFeedbackPath + "game_feedbackIncorrect.png" );

        var multiplierIndex;
        // ScoreType ScoreMultiplier
        if ( settings.scoreType === 1 )
        {
            for ( multiplierIndex = 0; multiplierIndex < settings.getMultiplierCount(); multiplierIndex++ )
            {
                cc.TextureCache.getInstance().addImage( baseFeedbackPath + "game_feedbackCorrectX" + settings.getMultiplier( multiplierIndex ) + ".png" );
            }

            this.flipbooks[ "pulseAnimation" ] = new GameWidgetFlipbook( "pulseAnimation", baseFeedbackPath + "game_feedbackPulse2Anim{0}.png", 15 );
        }
    },

    /**
     *  Creates this view's AnimationController from the game's settings file. That way of doing it
     *  is temporary, later we will want to pass the animation file name into this function. The
     *  controller is also automatically created with the shared game animation data.
     */
    _createAnimationController:function()
    {
        this.animationController = new AnimationController( "feedback_animations.json", this, this, [ this ] );
        this[ "feedback" ] = this.sprite;
        this[ "pulse" ] = this.pulseSprite;
    },

    /**
     * Sets image on the main sprite.
     *
     * @param correct
     * @param multiplier
     * @private
     */
    _setImage: function( correct, multiplier )
    {
        if ( this.sprite )
        {
            var tex = "feedback/";

            if ( correct )
            {
                if ( multiplier === undefined || multiplier <= 1 )
                {
                    tex += "game_feedbackCorrect.png";
                }
                else
                {
                    tex += "game_feedbackCorrectX" + multiplier + ".png";
                }
            }
            else
            {
                tex += "game_feedbackIncorrect.png";
            }

            this.sprite.setTexture( cc.TextureCache.getInstance().addImage( tex ) );
        }
    },

    /**
     * Plays feedback sequence. You may define a custom multiplier or custom pulse show, otherwise
     * it will poll the model to see what's the correct ones to show.
     *
     * @param correct Play feedback for correct answer
     * @param [multiplier=this._model.multiplier] Multiplier to show on the sprite
     * @param [showPulse=this._model.multiplier > meterValue] Determines if pulse rings will be shown.
     * @param [forceBonus=false] If true, will play this feedback like it's a bonus, regardless of other params.
     */
    play: function( correct, multiplier, showPulse, forceBonus )
    {
        this.stop();

        var defaultMultiplier = multiplier === undefined;
        var defaultPulse = showPulse === undefined;
        
        if ( defaultMultiplier )
        {
            multiplier = 1;
        }
        
        if ( defaultPulse )
        {
            showPulse = false;
        }
        
        if( !this._model.persistentData.isPracticeMode && Math.abs( this._model.meterValue - this._model.prevMeterValue ) > 1 )
        {
            if ( defaultMultiplier )
            {
                multiplier = this._model.multiplier;
            }
            
            if ( defaultPulse )
            {
                showPulse = true;
            }
        }

        this._setImage( correct, multiplier );


        // If feedback is not currently active, start from the beginning
        // Or feedback is currently active and still is on a starting animation
        if ( this._isPlaying === false || ( this._isPlaying === true && this._isPlayingIntro === true ) )
        {
            this._isPlaying = true;
            this._isPlayingIntro = true;

            if ( correct )
            {
                if ( !forceBonus && ( multiplier === undefined || multiplier <= 1 || showPulse === false ) )
                {
                    this.animationController.play( "answerCorrectIn", this._onCorrectInFinished, this );
                }
                else
                {
                    this.animationController.play( "answerCorrectBonusIn", this._onCorrectBonusInFinished, this );
                }
            }
            else
            {
                this.animationController.play( "answerIncorrectIn", this._onIncorrectInFinished, this );
            }
        }
        else
        {
            this._isPlaying = true;

            if ( correct )
            {
                if ( multiplier <= 1 || showPulse === false )
                {
                    this._onCorrectInFinished();
                }
                else
                {
                    this._onCorrectBonusInFinished();
                }
            }
            else
            {
                this._onIncorrectInFinished();
            }
        }
    },

    stop: function()
    {
        if ( this.animationController )
        {
            this.animationController.stopAll();
        }

        if ( this.flipbooks[ "pulseAnimation" ] && this.pulseSprite )
        {
            this.flipbooks[ "pulseAnimation" ].stopAnimation( this.pulseSprite );
        }
        
        this._hideAll();

    },

    setContentSize:function( contentSize )
    {
        var children = this.getChildren();
   
        this._super( contentSize );
        
        children.forEach( function( child )
        {
            child.setPosition( cc.p( contentSize.width >> 1, contentSize.height >> 1 ) );
        } );
    },

    _onCorrectInFinished: function()
    {
        this._isPlayingIntro = false;
        this._feedbackIsLarge();
        this.animationController.play( "answerCorrect", this._feedbackIsFinished, this );
    },

    _onCorrectBonusInFinished: function()
    {
        this._isPlayingIntro = false;
        this._feedbackIsLarge();
        this.animationController.play( "answerCorrectBonus", this._feedbackIsFinished, this );
    },

    _onIncorrectInFinished: function()
    {
        this._isPlayingIntro = false;
        this._feedbackIsLarge();
        this.animationController.play( "answerIncorrect", this._feedbackIsFinished, this );
    },
    
    _feedbackIsLarge:function()
    {
        if ( this._delegate && this._delegate.feedback_isLargest !== undefined )
        {
            this._delegate.feedback_isLargest( this );
        }
    },
    
    _feedbackIsFinished:function()
    {
        this._isPlaying = false;

        if ( this._delegate && this._delegate.feedback_isFinished !== undefined )
        {
            this._delegate.feedback_isFinished( this );
        }
    },

    cue_cueSettledCorrect: function()
    {
        if ( this._delegate && this._delegate.feedback_isSettled !== undefined )
        {
            this._delegate.feedback_isSettled( this );
        }
    },

    cue_cueSettledWrong: function()
    {
        if ( this._delegate && this._delegate.feedback_isSettled !== undefined )
        {
            this._delegate.feedback_isSettled( this );
        }
    },

    cue_cueShrinkingCorrect: function()
    {
        if ( this._delegate && this._delegate.feedback_onDisappearing !== undefined )
        {
            this._delegate.feedback_onDisappearing( this );
        }
    },

    cue_cueShrinkingWrong: function()
    {
        if ( this._delegate && this._delegate.feedback_onDisappearing !== undefined )
        {
            this._delegate.feedback_onDisappearing( this );
        }
    },

    cue_cueTinyWrong: function()
    {
        if ( this._delegate && this._delegate.feedback_onTiny !== undefined )
        {
            this._delegate.feedback_onTiny( this );
        }
    },

    cue_feedbackPulse: function()
    {
        if ( this.flipbooks[ "pulseAnimation" ] && this.pulseSprite )
        {
            this.flipbooks[ "pulseAnimation" ].play( this.pulseSprite, false );
        }
    },

    /**
     * Sets initial state for the widget. This is important to call before animations start,
     * because HUD widget needs to know how to position child elements and it will ignore content
     * changes from the flash file.
     *
     * @param position
     * @param contentSize
     */
    setInitialFlashState: function( position, contentSize )
    {
        this.setPosition( position );
        this.setContentSize( contentSize );

        if ( this.sprite )
        {
            this.sprite.setPosition( cc.p( contentSize.width / 2, contentSize.height / 2 ) );
        }

        if ( this.pulseSprite )
        {
            this.pulseSprite.setPosition( cc.p( contentSize.width / 2, contentSize.height / 2 ) );
        }
    },

    _hideAll: function()
    {
        if( this.sprite )
        {
            this.sprite.setVisible( false );
        }

        if( this.pulseSprite )
        {
            this.pulseSprite.setVisible( false );
        }
    },

    onPause: function()
    {
        "use strict";
        this._super();

        this.animationController.pause( this );
    },

    onResume: function()
    {
        "use strict";
        this._super();

        this.animationController.resume( this );
    },

    onRestart: function()
    {
        "use strict";
        this._super();

        this._isPlaying = false;
        this._isPlayingIntro = false;
        if( this.flipbooks[ "pulseAnimation" ] && this.pulseSprite )
        {
            this.flipbooks[ "pulseAnimation" ].stopAnimation( this.pulseSprite );
        }

        if ( this.animationController )
        {
            this.animationController.resume( this );
            this.animationController.stopAll();
        }

        this._hideAll();
    }
});