var require = require;

var cc = cc;

require( "src/Game/MemoryMatrixBoardTileModel.js" );
var MemoryMatrixBoardTileModel = MemoryMatrixBoardTileModel;

require( "src/Utility/LumosityUtils.js" );
var LumosityUtils = LumosityUtils;

var MemoryMatrixBoardModel = cc.Class.extend(
{
    /** @lends MemoryMatrixBoardModel.prototype */

    _boardSize: cc.size( 0, 0 ),
    _boardSetup: [],

    markedTilesCount: 0,
    remainingMarkedTilesCount: 0,

    /**
     * MemoryMatrix Board
     *
     * @constructs
     */
    ctor: function( boardSize )
    {
        this._boardSize = boardSize;

        var x;
        for ( x = 0; x < this._boardSize.width; x++ )
        {
            this._boardSetup[x] = [];
        }
    },

    /**
     * Returns size of the board.
     *
     * @returns {*}
     */
    getBoardSize: function()
    {
        return this._boardSize;
    },

    /**
     * Returns tile at specified position.
     *
     * @param boardPosition
     * @returns {undefined}
     */
    getTile: function( boardPosition )
    {
        var retval = undefined;

        if ( boardPosition.x >= 0 && boardPosition.x < this._boardSize.width && boardPosition.y >= 0 && boardPosition.y < this._boardSize.height )
        {
            retval = this._boardSetup[ boardPosition.x ][ boardPosition.y ];
        }
        else
        {
            cc.log( "Count not find board tile at (" + boardPosition.x + ", " + boardPosition.y + ")" );
        }

        return retval;
    },

    /**
     *  Creates a new board layout with a given size
     *
     *  @param round Round to create new board for.
     */
    generateNewBoard: function()
    {
        var x, y;
        for ( y = 0; y < this._boardSize.height; y++ )
        {
            for ( x = 0; x < this._boardSize.width; x++ )
            {
                this._boardSetup[x][y] = new MemoryMatrixBoardTileModel( x, y, false );
            }
        }
    },

    setRandomLocationsMarked: function( markedTileCount )
    {
        var possibleTiles = [];

        var x, y;
        for ( y = 0; y < this._boardSize.height; y++ )
        {
            for ( x = 0; x < this._boardSize.width; x++ )
            {
                possibleTiles.push( cc.p( x, y ) );
            }
        }

        var tile;
        while( possibleTiles.length > markedTileCount )
        {
            tile = possibleTiles[ LumosityUtils.getRandomInt( 0, possibleTiles.length - 1 ) ];
            if ( tile )
            {
                possibleTiles.remove( tile );
            }
        }

        this.markedTilesCount = markedTileCount;
        this.remainingMarkedTilesCount = 0;
        var i;
        for( i = 0; i < possibleTiles.length; i++ )
        {
            tile = this.getTile( possibleTiles[i] );
            if ( tile )
            {
                tile.isMarked = true;
                this.remainingMarkedTilesCount++;
            }
        }
    },

    getReaminingMarkedTiles: function()
    {
        var retval = [];

        var tileModel;
        var x, y;
        for ( y = 0; y < this._boardSize.height; y++ )
        {
            for ( x = 0; x < this._boardSize.width; x++ )
            {
                tileModel = this._boardSetup[x][y];
                if ( tileModel && tileModel.isMarked && !tileModel.isFlipped )
                {
                    retval.push( tileModel );
                }
            }
        }

        return retval;
    },

    getTileNeighbors: function( tileModel )
    {
        var tiles = [];

        var tileToAdd = undefined;

        if ( tileModel )
        {
            // left
            if ( tileModel.x > 0 )
            {
                tileToAdd = this.getTile( cc.p( tileModel.x - 1, tileModel.y ) );
                if ( tileToAdd )
                {
                    tiles.push( tileToAdd );
                }
            }

            // right
            if ( tileModel.x < this._boardSize.width - 1 )
            {
                tileToAdd = this.getTile( cc.p( tileModel.x + 1, tileModel.y ) );
                if ( tileToAdd )
                {
                    tiles.push( tileToAdd );
                }
            }

            // up
            if ( tileModel.y > 0 )
            {
                tileToAdd = this.getTile( cc.p( tileModel.x, tileModel.y - 1 ) );
                if ( tileToAdd )
                {
                    tiles.push( tileToAdd );
                }
            }

            // down
            if ( tileModel.y < this._boardSize.height - 1 )
            {
                tileToAdd = this.getTile( cc.p( tileModel.x, tileModel.y + 1 ) );
                if ( tileToAdd )
                {
                    tiles.push( tileToAdd );
                }
            }
        }

        return tiles;
    },

    getTopLeftTile: function()
    {
        "use strict";

        return this.getTile( cc.p( 0, this._boardSize.height - 1 ) );
    }
});