/*global cc*/
/*global require*/
/*global lu*/
/*global GameModel*/
/*global MemoryMatrixBoardModel*/
/*global metadataAggregateKeys*/


require( "src/Game/Base/GameModel.js" );
require( "src/Game/MemoryMatrixBoardModel.js" );
require( "src/Game/Component/GameComponentMetadata.js" );


var MemoryMatrixModel = GameModel.extend(
{
    /** @lends MemoryMatrixModel.prototype */

    currentBoard: {},
    currentBoardWin: false,

    /** Number of marked tiles to appear on the board. */
    currentMarkedTileCount: 3,
    markedTileCountHistoryArray: null,

    // Current Trial
    idleTimeTilShowPrompt: 0,
    remainingTileFlips: 3,
    currentTrialTilePreviewComplete: false,
    currentTrialSucceeded: false,
    currentTrialFailed: false,
    lastTileTapped: null,
    bonusPoints: 0,

    revealingRemainingMarkedTiles: false,
    revealingRemainingMarkedTilesCount: 0,

    activeRipplesCount: 0,

    // Trial Management
    trialCount: 0,

    // Trial Metadata
    trialsCorrect: 0,
    trialsCorrectStreak: 0,
    /*
     * Use single index pointer for both correct and incorrect.
     */
    // trialCorrectTaps: 0,
    // trialIncorrectTaps: 0,
    trialTaps: 0,

    // Metadata-ish
    maxTileCount: 0,

    // Stats
    LastTileCountStat: "lastTileCount",
    
    /**
     * The Brain Shift model. Mostly adds stuff dealing with top and bottom stimuli, vowels, and consonants.
     * @constructs
     */
    ctor:function( settings)
    {
        this._super();

        this.currentBoard = undefined;

        this.currentMarkedTileCount = 3;
        this.markedTileCountHistoryArray = [];

        this.remainingTileFlips = 3;
        this.currentTrialTilePreviewComplete = false;
        this.currentTrialSucceeded = false;
        this.currentTrialFailed = false;

        this.bonusPoints = 0;

        this.revealingRemainingMarkedTiles = false;
        this.revealingRemainingMarkedTilesCount = 0;

        this.activeRipplesCount = 0;

        this.trialCount = 0;

        this.trialsCorrect = 0;
        this.trialsCorrectStreak = 0;

        this.score = 0;

        this.minGameLevel = settings.startingMarkedTileCount;
        this.maxGameLevel = settings.getMaxTileCount();

        // Metadata-ish
        this.maxTileCount = 0;
    },

    resetForTrial: function()
    {
        this.remainingTileFlips = 3;
        this.currentTrialTilePreviewComplete = false;
        this.currentTrialSucceeded = false;
        this.currentTrialFailed = false;

        this.revealingRemainingMarkedTiles = false;
        this.revealingRemainingMarkedTilesCount = 0;

        this.activeRipplesCount = 0;

        this.lastTileTapped = undefined;

        this.trialTaps = 0;

        this.currentBoardWin = false;
    },

    getThresholdLevel: function()
    {
        var historyArray = this.markedTileCountHistoryArray;

        if (historyArray.length < 2)
        {
            return 0;
        }

        if (historyArray.length === 2)
        {
            return historyArray[1];
        }

        var tiles = historyArray[1];
        var prevTiles = historyArray[0];
        var minTiles = tiles;
        var maxTiles = tiles;
        var deltaTiles = tiles - prevTiles;
        var prevDeltaTiles = 0;
        var preFlatDeltaTiles = 0;
        var threshold = 0;
        var extrema = 0;

        var index;
        for ( index = 2; index < historyArray.length; index++ )
        {
            prevTiles = tiles;
            tiles = historyArray [ index ];
            minTiles = Math.min( minTiles, tiles );
            maxTiles = Math.max( maxTiles, tiles );

            prevDeltaTiles = deltaTiles;
            deltaTiles = tiles - prevTiles;

            if ( deltaTiles !== prevDeltaTiles )
            {
                if (deltaTiles < 0)
                {
                    if ( prevDeltaTiles > 0 || preFlatDeltaTiles > 0 )
                    {
                        threshold += prevTiles;
                        extrema++;
                    }
                    preFlatDeltaTiles = 0;
                }
                else if ( deltaTiles > 0 )
                {
                    if ( prevDeltaTiles < 0 || preFlatDeltaTiles < 0 )
                    {
                        threshold += prevTiles;
                        extrema++;
                    }
                    preFlatDeltaTiles = 0;
                }
                else if ( preFlatDeltaTiles === 0 )
                {
                    preFlatDeltaTiles = prevDeltaTiles;
                }
            }
        }

        if ( tiles === minTiles || tiles === maxTiles )
        {
            threshold += tiles;
            extrema++;
        }
        threshold /= extrema;
        return threshold;
    },


    //@Override
    createGameResultsData: function ( stats, bestStatKey )
    {
        var results = this._super( stats, bestStatKey );

        var metadata = this.metadata;

        if ( results && metadata )
        {
            results[ metadataAggregateKeys.endLevel ] = metadata[ metadataAggregateKeys.endLevel ];
            // results[ metadataAggregateKeys.sessionLevel ] = metadata[ metadataAggregateKeys.sessionLevel ];
            // reassigning an undefined value. sessionLevel is assigned and deleted in gameModel when super is called.
        }

        return results;
    },

    /**
     * Expects this.settings was set.
     * @return  Level - 3, at least level 3.
     */
    getNextSessionLevel: function(level)
    {
        level -= this.settings.startingMarkedTileOffsetFromPreviousLast;
        level = Math.max(this.settings.startingMarkedTileCount,
            Math.min(this.settings.getMaxTileCount(), level));
        return level;
    },

    /**
     * GAME-3659 Save user_level.  Test case:  2015-05-29 Zassmin Montes de Oca expects to find user_level key.
     * @Override
     */
    exportResults:function( stats, bestStatKey )
    {
        var stats = {};

        stats[ this.StatsKeys.TilesStat ] = this.maxTileCount;
        this.metadata.user_level = this.currentMarkedTileCount;
        this.metadata.session_level = this.metadata.session_level + this.settings.startingMarkedTileOffsetFromPreviousLast;
        return this._super( stats, this.StatsKeys.TilesStat );
    }
});
