var require = require;
var cc = cc;

require( "src/Game/Base/GameSettings.js" );
var GameSettings = GameSettings;

/**
 * The Brain Shift settings. Adds convenience functions for determining if things exist in the vowel and consonant lists.
 * @class
 * @name MemoryMatrixSettings
 */
var MemoryMatrixSettings = GameSettings.extend(
{
    /** @lends MemoryMatrixSettings.prototype */

    tileFlippingTime: 0.25,
    tilePreviewTime: 2,

    showScore: true,
    showTime: false,
    showTrials: true,

    promptPositionPercentage: 0.75,
    idlePromptTime: 7,

    enableRipple: true,
    rippleOriginSwitchBoardSize: cc.size( 4, 4 ),

    metadataTrialLayoutCorrectCharacterSequence: "abcdefghijklmnopqrstuvwxyz!@#$",
    metadataTrialLayoutIncorrectCharacterSequence: "123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ",
    metadataTrialLayoutDefaultCharacter: 0,

    tileBoardBorderSize: cc.size( 20, 20 ),
    tileSize: cc.size ( 100, 100 ),
    smallerTileSize: cc.size ( 98, 98 ),
    tileImageSize: cc.size ( 90, 90 ),
    smallerTileImageSize: cc.size ( 88, 88 ),

    boardBackgroundColor: cc.c4b ( 0, 0, 0, 140 ),


    /**
     * Returns max possible tile count that we have a setting for.
     *
     * @return {*}
     */
    getMaxTileCount: function ()
    {
        var retval = 3;

        if ( this.markedTiles /*&& this.MarkedTiles.length > 0*/ )
        {
            var markedTileInfo = this.markedTiles[ this.markedTiles.length - 1 ];
            if ( markedTileInfo )
            {
                retval = markedTileInfo.Count;
            }
        }

        return retval;
    },

    /**
     * Returns size of the board for specified tile count.
     * If settings file is not found or it doesn't contain needed information,
     * size is defaulted to 3 x 3.
     *
     * @param markedTileCount
     * @returns {*}
     */
    getBoardSize: function( markedTileCount )
    {
        var retval = cc.size( 3, 3 );

        if ( this.markedTiles /*&& this.MarkedTiles.length > 0*/ )
        {
            var i;
            for ( i = 0; i < this.markedTiles.length; i++ )
            {
                var markedTileInfo = this.markedTiles[ i ];
                if ( markedTileInfo && markedTileInfo.Count === markedTileCount )
                {
                    retval = cc.size( markedTileInfo.Width, markedTileInfo.Height );
                }
            }
        }

        return retval;
    },


    getTileSize: function( boardSize )
    {
        if ( boardSize.width >= 6 )
        {
            return this.smallerTileSize;
        }
        else
        {
            return this.tileSize;
        }
    },

    getTileImageSize: function ( boardSize )
    {
        if ( boardSize.width >= 6 )
        {
            return this.smallerTileImageSize;
        }
        else
        {
            return this.tileImageSize;
        }
    },

    getGameLengthInTrials: function ( isFitTestMode )
    {
        if ( !isFitTestMode )
        {
            return this._super();
        }
        else
        {
            if ( LumosityUtils.isShortGamesModeOn() )
            {
                return this.shortGameLengthInTrials || 3;
            }
            return this.fitTestMode.gameLengthInTrials;
        }
    }
});
