/*global cc*/
/*global ll*/
/*global lu*/
/*global GameView*/
/*global localize*/
/*global require*/
/*global GameWidgetHud*/
/*global GameWidgetCountdown*/
/*global GameWidgetFeedback*/
/*global GameWidgetPopup*/
/*global GameWidgetLevelCard*/
/*global LumosityLabel*/
/*global LumositySprite*/
/*global LumosityUtils*/
/*global AnimationChanges*/
/*global AnimationController*/
/*global LayoutLoader*/
/*global MemoryMatrixModel*/
/*global MemoryMatrixSettings*/
/*global InteractionType*/
/*global MemoryMatrixBoardModel*/
/*global MemoryMatrixBoardTileModel*/
/*global MemoryMatrixBoardTileView*/


var MemoryMatrixView;
(function()
{
    "use strict";

    require( "src/Utility/LumosityLogger.js" );
    require( "src/Utility/LumosityUtils.js" );
    require( "src/Game/Base/GameView.js" );
    require( "src/Game/Widget/GameWidgetHud.js" );
    require( "src/Game/Widget/GameWidgetCountdown.js" );
    require( "src/Game/Widget/GameWidgetFeedback.js" );
    require( "src/Game/Widget/GameWidgetPopup.js" );
    require( "src/Game/Widget/GameWidgetLevelCard.js" );
    require( "src/View/LumosityLabel.js" );
    require( "src/View/LumositySprite.js" );
    require( "src/Animation/AnimationController.js" );
    require( "src/Animation/LayoutLoader.js" );
    require( "src/Game/MemoryMatrixModel.js" );
    require( "src/Game/MemoryMatrixSettings.js" );
    require( "src/Game/MemoryMatrixBoardModel.js" );
    require( "src/Game/MemoryMatrixBoardTileModel.js" );
    require( "src/Game/MemoryMatrixBoardTileView.js" );

    MemoryMatrixView = GameView.extend(
    {
        _boardContainerView: null,
        _boardView: null,
        _boardBackground: null,
        _boardTiles: [],

        _promptIsVisible: false,
        _originalPromptText: "Keep tapping",
        _originalPromptTextKey: "promptLabelText",

        _zOrderBoardBackground: 10,
        _zOrderBoardTiles: 20,



        ctor:function( screenSize, settings, model, interactionResponder, preload )
        {
            this._boardContainerView = null;
            this._boardView = null;
            this._boardBackground = null;
            this._boardTiles = [];

            this._promptIsVisible = false;

            this._super( screenSize, settings, model, interactionResponder, preload );

            // Hud setup, this has to be done after views and widgets have been created
            if ( this._widgets )
            {
                // Hud
                var hudWidget = this._widgets[ GameWidgetHud.prototype.getType() ];
                if ( hudWidget )
                {
                    this[ "hud" ] = hudWidget;
                    //this.animationController.setNodeIgnoredChanges( "hud", [AnimationChanges.Scale] );
                }

                // Centering countdown widget at the center of the board.
                var trayPosition = this.tray.getPosition();
                var countdownWidget = this._widgets[ GameWidgetCountdown.prototype.getType() ];
                countdownWidget.setPosition( trayPosition );
            }

            // TODO: Improve
            // Text should not change content size
            if ( this[ "prompt" ] )
            {
                this.animationController.setNodeIgnoredChanges( "prompt", [AnimationChanges.Scale] );
            }
            if ( this[ "scorePoints" ] )
            {
                this.animationController.setNodeIgnoredChanges( "scorePoints", [AnimationChanges.Scale] );
            }
        },


        onPause: function()
        {
            this._super();

            this.pauseBoardTileAnimations( true );
        },

        onResume: function()
        {
            this._super();

            this.pauseBoardTileAnimations( false );
        },

        onRestart: function()
        {
            this._super();

            this._promptIsVisible = false;

            // Reset the hud to the normal state
            var hudWidget = this._widgets[ GameWidgetHud.prototype.getType() ];
            if ( hudWidget )
            {
                hudWidget.setScoreHighlighted( false );
            }

            var i;
            var tileView;
            for ( i = 0; i < this._boardTiles.length; i++ )
            {
                tileView = this._boardTiles[ i ];
                if ( tileView )
                {
                    tileView.resumeSchedulerAndActions();
                    tileView.stopAllActions();
                }
            }
        },


        setTrial: function( trialCount, maxTrials )
        {
            if ( this._widgets )
            {
                var hudWidget = this._widgets[ GameWidgetHud.prototype.getType() ];
                if ( hudWidget )
                {
                    hudWidget.setTrial( trialCount, maxTrials );
                }
            }
        },

        setMarkedTileCount: function( tileCount )
        {
            if ( this._widgets )
            {
                var levelCardWidget = this._widgets[ GameWidgetLevelCard.prototype.getType() ];
                if ( levelCardWidget )
                {
                    var labelString;
                    if ( tileCount === 1 )
                    {
                        labelString = localize( "LevelCard_Tile", "TILE" );
                    }
                    else
                    {
                        labelString = localize( "LevelCard_Tiles", "TILES" );
                    }

                    levelCardWidget.setLevelLabel ( labelString );
                    levelCardWidget.setLevelValue ( tileCount );
                }
            }
        },

        updateBonusPoints: function()
        {
            if ( this[ "scorePoints" ] )
            {
                this[ "scorePoints" ].setString( "+" + this.model.bonusPoints );
            }
        },

        // boardSlideInAnim: function()
        // {
        //     var screenSize = cc.Director.getInstance().getWinSize();
        //     this.tray.runAction( new cc.MoveTo( 2, cc.p( screenSize.width / 2, screenSize.height / 2 ) ) );
        // },

        _createExtraFeedback: function()
        {
            var zOrder = 100;

            var newFeedbackWidget = new GameWidgetFeedback( this.model, this.interactionResponder );
            var gameContentNode = this[ "gameContent" ] || this[ "centeredContent" ];
            if ( gameContentNode )
            {
                gameContentNode.addChild( newFeedbackWidget, zOrder );
            }
            else
            {
                this._gameStage.addChild( newFeedbackWidget, zOrder );
            }
            this.addWidget( newFeedbackWidget, false, true );
            return newFeedbackWidget;
        },

        _getNextAvailableFeedback: function()
        {
            var feedbackWidget = null;

            if ( this._widgets[ GameWidgetFeedback.prototype.getType() ] instanceof Array )
            {
                var feedbackWidgets = this._widgets[ GameWidgetFeedback.prototype.getType() ];

                var i;
                for ( i = 0; i < feedbackWidgets.length; i++ )
                {
                    feedbackWidget = feedbackWidgets[ i ];
                    if ( !feedbackWidget.isBusy )
                    {
                        break;
                    }
                    feedbackWidget = null;
                }
            }
            else
            {
                feedbackWidget = this._widgets[ GameWidgetFeedback.prototype.getType() ];
                if ( feedbackWidget.isBusy )
                {
                    feedbackWidget = null;
                }
            }

            if ( feedbackWidget === null )
            {
                feedbackWidget = this._createExtraFeedback();
            }

            return feedbackWidget;
        },

        showFeedbackOverTile: function( success, tileModel )
        {
            var feedbackWidget = this._getNextAvailableFeedback();
            if ( feedbackWidget )
            {
                var feedbackPosition = cc.p( 0, 0 );

                var boardContainerViewPosition = this._boardContainerView.getPosition();
                var boardContainerViewSize = this._boardContainerView.getContentSize();
                feedbackPosition.x += boardContainerViewPosition.x - boardContainerViewSize.width / 2;
                feedbackPosition.y += boardContainerViewPosition.y - boardContainerViewSize.height / 2;

                var boardViewPosition = this._boardView.getPosition();
                var boardViewSize = this._boardView.getContentSize();
                feedbackPosition.x += boardViewPosition.x - boardViewSize.width / 2;
                feedbackPosition.y += boardViewPosition.y - boardViewSize.height / 2;

                var tileViewPosition = tileModel.tileView.getPosition();
                feedbackPosition.x += tileViewPosition.x;
                feedbackPosition.y += tileViewPosition.y;

                feedbackWidget.setVisible( true );
                feedbackWidget.setAnchorPoint( cc.p( 0, 0 ) );
                feedbackWidget.setPosition( feedbackPosition );

                feedbackWidget.tileModel = tileModel;

                // Mark this feedback busy
                feedbackWidget.isBusy = true;

                feedbackWidget.play( success );
            }
        },

        hideFeedback: function( feedbackWidget )
        {
            if ( feedbackWidget )
            {
                feedbackWidget.stop();
                feedbackWidget.isBusy = false;
                feedbackWidget.setVisible( false );
            }
        },


        deconstructBoard: function()
        {
        },

        constructBoard: function( boardObject )
        {
            ll.error( "MemoryMatrix", "constructBoard", "Why u calling me?" );
        },



        //@Override
        _createViews:function()
        {
            ll.verbose( "MemoryMatrix", "createView()" );

            // Set up
            if ( this[ "tray" ] )
            {
                this._boardContainerView = this[ "tray" ];
            }
        },

        _layoutCreatingNode: function( nodeName, nodeData )
        {
            switch( nodeName )
            {
                case "settings":
                    return false;

                case "prompt":
                    this._originalPromptText = nodeData[ LayoutLoader.textDataKey ][ LayoutLoader.textStringKey ];
                    this._originalPromptTextKey = nodeData[ LayoutLoader.textDataKey ][ LayoutLoader.textKeyKey ];
                    return null;
            }

            return this._super( nodeName, nodeData );
        },


        _setupBoardView: function( boardSize )
        {
            if ( !this._boardView )
            {
                this._boardView = cc.Node.create();
                this._boardView.setAnchorPoint( cc.p( 0.5, 0.5 ) );
                this._boardView.nodeName = "BoardView";
                this._boardContainerView.addChild( this._boardView );
            }

            if ( this._boardView )
            {
                var tileSize = this.settings.getTileSize ( boardSize );
                var boardViewWidth = tileSize.width * boardSize.width; // + this.settings.tileBoardBorderSize.width * ( boardSize.width - 1 );
                var boardViewHeight = tileSize.height * boardSize.height; // + this.settings.tileBoardBorderSize.height * ( boardSize.height - 1 );

                var boardViewSize = cc.size( boardViewWidth, boardViewHeight );
                var boardViewPosition = cc.p( this._boardContainerView.getContentSize().width / 2, this._boardContainerView.getContentSize().height / 2 );

                this._boardView.setPosition( boardViewPosition );
                this._boardView.setContentSize( boardViewSize );
            }
        },

        _setupBoardBackground: function( boardSize )
        {
            var boardViewSize = this._boardView.getContentSize();
            var backgroundPosition = cc.p( boardViewSize.width / 2, boardViewSize.height / 2 );
            var tileSize = this.settings.getTileSize ( boardSize );

            var boardBackgroundImageWidth = tileSize.width * boardSize.width + this.settings.tileBoardBorderSize.width * 2;
            var boardBackgroundImageHeight = tileSize.height * boardSize.height + this.settings.tileBoardBorderSize.height * 2;
            var backgroundImageSize = cc.size( boardBackgroundImageWidth, boardBackgroundImageHeight );

            if ( this._boardBackground )
            {
                this._boardBackground.removeFromParent( true );
                this._boardBackground = null;
            }

            if ( !this._boardBackground )
            {
                this._boardBackground = lu.createRectangularSprite ( backgroundImageSize, this.settings.boardBackgroundColor );
                this._boardBackground.nodeName = "BoardBackground";
                this._boardBackground.setOpacity( this.settings.boardBackgroundColor.a );
                this._boardBackground.setAnchorPoint( cc.p( 0.5, 0.5 ) );
                this._boardBackground.setPosition( backgroundPosition );

                this._boardView.addChild( this._boardBackground, this._zOrderBoardBackground );
            }
        },

        generateTileBoard: function()
        {
            // Check if there is a container view for the board.
            if ( !this._boardContainerView )
            {
                return false;
            }

            ll.verbose( "MemoryMatrixView", "generateTileBoard", "Generating new board." );

            // Disable and hide existing buttons
            var i;
            for ( i = 0; i < this._boardTiles.length; i++ )
            {
                this._boardTiles[ i ].setVisible( false );
                this._boardTiles[ i ].setEnabled( false );
            }

            // Get current board model
            var boardObject = this.model.currentBoard;
            if ( !boardObject )
            {
                ll.error( "MemoryMatrixView", "generateTileBoard", "Missing board data!" );
            }

            // Setup board view
            this._setupBoardView( boardObject.getBoardSize() );

            // Setup board background
            this._setupBoardBackground( boardObject.getBoardSize() );

            // Board size
            var width = boardObject.getBoardSize().width;
            var height = boardObject.getBoardSize().height;
            // Add tiles
            var addX;
            var addY;
            var tilePosition;
            var tileModel;
            var tileView;

            var tileSize = this.settings.getTileSize ( boardObject.getBoardSize() );
            var tileImageSize = this.settings.getTileImageSize ( boardObject.getBoardSize() );

            // Reset iterator
            i = 0;

            for( addY = 0; addY < height; addY++ )
            {
                for( addX = 0; addX < width; addX++ )
                {
                    tileModel = boardObject.getTile( cc.p( addX, addY ) );
                    if ( tileModel )
                    {
                        // Calculate tile position on screen
                        tilePosition = cc.p( addX * ( tileSize.width ) + tileSize.width/2, addY * ( tileSize.height ) + tileSize.height/2 );

                        // Check if there is a tile we can reuse
                        if ( i < this._boardTiles.length )
                        {
                            tileView = this._boardTiles[ i ];
                            tileView.tileModel = tileModel;
                            tileView.setDefaultTexture();
                            tileView.setDefaultState();
                        }
                        // Or create a new one
                        else
                        {
                            tileView = new MemoryMatrixBoardTileView( tileModel, this.settings, this.interactionResponder );
                            if ( tileView )
                            {
                                this._boardView.addChild( tileView, this._zOrderBoardTiles );
                                this._boardTiles.push( tileView );
                                this.addButton( tileView );
                            }
                        }

                        // Setup tile view
                        if ( tileView )
                        {
                            tileModel.tileView = tileView;
                            tileView.setPosition( tilePosition );
                            tileView.setTileSize( tileSize, tileImageSize );
                            tileView.setVisible( true );
                            tileView.setEnabled( false );
                        }

                        i++;
                    }
                }
            }
        },


        /**
         * By default cells are created disabled, so that user could not tap them before
         * they are previewed.
         */
        setTilesEnabled: function( enable )
        {
            var width = this.model.currentBoard.getBoardSize().width;
            var height = this.model.currentBoard.getBoardSize().height;
            var x, y;
            var tileModel;
            for ( y = 0; y < height; y++ )
            {
                for ( x = 0; x < width; x++ )
                {
                    tileModel = this.model.currentBoard.getTile( cc.p( x, y ) );
                    if ( tileModel && tileModel.tileView )
                    {
                        tileModel.tileView.setEnabled( enable );
                    }
                }
            }
        },

        pauseBoardTileAnimations: function( paused )
        {
            var i;
            var tileView;
            for ( i = 0; i < this._boardTiles.length; i++ )
            {
                tileView = this._boardTiles[ i ];
                if ( tileView )
                {
                    if ( paused )
                    {
                        tileView.pauseSchedulerAndActions();
                    }
                    else
                    {
                        tileView.resumeSchedulerAndActions();
                    }
                }
            }
        },


        /**
         * Temporarily reveals marked tiles.
         */
        previewMarkedTiles: function()
        {
            var width = this.model.currentBoard.getBoardSize().width;
            var height = this.model.currentBoard.getBoardSize().height;
            var x, y;
            var tileModel;
            for ( y = 0; y < height; y++ )
            {
                for ( x = 0; x < width; x++ )
                {
                    tileModel = this.model.currentBoard.getTile( cc.p( x, y ) );
                    if ( tileModel && tileModel.tileView && tileModel.isMarked )
                    {
                        tileModel.tileView.previewTile();
                    }
                }
            }
        },



        /**
         * Moves prompt container somewhere between bottom of the board and the bottom of the screen.
         */
        _adjustPromptPosition: function ( aboveTheBoard )
        {
            if ( aboveTheBoard === undefined )
            {
                aboveTheBoard = false;
            }

            // Doing it here
            if ( this[ "promptContainer" ] && this._gameStage && this._boardBackground )
            {
                var promptPosition = this[ "promptContainer" ].getPosition();
                var promptAnchorPoint = this[ "promptContainer" ].getAnchorPoint();
                var promptContentSize = this[ "promptContainer" ].getContentSize();

                // Default
                if ( !aboveTheBoard )
                {
                    var bottomOfTheBoard = cc.p ( this._boardBackground.getContentSize().width / 2, 0 );
                    var bottomOfTheBoardInWorldSpace = lu.convertPointSpace ( bottomOfTheBoard, this._boardBackground, this._gameStage );

                    promptPosition.y = ( bottomOfTheBoardInWorldSpace.y - (promptContentSize.height *  (1 - promptAnchorPoint.y) ) ) * this.settings.promptPositionPercentage;
                }
                else
                {
                    var topOfTheBoard = cc.p ( this._boardBackground.getContentSize().width / 2, this._boardBackground.getContentSize().height );
                    var topOfTheBoardInWorldSpace = lu.convertPointSpace ( topOfTheBoard, this._boardBackground, this._gameStage );

                    promptPosition.y = topOfTheBoardInWorldSpace.y + (promptContentSize.height *  (1 - promptAnchorPoint.y) );
                }

                this[ "promptContainer" ].setPosition ( promptPosition );
            }
        },

        /**
         * Shows or hides prompt that lets user know that there are more tiles to flip.
         *
         * @param show
         */
        showPrompt: function( show )
        {
            if ( show )
            {
                if ( !this._promptIsVisible )
                {
                    if ( this.prompt )
                    {
                        this.prompt.setString ( localize ( this._originalPromptTextKey, this._originalPromptText ) );
                    }

                    this._adjustPromptPosition();
                    this._promptIsVisible = true;
                    this.playAnimation( "showPrompt" );
                }
            }
            else
            {
                if ( this._promptIsVisible )
                {
                    this._promptIsVisible = false;
                    this.playAnimation( "hidePrompt" );
                }
            }
        },


        showTutorialPrompt: function ( show, message )
        {
            if ( show )
            {
                if ( this._promptIsVisible )
                {
                    this._promptIsVisible = false;
                    this.playAnimation ( "hidePrompt", this.showTutorialPrompt.bind ( this, show, message ), this );
                }
                else
                {
                    if ( this.prompt )
                    {
                        this.prompt.setString ( localize ( message, message ) );
                    }

                    this.stopAnimation("hidePrompt");
                    this._adjustPromptPosition ( true );
                    this._promptIsVisible = true;
                    this.playAnimation( "showPrompt" );
                }
            }
            else
            {
                if ( this._promptIsVisible )
                {
                    this._promptIsVisible = false;
                    this.playAnimation( "hidePrompt" );
                }
            }
        },


        /**
         * Cue to highlight score portion of the hud.
         */
        cue_hudScoreHighlight: function()
        {
            if ( !this.model.persistentData.isPracticeMode )
            {
                if ( this._widgets )
                {
                    var hudWidget = this._widgets[ GameWidgetHud.prototype.getType() ];
                    if ( hudWidget )
                    {
                        hudWidget.setScore( this.model.score );
                        hudWidget.setScoreHighlighted( true );
                    }
                }
            }
        },

        /**
         * Cue to un-highlight score portion of the hud.
         */
        cue_hudScoreNormal: function()
        {
            if ( !this.model.persistentData.isPracticeMode )
            {
                if ( this._widgets )
                {
                    var hudWidget = this._widgets[ GameWidgetHud.prototype.getType() ];
                    if ( hudWidget )
                    {
                        hudWidget.setScoreHighlighted( false );
                    }
                }
            }
        }
    });
}());