/*global cc*/
/*global ll*/
/*global sys*/
/*global require*/
/*global PauseMenu*/
/*global Interaction*/
/*global LayoutLoader*/
/*global GameWidgetHud*/
/*global LumosityUtils*/
/*global LumosityButton*/
/*global InteractionType*/
/*global GameWidgetPopup*/
/*global InteractionPhase*/
/*global InteractionSource*/
/*global GameWidgetFeedback*/
/*global AnimationController*/
/*global LumosityButtonLayer*/
/*global GameWidgetHowToPlay*/
/*global GameWidgetCountdown*/
/*global GameWidgetPauseButton*/
/*global SwipeInteractionRecognizer*/
/*global localize*/
/*global LumosityButtonState*/
/*global AnimationChanges*/
/*global GameWidgetLevelCard*/
/*global lg*/
/*global LumosityDebugMenu*/
/*global lumosity*/
/*global GameWidgetSkipTutorialButton*/
/*global ButtonFactory*/
/*global ButtonStateData*/

var DisplayState = 
{
    App:            "App",
    Game:           "Game",
    Transition:     "Transition",
    PauseMenu:      "PauseMenu",
    HowToPlayMenu:  "HowToPlayMenu"
};

//these get populated in _initializeStagePositions() below
var StagePositions =
{
    RightWing:   cc.p( 0, 0 ), 
    RightCenter: cc.p( 0, 0 ),
    Center:      cc.p( 0, 0 ),
    LeftCenter:  cc.p( 0, 0 ),
    LeftWing:    cc.p( 0, 0 )
};

var GameView;
(function()
{
    "use strict";

    require( "src/Input/Interaction.js" );
    require( "src/Input/SwipeInteractionRecognizer.js" );
    require( "src/Game/Widget/GameWidgetPauseButton.js" );
    require( "src/Game/Widget/GameWidgetSkipTutorialButton.js" );
    require( "src/Game/Widget/GameWidgetHowToPlay.js" );
    require( "src/Game/Widget/GameWidgetCountdown.js" );
    require( "src/Game/Widget/GameWidgetHud.js" );
    require( "src/Game/Widget/GameWidgetFeedback.js" );
    require( "src/Game/Widget/GameWidgetPopup.js" );
    require( "src/Game/Widget/GameWidgetLevelCard.js" );
    require( "src/Utility/LumosityUtils.js" );
    require( "src/Utility/LumosityLogger.js" );
    require( "src/Animation/LayoutLoader.js" );
    require( "src/Animation/AnimationController.js" );
    require( "src/View/LumosityButtonLayer.js" );
    require( "src/View/LumosityButton.js" );
    require( "src/View/LumosityDebugMenu.js" );
    require( "src/View/PauseMenu.js" );
    require( "src/View/Factory/ButtonFactory.js" );

    GameView = LumosityButtonLayer.extend(
    {
        /** @lends GameView.prototype */
        
        /** The GameModel. */
        model:null,
        
        /** The settings for this. Probably a GameSettings. */
        settings:null,
        
        /** An object that responds to Interaction events. Probably is a Game. */
        interactionResponder:null,
        
        /** An object that responds to Animation events by default, if none is provided. */
        animationResponder: null,
        
        /** The background image that will be used. */
        _background: null,

        /** stages */
        _stages: [],

        /** Root node for the game content. Any node attached to this will be hidden during pause menu. */
        _gameStage: null,

        /** Menu stage */
        _menuStage: null,

        /** Used primarily for stages */
        _sideStage: null,

        _pauseMenu: null,

        _pauseButton: null,

        /** Debug menu (only in debug mode) */
        _debugMenu: null,
        
        /** A container for all flipbooks that exist in this view. */
        flipbooks:{},
        
        /** This view's AnimationController. */
        animationController:null,
        
        /** All the widgets that this view uses. */
        _widgets: {},

        /** Object that recognizes swipe interactions. */
        _swipeRecognizer: null,

        /** The previously known display state. */
        _prevDisplayState: null,

        /** Represents the current state of the display: [ game, pauseMenu, howToPlayMenu, etc] */
        _curDisplayState: null,

        /** True iff the popup in practiceMode is currently on screen. */
        guidedEndPopUpPlaying: false,
        
        /**
         *  A GameView is the main cc.Layer that contains all the Cocos objects that will be rendered.
         *  This should be the ONLY place you are using any Cocos objects of any kind, aside from Widgets.
         *  Also, the GameView has its own AnimationController and uses that to play back animations.
         *  Be careful when naming elements you've added, as anything created in the animations json file
         *  could have namespace collisions, in which case weird stuff will happen to your objects.
         *
         *  If the settings param is passed into the constructor, it can be used to set non-default
         *  background image and flash file locations. The keys are: "backgroundImage", "sharedAnimationFile",
         *  and "animationFile". The latter animation file gets concatted onto the former.
         *
         *  @constructs
         *  @param {cc.Size} screenSize The size of the screen that this view will be inserted into.
         *  @param {Object} [settings] The settings that we can pull view-based data from.
         *  @param {GameModel} [model] The GameModel that we are linked to.
         *  @param {Object} [interactionResponder] The object that we should send interaction events to. We also use this is a default animation event listener.
         *  @param {boolean} [preload=true] If true, will preload all flipbooks and other assets upon creation.
         */
        ctor:function( screenSize, settings, model, interactionResponder, preload )
        {
            this._super();
            this.init();
            
            this._initializeStagePositions();
            
            this.setAnchorPoint( cc.p( 0.5, 0.5 ) );
            
            this.model = model;
            this.settings = settings;

            this.interactionResponder = interactionResponder;

            this.nodeName = "GameView";

            this._prevDisplayState = DisplayState.App;
            this._curDisplayState = DisplayState.Game;
            
            //TODO - all this can be combined and refactored
            this._createBackground( screenSize );
            this._createCoreStages();
            this._createGameMenu();
            this._createWidgets();
            this._loadLayout();
            this._createAnimationController();
            this._createViews();
            this._positionWidgets();
            this._hideViews();
            this._createDebugMenu();

            if ( preload === undefined || preload )
            {
                this.preload();
            }
            
            if ( sys.capabilities.hasOwnProperty( "touches" ) )
            {
                this.setTouchEnabled( true );
            }

            //need to make sure this function exists, specifically for HTML5
            if ( this.setKeypadEnabled )
            {
                this.setKeypadEnabled( true );
            }
            
            //similarly, make sure keyboard functionality exists, for non-browser games
            if ( this.settings.keyboardEnabled && this.setKeyboardEnabled )
            {
                this.setKeyboardEnabled( true );
            }
            
            //prep the key button aliases to use the button key instead of the key code
            if ( this.settings.keyboardAliases )
            {
                //this map is created by looking for the name of the variable in cc.KEY in our keyboardAliases object,
                //then storing the key code as the key in another map, and pointing that to the button name
                this.settings.keyboardButtonNames = {};
                
                var aliasKey;
                for ( aliasKey in this.settings.keyboardAliases )
                {
                    var cocosKey;
                    for ( cocosKey in cc.KEY )
                    {
                        if ( cocosKey === aliasKey )
                        {
                            this.settings.keyboardButtonNames[ cc.KEY[ cocosKey ] ] = this.settings.keyboardAliases[ aliasKey ];
                        }
                    }
                }
            }
        },

         /**
         * Adds a new debug button to debug menu with specified options.
         *
         * @param {Object} buttonOptions Options used to create debug button.
         * @returns {LumosityButton}
         */
        addDebugMenuButton:function( buttonOptions )
        {
            var button;
            if( this._debugMenu )
            {
                button = this._debugMenu.addDebugButton( buttonOptions );
            }
            return button;
        },

        /**
         * Create debug menu if we're in debug mode.
         * @private
         */
        _createDebugMenu:function()
        {
            if( LumosityUtils.isDebugModeOn() )
            {
                this._debugMenu = new LumosityDebugMenu();
                this._gameStage.addChild( this._debugMenu, 100 );
            }
        },

        _createUserLevelDebugButtons:function()
        {
            if (this._debugMenu && !this._debugMenu._userLevelButton) 
            {
                this.model.debugUserLevel = this.model.minGameLevel;             
                this.addDebugMenuButton( { labelText: "+", 
                    callback: function()
                    {
                        this.model._incrementDebugUserLevel(1); 
                        this._debugMenu._updateButtonLabel(this._debugMenu._userLevelButton, "Set user_level " + this.model.debugUserLevel);
                    }, 
                    target: this, closeMenuOnCallback: false } );                
                this._debugMenu._userLevelButton = this.addDebugMenuButton( { labelText: "Set user_level " + this.model.debugUserLevel,
                    callback: function()
                    {
                        this.model.setUserLevel = true;
                    },
                    target: this, closeMenuOnCallback: true} );
                this.addDebugMenuButton( { labelText: "-", 
                    callback: function()
                    {
                        this.model._incrementDebugUserLevel(-1); 
                        this._debugMenu._updateButtonLabel(this._debugMenu._userLevelButton, "Set user_level " + this.model.debugUserLevel);
                    }, 
                    target: this, closeMenuOnCallback: false } ); 
            }
        },               

        dimLayer: {},

        createDimLayer: function ()
        {
            var windowSize = cc.Director.getInstance().getWinSize();
            var shadedRect = null;

            // NAF: JIRA GAME-1852
            // Reusing the existing shadedRect so that multiple versions
            // are not created and added as children.
            if( this.dimLayer.shadedRect === undefined ) 
            {
                shadedRect = cc.Sprite.create();
                this.dimLayer.shadedRect = shadedRect;
                this.addChild( shadedRect );
            }

            this.dimLayer.shadedRect.setOpacity( 0 );
            this.dimLayer.shadedRect.setColor( cc.black() );
            this.dimLayer.shadedRect.setTextureRect( cc.rect( 0, 0, windowSize.width * 3, windowSize.height * 3) );
            
            var fadeIn = cc.FadeTo.create( 0.5, 80 );
            this.dimLayer.shadedRect.runAction( fadeIn );
        },

        createFrontDimLayer: function ()
        {
            var windowSize = cc.Director.getInstance().getWinSize();
            var rect = cc.rect( 0, 0, windowSize.width, windowSize.height );

            var frontRect = cc.Sprite.create();

            this.dimLayer.frontRect = frontRect;

            frontRect.setColor( cc.black() );
                        
            frontRect.setTextureRect( cc.rect( 0, 0, windowSize.width * 3, windowSize.height * 3) );
            
            frontRect.setOpacity( 0 );
            
            this._gameStage.addChild( frontRect, 99 );

            var fadeIn = cc.FadeTo.create( 0.3, 140 );

            frontRect.runAction( fadeIn );

            this.guidedEndPopUpPlaying = true;

        },

        removeFrontDimLayer: function ()
        {
            var fadeFrontOut = cc.FadeTo.create( 0.3, 0 );

            this.dimLayer.frontRect.runAction( fadeFrontOut );

            this.guidedEndPopUpPlaying = false;

        },

        removeDimLayer: function ()
        {
            if ( this.dimLayer && this.dimLayer.shadedRect )
            {
                var fadeOut = cc.FadeTo.create( 0.5, 0 );
                this.dimLayer.shadedRect.runAction( fadeOut );
            }
        },

        _initializeStagePositions:function()
        {
            var winSize = cc.Director.getInstance().getWinSize();

            StagePositions.RightWing    = cc.p( -winSize.width * 3 / 2, winSize.height / 2 ); 
            StagePositions.RightCenter  = cc.p( -winSize.width / 2, winSize.height / 2 );
            StagePositions.Center       = cc.p( winSize.width / 2, winSize.height / 2 );
            StagePositions.LeftCenter   = cc.p( winSize.width * 3 / 2, winSize.height / 2 );
            StagePositions.LeftWing     = cc.p( winSize.width * 5 / 2, winSize.height / 2 );
        },
        
        /**
         *  Preloads all resources that this view might need. This will also preload all widgets.
         *  Generally, a View will be the owner of all images and sounds, and will preload them here.
         */
        preload:function()
        {
            var self = this;

            LumosityUtils.forEachIn( this._widgets, function( widgetKey, widget ) 
            {
                //cc.log( "GameView::preload widgetKey: " + JSON.stringify( widgetKey ) );
                if( widget instanceof Array )
                {
                    widget.forEach( function( w )
                    {
                        w.preload();
                    } );
                }
                else
                {
                    widget.preload( self.settings );
                }

            } );
        },
        
        /**
         * Called when the game is restarted. Tear down everything extra here.
         * By default, it stops all animations. Later, gameBegin will be played.
         */
        onRestart: function()
        {
            this.stopAllAnimations( true );
            this.restartWidgets();

            // Have to update the Feedback widgets model because a new one is created onRestart.
            var feedbackWidget = this._widgets[ GameWidgetFeedback.prototype.getType() ];
            feedbackWidget._model = this.model;
        },
        
        /**
         *  Set the object that will react to interaction events.
         *  This object should have the function interaction_received( interaction ).
         *  In addition, when playAnimation is called without an onDoneTarget but also
         *  with an onDoneFunction, the view will send the interaction 
         *
         *  @param {Object} responder The object to use as the interaction responder.
         */
        setInteractionResponder:function( responder )
        {
            this.interactionResponder = responder;
        },
        
        /**
         *  If you choose to call playAnimation() withot providing an onDoneTarget (but with
         *  an onDoneFunction) then this obejct will be used as the default target. In
         *  addition, this object is added to the animation controller as a receiver of
         *  cue events.
         *
         *  @param {Object} responder The object you want responding to animation events.
         */
        setAnimationResponder:function( responder )
        {
            if ( this.animationResponder )
            {
                this.animationController.removeCueTarget( this.animationResponder );
            }
            
            this.animationResponder = responder;
            
            this.animationController.addCueTarget( this.animationResponder );
        },
        
        /**
         *  Plays an animation on this view's AnimationController. Essentially,
         *  the same as calling this.animationController.play(). However, you
         *  should always call this function instead since games may override
         *  this behavior.
         *
         *  @param {string} anim The name of the animation to play.
         *  @param {function} [onDoneFunction] The function to call when the animation is finished.
         *  @param {Object} [onDoneTarget] The object to call the onDoneFunction on. If nothing is provided but a function was provided, will default to use this view's Game.
         */
        playAnimation:function( anim, onDoneFunction, onDoneTarget )
        {
            if ( onDoneFunction && !onDoneTarget )
            {
                onDoneTarget = this.animationResponder;
            }
            
            this.animationController.play( anim, onDoneFunction, onDoneTarget );
        },
        
        /**
         *  Stops an animation that is playing using this view's AnimationController.
         *  Essentially, the same as calling this.animationController.stop(). However,
         *  you should always call this function instead since games my override
         *  this behavior.
         *
         *  @param {string} anim The animation to stop playing.
         */
        stopAnimation:function( anim )
        {
            this.animationController.stop( anim );
        },
        
        /**
         *  Stops all animations that are playing using this view's AnimationController.
         *  Essentially, the same as calling this.animationController.stopAll(). However,
         *  you should always call this function instead since games my override
         *  this behavior.
         */
        stopAllAnimations:function( stopActionTarget )
        {
            this.animationController.stopAll( stopActionTarget );
        },
        
        /**
         * Returns whether or not the specified animation exists.
         *
         * @param {string} name The animation to check for.
         * @returns {boolean} True if the animation exists, false otherwise.
         */
        hasAnimation:function( anim )
        {
            return this.animationController.hasAnimation( anim );
        },
        
        /**
         *  Adds a widget to this view's list of widgets. Can also add it as a child or
         *  allow multiple of the same kind of widget. Be careful allowing multiples, however,
         *  as it will make that widget entry an array which may not be intended behavior.
         *  As such, the default is not to allow multiples.
         *
         *  @param {GameWidget} widget The widget you want to add.
         *  @param {boolean} [addAsChild=true] Whether or not you want to immediately add this widget as a child of this GameView.
         *  @param {boolean} [allowMultiples=false] Whether or not you want to allow multiple widgets of this type.
         */
        addWidget:function( widget, addAsChild, allowMultiples )
        {
            //ll.verbose( "GameView", "addWidget", "Adding widget " + widget.getType() );

            addAsChild = addAsChild === undefined ? true : addAsChild;
            allowMultiples = allowMultiples === undefined ? false : allowMultiples;

            var widgetAdded = false;
            
            var exists = !!(this._widgets[ widget.getType() ]);
            if ( exists )
            {
                if ( allowMultiples )
                {
                    //if it's not already an array, make it an array
                    if ( !( this._widgets[ widget.getType() ] instanceof Array ) )
                    {
                        this._widgets[ widget.getType() ] = [ this._widgets[ widget.getType() ] ];
                    }
                    
                    this._widgets[ widget.getType() ].push( widget );

                    widgetAdded = true;
                }
                else
                {
                    ll.warning( "GameView", "addWidget", "Cannot add a widget because you do not want to allow multiples and the type \"" + widget.getType() + "\" already exists." );
                }
            }
            else
            {
                this._widgets[ widget.getType() ] = widget;

                widgetAdded = true;
            }

            // If widget has any buttons, we need to add them to the game to receive touches.
            if ( widgetAdded )
            {
                // add buttons
                var i;
                var buttons = widget.getButtons();
                for ( i = 0; i < buttons.length; i+=1 )
                {
                    this.addButton( buttons[i] );
                }
            }
        },

        
        /////////////////////Interactions/////////////////////
        onTouchesBegan:function( touches, event )
        {
            this._super( touches, event );
            this._createInteractionFromTouch( touches, InteractionPhase.Began );
        },
        
        onTouchesCancelled:function( touches, event )
        {
            this._super( touches, event );
            this._createInteractionFromTouch( touches, InteractionPhase.Cancelled );
        },
        
        onTouchesMoved:function( touches, event )
        {
            this._super( touches, event );
            this._createInteractionFromTouch( touches, InteractionPhase.Changed );
        },

        onTouchesEnded:function( touches, event )
        {
            this._super( touches, event );
            this._createInteractionFromTouch( touches, InteractionPhase.Ended );
        },

        onTouchBegan:function( touch, event )
        {
            this._super( touch, event );
            this._createInteractionFromTouch( [ touch ], InteractionPhase.Began );
        },
        
        onTouchCancelled:function( touch, event )
        {
            this._super( touch, event );
            this._createInteractionFromTouch( [ touch ], InteractionPhase.Cancelled );
        },
        
        onTouchMoved:function( touch, event )
        {
            this._super( touch, event );
            this._createInteractionFromTouch( [ touch ], InteractionPhase.Changed );
        },

        onTouchEnded:function( touch, event )
        {
            this._super( touch, event );
            this._createInteractionFromTouch( [ touch ], InteractionPhase.Ended );
        },
        
        pressedKeys: {},
        onKeyDown:function( keycode )
        {
            if ( this.pressedKeys[ keycode ] )
            {
                this._recordButtonInteraction( this._getKeyButtonName( keycode ), InteractionPhase.Changed, InteractionSource.Keyboard, this._getRawKeyButtonName( keycode ) );
                this._doButtonInteraction( this._getKeyButtonName( keycode ), InteractionPhase.Changed, InteractionSource.Keyboard, this._getRawKeyButtonName( keycode ) );
            }
            else
            {
                this._recordButtonInteraction( this._getKeyButtonName( keycode ), InteractionPhase.Began, InteractionSource.Keyboard, this._getRawKeyButtonName( keycode ) );
                this._doButtonInteraction( this._getKeyButtonName( keycode ), InteractionPhase.Began, InteractionSource.Keyboard, this._getRawKeyButtonName( keycode ) );
            }
            
            this.pressedKeys[ keycode ] = true;
            
        },
        
        onKeyUp:function( keycode )
        {
            this._recordButtonInteraction( this._getKeyButtonName( keycode ), InteractionPhase.Ended, InteractionSource.Keyboard, this._getRawKeyButtonName( keycode ) );
            this._doButtonInteraction( this._getKeyButtonName( keycode ), InteractionPhase.Ended, InteractionSource.Keyboard, this._getRawKeyButtonName( keycode ) );
            this.pressedKeys[ keycode ] = false;
        },
        
        _getKeyButtonName:function( keycode )
        {
            if ( this.settings.keyboardButtonNames && this.settings.keyboardButtonNames[ keycode ] )
            {
                return this.settings.keyboardButtonNames[ keycode ];
            }
            return this._getRawKeyButtonName( keycode );
        },
        
        _getRawKeyButtonName:function( keycode )
        {
            return Interaction.keyboardPrefix + String( keycode );
        },

        menuClicked:function( )
        {
            this._doButtonInteraction( "system_menu", undefined, InteractionSource.DeviceButton );
        },

        backClicked:function( )
        {
            this._doButtonInteraction( "system_back", undefined, InteractionSource.DeviceButton );
        },
        
        setScore: function( score )
        {
            if ( this._widgets )
            {
                var hudWidget = this._widgets[ GameWidgetHud.prototype.getType() ];
                if ( hudWidget )
                {
                    hudWidget.setScore( score );
                }
            }
        },

        _handleTouchBegan: function( touch, handleRadioButtons )
        {
            var button = this.getTouchBeganButton( touch, handleRadioButtons );
            if( button === null )
            {
                return null;
            }

            // Record button interaction.
            this._recordButtonInteraction( button.nodeName, InteractionPhase.Began, InteractionSource.TouchScreen );

            // Call super after interaction. Super will perform the callback interaction which could cause the game to
            // end if it was the quit button. This allows us to record the action before the quit is performed.
            this._super( touch, handleRadioButtons );

            return button;
        },

        _handleTouchCancelled: function( touch )
        {
            var button = this.getTouchCancelledButton( touch );
            if( button === null )
            {
                return null;
            }

            this._recordButtonInteraction( button.nodeName, InteractionPhase.Cancelled, InteractionSource.TouchScreen );

            // Call super after interaction. Super will perform the callback interaction which could cause the game to
            // end if it was the quit button. This allows us to record the action before the quit is performed.
            this._super( touch );

            return button;
        },

        _handleTouchEnded: function( touch )
        {
            var button = this.getTouchEndedButton( touch );
            if( button === null )
            {
                return null;
            }

            // Record button interaction.
            this._recordButtonInteraction( button.nodeName, InteractionPhase.Ended, InteractionSource.TouchScreen );

            // Call super after interaction. Super will perform the callback interaction which could cause the game to
            // end if it was the quit button. This allows us to record the action before the quit is performed.
            this._super( touch );

            return button;
        },

        //TODO - _createInteractionFromTouch appears to take in an array of touches and handle multiple touches, but
        // in reality, it only handles a single touch (index 0, see SwipeInteractionRecognizer for more details on why.)
        // Also, phase is one single phase and not an array of phases, so even if it were to handle multiple touches, it
        // would probably need to handle multiple phases as well.
        
        /**
         *  Creates an Interaction object from touches. This is mainly for convenience
         *  so we're not copy/pasting code in all of the different possible input functions.
         *
         *  @param {cc.Touch|Array} touches Either an array of touches or a single touch.
         *  @param {InteractionPhase} phase The phase of the touch you are reflecting.
         */
        _createInteractionFromTouch:function( touches, phase )
        {
            if ( !this.interactionResponder )
            {
                return;
            }

            if( this.interactionResponder.usesInteractionType( InteractionType.Swipe, phase ) )
            {
                if( !this._swipeRecognizer )
                {
                    this._swipeRecognizer = new SwipeInteractionRecognizer();
                }
                this._swipeRecognizer.checkSwipeInteraction( touches, phase, this.interactionResponder );
            }

            //only send any events if we support either type
            if ( this.interactionResponder.usesInteractionType( InteractionType.Point, phase ) || this.interactionResponder.usesInteractionType( InteractionType.MultiplePoints, phase ) )
            {
                //copy all touch data into an array of positions. Note that the getLocation() call goes to the C++ layer
                var positions = [];
                var touchIndex;
                for ( touchIndex = 0; touchIndex < touches.length; touchIndex+=1 )
                {
                    positions.push( touches[ touchIndex ].getLocation() );
                }
                
                //if we found any positions, then send the interaction events
                if ( positions.length > 0 )
                {
                    var interactionData = {};
                    interactionData[ Interaction.getPositionKey() ] = positions[ 0 ];
                    this._sendInteractionToGame( new Interaction( InteractionType.Point, phase, interactionData ) );
                    
                    var multiInteractionData = {};
                    multiInteractionData[ Interaction.getPositionsKey() ] = positions;
                    this._sendInteractionToGame( new Interaction( InteractionType.MultiplePoints, phase, multiInteractionData ) );
                }
            }
        },

         /**
         *  Constructs an interaction.
         *
         *  @param {string} buttonName The name of the button to create an interaction from.
         *  @param {InteractionPhase} [phase=InteractionPhase.Ended] The phase of the button interaction to perform.
         *  @param {InteractionSource} [interactionSource] You can provide this to create an Interaction with a different source.
         *  @param {string} [rawButtonName] The raw name of the button. This doesn't take aliases into account.
         */
        _constructInteraction: function( buttonName, phase, interactionSource, rawButtonName )
        {
            var interactionData = {};
            interactionData[ Interaction.getButtonNameKey() ] = buttonName;
                
            if ( interactionSource !== undefined )
            {
                interactionData[ Interaction.getSourceKey() ] = interactionSource;
            }
                
            if ( rawButtonName !== undefined )
            {
                interactionData[ Interaction.getRawButtonNameKey() ] = rawButtonName;
            }
                
            var interaction = new Interaction( InteractionType.Button, phase, interactionData );
            return interaction;
        },
        
        /**
         *  Creates a Button interaction. Uses the Ended phase always, and a specific button name ID.
         *
         *  @param {string} buttonName The name of the button to create an interaction from.
         *  @param {InteractionPhase} [phase=InteractionPhase.Ended] The phase of the button interaction to perform.
         *  @param {InteractionSource} [interactionSource] You can provide this to create an Interaction with a different source.
         *  @param {string} [rawButtonName] The raw name of the button. This doesn't take aliases into account.
         *  @param {boolean} [isLegacy] Used to run legacy button interaction press code.
         */
        _doButtonInteraction:function( buttonName, phase, interactionSource, rawButtonName, isLegacy )
        {
            if ( !this.interactionResponder )
            {
                return;
            }
            
            phase = phase || InteractionPhase.Ended;
            if ( this.interactionResponder.usesInteractionType( InteractionType.Button, phase ) )
            {
                var interaction = this._constructInteraction(  buttonName, phase, interactionSource, rawButtonName );
                this._sendInteractionToGame( interaction, isLegacy );
            }
        },

        /**
         *  Records a Button interaction. Uses the Ended phase always, and a specific button name ID.
         *
         *  @param {string} buttonName The name of the button to create an interaction from.
         *  @param {InteractionPhase} [phase=InteractionPhase.Ended] The phase of the button interaction to perform.
         *  @param {InteractionSource} [interactionSource] You can provide this to create an Interaction with a different source.
         *  @param {string} [rawButtonName] The raw name of the button. This doesn't take aliases into account.
         */
        _recordButtonInteraction: function( buttonName, phase, interactionSource, rawButtonName )
        {
            if ( !this.interactionResponder )
            {
                return;
            }
            
            phase = phase || InteractionPhase.Ended;
            if ( this.interactionResponder.usesInteractionType( InteractionType.Button, phase ) )
            {
                var interaction = this._constructInteraction(  buttonName, phase, interactionSource, rawButtonName ); 
                this._recordInteractionByGame( interaction );
            }
        },

        /**
         *  Records an interaction to this view's Game using the function interaction_received( interaction ).
         *
         *  @param {Interaction} interaction The Interaction that we want to record by our Game.
         */
        _recordInteractionByGame: function( interaction )
        {
            if ( !this.interactionResponder )
            {
                return;
            }

            this.interactionResponder.interaction_record( interaction );
        },
        
        /**
         *  Sends an interaction to this view's Game using the function interaction_received( interaction ).
         *
         *  @param {Interaction} interaction The Interaction that we want to send to our Game.
         */
        _sendInteractionToGame:function( interaction, isLegacy )
        {
            if ( !this.interactionResponder )
            {
                return;
            }
            
            this.interactionResponder.interaction_received( interaction, isLegacy );
        },
        /////////////////////Interactions/////////////////////
        
        
        /////////////////////Private/////////////////////
        /**
         *  Creates the background image for this GameView from the game's settings file. That way of doing it
         *  is temporary, later we will want to pass the image name into this function. The background image
         *  is constructed to fit to the size of the screen, potentially with some letterboxing.
         *
         *  @param {cc.Size} screenSize The size of the screen to fill the background with.
         */
        _createBackground:function( screenSize )
        {
            var backgroundImage = "background.jpg";
            if ( this.settings )
            {
                if( this.settings.compressedBackgroundImage && lumosity.device.id !== "html5" )
                {
                    backgroundImage = this.settings.compressedBackgroundImage;
                }
                else if( this.settings.backgroundImage )
                {
                    backgroundImage = this.settings.backgroundImage;
                }
            }

            var background = cc.Sprite.create( backgroundImage );
            if( !background || ( background.textureLoaded && !background.textureLoaded() ) )
            {
                background = cc.Sprite.create( "background.jpg" );
            }
            //
            // Fall back on PNG as a default format. We need an asset manager. 
            //
            if ( !background || ( background.textureLoaded && !background.textureLoaded() ) )
            {
                background = cc.Sprite.create( "background.png" );
            }

            if( background )
            {
                background.setAnchorPoint( cc.p( 0.5, 0.5 ) );
                background.setPosition( cc.p( screenSize.width / 2, screenSize.height / 2 ) );
                var backgroundSize = background.getContentSize();

                //scale it to the width of the screen
                var backgroundScale = Math.max( screenSize.width / backgroundSize.width, screenSize.height / backgroundSize.height );
                background.setScale( backgroundScale );

                //add it to the scene
                this.addChild( background, 0 );
                this._background = background;
                this._background.nodeName = "Background";
            }


            return background;
        },

        /**
         *  Loads the view's layout from a layout.json file, if it exists. It's recommended you use this file for all games,
         *  since it has all resizing data within it also. Without this, stuff will likely not resize correctly.
         */
        _loadLayout: function()
        {
            var layoutFile = "layout.json";
            
            if ( this.settings && this.settings.layoutFile )
            {
                layoutFile = this.settings.layoutFile;
            }
            
            LayoutLoader.loadLayoutFile( layoutFile, this._gameStage, this, this );
        },
        
        /**
         *  DEPRECATED. Use layout_creatingNode( nodeName. nodeData ) instead.
         */
        _layoutCreatingNode: function( nodeName, nodeData )
        {
            var hudWidget = this._widgets[ GameWidgetHud.prototype.getType() ];
            var textData;
            var font;
            var fontSize;
            var colorData;
            var color;
            var levelCardWidget;
            
            switch( nodeName )
            {
                case "vignette":
                    var vignetteSprite = cc.Sprite.create( nodeData[ LayoutLoader.imageKey ] );
                    this.addChild( vignetteSprite );
                    return vignetteSprite;

                case "hud":
                    return this._widgets[ GameWidgetHud.prototype.getType() ];

                case "hudBackgroundColor":
                    if ( hudWidget )
                    {
                        textData = nodeData[ LayoutLoader.textDataKey ];
                        colorData = textData[ LayoutLoader.fontColorKey ];
                        color = LayoutLoader.getColor( colorData );

                        hudWidget.setBackgroundColor( color );
                    }
                    return false;

                case "hudBackgroundHighlight":
                    if ( hudWidget )
                    {
                        textData = nodeData[ LayoutLoader.textDataKey ];
                        colorData = textData[ LayoutLoader.fontColorKey ];
                        color = LayoutLoader.getColor( colorData );

                        hudWidget.setHighlightColor( color );
                    }
                    return false;

                case "hudText":
                    if ( hudWidget )
                    {
                        textData = nodeData[ LayoutLoader.textDataKey ];
                        colorData = textData[ LayoutLoader.fontColorKey ];
                        color = LayoutLoader.getColor( colorData );

                        hudWidget.setTextColor( color );
                    }
                    return false;
                    
                case "feedback":
                    if ( this.getFeedbackWidget() )
                    {
                        this.getFeedbackWidget().wasPositioned = true;
                        return this.getFeedbackWidget();
                    }
                    return null;
                
                case "countdown":
                    if ( this._widgets[ GameWidgetCountdown.prototype.getType() ] )
                    {
                        /*
                        calculating new origin and node size according to screen size for 
                        countdown widgets where position is customizable through game's 
                        layout and flash file

                        This is because the countdown widget is full screen and the game's 
                        flash countdown symbol is not
                        */

                        var screenSize = cc.Director.getInstance().getWinSize();
                        var centerPos = cc.p(nodeData.size.width / 2 + nodeData.origin.x, 
                                            nodeData.size.height / 2 + nodeData.origin.y);
                        var newOrigin = cc.p( centerPos.x - screenSize.width / 2 , centerPos.y - screenSize.height / 2 );

                        nodeData.origin = newOrigin;
                        nodeData.size = screenSize;

                        this._widgets[ GameWidgetCountdown.prototype.getType() ].wasPositioned = true;
                        return this._widgets[ GameWidgetCountdown.prototype.getType() ];
                    }
                    return null;
                    
                case "popup":
                    if ( this._widgets[ GameWidgetPopup.prototype.getType() ] )
                    {
                        this._widgets[ GameWidgetPopup.prototype.getType() ].wasPositioned = true;
                        return this._widgets[ GameWidgetPopup.prototype.getType() ];
                    }
                    return null;

                case "levelCard":
                    // create a level card widget in this node's place
                    levelCardWidget = this.getWidget( GameWidgetLevelCard.prototype.getType(), true );
                    levelCardWidget.setDesiredContentSize( nodeData.size );
                    return levelCardWidget;

                case "levelCardLabel":
                    levelCardWidget = this.getWidget( GameWidgetLevelCard.prototype.getType(), true );
                    if ( levelCardWidget )
                    {
                        textData = nodeData[ LayoutLoader.textDataKey ];

                        font = textData[ LayoutLoader.fontKey ];
                        fontSize = textData[ LayoutLoader.fontSizeKey ];

                        colorData = textData[ LayoutLoader.fontColorKey ];
                        color = LayoutLoader.getColor( colorData );

                        levelCardWidget.setLabelTextStyle( font, fontSize, color );
                    }
                    return false;

                case "levelCardNumber":
                    levelCardWidget = this.getWidget( GameWidgetLevelCard.prototype.getType(), true );
                    if ( levelCardWidget )
                    {
                        textData = nodeData[ LayoutLoader.textDataKey ];

                        font = textData[ LayoutLoader.fontKey ];
                        fontSize = textData[ LayoutLoader.fontSizeKey ];

                        colorData = textData[ LayoutLoader.fontColorKey ];
                        color = LayoutLoader.getColor( colorData );

                        levelCardWidget.setValueTextStyle( font, fontSize, color );
                    }
                    return false;
                    
                //for safety, don't let people override the settings reference
                case "settings":
                    return false;
                    
                //for safety, don't let people override the view reference
                case "model":
                    return false;
            }
            return null;
        },
        
        /**
         *  When _loadLayout() loads a layout file, we're the delegate, so this function is called before any of the 
         *  layout actions are actually performed. This is mainly so that you can can store off or even modify data
         *  before the LayoutLoader ever gets its mitts on it. It's useful, for example, in the case when you want
         *  to store off some data that will be used in the creation of another node in layout_creatingNode(). Since
         *  the order of the data being loaded is not guaranteed, you cannot do it all safely within the
         *  layout_creatingNode() function. Due to layout_readingData() always being called first, you can rely on it.
         *
         *  @param {string} nodeName The name of the node that will be created. This will correspond with the layout and animation files.
         *  @param {Object} nodeData The data that will be used to create the node. You can use the constant string keys in LayoutLoader to find data.
         */
        layout_readingData: function( nodeName, nodeData )
        {
            return undefined;
        },
        
        /**
         *  When _loadLayout() loads a layout file, we're the delegate, so this is the create function. That means,
         *  as the LayoutLoader creates nodes, it will first send the node data into this function. If you wish
         *  to create your own node instead of the default behavior, you'll want to override this function and
         *  look for what you want. If you return a node, then the LayoutLoader will use that intead (and apply
         *  all resizing data to that node). If you return null, then the LayoutLoader will create the default.
         *
         *  @param {string} nodeName The name of the node we want to create. This will correspond with the layout and animation files.
         *  @param {Object} nodeData The data that we will use to create the node. You can use the constant string keys in LayoutLoader to find data.
         *  @returns {cc.Node|boolean} If you've created a node, return that. Otherwise, return null and the LayoutLoader will create a default node. You may also return false to create nothing.
         */
        layout_creatingNode: function( nodeName, nodeData )
        {
            return this._layoutCreatingNode( nodeName, nodeData );
        },
        
        /**
         *  When the layout file is loaded, this function will be called whenever the loader is trying
         *  to add a node to a parent. You can do whatever you want in this case. If you return false,
         *  that means that the layout loader will not add the node.
         *
         *  @param {string} nodeName The name of the node that we'll be adding.
         *  @param {Object} nodeData The data from the layout file for this node.
         *  @param {cc.Node} node The node that we will be adding.
         *  @param {cc.Node} parentNode The node that we will be adding to.
         *  @returns {boolean} If you return true, the LayoutLoader will do its default behavior. If false, it will not.
         */
        layout_addingNodeToParent: function( nodeName /*, nodeData, node, parentNode */ )
        {
            if ( nodeName === "vignette" )
            {
                return false;
            }

            return true;
        },
        
        /**
         *  When the layout file is loaded, this function will be called whenever the loader is trying
         *  to write the node to the view dictionary. You can do whatever you want in this case. If you return false,
         *  that means that the layout loader will not write the node to the dictionary.
         *
         *  @param {string} nodeName The name of the node that we'll be writing.
         *  @param {Object} nodeData The data from the layout file for this node.
         *  @param {cc.Node} node The node that we will be writing.
         *  @param {Object} viewDictionary The dictionary that we will be writing the node to.
         *  @returns {boolean} If you return true, the LayoutLoader will do its default behavior. If false, it will not.
         */
        layout_writingNodeToDictionary: function( /* nodeName, nodeData, node, viewDictionary */ )
        {
            return true;
        },
        
        /**
         *  When the layout file is loaded, this function will be called whenever the loader is trying
         *  to set content size on the node. You can do whatever you want in this case. If you return false,
         *  that means that the layout loader will not set layoutSize or contentSize.
         *
         *  @param {string} nodeName The name of the node that we'll be setting to.
         *  @param {Object} nodeData The data from the layout file for this node.
         *  @param {cc.Node} node The node that we will be setting to.
         *  @param {cc.Size} size The size that we want to set.
         *  @returns {boolean} If you return true, the LayoutLoader will do its default behavior. If false, it will not.
         */
        layout_settingNodeSize: function( /* nodeName, nodeData, node, size */ )
        {
            return true;
        },
        
        /**
         *  When the layout file is loaded, this function will be called whenever the loader is trying
         *  to set an anchor on the node. You can do whatever you want in this case. If you return false,
         *  that means that the layout loader will not set an anchor.
         *
         *  @param {string} nodeName The name of the node that we'll be setting to.
         *  @param {Object} nodeData The data from the layout file for this node.
         *  @param {cc.Node} node The node that we will be setting to.
         *  @param {cc.Point} anchor The anchor that we want to set.
         *  @returns {boolean} If you return true, the LayoutLoader will do its default behavior. If false, it will not.
         */
        layout_settingNodeAnchor: function( /* nodeName, nodeData, node, anchor */ )
        {
            return true;
        },
        
        /**
         *  When the layout file is loaded, this function will be called whenever the loader is trying
         *  to set resizing info on the node. You can do whatever you want in this case. If you return false,
         *  that means that the layout loader will not set resizing info.
         *
         *  @param {string} nodeName The name of the node that we'll be setting to.
         *  @param {Object} nodeData The data from the layout file for this node.
         *  @param {cc.Node} node The node that we will be setting to.
         *  @param {Object} resizingData The resizingData that we want to set.
         *  @returns {boolean} If you return true, the LayoutLoader will do its default behavior. If false, it will not.
         */
        layout_settingNodeResizingInfo: function( /* nodeName, nodeData, node, resizingData */ )
        {
            return true;
        },
        
        /**
         *  When the layout file is loaded, this function will be called whenever the loader is trying
         *  to set flags on the node. You can do whatever you want in this case. If you return false,
         *  that means that the layout loader will not set flags.
         *
         *  @param {string} nodeName The name of the node that we'll be setting to.
         *  @param {Object} nodeData The data from the layout file for this node.
         *  @param {cc.Node} node The node that we will be setting to.
         *  @returns {int} Returns the desired flags.
         */
        layout_settingNodeFlags: function( /* nodeName, nodeData, node */ )
        {
            return 0xFF;
        },
        
        /**
         *  When the layout file is loaded, this function will be called whenever the loader is trying
         *  to set the intial position and scale of the node. You can do whatever you want in this case.
         *  If you return false, that means that the layout loader will not set the position or scale.
         *
         *  @param {string} nodeName The name of the node that we'll be setting to.
         *  @param {Object} nodeData The data from the layout file for this node.
         *  @param {cc.Node} node The node that we will be setting to.
         *  @param {cc.Point} origin The position in the design resolution to set the node to.
         *  @param {cc.Size} size The size in the design resolution to set the node to.
         *  @returns {boolean} If you return true, the LayoutLoader will do its default behavior. If false, it will not.
         */
        layout_settingNodeFrame: function( /* nodeName, nodeData, node, origin, size */ )
        {
            return true;
        },
        
        /**
         *  When the layout file has finished loading and doing everything to a particular node, this
         *  function will be called. The LayoutLoader itself does nothing here, this is merely a place
         *  you can intercept in case you want to do something to a node after it is fully created.
         *
         *  @param {string} nodeName The name of the node that has finished being created.
         *  @param {Object} nodeData The data from the layout file for this node.
         *  @param {cc.Node} node The node that has finished being created.
         */
        layout_wrappingUp: function( nodeName, nodeData, node )
        {
            return undefined;
        },
        
        /**
         *  Creates this view's AnimationController from the game's settings file. That way of doing it
         *  is temporary, later we will want to pass the animation file name into this function. The
         *  controller is also automatically created with the shared game animation data.
         */
        _createAnimationController:function()
        {
            var sharedAnimFile = "shared_animations.json";
            if ( this.settings && this.settings.sharedAnimationFile )
            {
                sharedAnimFile = this.settings.sharedAnimationFile;
            }
            
            var animFile = "animations.json";
            if ( this.settings && this.settings.animationFile )
            {
                animFile = this.settings.animationFile;
            }
            
            this.animationController = new AnimationController( [ sharedAnimFile, animFile ], this, this, [ this ] );
        },

        /**
         * Creates root game content node, everything but pause button, game menu and few custom things
         * should be attached to this node. This node slides in and out of the view when the game is paused.
         *
         * @private
         */
        _createCoreStages: function()
        {
            // Core stages should be the same size as the screen
            var screenSize = cc.Director.getInstance().getWinSize();

            this._gameStage = cc.Node.create();
            this._gameStage.setPosition( cc.p( screenSize.width / 2, screenSize.height / 2 ) );
            this._gameStage.setContentSize( cc.size( screenSize.width, screenSize.height ) );
            this._gameStage.setAnchorPoint( cc.p( 0.5, 0.5 ) );
            this._gameStage.nodeName = "GameStage";
            this.addChild( this._gameStage, 100 );

            this._menuStage = cc.Node.create();
            this._menuStage.setPosition( cc.p( -screenSize.width / 2, screenSize.height / 2 ) );
            this._menuStage.setContentSize( cc.size( screenSize.width, screenSize.height ) );
            this._menuStage.setAnchorPoint( cc.p( 0.5, 0.5 ) );
            this._menuStage.setVisible( false );
            this._menuStage.nodeName = "MenuStage";
            this.addChild( this._menuStage, 100 );

            this._sideStage = cc.Node.create();
            this._sideStage.setPosition( cc.p( -screenSize.width * 3 / 2, screenSize.height / 2 ) );
            this._sideStage.setContentSize( cc.size( screenSize.width, screenSize.height ) );
            this._sideStage.setAnchorPoint( cc.p( 0.5, 0.5 ) );
            this._sideStage.setVisible( false );
            this._sideStage.nodeName = "SideStage";
            this.addChild( this._sideStage, 100 );
        },

        /**
         *  Creates the game pause menu.
         */
        _createGameMenu: function()
        {
            var self = this;
            this._pauseMenu = new PauseMenu( this.interactionResponder );
            this._menuStage.addChild( this._pauseMenu );

            // Add buttons so they monitor for touch events
            this._pauseMenu.buttons.forEach( function( button )
            {
                self.addButton( button );
            } );
        },

        /**
         *  Returns the game pause menu.
         */
        getPauseMenu: function()
        {
            return this._pauseMenu;
        },
        
        /**
         *  Override this to create all the views that your GameView will need.
         *  Eventually, we will do this automatically from the Flash data.
         */
        _createViews: function()
        {
            ll.verbose( "GameView", "_createViews" );
            //implement in subclasses
        },
        
        /**
         *  Create a Node and attach it to this GameView.
         *
         *  @param {cc.Point} position The position at which to create the Node.
         *  @param {cc.Size} size The size to create the Node with.
         *  @returns {cc.Node} The created Node.
         */
        _createNode:function( position, size )
        {
            //Create an empty CCNode
            var node = cc.Node.create();
            var parent_height = this.getContentSize().height;
            node.setPosition( cc.p( position.x, parent_height - ( position.y + size.height/2 ) ) );
            node.setContentSize( size );
            node.setAnchorPoint( cc.p( 0.5, 0.5 ) );
            return node;
        },

        /**
         *  Create a Sprite.  Does not attach it to this GameView.
         *
         *  @param {string} filename The file name of the Sprite that you want to load.
         *  @returns {cc.Sprite} The created Sprite.
         */
        _createSprite:function( filename )
        {
            var sprite = cc.Sprite.create( filename );
            
            if ( sprite )
            {
                sprite.setAnchorPoint( cc.p( 0.5,0.5 ) );
            }
            else
            {
                ll.warning( "Tried to _createSprite with filename " + filename + ", but could not." );
            }
            
            return sprite;
        },

        /**
         *  Create a button and attach it to this GameView.
         *
         *  @param {string} upFilename The file name for the up (unpressed) image for this button.
         *  @param {string} [downFilename] The file name for the down (pressed) image for this button.
         *  @param {function|string} [onClickFunctionOrButtonName] If this is a function, then that function will be called when this button is pressed. If it's a string, then an interaction event will be fired with that button name.
         *  @param {string} [text] The text to show in the button label.
         *  @param {string} [font] The font to show in the button label.
         *  @param {number} [textSize] The size of the text in the button label.
         *  @param {cc.Color} [textColor] The color of the text in the button label.
         *  @param {cc.Color} [textDownColor] The color of the text in the button label when pressed.
         *  @param {cc.size} [contentSize] The content size of the button.
         *  @returns {cc.MenuItemImage} The created button.
         */
        _createButton:function( upFilename, downFilename, onClickFunctionOrButtonName, text, font, textSize, textColor, textDownColor, contentSize )
        {
            var backgroundData = new ButtonStateData( upFilename, downFilename );
            var textColorData = new ButtonStateData( textColor, textDownColor );
            var button = ButtonFactory.createImageTextButton( backgroundData, undefined, text, textColorData, font, textSize, contentSize );

            button.setTarget( this );
            
            //if you passed a string in, then that means we want to use the button interaction behavior
            if ( typeof onClickFunctionOrButtonName === "string" )
            {
                // NAF: Should be noted that this is legacy and games that go through this way of handling button presses will not work with automated scripts.
                button.setPressedCallback( this._doButtonInteraction.bind( this, onClickFunctionOrButtonName, InteractionPhase.Began, InteractionSource.TouchScreen, onClickFunctionOrButtonName, true ) );
                button.setHeldCallback( this._doButtonInteraction.bind( this, onClickFunctionOrButtonName, InteractionPhase.Changed, InteractionSource.TouchScreen, onClickFunctionOrButtonName, true ) );
                button.setReleasedCallback( this._doButtonInteraction.bind( this, onClickFunctionOrButtonName, InteractionPhase.Ended, InteractionSource.TouchScreen, onClickFunctionOrButtonName, true ) );
            }
            //otherwise, we assume you passed in a custom function
            else if ( onClickFunctionOrButtonName )
            {
                button.setReleasedCallback( onClickFunctionOrButtonName );
            }
            
            this.addButton( button );
            
            //also add in held and released callbacks
            
            return button;
        },

        /**
         *  Create a Label and attach it to either this GameView or a parent.
         *
         *  @param {string} text The text to show in the label.
         *  @param {string} font The font to set for the label.
         *  @param {number} size The font size in points for the label.
         *  @param {cc.c4b} color The color for the text in the label.
         *  @param {cc.Size} screenSize The size of the screen.
         *  @param {cc.Node} [parent=this] The parent to add this label to. 
         *  @returns {cc.Sprite} The created Sprite.
         */
        _createLabel:function( text, font, size, color, screenSize, parent )
        {
            var label = cc.LabelTTF.create( text, font, size );
            label.setColor( color );
            label.setAnchorPoint( cc.p( 0.5, 0.5 ) );
            // position the label off screen
            label.setPosition( cc.p( screenSize.width / 2, screenSize.height ) );
            label.setHorizontalAlignment( cc.TEXT_ALIGNMENT_CENTER );
            label.setVerticalAlignment( cc.VERTICAL_TEXT_ALIGNMENT_CENTER );
            label.setVisible( false );
            
            AnimationChanges.addNodeIgnoredChanges( label, AnimationChanges.Scale );

            if ( !parent )
            {
                parent = this;
            }
            
            // add the label as a child to this layer
            parent.addChild( label );
            return label;
        },

        
        /**
         *  Override this in order to create all the widgets that this GameView uses.
         */
        _createWidgets:function()
        {
            var zOrder = 100;

            // Pause menu
            var pauseMenu = new GameWidgetPauseButton( this.model, this.settings, this.interactionResponder, true );
            this.addChild( pauseMenu, zOrder );
            this.addWidget( pauseMenu, false, false );

            // How To Play
            var sideStageContentSize = this._sideStage.getContentSize();
            var howToPlay = new GameWidgetHowToPlay( this.model, this.settings, this.interactionResponder, this, true );
            howToPlay.setPosition( cc.p( sideStageContentSize.width / 2, sideStageContentSize.height / 2 ) );
            this._sideStage.addChild( howToPlay );
            this.addWidget( howToPlay );

            // Hud
            this.hud = new GameWidgetHud( this.settings, this.interactionResponder, true );
            this._gameStage.addChild( this.hud, zOrder );
            this.addWidget( this.hud );

            // Skip tutorial button
            var skipTutorial = new GameWidgetSkipTutorialButton( this.model, this.settings, this.interactionResponder, true );
            this._gameStage.addChild( skipTutorial, zOrder + 1 );
            this.addWidget( skipTutorial );

            // Count down
            var countDownWidget = new GameWidgetCountdown( this.settings, this.interactionResponder, true );
            this._gameStage.addChild( countDownWidget, zOrder );
            this.addWidget( countDownWidget );

            
            // Feedback (Placement and what the parent is changes from game to game).
            var feedbackWidget = new GameWidgetFeedback( this.model, this.interactionResponder );
            this._gameStage.addChild( feedbackWidget, zOrder );
            this.addWidget( feedbackWidget );

            // Popup
            var gameStageContentSize = this._gameStage.getContentSize();
            var feedbackPopup = new GameWidgetPopup( this.model, this.interactionResponder );
            feedbackPopup.setPosition( cc.p( gameStageContentSize.width / 2, gameStageContentSize.height / 2 ) );
            this._gameStage.addChild( feedbackPopup, zOrder );
            this.addWidget( feedbackPopup );
        },

        _positionWidgets: function()
        {
            var gameContentNode = this.gameContent || this.centeredContent;
            if ( gameContentNode )
            {
                var gameContentSize = gameContentNode.getContentSize();

                // Countdown
                var countdownWidget = this._widgets[ GameWidgetCountdown.prototype.getType() ];

                if ( countdownWidget && !countdownWidget.wasPositioned )
                {
                    countdownWidget.removeFromParent( false );
                    gameContentNode.addChild( countdownWidget );

                    countdownWidget.setPosition( cc.p( gameContentSize.width / 2, gameContentSize.height / 2 ) );
                }

                // Feedback
                var feedbackWidget = this._widgets[ GameWidgetFeedback.prototype.getType() ];
                if ( feedbackWidget && !feedbackWidget.wasPositioned  )
                {
                    feedbackWidget.removeFromParent( false );
                    gameContentNode.addChild( feedbackWidget );

                    feedbackWidget.setPosition( cc.p( gameContentSize.width / 2, gameContentSize.height / 2 ) );
                }
            }
        },

        getWidget: function ( type, createIfNotFound )
        {
            var widget = this._widgets[ type ];
            if ( !widget && createIfNotFound )
            {
                if ( type === GameWidgetLevelCard.prototype.getType() )
                {
                    widget = new GameWidgetLevelCard( this.model, this.interactionResponder, true );
                }

                if ( widget )
                {
                    this.addWidget( widget );
                }
            }

            return widget;
        },
        
        _hideViews: function()
        {
            //hide everything in the GameRoot
            var childIndex;
            var children = this._gameStage.getChildren();
            for ( childIndex = 0; childIndex < children.length; childIndex+=1 )
            {
                children[ childIndex ].setVisible( false );
            }
        },

        /**
         * This cue appears in all of our game flash files and signifies start of the countdown to the start of the game.
         */
        cue_startCountdown: function()
        {
            if ( !this.model.restartToPracticeMode )
            {
                this.showCountdown();
            }
        },

        /**
         * If there is a countdown widget, this will start countdown sequence.
         */
        showCountdown: function()
        {
            if ( this._widgets )
            {
                var countDownWidget = this._widgets[ GameWidgetCountdown.prototype.getType() ];
                if ( countDownWidget )
                {
                    countDownWidget.play();
                }
                else
                {
                    ll.error( "GameView", "showCountdown", "Error! Could not find Game Widget Countdown." );
                }
            }
        },

        /**
         * Standard cue to show popup after the game.
         */
        cue_popUp2Lines1Slide: function()
        {
            if ( !this.model.persistentData.isPracticeMode )
            {
                this.showPostGamePopUp();
            }
            else
            {
                this.showPostPracticePopUp();
            }
        },


        showPostGamePopUp: function ()
        {
            var popupWidget = this._widgets[ GameWidgetPopup.prototype.getType() ];
            if ( popupWidget )
            {
                var string = localize( this.settings.popupTopTextKeys.getRandomElement() );
                var strings = string.split("\n");
                strings.push( this.model.score + " " + localize( "PopupBottomTextPoints" ) );
                var stringsLength = strings.length;
                var lines = 0;

                if( stringsLength >= 1 )
                {
                    popupWidget.setFirstText( strings[ 0 ] );
                    lines++;
                }

                if( stringsLength >= 2 )
                {
                    popupWidget.setSecondText( strings[ 1 ] );
                    lines++;
                }

                if( stringsLength >= 3 )
                {
                    popupWidget.setThirdText( strings[ 2 ] );
                    lines++;
                }

                if( stringsLength >= 4 )
                {
                    popupWidget.setFourthText( strings[ 3 ] );
                    lines++;
                }

                popupWidget.play( 1, lines );
            }
        },

        showPostPracticePopUp:function ()
        {
            var popupWidget = this._widgets[ GameWidgetPopup.prototype.getType() ];
            if ( popupWidget )
            {
                var topText = localize( "PracticePopUpTop" );
                var bottomText = localize( "PracticePopUpBottom" );
                var strings = topText.split("\n");
                strings = strings.concat( bottomText.split("\n") );

                cc.log( "strings" + JSON.stringify( strings ) );
                var stringsLength = strings.length;
                var lines = 0;

                if( stringsLength >= 1 )
                {
                    popupWidget.setFirstText( strings[ 0 ] );
                    lines++;
                }

                if( stringsLength >= 2 )
                {
                    popupWidget.setSecondText( strings[ 1 ] );
                    lines++;
                }

                if( stringsLength >= 3 )
                {
                    popupWidget.setThirdText( strings[ 2 ] );
                    lines++;
                }

                if( stringsLength >= 4 )
                {
                    popupWidget.setFourthText( strings[ 3 ] );
                    lines++;
                }

                popupWidget.play( 1, lines );
            }
        },

        showPostPracticeUnguidedPopUp:function ()
        {
            var popupWidget = this._widgets[ GameWidgetPopup.prototype.getType() ];
            if ( popupWidget )
            {
                var topText = localize( "PracticeUnGuidedPopUpTop" );
                var bottomText = localize( "PracticeUnGuidedPopUpBottom" );
                var strings = topText.split("\n");
                strings = strings.concat( bottomText.split("\n") );

                var stringsLength = strings.length;
                var lines = 0;

                if( stringsLength >= 1 )
                {
                    popupWidget.setFirstText( strings[ 0 ] );
                    lines++;
                }

                if( stringsLength >= 2 )
                {
                    popupWidget.setSecondText( strings[ 1 ] );
                    lines++;
                }

                if( stringsLength >= 3 )
                {
                    popupWidget.setThirdText( strings[ 2 ] );
                    lines++;
                }

                if( stringsLength >= 4 )
                {
                    popupWidget.setFourthText( strings[ 3 ] );
                    lines++;
                }

                popupWidget.play( 1, lines );
            }
        },

        showEndTutorialPopup:function( skipped )
        {
            var popupWidget = this._widgets[ GameWidgetPopup.prototype.getType() ];
            if( popupWidget )
            {
                if( skipped )
                {
                    this.showSkippedEndTutorialPopup();
                }
                else
                {
                    this.showCompletedEndTutorialPopup();
                }
            }
        },

        showSkippedEndTutorialPopup: function()
        {
            var popupWidget = this._widgets[ GameWidgetPopup.prototype.getType() ];

            if( popupWidget )
            {
                var strings = LumosityGlobals.localizeLines( ["skipTutorial_line1", "skipTutorial_line2"] )
                
                var stringsLength = strings.length;
                var lines = 0;

                if( stringsLength >= 1 )
                {
                    popupWidget.setFirstText( strings[ 0 ] );
                    lines++;
                }

                if( stringsLength >= 2 )
                {
                    popupWidget.setSecondText( strings[ 1 ] );
                    lines++;
                }

                if( stringsLength >= 3 )
                {
                    popupWidget.setThirdText( strings[ 2 ] );
                    lines++;
                }

                if( stringsLength >= 4 )
                {
                    popupWidget.setFourthText( strings[ 3 ] );
                    lines++;
                }

                popupWidget.play( 1, lines );
            }
        },

        showCompletedEndTutorialPopup: function()
        {
            var popupWidget = this._widgets[ GameWidgetPopup.prototype.getType() ];

            if( popupWidget )
            {
                var strings = LumosityGlobals.localizeLines( ["completeTutorial_line1", "completeTutorial_line2"] )
                
                var stringsLength = strings.length;
                var lines = 0;

                if( stringsLength >= 1 )
                {
                    popupWidget.setFirstText( strings[ 0 ] );
                    lines++;
                }

                if( stringsLength >= 2 )
                {
                    popupWidget.setSecondText( strings[ 1 ] );
                    lines++;
                }

                if( stringsLength >= 3 )
                {
                    popupWidget.setThirdText( strings[ 2 ] );
                    lines++;
                }

                if( stringsLength >= 4 )
                {
                    popupWidget.setFourthText( strings[ 3 ] );
                    lines++;
                }

                popupWidget.play( 1, lines );
            }
        },

        /**
         * Runs the play method on the feedback widget.
         * 
         * @param  {boolean} correct   Whether or not we should display "correct" feedback.
         * @param  {int} multiplier What number should be shown in "correct" feedback.
         * @param  {boolean} showPulse  Play the pulsing animation along with the scaled animation of feedback. 
         * @param  {boolean} [forceBonus=false] If true will force the bonus effect, regardless of other params.
         */
        showFeedback: function( correct, multiplier, showPulse, forceBonus )
        {
            if( LumosityUtils.isFitTestModeOn() === true && this.model.persistentData.isPracticeMode === true )
            {
                multiplier = 0;
                showPulse = false;
                forceBonus = false;
            }
            this.getFeedbackWidget().play( correct, multiplier, showPulse, forceBonus );
        },

        /**
         * Convenience method for getting the feedback widget.
         * 
         * @return {object} Sends back the feedback widget.
         */
        getFeedbackWidget: function()
        {
            return this._widgets[ GameWidgetFeedback.prototype.getType() ];
        },

        /**
         * Convenience method for getting the countdown widget.
         *
         * @returns {*} Sends back the countdown widget.
         */
        getCountdownWidget: function()
        {
            return this._widgets[ GameWidgetCountdown.prototype.getType() ];
        },
        
        /**
         * Convenience method for getting the hud widget.
         * 
         * @return {object} Sends back the hud widget.
         */
        getHudWidget: function()
        {
            return this._widgets[ GameWidgetHud.prototype.getType() ];
        },

        /**
         * Convenience method for getting the how to play widget.
         * 
         * @return {object} Sends back the how to play widget.
         */
        getHowToPlayWidget: function()
        {
            return this._widgets[ GameWidgetHowToPlay.prototype.getType() ];
        },

        /**
         * Slides between cards in the how to play view.
         *
         * @param {boolean} next True if user wants to see next card, false if previous.
         */
        slideHowToPlayCards: function( next )
        {
            var howToPlayWidget = this._widgets[ GameWidgetHowToPlay.prototype.getType() ];
            if ( howToPlayWidget )
            {
                if ( next )
                {
                    howToPlayWidget.nextPage();
                }
                else
                {
                    howToPlayWidget.previousPage();
                }
            }
        },

        /**
         * Shows debug info in the pause menu.
         */
        showDebugInfo:function()
        {
            this._pauseMenu.showDebugInfo();
        },

        onPause: function()
        {
            if ( this.animationController )
            {
                this.animationController.pause( true );
            }
        },

        onResume: function()
        {
            if ( this.animationController )
            {
                this.animationController.resume( true );
            }
        },


        pauseWidgets: function()
        {
            LumosityUtils.callFunctionOnCollection( this._widgets, "onPause" );
        },

        resumeWidgets: function()
        {
            LumosityUtils.callFunctionOnCollection( this._widgets, "onResume" );
        },

        restartWidgets: function()
        {
            if ( !this.model.restartToPracticeMode )
            {
                var countDownWidget = new GameWidgetCountdown( this.settings, this.interactionResponder, true );
                this._gameStage.addChild( countDownWidget, 100 );
                this.addWidget( countDownWidget );
            }
            LumosityUtils.callFunctionOnCollection( this._widgets, "onRestart" );
        },

        /**
         * Cue from flash file to show The Pause Button.
         */
        cue_showPauseButton: function()
        {
            var pauseButtonWidget = this._widgets[ GameWidgetPauseButton.prototype.getType() ];
            if ( pauseButtonWidget )
            {
                pauseButtonWidget.showPauseButton( true );
            }
        },

        /**
         * Cue from flash file to hide The Pause Button.
         */
        cue_hidePauseButton: function()
        {
            var pauseButtonWidget = this._widgets[ GameWidgetPauseButton.prototype.getType() ];
            if ( pauseButtonWidget )
            {
                pauseButtonWidget.showPauseButton( false );
            }
        },

        /**
         * Cue from flash file to show the skip tutorial button.
         */
        cue_showSkipTutorialButton: function()
        {
            var skipTutorialButtonWidget = this._widgets[ GameWidgetSkipTutorialButton.prototype.getType() ];
            if ( skipTutorialButtonWidget )
            {
                skipTutorialButtonWidget.showButton( true );
            }
        },

        /**
         * Cue from flash file to hide the skip tutorial button.
         */
        cue_hideSkipTutorialButton: function()
        {
            cc.log( "cue_hideSkipTutorialButton" );
            var skipTutorialButtonWidget = this._widgets[ GameWidgetSkipTutorialButton.prototype.getType() ];
            if ( skipTutorialButtonWidget )
            {
                skipTutorialButtonWidget.showButton( false );
            }
        },

        _moveStage: function( stage, newStagePosition, callback, instant )
        {
            if ( instant === undefined )
            {
                instant = false;
            }

            if ( !instant )
            {
                var sequence = null;
                var visibilityAction = null;
                var moveAction = cc.MoveTo.create( this.settings.stageSlideDuration, newStagePosition );

                if( newStagePosition === StagePositions.Center )
                {
                    stage.setVisible( true );
                    sequence = cc.Sequence.create( moveAction, callback );
                }
                else
                {
                    visibilityAction = cc.CallFunc.create( function(node) { node.setVisible( false ); }, stage );
                    sequence = cc.Sequence.create( moveAction, visibilityAction, callback );
                }

                stage.runAction( sequence );
            }
            else
            {
                stage.setPosition( newStagePosition );

                if( newStagePosition === StagePositions.Center )
                {
                    stage.setVisible( true );
                }
                else
                {
                    stage.setVisible( false );
                }

                stage.runAction( callback );
            }
        },

        _updateStageWidgets: function( displayState, instant )
        {
            if ( instant === undefined )
            {
                instant = false;
            }

            var howToPlayWidget = this._widgets[ GameWidgetHowToPlay.prototype.getType() ];
            var pauseButtonWidget = this._widgets[ GameWidgetPauseButton.prototype.getType() ];

            // If coming from how to plays, it should shrink the button instantly, otherwise it should animate
            var playShowButtonAnim = true;
            var playShrinkAnim = true;
            if ( instant === true )
            {
                playShrinkAnim = false;
                playShowButtonAnim = false;
            }
            else if ( this._prevDisplayState === DisplayState.HowToPlayMenu && displayState === DisplayState.Game )
            {
                playShrinkAnim = false;
            }

            pauseButtonWidget.setPaused( displayState !== DisplayState.Game, playShrinkAnim );
            pauseButtonWidget.showPauseButton( displayState === DisplayState.Game || displayState === DisplayState.PauseMenu, playShowButtonAnim );
            howToPlayWidget.slideButtons( displayState === DisplayState.HowToPlayMenu );
        },

        _updateStages: function( transitionToDisplayState, gameStagePosition, menuStagePosition, sideStagePosition, instant )
        {
            if( this.getDisplayState() === DisplayState.Transition ){ return; }

            this._updateDisplayStates( DisplayState.Transition );

            this._moveStage( this._gameStage, gameStagePosition, cc.CallFunc.create( this._stagesMoveFinished.bind( this, this._gameStage, transitionToDisplayState ), this ), instant );
            this._moveStage( this._menuStage, menuStagePosition, cc.CallFunc.create( this._stagesMoveFinished.bind( this, this._menuStage, transitionToDisplayState ), this ), instant );
            this._moveStage( this._sideStage, sideStagePosition, cc.CallFunc.create( this._stagesMoveFinished.bind( this, this._sideStage, transitionToDisplayState ), this ), instant );

            this._updateStageWidgets( transitionToDisplayState, instant );
        },

        _stagesMoveFinished: function( stage, transitionToDisplayState )
        {
            this._gameStageFinished = this._gameStageFinished || Boolean( stage === this._gameStage );
            this._menuStageFinished = this._menuStageFinished || Boolean( stage === this._menuStage );
            this._sideStageFinished = this._sideStageFinished || Boolean( stage === this._sideStage );

            if( this._gameStageFinished && this._menuStageFinished && this._sideStageFinished )
            {
                this._updateDisplayStates( transitionToDisplayState );
            }
        },

        _updateDisplayStates: function( displayState )
        {
            this._prevDisplayState = this._curDisplayState;
            this._curDisplayState = displayState;
        },

        centerOnGameStage: function( instant )
        {
            if ( this.getDisplayState() !== DisplayState.Game )
            {
                this._updateStages( DisplayState.Game, StagePositions.Center, StagePositions.RightCenter, StagePositions.RightWing, instant );
            }
        },

        centerOnGameMenu: function( instant )
        {
            this._updateStages( DisplayState.PauseMenu, StagePositions.LeftCenter, StagePositions.Center, StagePositions.RightCenter, instant );
        },

        centerOnSideStage: function( instant )
        {
            this._updateStages( DisplayState.HowToPlayMenu, StagePositions.LeftWing, StagePositions.LeftCenter, StagePositions.Center, instant );
        },

        getDisplayState: function()
        {
            return this._curDisplayState;
        },

        getPrevDisplayState: function()
        {
            return this._prevDisplayState;
        },

        goToNextHowToPlaySlide: function()
        {
            this._widgets[ GameWidgetHowToPlay.prototype.getType() ].nextPage();
        }
    });

}());
