/*global require*/
/*global cc*/
/*global audio*/
/*global localize*/
/*global GameComponentTimer*/
/*global GameWidgetHud*/
/*global GameWidgetHudElementType*/

require( "src/Game/Component/GameComponentTimer.js" );
require( "src/Game/Widget/GameWidgetHud.js" );

var GameComponentTimeBonus;
( function()
{
    "use strict";

    GameComponentTimeBonus = GameComponentTimer.extend(
    {
        _timeBonusRemaining: 0,
        _timeBonusDecreasePerSecond: 0,
        _hudComponentKey: "TimeBonus",

        /** @lends GameComponentTimeBonus.prototype */
        
        /**
         * A GameComponentTimeBonus is a subclass of GameComponentTimer that can be used to set up games with
         * limited time for a bonus. Provides functionality to be fired every frame or every second.
         * @constructs
         */
        ctor:function( controller )
        {
            this._super( controller );

            var model = this._controller.model;
            var settings = this._controller.settings;
            var view = this._controller.view;
            var hudWidget = view._widgets[ GameWidgetHud.prototype.getType() ];

            // Initialize the model and hud widget timer value
            this._timeBonusRemaining = settings.timeBonus;
            this._timeBonusDecreasePerSecond = settings.timeBonusDecreasePerSecond;
            
            model.timeBonusRemaining = this._timeBonusRemaining;

            this._createTimeBonusComponent();
            hudWidget.setCustomComponentValue( this._hudComponentKey, model.timeBonusRemaining );
        },

        /**
        *  Creates the time bonus component view.
        */
        _createTimeBonusComponent: function()
        {
            var labelString = localize ( "HUD_TimeBonusText", "BONUS" );
            var view = this._controller.view;
            var hudWidget = view._widgets[ GameWidgetHud.prototype.getType() ];

            hudWidget.addCustomComponent( this._hudComponentKey, GameWidgetHudElementType.HudElementLabelValue, labelString, "8888" );
            hudWidget.setComponentOrder( this._hudComponentKey, 1 );
        },

        getTimeBonusRemaining: function()
        {
            return this._timeBonusRemaining;
        },

        /**
        * Returns Component type as a string.
        *
        * @returns {string}
        */
        getType:function()
        {
            return "GameComponentTimeBonus";
        },

        /**
        *  Updates the visual timer.
        */
        _updateTimer: function()
        {
            var view = this._controller.view;
            var hudWidget = view._widgets[ GameWidgetHud.prototype.getType() ];
            var model = this._controller.model;

            this._timeBonusRemaining -= this._timeBonusDecreasePerSecond;
            model.timeBonusRemaining = this._timeBonusRemaining;

            hudWidget.setCustomComponentValue( this._hudComponentKey, model.timeBonusRemaining );

            if( model.timeBonusRemaining <= 0 )
            {
                this._hasTimerEnded = true;
            }
        }
    } );
}() );