/*global cc
global require
global lumosity
global createGame*/

/**
*  Main app entrypoint, for now. Creates a scene and a game and adds the game to the scene.
*/

require ( "src/launcher.js" );

var audio = cc.AudioEngine.getInstance();

var sounds = {
    'answerCorrect.mp3' : 'sounds/answerCorrect.mp3',
    'answerIncorrect.mp3' : 'sounds/answerWrong.mp3',
    'cardSet.mp3' : 'sounds/cardSet.mp3',
    'cardSlide.mp3' : 'sounds/cardSlide.mp3',
    'countdown.mp3' : 'sounds/countdown.mp3',
    'gameStart.mp3' : 'sounds/gameStart.mp3'
};

//TODO - move this to animations
function preloadSoundEffects() {
    var key;
    for( key in sounds )
    {
        audio.preloadEffect(sounds[key]);
    }
}

//TODO - move this to animations
function unloadSoundEffects()
{
    var key;
    for( key in sounds )
    {
        audio.unloadEffect(sounds[key]);
    }
}

/**
 *  This is the main Scene that starts everything up.
 *  
 *  @class
 *  @name MainScene
 */
var MainScene = cc.Scene.extend(
{
    /** @lends MainScene.prototype */
    
    /** Previous time value. Used to calculate a precise time. */
    _prevTime: 0,

    /** The game we are loading. */
    game:null,
    
    onEnter:function()
    {
        this._super();
        preloadSoundEffects();
        
        this.startGame( this._allowHowToPlay() );
        
        //schedule updates. I would do this within Game, but you must schedule it on a Cocos object.
        this.scheduleUpdate();
    },

    _allowHowToPlay: function()
    {
        var retVal = false;
        if( lumosity.game.params )
        {
            if( lumosity.game.params.fit_test_mode === "true" || lumosity.game.params.fit_test_mode === true )
            {
                retVal = false;
            }
            else
            {
                retVal = true;
            }
        }
        return retVal;
    },
                              
    onExit:function()
    {
        this._super();
        unloadSoundEffects();
    },
    
    update:function( delta )
    {
        //for html5 - JS drives everything, so js needs to set its own frametime etc
        if ( lumosity.game.jsDrivenUpdates )
        {
            lumosity.game.frametime += delta;
        }
        
        var currTime = 0;
        var timeDelta = 0;

        this.game.onUpdate( delta );

        // In place of cc.Scheduler. More precise time.
        // ( this.schedule( this.updateEachSecond, 1 ) );
        currTime = lumosity.game.frametime;
        if( ( currTime - this._prevTime ) >= 1.0 )
        {
            this.updateEachSecond();
            timeDelta = ( ( currTime - this._prevTime ) - 1.0 );
            this._prevTime = ( currTime - timeDelta );
        }
    },
    
    updateEachSecond:function()
    {
        this.game.onUpdateEachSecond();
    },
    
    startGame:function( allowHowToPlay )
    {
        if ( this.game && this.game.view )
        {
            this.game.view.removeFromParent();
        }
        
        this.game = createGame();
        
        var layer = this.game.view;
        this.addChild( layer );
        
        if( allowHowToPlay && ( lumosity.game.params.how_to_play === "true" || lumosity.game.params.how_to_play === true ) )
        {
            this.game.showHowToPlay( true );
        }
        else
        {
            this.game.onBegin();
        }
    }
});
