/*global cc*/
/*global require*/
/*global WebSocket*/
/*global global*/

//socket.io
//require( "src/Server/socket.io.js" );
/*global io*/

var ServerHandler;
( function()
{
    "use strict";
    
    ServerHandler =
    {
        /**
         * Use this to send this.socket.io messages back to the server. Make sure that you have called connectToServer() first.
         */
        socket: null,

        delegate: null,

        //the delegate just gets all the server response calls as well, why not
        setDelegate: function( setDelegate )
        {
            this.delegate = setDelegate;
        },

        /**
         * This must be called to connect to the server and begin communication through this.socket.io.
         */
        connectToServer: function( gameId, cb )
        {
            cc.log( "ServerHandler::connectToServer" );
        
            this.connectCb = cb;
    
            var ip = "10.11.0.11:13375";
    
            //websocket - MozWebSocket is for some browsers
            if ( global.MozWebSocket )
            {
                this.socket = new global.MozWebSocket( "ws://" + ip );
            }
            else
            {
                this.socket = new WebSocket( "ws://" + ip );
            }
        
            //socket.io
            //this.socket = io.connect( ip );

            if ( !this.socket )
            {
                if ( cb )
                {
                    cb( "Could not create a socket." );
                }
                return;
            }
        
            //websocket
            this.socket.onopen = function( gameId )
            {
                this.socketResponseFunctions.connect.call( this, gameId );
            }.bind( this, gameId );
        
            //websocket
            this.socket.onerror = function( err )
            {
                this.socketResponseFunctions.error.call( this, err );
            }.bind( this );
        
            //websocket
            this.socket.onmessage = function( message )
            {
                var messageData = null;
                
                try
                {
                    messageData = JSON.parse( message.data );
                }
                catch ( err )
                {
                    cc.log( "WARN: Received mangled JSON from the server: " + message );
                }
                
                if ( messageData && messageData.t )
                {
                    if ( this.socketResponseFunctions[ messageData.t ] )
                    {
                        this.socketResponseFunctions[ messageData.t ].call( this, messageData.d );
                    }
                
                    if ( this.delegate && this.delegate[ "server_" + messageData.t ] )
                    {
                        this.delegate[ "server_" + messageData.t ].call( this.delegate, messageData.d );
                    }
                }
                else if ( !messageData )
                {
                    cc.log( "WARN: In a message from the server, failed to parse the message data." );
                }
                else
                {
                    cc.log( "WARN: In a message from the server, there is not a 't' element specifying the function to call." );
                }
            }.bind( this );
        
            //socket.io
            // //automatically register response functions for _respondTo_ functions we have defined in this object.
            // var key;
            // for ( key in this.socketResponseFunctions )
            // {
            //     this.socket.on( key, this.socketResponseFunctions[ key ].bind( this ) );
            // }
        },

        /**
         * Sends a message to the server with this.socket.io. This is convenient to use and is the recommended way
         * to get messages to the server. The server can easily respond back if desired. If you want to add a
         * new type of socket message in, then in lumos_modules edit serverSocketHandler.js.
         *
         * @param {string} type The type of message to send. This will directly match the function in serverSocketHandler.js which will respond to it.
         * @param {*} [data] Any data to send with the message. Can be any type.
         */
        sendMessage: function( type, data )
        {
            //websocket
            var sentData = { t: type };
            if ( data )
            {
                sentData.d = data;
            }
        
            var sendStr = JSON.stringify( sentData );
            cc.log( "About to send " + sendStr );
        
            this.socket.send( sendStr );
        
            //socket.io
            //this.socket.emit( type, data );
        },
    
        /**
         * These functions will be called whenever the server emits a message matching the function name in serverSocketHandler.js.
         * They can receive any objects the server might have sent.
         */
        socketResponseFunctions:
        {
            connect: function( gameId )
            {
                cc.log( "ServerHandler::socketResponseFunctions connect" );
            
                //now that we're connected, tell the server that we want to join our game
                this.sendMessage( "joinGame", gameId );
            
                if ( this.connectCb )
                {
                    this.connectCb();
                    this.connectCb = null;
                }
            },
    
            error: function( data )
            {
                cc.log( "ServerHandler::socketResponseFunctions error" + JSON.stringify( data ) );
            
                if ( this.connectCb )
                {
                    this.connectCb( data );
                    this.connectCb = null;
                }
            },
    
            message: function( data )
            {
                cc.log( "ServerHandler::socketResponseFunctions message: \"" + data + "\"" );
            }
        }
    };
}() );