/*global cc */
/*global require*/
/*global lumosity*/
/*global Random*/

require( "src/Utility/Random.js" );

/**
 *  Functionality for setting up an automated test.
 *  @name AutomatedTester
 *  @namespace
 */
var AutomatedTester =
{
	script: null,
	randomSeed: 1,
    filePath: "",
	isRecording: false,
	isPlayingBack: false,

	initialize: function()
	{
		"use strict";

		// Read automated test data into memory.
		if( this.read() )
		{
			// Assign games random seed.
			Random.seed = Random.initialSeed = this.randomSeed;
		}
	},

 	/**
     *  Reads the automated test data from file.
     *  Tests are stored within tests/ folder inside each game.
     */
    read: function()
    {
    	"use strict";

        var folderPath = "tests/" + lumosity.game.locale + "/";
        var fileData = null;
        var jsonData = null;

        if( !lumosity.game.params || !lumosity.game.params.interaction_script )
        {
            return false;
        }

        this.filePath = folderPath + lumosity.game.params.interaction_script;
        fileData = lumosity.io_read( this.filePath );
        if( !fileData )
        {
            cc.log( "AutomatedTester::read Can't load file data for file: " + this.filePath );
            return false;
        }

        jsonData = JSON.parse( fileData );
        this.script = jsonData.script;
        this.randomSeed = jsonData.seed;

        cc.log( "AutomatedTester::read Was able to read script: " + this.filePath + " fileData: " +  JSON.stringify( fileData ) );

        return true;
    },

    /**
     *  For automated support, you can hook into this to begin automation.
     *  Boiler currently provides a record function to gameplayBegan to start
     *  recording an automated test.
     */
    record: function()
    {
    	"use strict";

        if( !lumosity.game.gameplayBegan )
        {
            return false;
        }

        if( lumosity.game.params.interaction_script )
        {
            return false;
        }

        if( this.script )
        {
        	return false;
        }

        this.isRecording = true;
        lumosity.game.gameplayBegan();
        return true;
    },

    /**
     * Plays back automated script that was read.
     */
    playback: function()
    {
    	"use strict";

        if( !lumosity.game.playInteractionScript )
        {
            return false;
        }

        if( !this.script )
        {
        	return false;
        }

        this.isPlayingBack = true;
        lumosity.game.playInteractionScript( this.script );
        return true;
    },

    isRunning: function() 
    {
    	"use strict";
    	
    	return this.isRecording || this.isPlayingBack;
    }
};

var at = AutomatedTester;