/*global require*/
/*global cc*/
/*global localize*/
/*global LumosityButtonState*/
/*global LumosityButton*/
/*global LayoutLoader*/
/*global LumosityUtils*/
/*global lg*/

require( "src/View/LumosityButton.js" );
require( "src/Animation/LayoutLoader.js" );
require( "src/Utility/LumosityUtils.js" );

var PauseMenu;
( function()
{
    "use strict";

    PauseMenu = cc.Node.extend(
    {
        _target: null,

        _buttonContainer: null,

        _resumeTextData: null,

        _restartTextData: null,

        _quitTextData: null,

        _howToPlayTextData: null,

        _buttonPosition: cc.PointZero(),

        _buttonLabelPosition: cc.PointZero(),

        _buttonIconPosition: cc.PointZero(),

        _buttonDesignSize: cc.SizeZero(),

        _buttonIconImageDesignSize: cc.SizeZero(),

        _buttonPaddingY: 0,

        buttons: null,

        /** @lends PauseMenu.prototype */
        
        /**
         *  PauseMenu is the main class for the pause menu view!
         *
         *  @constructs
         */
        ctor: function( target )
        {
            this._super();

            this._target = target;

            this._buttonContainer = cc.Node.create();
            this._buttonContainer.setAnchorPoint( cc.p( 0.5, 0.5 ) );

            var screenSize = cc.Director.getInstance().getWinSize();
            this.setContentSize( screenSize );
            this.setAnchorPoint( cc.p( 0.5, 0.5 ) );
            this.setPosition( cc.p( screenSize.width >> 1, screenSize.height >> 1 ) );
            this.nodeName = "PauseMenu";

            this.buttons = [];

            this._loadLayout();
            this._createViews();
            
            if ( this.buildNumberText )
            {
                this.buildNumberText.setVisible( false );
            }            
        },

        /**
         * Shows the debug info. Currently this is just the build number.
         */
        showDebugInfo:function()
        {
            if ( this.buildNumberText )
            {
                this.buildNumberText.setString( "build: #" + lg.buildNumber + "  asset: #" + lg.assetNumber);
                this.buildNumberText.setVisible( true );
                this.buildNumberText.runAction( cc.FadeIn.create( 0.3 ) );
                
                if( LumosityUtils.isDebugModeOn() )
                {
                    this.setMenuButtonEnabled( "CrashGameButton", true );
                }
            }
        },

        _loadLayout: function()
        {
            LayoutLoader.loadLayoutFile( "pauseMenu_layout.json", this, this , this );
        },

        _createViews: function()
        {
            var resumeButton = null;
            var restartButton = null;
            var quitButton = null;
            var howToPlayButton = null;
            var crashButton = null;
            var buttonPos = cc.PointZero();
            var buttonIconPos = cc.PointZero();
            var buttonLabelPos = cc.PointZero();
            var menuContainerContentSize = cc.SizeZero();

            // Get layout created nodes data and them destroy the nodes.
            // Must do this because nodes information isn't converted from design
            // space until after the node is created. 
            this._buttonPosition = this.menuButton.getPosition();
            this._buttonDesignSize = this.menuButton.getContentSize();
            
            this._buttonIconPosition = this.menuButtonIcon.getPosition();
            this._buttonIconImageDesignSize = this.menuButtonIcon.getContentSize();

            this._buttonLabelPosition = this.menuButtonText.getPosition();

            this._buttonPaddingY = this.menuButtonPadding.getContentSize().height;

            // Convert button icon position.
            buttonIconPos = this._buttonIconPosition;
            buttonIconPos = this.menuContainer.convertToWorldSpace( buttonIconPos );
            buttonIconPos = this.menuButton.convertToNodeSpace( buttonIconPos );

            // Convert button label position.
            buttonLabelPos = this._buttonLabelPosition;
            buttonLabelPos = this.menuContainer.convertToWorldSpace( buttonLabelPos );
            buttonLabelPos = this.menuButton.convertToNodeSpace( buttonLabelPos );

            // Remove all the flash generated nodes
            this.menuButton.removeFromParent();
            this.menuButtonIcon.removeFromParent();
            this.menuButtonText.removeFromParent();
            this.menuButtonPadding.removeFromParent();

            // Align buttonCointainer in the center of menuContainer.
            // buttonContainer will house all the menu buttons.
            menuContainerContentSize = this.menuContainer.getContentSize();
            this._buttonContainer.setPosition( menuContainerContentSize.width / 2, menuContainerContentSize.height / 2 );
            this.menuContainer.addChild( this._buttonContainer );

            if( LumosityUtils.isDebugModeOn() )
            {
                // Crash Button ( Only set visible in debug mode with three swipes )
                crashButton = this.createMenuButton( "menuIcons/menuButtonIconsHowToPlayUp.png",
                                                     "menuIcons/menuButtonIconsHowToPlayDown.png",
                                                     this._crashButtonTextData,
                                                     buttonPos,
                                                     buttonIconPos,
                                                     buttonLabelPos,
                                                     this._target,
                                                     function() { throw new Error("Debug Crash! Game was intentionally crashed!"); } );
                crashButton.nodeName = "CrashGameButton";
                this.addMenuButton( crashButton );
                this.setMenuButtonEnabled( "CrashGameButton", false );
            }

            // How To Play Button
            howToPlayButton = this.createMenuButton( "menuIcons/menuButtonIconsHowToPlayUp.png",
                                                      "menuIcons/menuButtonIconsHowToPlayDown.png",
                                                      this._howToPlayTextData,
                                                      buttonPos,
                                                      buttonIconPos,
                                                      buttonLabelPos,
                                                      this._target,
                                                      this._target.gamePauseMenu_howToPlay );
            howToPlayButton.nodeName = "HowToPlayButton";
            this.addMenuButton( howToPlayButton );

            // Quit Button
            quitButton = this.createMenuButton( "menuIcons/menuButtonIconsQuitUp.png",
                                                     "menuIcons/menuButtonIconsQuitDown.png",
                                                     this._quitTextData,
                                                     buttonPos,
                                                     buttonIconPos,
                                                     buttonLabelPos,
                                                     this._target,
                                                     this._target.gamePauseMenu_quitGame );
            quitButton.nodeName = "QuitButton";
            this.addMenuButton( quitButton );

            // Restart Button
            restartButton = this.createMenuButton( "menuIcons/menuButtonIconsRestartUp.png",
                                                    "menuIcons/menuButtonIconsRestartDown.png",
                                                    this._restartTextData,
                                                    buttonPos,
                                                    buttonIconPos,
                                                    buttonLabelPos,
                                                    this._target,
                                                    this._target.gamePauseMenu_restartGame );
            restartButton.nodeName = "RestartButton";
            this.addMenuButton( restartButton );

            // Resume Button
            resumeButton = this.createMenuButton( "menuIcons/menuButtonIconsResumeUp.png",
                                                   "menuIcons/menuButtonIconsResumeDown.png",
                                                   this._resumeTextData,
                                                   buttonPos,
                                                   buttonIconPos,
                                                   buttonLabelPos,
                                                   this._target,
                                                   this._target.gamePauseMenu_closeMenu );
            resumeButton.nodeName = "ResumeButton";
            this.addMenuButton( resumeButton );
        },

        layout_creatingNode: function( nodeName, nodeData )
        {
            var result = null;

            if( nodeName === "resumeText" )
            {
                this._resumeTextData = nodeData[ LayoutLoader.textDataKey ];
                result = false;
            }
            else if( nodeName === "restartText" )
            {
                this._restartTextData = nodeData[ LayoutLoader.textDataKey ];
                result = false;
            }
            else if( nodeName === "quitText" )
            {
                this._quitTextData = nodeData[ LayoutLoader.textDataKey ];
                result = false;
            }
            else if( nodeName === "howToPlayText" )
            {
                this._howToPlayTextData = nodeData[ LayoutLoader.textDataKey ];

                //nabbing the information from the how to play .json piece for the crash button
                this._crashButtonTextData = LumosityUtils.duplicateObject( nodeData[ LayoutLoader.textDataKey ], true );
                this._crashButtonTextData[ LayoutLoader.textStringKey ] = "Don't Touch Me!!!";
                this._crashButtonTextData[ LayoutLoader.textKeyKey ] = this._crashButtonTextData[ LayoutLoader.textStringKey ];

                result = false;
            }

            return result;
        },

        _createButton: function( bgUpImage, bgDownImage, iconUpImage, iconDownImage, text, fontUpColor, fontDownColor )
        {   
            var button = null;

            button = new LumosityButton();
            button.setTitleForState( text, LumosityButtonState.ButtonStateNormal );
            button.setTitleColorForState( fontUpColor, LumosityButtonState.ButtonStateNormal );
            button.setTitleColorForState( fontDownColor, LumosityButtonState.ButtonStateSelected );
            button.setBackgroundImageForState( bgUpImage, LumosityButtonState.ButtonStateNormal );
            button.setBackgroundImageForState( bgDownImage, LumosityButtonState.ButtonStateSelected );
            button.setImageForState( iconUpImage, LumosityButtonState.ButtonStateNormal );
            button.setImageForState( iconDownImage, LumosityButtonState.ButtonStateSelected );            

            return button;
        },

        createMenuButton: function( iconUpImageName, iconDownImageName, textData, position, iconPosition, labelPosition, target, callback )
        {
            var button = null;
            var bgUpImageName = "buttons/menuButtonBackgroundUp.png";
            var bgDownImageName = "buttons/menuButtonBackgroundDown.png";
            var text = "";
            var textKey = "";
            var localizedText = "";
            var font = "";
            var fontSize = "";
            var fontUpColor = cc.c3b( 255, 255, 255 );
            var fontDownColor = cc.c3b( 255, 255, 255 );
            var colorData = null;
            var bgUpImage = null;
            var bgDownImage = null;
            var iconUpImage = null;
            var iconDownImage = null;

            bgUpImage = cc.Sprite.create( bgUpImageName );
            bgDownImage = cc.Sprite.create( bgDownImageName );
            iconUpImage = cc.Sprite.create( iconUpImageName );
            iconDownImage = cc.Sprite.create( iconDownImageName );
            text = textData[ LayoutLoader.textStringKey ];
            textKey = textData[ LayoutLoader.textKeyKey ];
            localizedText = localize( textKey, text );
            font = LumosityUtils.getFontLocation( textData[ LayoutLoader.fontKey ] );
            fontSize = textData[ LayoutLoader.fontSizeKey ];
            colorData = textData[ LayoutLoader.fontColorKey ];
            fontUpColor = LayoutLoader.getColor( colorData );

            button = this._createButton( bgUpImage,
                                         bgDownImage,
                                         iconUpImage,
                                         iconDownImage,
                                         localizedText,
                                         fontUpColor,
                                         fontDownColor );

            button.setContentSize( this._buttonDesignSize, false );
            button.setAnchorPoint( cc.PointZero() );
            button.setPosition( position );
            button.setTarget( target );
            button.setReleasedCallback( callback );

            // Setup label.
            button.titleLabel.setFontName( font );
            button.titleLabel.setInitialFontSize( fontSize );
            button.titleLabel.setAnchorPoint( cc.PointZero() );
            button.titleLabel.setPosition( labelPosition );
            button.titleLabel.setHorizontalAlignment( cc.TEXT_ALIGNMENT_LEFT );

            // Setup icon.
            LumosityUtils.scaleForContentSize( iconUpImage, this._buttonIconImageDesignSize );
            iconUpImage.setPosition( iconPosition );
            LumosityUtils.scaleForContentSize( iconDownImage, this._buttonIconImageDesignSize );
            iconDownImage.setPosition( iconPosition );

            return button;
        },

        /**
         * Adds a button.
         * 
         * @param {LumosityButton} menuButton The button to be added.
         */
        addMenuButton: function( menuButton )
        {
            this._buttonContainer.addChild( menuButton );
            this.buttons.push( menuButton );

            this._updateView();
        },

        /**
         * Returns a button by nodeName.
         * 
         * @param  {string} nodeName The nodeName of the button.
         * @return {LumosityButton}  The button.
         */
        getMenuButton: function( nodeName )
        {
            var i;
            var button = null;

            for( i = 0; i < this.buttons.length; i++ )
            {
                button = this.buttons[ i ];
                if( button.nodeName === nodeName )
                {
                    break;
                }
            }

            return button;
        },

        /**
         * Sets the enabled/disabled of the button.
         * 
         * @param {string} nodeName   The nodeName of the button.
         * @param {Boolean} isEnabled Whether to enable or disable the button.
         */
        setMenuButtonEnabled: function( nodeName, isEnabled )
        {
            var i;
            var button = null;

            for( i = 0; i < this.buttons.length; i++ )
            {
                button = this.buttons[ i ];
                if( button.nodeName === nodeName )
                {
                    button.setVisible( isEnabled );
                    button.setEnabled( isEnabled );
                    break;
                }
            }

            this._updateView();
        },

        setFitTestModeEnabled: function( isEnabled )
        {
            this.setMenuButtonEnabled( "HowToPlayButton", !isEnabled );
        },

        setTutorialModeEnabled: function( isEnabled )
        {
            this.setMenuButtonEnabled( "HowToPlayButton", !isEnabled );
        },

        _updateView: function()
        {
            var i;
            var button = null;
            var buttonPos = cc.PointZero();
            var contentSize = cc.SizeZero();

            // Calculate the content size of the buttonContainer.
            for( i = 0; i < this.buttons.length; i++ )
            {
                button = this.buttons[ i ];

                // Continue if the button is disabled.
                // This is so it's content size isn't included in the next buttons position.
                if( !button.getIsEnabled() )
                {
                    continue;
                }

                buttonPos = cc.p( 0, contentSize.height );
                contentSize = cc.size( button.getContentSize().width, contentSize.height + button.getContentSize().height );

                // Don't add button padding if it is the last button in the list.
                if( i !== ( this.buttons.length - 1 ) )
                {
                    contentSize = cc.size( button.getContentSize().width, contentSize.height + this._buttonPaddingY );
                }

                button.setPosition( buttonPos );
            }

            this._buttonContainer.setContentSize( contentSize );
        }
    } );
}() );
