/*global cc*/
/*global require*/
/*global LumosityUtils*/
/*global lg*/
/*global InteractionType*/
/*global InteractionSource*/
/*global CardinalDirections*/
/*global GameComponentScoreMultiplier*/
/*global GameComponentTimer*/
/*global metadataAggregateKeys*/
/*global metadataTrialKeys*/
/*global lumosity*/
/*global audio*/

require( "src/Game/Base/Game.js" );
/*global Game*/

require( "src/Game/SpeedMatchSettings.js" );
/*global SpeedMatchSettings*/

require( "src/Game/SpeedMatchModel.js" );
/*global SpeedMatchModel*/

require( "src/Game/SpeedMatchView.js" );
/*global SpeedMatchView*/

var SpeedMatch;
(function()
{
    "use strict";

    SpeedMatch = Game.extend(
    {
        ctor: function()
        {
            this._super();
        },
        
        _createSettings: function()
        {
            return new SpeedMatchSettings();
        },

        _createModel: function()
        {
            return new SpeedMatchModel( this.settings );
        },

        _createView: function( screenSize )
        {
            return new SpeedMatchView( screenSize, this.settings, this.model, this );
        },
        
        onBegin: function()
        {
            this._super();
            this.feedback = this.view.getFeedbackWidget();
            this.allowButtonInput = false;
            this.view.enableButtons( false );
            this.model.chooseNextSymbol();
            
            this.view.updateSymbols();
            this.view.updateText( true );

            this.view.animCardIn( this.view.outCard );

            if ( this.model.persistentData.isPracticeMode && !LumosityUtils.isFitTestModeOn() )
            {
                // this.view._widgets[ GameWidgetHud.prototype.getType() ]._hideAll();

                delete this._components[ ( new GameComponentTimer( this ) ).getType() ];
            }
            else
            {
                if ( !this.model.persistentData.isPracticeMode )
                {
                    this._components = {};
                    this._createComponents();
                    this.view._widgets[ GameWidgetHud.prototype.getType() ].showScore();
                    this.view._widgets[ GameWidgetHud.prototype.getType() ].showTime();
                }
            }

            this.scoring = this.getScoringComponent();
        },
        
        onStartGameplay: function()
        {
            if ( !this.model.tutorialSkipped ) {
                this._super();
                this.view.enableButtons( true );
                this.model.markTrialStartTime();
                
                this._showNewStimulus();
            }
        },
    
        playStartingAnimation: function()
        {
            if ( this.model.restartToPracticeMode || ( this.model.persistentData.isPracticeMode && LumosityUtils.isFitTestModeOn() ) )
            {
                this.view.playAnimation( "practiceBegin", this.onStartGameplay );
            }
            else if ( this.model.persistentData.isPracticeMode && !this.model.isFitTestMode ) 
            {
                this.view.animationController.play( "startTutorial", this.onStartGameplay, this );
            }
            //otherwise, play the game begin animation
            else
            {
                this.view.playAnimation( "gameBegin", this.onStartGameplay );
            }
        },

        playEndAnimation: function ()
        {
            if ( !this.model.persistentData.isPracticeMode )
            {
                var results = this.exportResults();
                var automatedTestReport = this.exportAutomatedTestReport();
                if( automatedTestReport )
                {
                    // Append automated test report if we have one.
                    results.automatedTestReport = automatedTestReport;
                }

                lumosity.game.report( JSON.stringify( results ) );
                this.view.playAnimation( "gameEnd", this.onFinish.bind( this, false ) );
            }
            else if ( LumosityUtils.isFitTestModeOn() )
            {
                this.view.playAnimation( "gameEnd", this.onStartFitTest );
            }
        },

        onRestart: function ()
        {
            this.view.inCard.stopAllActions();
            this.view.outCard.stopAllActions();
            this.view.inCard.setPosition( this.view.inCardStartPosition );
            this.view.outCard.setPosition( this.view.inCardStartPosition );
            this._super();
        },

        /** Called when a game should begin its tutorial completion sequence. **/
        onCompleteTutorial: function( skipped )
        {
            cc.log( "Game::onCompleteTutorial" );

            this.model.tutorialSkipped = skipped;

            this.view.inCard.stopAllActions();
            this.view.outCard.stopAllActions();
            this.view.animCardOut( this.view.outCard );
            this.view.animCardOut( this.view.inCard );
            this.getMetadataComponent()._trialCount = this.model.totalResponseCount;

            this.view.playAnimation( "endTutorial" );
        },

        skipTutorialButton_Tapped: function()
        {
            if ( !this.model.tutorialSkipped )
            {
                audio.playEffect( "sounds/cardSlide.mp3" );
                this.view.enableButtons( false );
                this.view.stopAnimation( "problem" );

                this.tutorialMetadata.finishes += 1;
                // this.metadataComponentTemp = this.getMetadataComponent();
                // this.metadataComponentTemp.tutorialMetadata = this.tutorialMetadata;
                this.getMetadataComponent().tutorialMetadata = this.tutorialMetadata; 
                LumosityUtils.callFunctionOnCollection( this._components, "onEndGameplay" );

                this.onCompleteTutorial( true );            
            }
        },

        onEndGameplay:function()
        {
            //we need to make sure button mashing etc won't carry on after the timer runs out
            this.view.enableButtons( false );
            this.view.stopAnimation( "problem" );

            this.view.inCard.stopAllActions();
            this.view.outCard.stopAllActions();
            this.view.animCardOut( this.view.outCard );
            this.view.animCardOut( this.view.inCard );
            this.getMetadataComponent()._trialCount = this.model.totalResponseCount;

            // Overriden from super class
            this.model.isGameRunning = false;
            this.model.isGameOver = true;

            this.finalizeMetadata();

            if ( this.model.persistentData.isPracticeMode )
            {
                this.tutorialMetadata.finishes += 1;

                if ( !LumosityUtils.isFitTestModeOn() )
                {
                    this.onCompleteTutorial( false );
                }
            }

            if ( LumosityUtils.isFitTestModeOn() || ( !LumosityUtils.isFitTestModeOn() && this.model.persistentData.isPracticeMode ) )
            {
                this.getMetadataComponent().tutorialMetadata = this.tutorialMetadata; 
            }

            LumosityUtils.callFunctionOnCollection( this._components, "onEndGameplay" );

            this.playEndAnimation();

            //finish the interaction script in case there is one (if you want it absolutely tied to the session)
            //this is harmless to call if nothing is being recorded.
            if( this._interactionAutomator )
            {
                this._interactionAutomator.finishRecordingScript();
            }

            //for automation support, you can hook into this to end automation
            if ( lumosity.game.gameplayEnded )
            {
                lumosity.game.gameplayEnded();
            }
            cc.log("METADATA END", this.getMetadataComponent().tutorialMetadata)
        },
        
        handleResponse: function( correct, userAnsweredYes )
        {


            if ( !this.allowButtonInput || !this.model.isGameRunning )
            {
                return;
            }
            
            if ( LumosityUtils.isDebugModeOn() )
            {
                correct = true;
            }
            
            //this must be called before the multiplier etc are increased
            this._recordTrialMetadata( correct, userAnsweredYes );
            
            this.model.markResponse( correct );
            
            if ( correct )
            {
                this.scoring.increaseMultiplierProgress();
                
                //update the text in practice mode, but don't do it on the last correct since the text will fade out
                if ( this.model.persistentData.isPracticeMode && !this.model.hasFinishedPractice() )
                {
                    this.view.updateText( false, false, userAnsweredYes );
                }
            }
            else
            {
                this.scoring.decreaseMultiplierProgress();
                
                //if in practice mode, we want to show feedback about what they did wrong
                if ( this.model.persistentData.isPracticeMode )
                {
                    this.view.updateText( false, true, !userAnsweredYes );
                    this.model.maintainString = true;
                    
                    //don't advance to the new card until they get it right
                    this.feedback.play( correct);
                    return;
                }
            }
            this.feedback.play( correct );
            
            this.allowButtonInput = false;
            
            //don't show anything else if we have finished gameplay in practice mode
            if ( this.model.hasFinishedPractice() )
            {
                return;
            }
            
            this.model.chooseNextSymbol();
            this.view.updateSymbols();
            this.model.markTrialStartTime();
            
            //we can't use stopAnimation because that stops all animations on all the cards, so we do this instead
            //this.view.card1.stopAllActions();
            //this.view.card2.stopAllActions();

            this._showNewStimulus();
        },
        
        cue_allowInput: function()
        {
            if ( this.model.hasFinishedPractice() )
            {
                return;
            }
            
            this.model.hasAllowedInput = true;
            
            //bleh kinda hacky
            if ( !this.model.maintainString )
            {
                this.view.updateText( false );
            }
            this.model.maintainString = false;
            
            this.allowButtonInput = true;
            this.model.trialStartTime = lumosity.game.frametime;
            this.beginTrial();
        },
        
        _showNewStimulus: function()
        {
            this.view.playAnimation( "problem" );
            this.view.inCard.stopAllActions();
            this.view.outCard.stopAllActions();
            this.view.outCard.setPosition(this.view.activeCardPosition);
            this.view.animCardOut( this.view.outCard );
            this.view.inCard.setPosition(this.view.inCardStartPosition);
            this.view.animCardIn( this.view.inCard );

        },
        
        _recordTrialMetadata: function( correct, userAnsweredYes )
        {
            //trial metadata
            var trial =
            {
                stimulus : this.settings.stimulusMetadataIndices[ this.model.currentSymbolId ],
                response : userAnsweredYes ? "T" : "F",
                correct : correct ? "T" : "F",
                multiplier : this._components[ GameComponentScoreMultiplier.prototype.getType() ]._multiplier,
                progress : this._components[ GameComponentScoreMultiplier.prototype.getType() ].getProgressMetadata(),
                back : this.settings.stimulusMetadataIndices[ this.model.lastSymbolId ],
                match : this.model.doSymbolsMatch() ? "T" : "F" 
            };
            
            // NAF: Removing for now because simulating a button press rather than going through the interaction reciever doesn't
            //      allow us to pass through the sourceInteraction to determine whether it was a button or keyboard input.
            // var inputType = "T";
            // if ( sourceInteraction && sourceInteraction.getSource() === InteractionSource.Keyboard )
            // {
            //     if ( sourceInteraction.isKeyboardButton( cc.KEY.w, true ) || sourceInteraction.isKeyboardButton( cc.KEY.a, true ) ||
            //          sourceInteraction.isKeyboardButton( cc.KEY.s, true ) || sourceInteraction.isKeyboardButton( cc.KEY.d, true ) )
            //     {
            //         inputType = "L";
            //     }
            //     else
            //     {
            //         inputType = "R";
            //     }
            // }
            // trial.input_type = inputType;
            
            this.endTrial( trial, correct, false, false, false, false, true );
        },
        
        feedback_isFinished: function()
        {
            //end the gameplay if we finished practice
            if ( this.model.hasFinishedPractice() )
            {
                this.onEndGameplay();
            }
        },
        
        interaction_alias:function( alias, data )
        {
            this._super( alias, data );
            
            if ( alias === "correct" )
            {
                this.handleResponse( true, this.model.doSymbolsMatch(), null );
            }
            else if ( alias === "incorrect" )
            {
                this.handleResponse( false, !this.model.doSymbolsMatch(), null );
            }
        }
    } );
}() );
