/*global cc*/
/*global require*/
/*global lumosity*/
/*global Random*/

require( "src/Game/Base/GameModel.js" );
/*global GameModel*/

require( "src/Utility/LumosityUtils.js" );
/*global lu*/

var SpeedMatchModel;
(function()
{
    "use strict";

    SpeedMatchModel = GameModel.extend(
    {
        ctor: function( settings )
        {
            this._super();
            
            this.totalResponseCount = 0;
            this.totalCorrectResponseCount = 0;
            this.consecutiveResponsesCorrect = 0;
            this.trialStartTime = -1;
            this.reactionTimes = [];
            this.currentTrialSet = settings.trials[ settings.trialIds.getRandomElement() ];
            this.scriptedTrials = settings.tutorialScriptedTrials;
            this.currentSymbolId = this.currentTrialSet.symbolIds.getRandomElement();
            this.lastSymbolId = this.currentSymbolId;
            this.lastTrialWasMatch = false;
            this.trialStartTime = 0; //this may be redundant but whatever
            this.practiceProgress = 0;
            this.hasAllowedInput = false;
            
            //just put these in a separate thing to keep them a little more organized
            this.responseTimesMatch = [];
            this.responseTimesNoMatch = [];
            // this.metadata =
            // {
            //     num_match: 0,
            //     response_time_match: 0,
            //     response_time_no_match: 0
            // };
        },
        
        exportResults: function()
        {
            var stats =
            {
                cards: this.totalCorrectResponseCount,
                total: this.totalResponseCount,
                correct: this.totalCorrectResponseCount
            };
            
            return this._super( stats, "correct" );
        },

        doSymbolsMatch: function()
        {
            return this.lastSymbolId === this.currentSymbolId;
        },

        setSymbolId: function( symbolId )
        {
            this.lastTrialWasMatch = ( symbolId === this.currentSymbolId );
            this.lastSymbolId = this.currentSymbolId;
            this.currentSymbolId = symbolId;
        },

        chooseNextSymbol: function()
        {
            var trialIds = this.currentTrialSet.getSymbolIds();
            
            if ( this.currentSymbolId === "" )
            {
                this.setSymbolId( trialIds.getRandomElement() );
            }
            else
            {
                var showMatch = Math.random() < 0.5;
                
                //in practice mode, show a scripted trial (matching / not matching in a fixed order)
                if ( this.persistentData.isPracticeMode && !this.hasFinishedPractice() )
                {
                    showMatch = !!this.scriptedTrials[ this.practiceProgress ];
                }
                
                if ( showMatch )
                {
                    this.setSymbolId( this.currentSymbolId );
                }
                else
                {
                    var currentIndex = trialIds.indexOf( this.currentSymbolId );
    
                    if ( currentIndex === -1 )
                    {
                        cc.log("Could not find current symbol id in SpeedMatch::getNextTrialSymbol(), returning");
                        return "";
                    }
    
                    var nextTrialIndex = lu.getRandomInt( 0, trialIds.length - 1, false, true );
    
                    if ( nextTrialIndex >= currentIndex )
                    {
                        nextTrialIndex += 1;
                    }
    
                    this.setSymbolId( trialIds[ nextTrialIndex ] );
                }
            }
        },

        markTrialStartTime: function()
        {
            this.trialStartTime = lumosity.game.frametime;
        },

        markTrialFinishTime: function()
        {
            this.reactionTimes.push( lumosity.game.frametime - this.trialStartTime );
        },

        markResponse: function( correct )
        {
            this.totalResponseCount += 1;
            if( this.lastTrialWasMatch ) 
            {
                this.metadata.num_match++;
            } 
            if ( correct )
            {
                this.totalCorrectResponseCount += 1;
                this.consecutiveResponsesCorrect += 1;
                
                if ( this.lastTrialWasMatch )
                {
                    this.responseTimesMatch.push( lumosity.game.frametime - this.trialStartTime );
                    this.metadata.response_time_match = Math.round( lu.convertSecondsToMilliseconds( this.findAverageReactionTimeMatch() ) );
                }
                else
                {
                    this.responseTimesNoMatch.push( lumosity.game.frametime - this.trialStartTime );
                    this.metadata.response_time_no_match = Math.round( lu.convertSecondsToMilliseconds( this.findAverageReactionTimeNoMatch() ) );
                }
            }
            else
            {
                this.consecutiveResponsesCorrect = 0;
            }
            
            //in practice mode, we track the progress of corrects, and incorrects reset this to 0
            if ( this.persistentData.isPracticeMode && !this.hasFinishedPractice() )
            {
                if ( correct )
                {
                    this.practiceProgress++;
                }
                //nevermind, turns out they don't want this
                // else
                // {
                //     this.practiceProgress = 0;
                // }
            }
        },
        
        hasFinishedPractice:function()
        {
            return this.persistentData.isPracticeMode && ( !this.scriptedTrials || this.practiceProgress >= this.scriptedTrials.length );
        },

        //this isn't even used!
        // accuracyPercent: function()
        // {
        //     if ( this.totalResponses > 0 )
        //     {
        //         return Math.floor( 100 * this.totalResponsesCorrect / this.totalResponses );
        //     }
        //     else
        //     {
        //         return 0;
        //     }
        // },

        //this isn't even used!
        // averageReactionTime: function()
        // {
        //     return this._findAverageReactionTime( this.reactionTimes );
        // },
        
        findAverageReactionTimeMatch: function()
        {
            return this._findAverageReactionTime( this.responseTimesMatch );
        },
        
        findAverageReactionTimeNoMatch: function()
        {
            return this._findAverageReactionTime( this.responseTimesNoMatch );
        },
        
        _findAverageReactionTime: function( array )
        {
            var averageReactionTime = 0;
            
            if ( array.length > 0 )
            {
                var reactionTimeIndex;
                for ( reactionTimeIndex = 0; reactionTimeIndex < array.length; reactionTimeIndex++ )
                {
                    averageReactionTime += array[ reactionTimeIndex ];
                }
                averageReactionTime /= array.length;
            }
            return averageReactionTime;
        }
    } );
}() );
