/*global require*/
/*global cc*/
/*global lumosity*/
/*global LumosityUtils*/
/*global localize*/

require( "src/Game/Base/GameView.js" );
/*global GameView*/

require("src/Game/Widget/GameWidgetFeedback.js");
/*global GameWidgetFeedback*/

require("src/Game/Widget/GameWidgetHud.js");
/*global GameWidgetHud*/

require( "src/Animation/AnimationChanges.js" );
/*global AnimationChanges*/


var SpeedMatchView;
(function()
{
    "use strict";

    SpeedMatchView = GameView.extend(
    {
        ctor: function( screenSize, settings, model, interactionResponder, preload )
        {
            this._super( screenSize, settings, model, interactionResponder, preload );

            this.lastShapes = {};
            this.currShapes = {};
            this.lastShape = "";
            this.currShape = "";
            this.desiredText = "";

            //determine important card positions
            this.winSize = cc.Director.getInstance().getWinSize();
            this.inCardStartPosition = cc.p (this.winSize.width*1.5, this.bottomUI.getContentSize().height + (( this.winSize.height - this.hud.getContentSize().height - this.bottomUI.getContentSize().height ) / 2 ));
            this.activeCardPosition = cc.p(this.winSize.width/2 , this.bottomUI.getContentSize().height + (( this.winSize.height - this.hud.getContentSize().height - this.bottomUI.getContentSize().height ) / 2 ));
            this.outCardEndPosition = cc.p (this.winSize.width*-1.5, this.bottomUI.getContentSize().height + (( this.winSize.height - this.hud.getContentSize().height - this.bottomUI.getContentSize().height ) / 2 ));

            //put the blank card offscreen
            this.inCard = cc.Sprite.create ("speedMatch_shapeBackground.png");
            this.inCard.nodeName = "inCard";
            this.inCard.setAnchorPoint( cc.p(.5,.5));
            this.inCard.setPosition( this.inCardStartPosition );
            lu.scaleForContentSize ( this.inCard, cc.size(396 * this.centeredContent.getScaleX(),390 * this.centeredContent.getScaleY()) )
            this._gameStage.addChild(this.inCard);

            //prepare the old card card offscreen
            this.outCard = this._createSprite ("speedMatch_shapeBackground.png");
            this.outCard.nodeName = "outCard"
            this.outCard.setAnchorPoint( cc.p(.5,.5));
            this.outCard.setPosition( this.inCardStartPosition );
            lu.scaleForContentSize ( this.outCard, cc.size(396 * this.centeredContent.getScaleX(),390 * this.centeredContent.getScaleY()) )
            this._gameStage.addChild(this.outCard);


            //I believe this block creates sprites for every symbol, which will be reused later instead of constantly creating sprites
            var key;
            for ( key in this.model.currentTrialSet.symbols )
            {
                var symbolImage = this.model.currentTrialSet.symbols[ key ];

                var curr = this._createSprite( "stimuli/" + symbolImage );
                this.currShapes[ key ] = curr;
                curr.setAnchorPoint( cc.p(.5,.5) );
                curr.setPosition( cc.p( this.inCard.getContentSize().width/2, this.inCard.getContentSize().height/2 ));
                curr.setVisible( false );
                this.inCard.addChild( curr );

                var last = this._createSprite( "stimuli/" + symbolImage );
                this.lastShapes[ key ] = last;
                last.setAnchorPoint( cc.p(.5,.5) );
                last.setPosition( cc.p( this.outCard.getContentSize().width/2, this.outCard.getContentSize().height/2 ));

                last.setVisible( false );
                this.outCard.addChild( last );
            }

            if ( this.settings.useHudTextColor && this.settings.hudTextColor )
            {
                this.getHudWidget().setTextColor( this.settings.hudTextColor );
            }

            //html5, we don't want the yes and no buttons
            if ( lumosity.device.id === "html5" )
            {
                AnimationChanges.addNodeIgnoredChanges( this.yesButton, AnimationChanges.Visibility );
                AnimationChanges.addNodeIgnoredChanges( this.noButton , AnimationChanges.Visibility );
                this.yesButton.setVisible( false );
                this.noButton.setVisible( false );
            }

        },

        layout_readingData : function( name, data )
        {
            this._super( name, data );

            switch ( name )
            {
                case "btnText":
                    this.btnFace = LumosityUtils.getFontLocation( data.textData.face );
                    this.btnSize = data.textData.size;
                    this.btnColor = cc.c4b(data.textData.color.Red, data.textData.color.Green, data.textData.color.Blue, data.textData.color.Alpha);
                break;
            }
        },

        layout_creatingNode : function( name, data )
        {
            var button;
            switch (name)
            {
                case "noButton":
                    button = this._createButton("buttons/game_buttonInputUp.png", "buttons/game_buttonInputDown.png", null, localize("No_Label", "NO"), this.btnFace, this.btnSize, this.btnColor, cc.c4b(255, 255, 255, 255), data.size);
                    button.setPressedCallback(this.onButtonPressedCallback);
                    return button;
                case "yesButton":
                    button = this._createButton("buttons/game_buttonInputUp.png", "buttons/game_buttonInputDown.png", null, localize("Yes_Label", "YES"), this.btnFace, this.btnSize, this.btnColor, cc.c4b(255, 255, 255, 255), data.size);
                    button.setPressedCallback(this.onButtonPressedCallback);
                    return button;
                //case "card1":
                //    return this._createSprite("speedMatch_incorrect.png");
                //case "card2":
                //    return this._createSprite("speedMatch_shapeBackground.png");
            }

            return this._super( name, data );
        },

        //swipes a new card into the active position
        animCardIn: function ( card )
        {
            var moveAction = cc.MoveTo.create(.3, this.activeCardPosition );
            card.runAction( moveAction );
        }   ,

        //swipes the active card offscreen
        animCardOut: function ( card )
        {
            var moveAction = cc.MoveTo.create(.2, this.outCardEndPosition );
            card.runAction( moveAction );
        }   ,

        onButtonPressedCallback: function( button )
        {
            // if( !this.interactionResponder.allowButtonInput || !this.model.isGameRunning )
            // {
            //     return;
            // }

            switch( button.nodeName )
            {
                case "yesButton":
                    return this.interactionResponder.handleResponse( this.model.doSymbolsMatch(), true );
                case "noButton":
                    return this.interactionResponder.handleResponse( !this.model.doSymbolsMatch(), false );
            }
        },

        enableButtons : function(isEnabled)
        {
            this.noButton.setEnabled(isEnabled);
            this.yesButton.setEnabled(isEnabled);
        },

        updateSymbols : function()
        {
            var last = this.model.lastSymbolId;
            var curr = this.model.currentSymbolId;
            if ( last !== this.lastShape )
            {
                if ( this.lastShape !== "" )
                {
                    this.lastShapes[ this.lastShape ].setVisible( false );
                }
                this.lastShape = last;
                this.lastShapes[ last ].setVisible( true );
            }

            if ( curr !== this.currShape )
            {
                if ( this.currShape !== "" )
                {
                    this.currShapes[ this.currShape ].setVisible( false );
                }
                this.currShape = curr;
                return this.currShapes[ curr ].setVisible( true );
            }
        },

        /**
         * Updates the text in the prompt, using model values to decide what to show.
         *
         * @param {boolean} [immediate=false] If immediate, we set the text immediately. If not, we play the fadePrompt animation, then change the text on a cue.
         * @param {boolean} [wasIncorrect=false] If this was an incorrect trial in the tutorial, we show a message telling the user what they did wrong.
         * @param {boolean} [wasMatch=false] If this was incorrect, we need to know if it was a match or not to decide what message to show.
         */
        updateText: function( immediate, wasIncorrect, wasMatch )
        {
            this.desiredText = localize( "promptCaption" );

            //in practice mode, we show different strings on the bottom
            if ( this.model.persistentData.isPracticeMode )
            {
                if ( !wasIncorrect )
                {
                    //basically we show this until the user can enter input from the first card
                    if ( this.model.practiceProgress === 0 && !this.model.hasAllowedInput )
                    {
                        this.desiredText = localize( "tutorialRememberCard" );
                    }
                }
                //when incorrect, tell the user why they are a screw-up
                else
                {
                    if ( wasMatch )
                    {
                        this.desiredText = localize( "tutorialIncorrectMatch" );
                    }
                    else
                    {
                        this.desiredText = localize( "tutorialIncorrectNoMatch" );
                    }
                }
            }

            if ( this.prompt.getString() !== this.desiredText )
            {
                if ( immediate )
                {
                    this.cue_changePrompt();
                }
                else
                {
                    this.playAnimation( "fadePrompt" );
                }
            }
            else
            {
                this.desiredText = "";
            }
        },

        cue_changePrompt: function()
        {
            if ( !this.desiredText )
            {
                return;
            }

            this.prompt.setString( this.desiredText );
            this.desiredText = "";
        }, 

        cue_hideSkipTutorialButton: function()
        {
            this._super();
            // this.getFeedbackWidget().setVisible( false );
        }
    } );
}() );
