// Generated by CoffeeScript 1.7.1
(function() {
  var Game, StatusCountdown, StatusGameEnd, StatusGameplay, StatusHidingBoard, StatusLevelSelect, StatusNewLevel, TrainOfThoughtModel, TrainOfThoughtSettings, TrainOfThoughtView, audio, cc, ip, it, tt, _ref, engineSoundPlaying,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  this.require('src/Game/Base/Game.js');

  this.require('src/Game/TrainOfThoughtUtil.js');
  this.require('src/Game/TrainOfThoughtSettings.js');
  this.require('src/Game/TrainOfThoughtModel.js');
  this.require('src/Game/TrainOfThoughtView.js');

  this.require('src/Game/SimpleTutorScene.js');
  this.require('src/Game/DebugMenu.js');

  cc = this.cc;

  TrainOfThoughtSettings = this.TrainOfThoughtSettings;

  TrainOfThoughtModel = this.TrainOfThoughtModel;

  TrainOfThoughtView = this.TrainOfThoughtView;

  tt = this.TrainOfThoughtUtil;

  audio = cc.AudioEngine.getInstance();

  it = this.InteractionType;

  ip = this.InteractionPhase;

  Game = this.Game;


  /**
  * Status enum
   */

  _ref = [1, 2, 3, 4, 5, 6, 7], StatusLevelSelect = _ref[0], StatusCountdown = _ref[1], StatusGameplay = _ref[2], StatusHidingBoard = _ref[3], StatusNewLevel = _ref[4], StatusGameEnd = _ref[5];
  engineSoundPlaying = false;


  /**
   * The root of the Train of Thought game.
   * @class
   * @name TrainOfThought
   */

  this.TrainOfThought = (function(_super) {
    __extends(TrainOfThought, _super);


    /**
    	* @constructor
     */

    function TrainOfThought() {

      TrainOfThought.__super__.constructor.call(this);
      this.timer = this._components.GameComponentTimer;
      this.duration = this.settings.getGameLengthInSeconds();

      this._createTutorScene();
      this.debugMenu = new DebugMenu(this);

      // debug menu
      //this.view.addDebugMenuButton( { labelText: "Invincible", callback: this.toggleInvincible.bind( this ), target: this, closeMenuOnCallback: true } );
      //this.view.addDebugMenuButton( { labelText: "Unlock More", callback: this.unlockMore.bind( this ), target: this, closeMenuOnCallback: true } );                                
    
    };

    TrainOfThought.prototype._createTutorScene = function() {
      var parent = this.view._gameStage;
      this.tutorScene = new SimpleTutorScene(this, parent);
      TutorScene.extend(TrainOfThought, this, this.tutorScene);
      TutorScene.disableDimLayer(this.view);
      TutorScene.extendHud(this.view.hud);
    };

    TrainOfThought.prototype._createSettings = function() {
      return new TrainOfThoughtSettings();
    };

    TrainOfThought.prototype._createModel = function() {
      return new TrainOfThoughtModel(this.settings);
    };

    TrainOfThought.prototype._createView = function(screenSize) {
      var view = new TrainOfThoughtView(screenSize, this.settings, this.model, this);
      return view;
    };

    /**
    	* Initialize status
    	* @method
    	* @name onBegin
     */

    TrainOfThought.prototype.onBegin = function() {
      this.model.onBegin();
      TrainOfThought.__super__.onBegin.call(this);
      this.status = StatusLevelSelect;
      this.engineSfxId = 0;
      this.tutorScene.onBegin();
      // cc.TextureCache.getInstance().dumpCachedTextureInfo();  // XXX test only
    };


    /**
     * When countdown finishes update status and 
     * begin the actual game loop
     * GAME-2160 Only if not already in gameplay, then pause game.  The base game calls onStartGameplay after the tutorial has already called this method.  Test case:  2015-03-05 Begin fit test tutorial.  [~aaron] expects first train enters immediately.
     */
    TrainOfThought.prototype.onStartGameplay = function() {
      if (!this.model.isGameRunning) {
        TrainOfThought.__super__.onStartGameplay.call(this);
        // cc.TextureCache.getInstance().dumpCachedTextureInfo();  // XXX test only
        // TrainOfThought._dumpCacheAndCrash()  // XXX this crashes
        if (this.status != StatusGameplay) {
          // ll.log("TrainOfThought.onStartGameplay: pause");
          this.model.isGamePaused = true;
        }
      }
      this.countdownOver = false;
    };

    TrainOfThought._dumpCacheAndCrash = function()
    {
        cc.TextureCache.getInstance().dumpCachedTextureInfo();
        cc.TextureCache.getInstance().removeUnusedTextures();
        cc.SpriteFrameCache.getInstance().removeUnusedSpriteFrames(); // XXX this crashes
        cc.Director.getInstance().purgeCachedData();
        cc.TextureCache.getInstance().dumpCachedTextureInfo();
    };

    TrainOfThought.prototype._buttonReleased = function(name) {
      var btn;
      TrainOfThought.__super__._buttonReleased.call(this, name);
      if (!this.model.isGameRunning) {
        return;
      }
      if (name in this.view.nameToButton) {
        btn = this.view.nameToButton[name];
        for(var i = 0; i < this.view.stationOutlines.length; i++)
        {
            this.view.stationOutlines[i].setVisible(false);
        }
        var i = parseInt(name.substring("station_".length));
        i = i - 3;
        this.view.stationOutlines[i].setVisible(true);
        this.view.selectedButton = btn;

        if (this.view.selectedButton !== null && this.status === StatusLevelSelect) {
            return this.onLevelSelected(this.view.selectedButton.numStations);
        }

        return;
      }
      switch (name) {
        case "btnNext":
          if (this.view.selectedButton !== null && this.status === StatusLevelSelect) {
            return this.onLevelSelected(this.view.selectedButton.numStations);
          }
      }
    };

    TrainOfThought.prototype._backButtonTapped = function() {

      TrainOfThought.__super__._backButtonTapped.call(this);
      if(this.status === StatusLevelSelect)
      {
          this.onFinish(false);
      }
    };


    /**
     * Invoked by the view when the player chooses their difficulty
     * leve
   	 * @method
     * @name onLevelSelected
     *
     * GAME-2159 If in practice mode, ignore level select button taps. Unfortunately every LumosityButton responds to a tap even though its ancestor is invisible.  Test case:  2015-03-03 How to Play.  As board is sliding into view tap somewhere near center.  Not sure where; maybe where station 3 button might be or might animating horizontally.  Expect no response.  Got board of the level appears on top of tutorial board.  Trains appear or one train appears.  And tutorial board is lower.  Can reproduce about 10% of the time in Xcode Simulator iPad Retina or Boiler.
     */
    TrainOfThought.prototype.onLevelSelected = function(numStations) {
      if (this.model.persistentData.isPracticeMode)
      {
        return;
      }
      for (name in this.view.nameToButton)
      {
          var btn = this.view.nameToButton[name];
          btn.setEnabled(false);
      }
      this.selectLevel(numStations);
      return this.view.playAnimation('transition');
    };

    /**
     * GAME-2928 At start of fit test, set to countdown state, to trigger disposing of the view.  Otherwise, although the boardView no longer exists, the previous 7 stations are visible.  Test case:  2015-03-11 iOS.  Fit test.  Short game.  Countdown.  Restart.  [~rivey] would expect to see 3 stations in tutor.  Got 7 stations in tutor.
     */
    TrainOfThought.prototype.selectLevel = function(numStations) {
      this.status = StatusCountdown;
      this.model.selectNumStations(numStations);
      this.view.showBoard();
    };

    TrainOfThought.prototype.animationFinished_gameplay = function() {
        this.model.isGamePaused = false;
        this.status = StatusGameplay;
    };

    TrainOfThought.prototype.animationFinished_transition = function() {
      this.model.initSpawn();
      return this.view.playAnimation('gameplay');
    };

    /**
     * During gameplay the model and the view need to be 
     * ticked every frame.
     * @method
     * @name onUpdate
     * @param delta - delta time (in milliseconds)
     */
    TrainOfThought.prototype.onUpdate = function(delta) {
      var result;
      delta = this.debugMenu.speedUp(delta, 10.0);
      if (this.status === StatusGameplay && !this.model.isGamePaused && !this.tutorScene.isGamePaused) {
        TrainOfThought.__super__.onUpdate.call(this, delta);
        result = this.tickGameplay(delta);
      }
      this.tutorScene.onUpdate();
      return result;
    };

    TrainOfThought.prototype.playMusic = function() {
      if(!engineSoundPlaying) {
        audio.setMusicVolume( 1 );
        this.engineSfxId = audio.playMusic("trainEngineLoop.m4a", true);
        engineSoundPlaying = true;
      }
    }

    TrainOfThought.prototype.stopMusic = function() {
      if(engineSoundPlaying) {
        audio.stopMusic(this.engineSfxId);
        engineSoundPlaying = false;
      }
    }

    /**
    	* Update the game one frame
    	* @method
    	* @name tickGameplay
    	* @param delta - tick time (in milliseconds)
     */

    TrainOfThought.prototype.tickGameplay = function(delta) {
      var arriving, newTraveller, traveller, _i, _j, _len, _len1, _ref1;
      this.model.now += delta * 1000.0;
      if (!this.countdownOver) {

        var now =  this.getMetadataComponent().getRunningTime();
        newTraveller = this.model.checkSpawn(now);
        if (newTraveller !== null) {

          this.beginTrial();
          this.view.createTravellerView(newTraveller);
          this.playMusic();
        }
      }
      arriving = null;
      _ref1 = this.model.travellers;
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        traveller = _ref1[_i];
        if (traveller.step()) {
          if (!traveller.finished) {
            traveller.view.move();
            if (traveller.arrivalStatus === 1) {
              audio.playEffect("trainWhistle.mp3");
              this.view.showArrivalEffectsFor(traveller);
            }
          } else if (arriving === null) {
            arriving = [traveller];
          } else {
            arriving.push(traveller);
          }
        }
      }
      if (arriving !== null) {
        for (_j = 0, _len1 = arriving.length; _j < _len1; _j++) {
          traveller = arriving[_j];
          var now = this.getMetadataComponent().getRunningTime();
          var trialData = this.model.processArrival(traveller, now);
          var startTime =  parseInt(trialData["spawn_time_offset"]);

          if(this.model.travellers.length === 1)
          {
              // last trial
              // set level to session level
              var nextLev =  this.model.getNextLevel(this.model.level, this.model.level, this.model.numCorrect, this.model.numTries);
              if(nextLev < this.model.sessionLevel)
              {
                  nextLev = this.model.sessionLevel;
              }
              trialData["level"] = nextLev;
          }

          this.getMetadataComponent().endTrialWithTimes(trialData, (traveller.arrivalStatus === 1), startTime );

          this.view.disposeOf(traveller.view);
          this.model.disposeOf(traveller);
        }
      }
      this.model.update(this.model.timeRemaining);
      this.view.showScore(this.model.numCorrect, this.model.numTries);
      this.view.update(delta);
      if (this.countdownOver && this.model.travellers.length === 0) {
        audio.stopMusic(this.engineSfxId);
        return this.doEndGameplay();
      }
    };

    TrainOfThought.prototype.cue_setScoreHud = function() {
      this.view.showScore(this.model.numCorrect, this.model.numTries);
    }

    /**
    	* Invoked when the game timers runs out to freeze interaction and 
    	* trigger the end game dialog.
    	* @method
    	* @name onCountdownCompleted
     */

    TrainOfThought.prototype.onEndGameplay = function() {
      this.superEndGameplay = this._super;
      this.countdownOver = true;
      if (this.model.travellers.length === 0) {
        return this.doEndGameplay();
      }
    };

   TrainOfThought.prototype.onRestart = function() {
      this.model.onBegin();
       // reset selection and buttons
       audio.setMusicVolume( 1 );
       for (name in this.view.nameToButton)
       {
           var btn = this.view.nameToButton[name];
           btn.setEnabled(true);
       }
       for(var i = 0; i < this.view.stationOutlines.length; i++)
       {
           if (i + 3 === this.model.numStationsSelected)
           {

               continue;
           }
           this.view.stationOutlines[i].setVisible(false);
       }

       var hudWidget = this.view.getHudWidget();
       if(hudWidget) {
           hudWidget.setCustomComponentValue("CORRECT", "0 of 0");
       }
       else
       {
           ll.verbose("no hudwidget to reset");
       }

       var boardView = this.view.trialRoot.boardView;
       var isSeeingBoard = this.status === StatusGameplay || this.status === StatusCountdown
            || this.tutorScene.isVisible
            || (undefined != boardView && 1 <= boardView.getChildrenCount());
       ll.log("TrainOfThought.onRestart: isSeeingBoard: " + isSeeingBoard);
       if (isSeeingBoard)
       {
           audio.stopMusic(this.engineSfxId);
           engineSoundPlaying = false;
           this.view.dispose();
       }
       else
       {
           // in level select
           //this.view.dispose();
       }
       TrainOfThought.__super__.onRestart.call(this);
   };

   /**
    * Called when a Game has been paused.
    *
    * GAME-2861 When pausing, pause the tutor scene, which is overriding game pause to pause trains.  Test case:  2015-03-12 Fit test.  Red train travels toward red station.  Pause.  Wait 4 seconds.  Resume.  [~rivey] expects train has traveled less than one cell in the grid.  Got train has continued a couple of cells in the grid.
    */
   TrainOfThought.prototype.onPause = function(instant)
   {
       //audio.pauseMusic();
       audio.setMusicVolume( 0 );
       cc.BuilderReaderHelper.pause(this.tutorScene.root);
       TrainOfThought.__super__.onPause.apply(this, arguments);
   },

   /**
    * Called when a Game has been unpaused.
    *
    * GAME-2832 On pause, pause tutor scene animation.  Test case:  2015-03-02 Level select screen.  Start tutorial.  Apple home button.  Lumosity App minimizes.  Refocus on Lumosity.  Resume.  [~rivey] expects red train still paused.  Got red train travels and switch is not toggled toward red station.  So tutorial now feels confusing.
    */
   TrainOfThought.prototype.onResume = function()
   {
       TrainOfThought.__super__.onResume.call(this);
       cc.BuilderReaderHelper.resume(this.tutorScene.root);
       if (this.status === StatusGameplay) {
           audio.setMusicVolume( 1 );
       }
   },

    /**
    	* Actually end gameplay (since play continues until last train disappears)
    	* @method
    	* @name doEndGameplay
     */

    TrainOfThought.prototype.doEndGameplay = function() {
      if (this.status !== StatusGameplay) {
        TutorScene.mayCompleteTutorial(this);
        return;
      }
      this.status = StatusHidingBoard;
      return this.view.playAnimation('hideBoard');
    };

    /**
     * GAME-2160 If in fit test, do not level up.  Test case:  2015-03-03 Fit test.  Short game.  Play perfectly.  Expect no level unlocked screen.
     *
     * GAME-845 GAME-2160 Record last and max destinations.  Test case:  2014-05-05 In fit test, [~aaron] expects to record last destination count and max destination count.
     *
     * To test fit test next session level, save last destination count to user_level.  It might be profitable to split test the user_level to see what effect this has on engagement for Lumosity's most popular game.  Test case:  2015-06-01 Fit test.  Zassmin Montes de Oca expects to save user_level from fit test estimate of next session level.
     */
    TrainOfThought.prototype.animationFinished_hideBoard = function() {
      if (this.status !== StatusHidingBoard) {
        return;
      }
      var model = this.model;
      var board = this.model.board;
      var metadata = model.metadata;
      metadata[ "switch_csv" ] = LumosityUtils.jsToCsv( model.switchMetadata );

      metadata["diagram"] = board.diagram;
      metadata["last_destination_count"] = board.maxDestinationCount;
      metadata["max_destination_count"] = board.maxDestinationMax;
      metadata["switch_counts"] = board.switchCounts.toString();

      metadata["isomorph"] = model.isomorph;
      metadata["switch_variance"]= model.getVariance();
      metadata["selected_level"] = model.level;

      if (!LumosityUtils.isFitTestModeOn() && model.completeLevel()) {
        this.status = StatusNewLevel;
        this.view.levelUnlocked.setVisible(true);
        return this.view.playAnimation('levelUnlocked');
      } else {
        if (LumosityUtils.isFitTestModeOn()) {
          this.model.sessionLevel = Math.max(30, 10 * board.maxDestinationCount);
        }
        this.status = StatusGameEnd;
        return Game.prototype.onEndGameplay.call(this);
      }
    };

    TrainOfThought.prototype.animationFinished_levelUnlocked = function() {
      this.status = StatusGameEnd;
      return Game.prototype.onEndGameplay.call(this);
    };


    /**
    	* Collect single-touches during gameplay
    	* @method
    	* @name usesInteractionType
    	* @param type - numeric identifier for the type of interaction
     */

    TrainOfThought.prototype.usesInteractionType = function(type) {
      return (this.status === StatusLevelSelect && type === it.Button) || (this.status === StatusGameplay && type === it.Point) || TrainOfThought.__super__.usesInteractionType.call(this, type);
    };


    /**
    	* If it's a single touch, determine which switch was
    	* triggered.
    	* @method
    	* @name interaction_received
    	* @param event - interaction event object
     */

    TrainOfThought.prototype.interaction_received = function(event) {
      var distSq, dx, dy, halfTileWidth, i, minDistSq, minSwitch, phase, pos, relaxedDistance, sp, switchView, _i, _len, _ref1;
      TrainOfThought.__super__.interaction_received.call(this, event, true);
      if (event.getType() !== it.Point) {
        return;
      }
      phase = event.getPhase();
      if (phase === ip.Began) {
        this.view.touchCount += 1;
        pos = this.view.boardView.convertToNodeSpace(event.getPosition());
        halfTileWidth = 0.5 * this.settings.tileWidth;
        minSwitch = null;
        minDistSq = 0;
        _ref1 = this.view.switches;
        for (i = _i = 0, _len = _ref1.length; _i < _len; i = ++_i) {
          switchView = _ref1[i];
          sp = switchView.getPosition();
          dx = Math.abs(pos.x - sp.x);
          dy = Math.abs(pos.y - sp.y);
          distSq = dx * dx + dy * dy;
          if (dx + dy < halfTileWidth) {
            this.onTappedSwitch(switchView);
            return;
          } else if (minSwitch === null || distSq < minDistSq) {
            minSwitch = switchView;
            minDistSq = distSq;
          }
        }
        relaxedDistance = 0.75 * this.settings.tileWidth;
        if (minSwitch !== null && minDistSq < relaxedDistance * relaxedDistance) {
          return this.onTappedSwitch(minSwitch);
        }
      } else if (phase === ip.Ended || phase === ip.Cancelled) {
        return this.view.touchCount -= 1;
      }
    };


    /**
    	* Emitted by interaction_received - attempt
    	* to actually toggle the given switch
     */

    TrainOfThought.prototype.onTappedSwitch = function(switchView) {
      if (this.model.checkToggle(switchView.address, this.getMetadataComponent().getRunningTime(), switchView.toggled)) {
        audio.playEffect("trackSwitch.mp3");
        return this.view.toggle(switchView);
      }
    };

    return TrainOfThought;

  })(Game);

}).call(this);
