/*jslint node: true */
"use strict";


require("src/Utility/LumosityUtils.js");
require("src/Utility/Random.js");

var FitTest = FitTest || {};

FitTest.init = function()
{
  if (LumosityUtils.isFitTestModeOn()) {
      Random.initialize(0);
  }
};

/**
 * @return  Uniformly distributed random floating point number [0..1)
 *
 * GAME-843 GAME-2160 In fit test only, use pseudo-random number generator (sin-transform, which Eli has used to seed Lumosity Party) if and only if called in the same order; for example, branching user behavior branches the number of calls to the number generator.  Test case:  2014-05-05 Fit Test.  To diminish variability, Ethan expects repeatable sequence.  Aaron expects fit test is only taken once by a user.
 */
FitTest.random = function() {
  if (LumosityUtils.isFitTestModeOn()) {
    return Random.value();
  }
  else {
    return Math.random();
  }
};


// Generated by CoffeeScript 1.7.1
(function() {
  var BIG_INT, BoardModel, TravellerModel, WEE_INT, addressDestinations, arcNames, arcs, assert, at, boardUntilTurn, charList, clamp01, clampDestinationCount, corners, countSwitch, countSwitches, defaultTravellerDistribution, destinationNames, doClassify, doRegulateDensity, doStep, doToggle, due, dueIndexes, flipHorizontal, flipVertical, from, getDensity, leftRight, modHelper, modelApproachingSource, modelDestinations, modelHistorySlice, modelMaxConcurrent, modelMaxSpawn, modelMinDestinationCount, modelNoIndex, modelOccupied, modelOffC, modelOffR, modelRestIndex, modelSources, modelSpawnPeriod, modelSwitches, noNeighbor, occupiedColumns, occupiedRows, outgoing, parentsOf, pushLeaf, randomElementFrom, rc, refBasePoints, refDifficultyDensity, refDifficultyVariety, refEquilibriumAccuracy, refInit, refLevelChanges, refMaxDifficulty, refMaxLevel, refMinLevel, refNextLevel, refPeriod, refProgress, refScaleSpawnPeriod, refScaleSpawnPeriodSuper, refSetEndMessage, refSetWrongLevelUp, refSpawnPeriod, refStartDifficulty, refWrongLevelUp, seeCorrect, seeWaypointLeaves, selectIsomorph, shuffle, shuffleDestinations, sourceNames, switchNames, tableToText, textToTable, tileTreeMaxLevel, tileTreeMinLevel, toVectors, topBottom, transpose, tt, vacantCorners, variance, vary, visitParents,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  this.require("src/Game/TrainOfThoughtUtil.js");

  tt = this.TrainOfThoughtUtil;

  assert = tt.assert;

  BIG_INT = tt.BIG_INT;

  WEE_INT = tt.WEE_INT;

  charList = function(chars) {
    return chars.split('');
  };


  /**
  * Pictograms representing pointing in a direction, 
  * as if a clock pointing to the right and turning clockwise by a quarter turn.
  * This aligns to Flash's representation of 0 degrees rotation pointing to the right.
   */

  arcs = charList(">v<^");


  /**
  * "A", "C" and "b" are reserved.
  * Only 14 modelDestinations fit in 7x5 grid.
   */

  modelDestinations = charList("0123456789!@#$");

  modelMinDestinationCount = 3;

  tileTreeMinLevel = 2;

  tileTreeMaxLevel = modelDestinations.length;

  clampDestinationCount = function(count) {
    return Math.max(modelMinDestinationCount, Math.min(tileTreeMaxLevel, count));
  };

  modelOffR = [0, 1, 0, -1];

  modelOffC = [1, 0, -1, 0];


  /**
  * Pictograms representing a source pointing in a direction.
  * Example "U" points down.
   */

  modelSources = charList("DUCA");


  /**
  * Pictograms representing switch of between two different directions.
  * Example "p" may point right or down.
   */

  modelSwitches = charList("pqdb");


  /**
  * Convert the diagram string to a 2D Array of Pictograms
   */

  textToTable = function(diagram) {
    var line, _i, _len, _ref, _results;
    _ref = diagram.split('/');
    _results = [];
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      line = _ref[_i];
      _results.push(charList(line));
    }
    return _results;
  };

  tableToText = function(tab) {
    var li;
    return ((function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = tab.length; _i < _len; _i++) {
        li = tab[_i];
        _results.push(li.join(''));
      }
      return _results;
    })()).join('/');
  };


  /**
  * Swap columns and substitute left-right facing cells.
   */

  flipHorizontal = function(diagram) {
    var c, c1, middle, p, palindrome, right, row, rows, temp, _i, _j, _len;
    rows = textToTable(diagram);
    palindrome = charList(">DpbdqC<");
    for (_i = 0, _len = rows.length; _i < _len; _i++) {
      row = rows[_i];
      right = row.length - 1;
      middle = Math.floor(right / 2);
      for (c = _j = 0; 0 <= middle ? _j <= middle : _j >= middle; c = 0 <= middle ? ++_j : --_j) {
        temp = row[c];
        c1 = right - c;
        row[c] = row[c1];
        row[c1] = temp;
        p = palindrome.indexOf(row[c]);
        if (0 <= p) {
          row[c] = palindrome[palindrome.length - p - 1];
        }
        if (c !== c1) {
          p = palindrome.indexOf(row[c1]);
          if (0 <= p) {
            row[c1] = palindrome[palindrome.length - p - 1];
          }
        }
      }
    }
    return tableToText(rows);
  };


  /**
  * Swap rows and substitute up-down facing cells.
   */

  flipVertical = function(diagram) {
    var bottom, c, middle, p, palindrome, r, r1, rows, temp, _i, _j, _ref;
    rows = textToTable(diagram);
    palindrome = charList("^ApqdbUv");
    bottom = rows.length - 1;
    middle = Math.floor(bottom / 2);
    for (r = _i = 0; 0 <= middle ? _i <= middle : _i >= middle; r = 0 <= middle ? ++_i : --_i) {
      r1 = bottom - r;
      for (c = _j = 0, _ref = rows[r].length - 1; 0 <= _ref ? _j <= _ref : _j >= _ref; c = 0 <= _ref ? ++_j : --_j) {
        temp = rows[r][c];
        rows[r][c] = rows[r1][c];
        rows[r1][c] = temp;
        p = palindrome.indexOf(rows[r][c]);
        if (0 <= p) {
          rows[r][c] = palindrome[palindrome.length - p - 1];
        }
        if (r !== r1) {
          p = palindrome.indexOf(rows[r1][c]);
          if (0 <= p) {
            rows[r1][c] = palindrome[palindrome.length - p - 1];
          }
        }
      }
    }
    return tableToText(rows);
  };


  /**
  * Array-shufflin' utility
   */
  shuffle = function(array, random) {
    if (undefined === random) {
      random = Math.random;
    }
    var currentIndex, randomIndex, temporaryValue, _results;
    currentIndex = array.length;
    _results = [];
    while (0 !== currentIndex) {
      randomIndex = Math.floor(FitTest.random() * currentIndex);
      currentIndex -= 1;
      temporaryValue = array[currentIndex];
      array[currentIndex] = array[randomIndex];
      _results.push(array[randomIndex] = temporaryValue);
    }
    return _results;
  };


  /**
  * Randomize the modelDestinations
   */
  shuffleDestinations = function(diagram) {
    var c, characters, count, destinies, _i, _j, _ref, _ref1;
    characters = charList(diagram);
    count = 0;
    for (c = _i = 0, _ref = characters.length - 1; 0 <= _ref ? _i <= _ref : _i >= _ref; c = 0 <= _ref ? ++_i : --_i) {
      if (0 <= modelDestinations.indexOf(characters[c])) {
        characters[c] = "?";
        count += 1;
      }
    }
    destinies = modelDestinations.slice(0, count);
    shuffle(destinies, FitTest.random);
    for (c = _j = 0, _ref1 = characters.length - 1; 0 <= _ref1 ? _j <= _ref1 : _j >= _ref1; c = 0 <= _ref1 ? ++_j : --_j) {
      if ("?" === characters[c]) {
        characters[c] = destinies.pop();
      }
    }
    return characters.join("");
  };

  vary = function(diagram) {
    if (FitTest.random() < 0.5) {
      diagram = flipHorizontal(diagram);
    }
    if (FitTest.random() < 0.5) {
      diagram = flipVertical(diagram);
    }
    return shuffleDestinations(diagram);
  };

  randomElementFrom = function(li) {
    return li[Math.floor(li.length * FitTest.random())];
  };


  /**
  * Randomly varied, balanced isomorph at this level.
   */

  selectIsomorph = function(level, boards) {
    var idx;
    idx = Math.floor(Math.max(0, Math.min(tileTreeMaxLevel, level / 10) - tileTreeMinLevel));
    return randomElementFrom(boards[idx]);
  };


  /**		
  * Array destination addresses.  For each: backtrack to source.  At each waypoint insert unique destination.
  * @param   table   2D representation of diagram.
  * @return  which modelDestinations this waypoint might end up at.
   */

  seeWaypointLeaves = function(table) {
    return visitParents(table, addressDestinations(table), pushLeaf);
  };


  /**
  * Search the able for 10-pitch addresses of modelDestinations
   */

  addressDestinations = function(table) {
    var c, col, i, r, result, row, _i, _j, _len, _len1;
    result = {};
    for (r = _i = 0, _len = table.length; _i < _len; r = ++_i) {
      row = table[r];
      for (c = _j = 0, _len1 = row.length; _j < _len1; c = ++_j) {
        col = row[c];
        i = modelDestinations.indexOf(col);
        if (0 <= i) {
          result[10 * r + c] = i;
        }
      }
    }
    return result;
  };


  /**
  * @param   atParent    parameters:  leaves, parent, destination, table[r][c]
   */

  visitParents = function(table, aDestinations, atParent) {
    var address, c, destination, leaves, parent, parents, r, step;
    leaves = {};
    for (address in aDestinations) {
      destination = aDestinations[address];
      parents = [address];
      step = 0;
      while (parents.length) {
        parent = parents.pop();
        r = Math.floor(parent / 10);
        c = parent % 10;
        atParent(leaves, parent, destination, table[r][c]);
        parents = parents.concat(parentsOf(table, r, c));
        step += 1;
        assert(step < 128, "Expected to find source in 5x7 table.");
      }
    }
    return leaves;
  };

  pushLeaf = function(leaves, parent, destination, cell) {
    if (parent in leaves) {
      if (__indexOf.call(leaves[parent], destination) < 0) {
        return leaves[parent].push(destination);
      }
    } else {
      return leaves[parent] = [destination];
    }
  };


  /**
  * Population variance in tree height of each leaf from the root.
   */

  variance = function(numbers) {
    var length, mean, n, sum, _i, _j, _len, _len1;
    length = numbers.length;
    if (!(1 <= length)) {
      return 0;
    }
    sum = 0;
    for (_i = 0, _len = numbers.length; _i < _len; _i++) {
      n = numbers[_i];
      sum += n;
    }
    mean = sum / length;
    sum = 0;
    for (_j = 0, _len1 = numbers.length; _j < _len1; _j++) {
      n = numbers[_j];
      sum += Math.pow(n - mean, 2);
    }
    return sum / length;
  };


  /**
  * @return  Array of adjacent integer addresses point here, numerically sorted.
   */

  parentsOf = function(table, r, c) {
    var a, co, o, result, reverse, ro, _i, _ref;
    result = [];
    for (o = _i = 0, _ref = modelOffR.length - 1; 0 <= _ref ? _i <= _ref : _i >= _ref; o = 0 <= _ref ? ++_i : --_i) {
      ro = r + modelOffR[o];
      co = c + modelOffC[o];
      if (!(ro < 0 || table.length <= ro || co < 0 || table[ro].length <= co)) {
        reverse = (o + modelOffR.length / 2) % modelOffR.length;
        a = table[ro][co];
        if (reverse === modelSources.indexOf(a) || reverse === arcs.indexOf(a) || reverse === modelSwitches.indexOf(a) || reverse === "bpqd".indexOf(a)) {
          result.push(10 * ro + co);
        }
      }
    }
    result.sort(Array.NUMERIC);
    return result;
  };


  /**
  * Assumes modelDestinations indexed in continguous order 0,1,2...
  * @return  Count of modelSwitches to that destination, ordered by destination indexes.
   */

  countSwitches = function(diagram) {
    var count, counts, destinationCounts, dst, table;
    table = textToTable(diagram);
    destinationCounts = visitParents(table, addressDestinations(table), countSwitch);
    counts = [];
    for (dst in destinationCounts) {
      count = destinationCounts[dst];
      counts[dst] = count;
    }
    return counts;
  };

  countSwitch = function(counts, parent, destination, cell) {
    if (!(destination in counts)) {
      counts[destination] = 0;
    }
    if (__indexOf.call(modelSwitches, cell) >= 0) {
      return counts[destination] += 1;
    }
  };

  modelMaxSpawn = 20;

  modelMaxConcurrent = 20;

  modelNoIndex = -2;

  modelRestIndex = -1;

  modelApproachingSource = -1;

  modelHistorySlice = -12;

  modelSpawnPeriod = 0;

  modelOccupied = -1;


  /**
  * @param   directions     Each key is 2D address represented as 10*row + column.  
  *                      Modified in place.  Value represents. -1: left, 1: right.
  * @return   vectors     2D table of indexes of directions. -1 denotes to delete.
   */

  toVectors = function(model, diagram) {
    var a, address, c, destination, i, o, r, row, _i, _j, _len, _len1, _ref;
    model.diagram = diagram;
    model.table = textToTable(diagram);
    model.vectors = [];
    model.leaves = seeWaypointLeaves(model.table);
    model.destinations = {};
    model.travellerDistribution = defaultTravellerDistribution(modelDestinations.length + 1);
    _ref = model.table;
    for (r = _i = 0, _len = _ref.length; _i < _len; r = ++_i) {
      row = _ref[r];
      model.vectors[r] = [];
      for (c = _j = 0, _len1 = row.length; _j < _len1; c = ++_j) {
        a = row[c];
        address = 10 * r + c;
        o = -1;
        if ('.' === a) {

        } else if (__indexOf.call(arcs, a) >= 0) {
          o = arcs.indexOf(a);
        } else if (__indexOf.call(modelSwitches, a) >= 0) {
          o = modelSwitches.indexOf(a);
          model.directions[address] = 1;
        } else if (__indexOf.call(modelDestinations, a) >= 0) {
          i = modelDestinations.indexOf(a);
          model.destinations[address] = i;
          model.travellerDistribution[i] += 1;
        } else if (__indexOf.call(modelSources, a) >= 0) {
          o = modelSources.indexOf(a);
          model.source = address;
          model.spawnRow = Math.floor(address / 10);
          model.spawnColumn = address % 10;
        } else {
          tt.log("Did not expect <" + a + "> in table");
        }
        model.vectors[r][c] = o;
      }
    }
    model.totalDestinationCount = 0;
    for (destination in model.destinations) {
      model.totalDestinationCount += 1;
    }
    model.maxDestinationCount = model.totalDestinationCount;
    model.switchCounts = countSwitches(diagram);
    return model.vectors;
  };

  defaultTravellerDistribution = function(length) {
    var t, travellerDistribution, _i, _ref;
    travellerDistribution = [];
    for (t = _i = 0, _ref = length - 1; 0 <= _ref ? _i <= _ref : _i >= _ref; t = 0 <= _ref ? ++_i : --_i) {
      travellerDistribution[t] = 0;
    }
    travellerDistribution[length - 1] = 2;
    return travellerDistribution;
  };


  /**
  * Move each waypoint to its next 2D address.
  * To animate at arbitrary time, board expects model has previous step times array.
  * @param   waypoints   2D address represented as 10*row + column.  
              Above 100 denotes to delete.
              negative number (-1) reserved to approach source.
              Modified in-place.
  * @param   vectors     2D table of indexes of directions. -1 denotes to delete.
  * @param   source      2D address of source.
  * @param   now         Current time.  If 0, move all.
  * @param   times       Last movements, same order as waypoints. Only moves those after period.
  * @param   moves       If times, marks each due element at time in parameter "now".
   */

  doStep = function(w, waypoints, vectors, source, now, times, period, moves) {
    var c, o, r, waypoint;
    if (source == null) {
      source = 0;
    }
    if (now == null) {
      now = 0;
    }
    if (times == null) {
      times = null;
    }
    if (period == null) {
      period = 0;
    }
    if (moves == null) {
      moves = null;
    }
    if (waypoints.length <= w || !(due((times === null ? -1 : times[w]), now, period))) {
      return waypoints;
    }
    if (modelApproachingSource === waypoints[w]) {
      waypoints[w] = source;
    } else {
      waypoint = waypoints[w] % 100;
      r = Math.floor(waypoint / 10);
      c = waypoint % 10;
      o = vectors[r][c];
      if (-1 === o) {
        waypoints[w] += 100;
      } else {
        r += modelOffR[o];
        c += modelOffC[o];
        waypoints[w] = 10 * r + c;
      }
    }
    return waypoints;
  };


  /**
  * Up to 16 ms earlier than full period to account for Flash player frame drift.
   */

  due = function(prev, now, period) {
    if (1 <= now && (0 <= prev && now < prev + period - 16)) {
      return false;
    } else {
      return true;
    }
  };


  /**
  * @param   times       Modified in place.  Schedules next time.
  * @return indexes of times that are due.
   */

  dueIndexes = function(times, now, period) {
    var indexes, t, time, _i, _len;
    indexes = [];
    for (t = _i = 0, _len = times.length; _i < _len; t = ++_i) {
      time = times[t];
      if (due(time, now, period)) {
        indexes.push(t);
      }
    }
    return indexes;
  };


  /**
  * @return The previous waypoint that would arrive at this waypoint after one step.
   */

  from = function(waypoint, vectorIndex) {
    var c, o, r;
    if (-1 !== vectorIndex) {
      r = Math.floor(waypoint / 10);
      c = waypoint % 10;
      o = (vectorIndex + 0.5 * modelOffR.length) % modelOffR.length;
      r += modelOffR[o];
      c += modelOffC[o];
      waypoint = 10 * r + c;
    }
    return waypoint;
  };

  modHelper = function(dividend, divisor) {
    var remainder;
    remainder = dividend % divisor;
    if (remainder < 0) {
      remainder += divisor;
    }
    return remainder;
  };


  /**
  * Toggle a switch of a vector between positive numbers.
  * @param   vectors     2D table of indexes of directions. -1 denotes to delete.
  * @param   directions     Each key is 2D address represented as 10*row + column.  
  *                      Modified in place.  Value represents. -1: left, 1: right.
   */

  doToggle = function(vectors, directions, address) {
    var c, r;
    r = Math.floor(address / 10);
    c = address % 10;
    vectors[r][c] = modHelper(vectors[r][c] + directions[address], modelOffR.length);
    return directions[address] *= -1;
  };


  /**
  * @param   history   Calculate density from ratio of non-rest travellers.
  * @param   per     If 100, round to integer percent.  If 0, floating point [0.0..1.0].
   */

  getDensity = function(history, per) {
    var count, density, hist, t, _i, _len;
    if (per == null) {
      per = 0;
    }
    density = 0;
    if (1 <= history.length) {
      count = 0;
      for (t = _i = 0, _len = history.length; _i < _len; t = ++_i) {
        hist = history[t];
        if (modelRestIndex !== hist) {
          count += 1;
        }
      }
      density = count / history.length;
    }
    if (1 <= per) {
      density = Math.floor(Math.round(per * density));
    }
    return density;
  };


  /**
  * Functions analogously to a rest in a musical score.
  * Balance the density of current travellers on variable layouts.
  * @return pending without rests except maybe rest (-1) pushed to uniformly distribute at density.
   */

  doRegulateDensity = function(density, history, pending) {
    var current, interval, next, rest, s, spliced, t;
    current = getDensity(history);
    if (density <= 0.0) {
      next = false;
    } else if (1.0 <= density || current <= 0.0) {
      next = true;
    } else {
      next = current <= density;
      if (density <= 0.5) {
        interval = Math.max(0, history.length - Math.floor(0.5 / density));
        t = history.length - 1;
        while (interval <= t) {
          if (modelRestIndex !== history[t]) {
            next = false;
            break;
          }
          t -= 1;
        }
      } else {
        rest = false;
        interval = Math.max(0, history.length - Math.floor(1.0 / (1.0 - density)));
        t = history.length - 1;
        while (interval < t) {
          if (modelRestIndex !== history[t]) {
            rest = false;
          } else {
            rest = true;
            break;
          }
          t -= 1;
        }
        next = next && rest;
      }
    }
    spliced = pending.slice();
    s = spliced.length - 1;
    while (s >= 0) {
      if (modelRestIndex === spliced[s]) {
        spliced.pop();
      }
      s -= 1;
    }
    if (!next) {
      spliced.push(modelRestIndex);
    }
    return spliced;
  };


  /**
  * Identify a new traveller going toward certainly correct/wrong path 
  * for the first step of that traveller.  
  *
  * If a waypoint is in leaves, 
  * and the traveller index is not among them, and is not a rest, then it is wrong.
  * For each traveller, store a parallel array:  
  *     0: maybe on correct path, -1: on wrong path.  -2: second turn on wrong path, etc.
  * For each entering wrong path (-1), regress, 
  * for each entering correct path (1), progress.
  * 
  * @param   leaves      Expects each waypoint integer has array of possible modelDestinations.
  * @param   waypoint    Numerical address of next step. +100 if at destination. 
  *                      Negative if approaching source.
  * @param   traveling       Indexes of destination for that traveller.
  * @param   seeCorrects     Parallel to waypoints and traveling.  Modified in-place.
  * @param   now         Current time.  If 0, move all.
  * @param   times       Last movements, same order as waypoints. Only moves those after period.
  * @return  seeCorrects, each: incremented if correct, or decremented if wrong.
   */

  seeCorrect = function(t, waypoints, traveling, leaves, seeCorrects) {
    var address, _ref;
    if (modelRestIndex !== traveling[t]) {
      address = rc(waypoints[t]);
      if (address in leaves) {
        if (!(_ref = traveling[t], __indexOf.call(leaves[address], _ref) >= 0)) {
          seeCorrects[t] = Math.min(-1, seeCorrects[t]);
        } else if (leaves[address].length === 1) {
          seeCorrects[t] = Math.max(1, seeCorrects[t]);
        }
      }
    }
    return seeCorrects;
  };

  rc = function(waypoint) {
    return Math.abs(waypoint % 100);
  };

  at = function(array2D, rowColumn) {
    var c, r, rowCol;
    rowCol = parseInt(rowColumn);
    r = int(rowCol / 10);
    c = rowCol % 10;
    return array2D[r][c];
  };

  occupiedRows = function(diagram) {
    var bounds;
    bounds = topBottom(diagram);
    if (bounds.length <= 0) {
      return 0;
    } else {
      return bounds[0] + bounds[1] + 1;
    }
  };

  occupiedColumns = function(diagram) {
    var bounds;
    bounds = leftRight(diagram);
    if (bounds.length <= 0) {
      return 0;
    } else {
      return bounds[0] + bounds[1] + 1;
    }
  };


  /**
  * @return [top left,   top right,   bottom left,   bottom right]
   */

  corners = function(diagram) {
    var c, columns, r, result, rows, _i, _j, _len, _len1;
    result = [];
    columns = leftRight(diagram);
    rows = topBottom(diagram);
    for (_i = 0, _len = rows.length; _i < _len; _i++) {
      r = rows[_i];
      for (_j = 0, _len1 = columns.length; _j < _len1; _j++) {
        c = columns[_j];
        result.push(10 * r + c);
      }
    }
    return result;
  };

  /**
  * @return -1 in that corner if not vacant.
   */

  vacantCorners = function(diagram) {
    var address, table, vacancies, _i, _len, _ref;
    vacancies = [];
    table = textToTable(diagram);
    _ref = corners(diagram);
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      address = _ref[_i];
      if (noNeighbor(table, Math.floor(address / 10), address % 10)) {
        vacancies.push(address);
      } else {
        vacancies.push(modelOccupied);
      }
    }
    return vacancies;
  };

  noNeighbor = function(table, r, c) {
    var co, o, ro, _i, _ref;
    for (o = _i = 0, _ref = modelOffR.length - 1; 0 <= _ref ? _i <= _ref : _i >= _ref; o = 0 <= _ref ? ++_i : --_i) {
      ro = r + modelOffR[o];
      co = c + modelOffC[o];
      if (ro < 0 || table.length <= ro || co < 0 || table[ro].length <= co) {

      } else if ('.' !== table[ro][co]) {
        return false;
      }
    }
    return true;
  };

  topBottom = function(diagram) {
    var bottom, c, cell, empty, r, row, table, top, _i, _j, _len, _len1;
    table = textToTable(diagram);
    top = 0;
    bottom = -1;
    empty = true;
    for (r = _i = 0, _len = table.length; _i < _len; r = ++_i) {
      row = table[r];
      if (empty) {
        top = r;
      }
      for (c = _j = 0, _len1 = row.length; _j < _len1; c = ++_j) {
        cell = row[c];
        if ('.' !== cell) {
          empty = false;
          bottom = r;
        }
      }
    }
    if (empty) {
      return [];
    } else {
      return [top, bottom];
    }
  };

  leftRight = function(diagram) {
    var c, cell, empty, max, min, r, row, table, _i, _j, _len, _len1;
    table = textToTable(diagram);
    min = BIG_INT;
    max = -1;
    empty = true;
    for (r = _i = 0, _len = table.length; _i < _len; r = ++_i) {
      row = table[r];
      for (c = _j = 0, _len1 = row.length; _j < _len1; c = ++_j) {
        cell = row[c];
        if ('.' !== cell) {
          empty = false;
          max = Math.max(c, max);
          min = Math.min(c, min);
        }
      }
    }
    if (empty) {
      return [];
    } else {
      return [min, max];
    }
  };

  outgoing = function(table, r, c) {
    var a, backward, co, i, o, ro, _i, _ref;
    backward = 2;
    for (o = _i = 0, _ref = modelOffR.length - 1; 0 <= _ref ? _i <= _ref : _i >= _ref; o = 0 <= _ref ? ++_i : --_i) {
      ro = r + modelOffR[o];
      co = c + modelOffC[o];
      if (ro >= 0 && table.length > ro && co >= 0 && table[ro].length > co) {
        a = table[ro][co];
        i = arcs.indexOf(a);
        if (0 <= i && i === (o + backward) % 4) {
          return o;
        }
        i = modelSwitches.indexOf(a);
        if (0 <= i && (i === (o + backward) % 4 || i === (o + backward - 1) % 4)) {
          return o;
        }
        i = modelSources.indexOf(a);
        if (0 <= i && i === (o + backward) % 4) {
          return o;
        }
      }
    }
    return NaN;
  };

  switchNames = ['SwitchEastWest', 'SwitchEastNorth', 'SwitchSouthNorth', 'SwitchSouthEast', 'SwitchWestEast', 'SwitchWestNorth', 'SwitchNorthSouth', 'SwitchNorthWest'];

  arcNames = ['PathEastSouth', 'PathEastWest', 'PathEastNorth', 'PathSouthWest', 'PathSouthNorth', 'PathWestNorth'];

  destinationNames = ['PathEast', 'PathSouth', 'PathWest', 'PathNorth'];

  sourceNames = ['PathEastWest', 'PathSouthNorth', 'PathEastWest', 'PathSouthNorth'];

  doClassify = function(table, r, c) {
    var a, i, j, min;
    a = table[r][c];
    if ('.' === a) {
      return null;
    }
    if (__indexOf.call(modelSwitches, a) >= 0) {
      i = modelSwitches.indexOf(a);
      j = (i + outgoing(table, r, c)) % 2;
      return switchNames[2 * i + j];
    }
    if (__indexOf.call(arcs, a) >= 0) {
      i = arcs.indexOf(a);
      j = outgoing(table, r, c);
      min = i < j ? i : j;
      j = i < j ? j : i;
      i = min;
      return arcNames[i * (4 - i) + j - i - Math.min(1, 2 - i)];
    }
    if (__indexOf.call(modelDestinations, a) >= 0) {
      return destinationNames[outgoing(table, r, c)];
    }
    if (__indexOf.call(modelSources, a) >= 0) {
      return sourceNames[modelSources.indexOf(a)];
    }
    return null;
  };


  /**
  * Conceptual boardgame version of how travelers spawn and approach a destination.
  * Only responsible for updating the state of each cell in the grid.
   */

  BoardModel = (function() {
    function BoardModel(diagram) {
      if (diagram == null) {
        diagram = void 0;
      }

      this.directions = {};
      this.waypoints = [];
      this.traveling = [];
      this.history = [];
      this.moves = [];
      this.seeCorrects = [];
      this.times = [];
      this.pending = [];
      this.spawnCount = 0;
      this.density = 0.25;
      this.period = refPeriod;
      this.spawnPeriod = modelSpawnPeriod;
      this.spawns = [];
      this.maxDestinationCount = BIG_INT;
      this.maxDestinationMax = WEE_INT;
      this.totalDestinationCount = BIG_INT;
      if (diagram) {
        toVectors(this, diagram);
      }
      this.travelTime = -100;
    }

    BoardModel.prototype.toggle = function(address) {
      return doToggle(this.vectors, this.directions, address);
    };

    BoardModel.prototype.getTravellerDistribution = function() {
      var r, restIndex, sum, _i;
      sum = 0;
      restIndex = this.travellerDistribution.length - 1;
      for (r = _i = 0; 0 <= restIndex ? _i <= restIndex : _i >= restIndex; r = 0 <= restIndex ? ++_i : --_i) {
        sum += this.travellerDistribution[r];
      }
      return (sum - this.travellerDistribution[restIndex]) / sum;
    };

    BoardModel.prototype.setTravellerDistribution = function(density) {
      var rest, restIndex, sum, t, _i, _ref;
      assert(density > 0 && density <= 1, "BoardModel.setTravellerDistribution: Expected density was in (0..1]");
      restIndex = this.travellerDistribution.length - 1;
      sum = 0;
      for (t = _i = 0, _ref = restIndex - 1; 0 <= _ref ? _i <= _ref : _i >= _ref; t = 0 <= _ref ? ++_i : --_i) {
        sum += this.travellerDistribution[t];
      }
      rest = Math.max(0, Math.round((sum / density) - sum));
      this.travellerDistribution[restIndex] = rest;
      return rest;
    };


    /**
    	* Shuffle, without gaps.
     */

    BoardModel.prototype.shufflePending = function() {
      var num, t, _i, _ref;
      this.pending = [];
      for (t = _i = 0, _ref = this.travellerDistribution.length - 2; 0 <= _ref ? _i <= _ref : _i >= _ref; t = 0 <= _ref ? ++_i : --_i) {
        num = this.travellerDistribution[t];
        while (num > 0) {
          this.pending.push(t);
          num -= 1;
        }
      }
      shuffle(this.pending);
      return this.pending;
    };


    /**
    	* During step, model may detect first waypoint toward wrong path.
    	* To move only those due, board expects model has previous step times array.
     */

    BoardModel.prototype.step = function(index, now) {
      if (now == null) {
        now = 0;
      }
      this.waypoints = doStep(index, this.waypoints, this.vectors, this.source, now, this.times, this.period, this.moves);
      return this.seeCorrects = seeCorrect(index, this.waypoints, this.traveling, this.leaves, this.seeCorrects);
    };


    /**
    	* During move, each next switch queues to animate commit.
    	* That is available in waypoints that are switches after step.
    	* @return  switch addresses that will play commit animation.
     */

    BoardModel.prototype.nextSwitches = function(now) {
      var result, w, waypoint, wp, _i, _len, _ref;
      if (now == null) {
        now = 0;
      }
      result = [];
      _ref = this.waypoints;
      for (w = _i = 0, _len = _ref.length; _i < _len; w = ++_i) {
        wp = _ref[w];
        if (this.traveling[w] !== modelRestIndex && due(this.times[w], now, this.period)) {
          waypoint = rc(wp);
          if (waypoint in this.directions) {
            result.push(waypoint);
          }
        }
      }
      return result;
    };


    /**
    	* Up to max concurrent travel at a time.  Increment spawn count.
    	* If no pending, shuffle and redeal pending.
    	* Also limited by maxDestinationCount, useful for limiting difficulty.
    	* @param   excluding   Do not spawn this index.  May result in no spawn if no others pending.
    	*                      Useful to ensure no delay before first spawn.
     */

    BoardModel.prototype.spawn = function(excluding, currentTimeOffset) {
      var selected, traveller, tries;
      if (excluding == null) {
        excluding = WEE_INT;
      }
      traveller = modelNoIndex;
      if (this.maySpawn()) {
        tries = 0;
        selected = BIG_INT;
        while (excluding === selected || this.maxDestinationCount <= selected) {
          if (!(this.pending.length > 0)) {
            this.pending = this.shufflePending();
          }
          selected = this.pending.pop();
          tries += 1;
          assert(tries < 128, "BoardModel.spawn: Expected to find an included index in a few tries");
        }
        traveller = selected;
        this.history.push(traveller);
        if (traveller !== modelRestIndex) {
          this.seeCorrects.push(0);
          this.moves.push(modelApproachingSource);
          this.waypoints.push(this.source);
          this.times.push(modelApproachingSource);
          this.traveling.push(traveller);
          this.spawnCount += 1;
          this.spawns.push({
            traveller: selected,
            spawn_density: this.getDensity(100),
            spawn_variety: this.maxDestinationCount,
            spawn_time_offset: currentTimeOffset
          });
        }
      }
      // ll.log("BoardModel.spawn: " + traveller);
      return traveller;
    };

    BoardModel.prototype.splice = function(index) {
      this.moves.splice(index, 1);
      this.seeCorrects.splice(index, 1);
      this.times.splice(index, 1);
      this.traveling.splice(index, 1);
      this.waypoints.splice(index, 1);
      return this.spawns.splice(index, 1);
    };

    BoardModel.prototype.getDensity = function(per) {
      if (per == null) {
        per = 0;
      }
      return getDensity(this.history.slice(modelHistorySlice), per);
    };


    /**
    	* Regulate density by history of latest spawns (or rests), 
    	* rather than by currently active spawns (or rests), which has a variable length.
    	* Edit pending in-place.
     */

    BoardModel.prototype.regulateDensity = function() {
      return this.pending = doRegulateDensity(this.density, this.history.slice(modelHistorySlice), this.pending);
    };

    /**
     * Maximum spawn.
     */
    BoardModel.prototype.maySpawn = function() {
      if (modelMaxConcurrent <= this.waypoints.length
       || modelMaxSpawn <= this.spawnCount) {
        return false;
      } else {
        return !(__indexOf.call(this.waypoints, modelApproachingSource) >= 0);
      }
    };


    /**
   	 * @return  0: not arrived. -1: wrong destination. 1: correct destination.
     */

    BoardModel.prototype.mayArrive = function(index) {
      if (this.waypoints[index] in this.destinations) {
        if (this.traveling[index] === this.destinations[this.waypoints[index]] || LumosityUtils.isDebugModeOn() || this.invincibleMode === true) {
          return 1;
        } else {
          return -1;
        }
      } else {
        return 0;
      }
    };

    /**
     * GAME-845 Interpolate fraction of total destinations, starting at 2 stations.  Test case:  2015-05-04 With 7 stations at level 70, Kacey Ballard expects to begin with 3 stations and ramp up to 7.  Ethan expects to begin with 2 stations and ramp up to 7, after the first train which is the 7th station.
     */
    BoardModel.prototype.setDestinationFraction = function(fraction) {
      this.maxDestinationCount = Math.max(2, Math.round(Math.min(1.0, fraction) * this.totalDestinationCount));
      this.maxDestinationMax = Math.max(this.maxDestinationMax, this.maxDestinationCount);
    };

    BoardModel.prototype.empty = function() {
      var noMore, t, trav, _i, _len, _ref;
      noMore = modelMaxSpawn <= this.spawnCount;
      if (noMore) {
        _ref = this.traveling;
        for (t = _i = 0, _len = _ref.length; _i < _len; t = ++_i) {
          trav = _ref[t];
          if (modelRestIndex !== trav) {
            noMore = false;
          }
        }
      }
      return noMore;
    };

    /**
     * Clear history and set to maximum density.  Test case:  2013-06 Tutor.  Spawn traveller 0.  Spawn traveller 1.  Expect traveller 1 appears immediately.
     */
    BoardModel.prototype.spawnTraveller = function(index) {
      // ll.log("BoardModel.spawnTraveller: " + index);
      modelMaxSpawn = this.spawnCount + 1;
      this.travelTime = WEE_INT;
      this.setTravellerDistribution(1.0);
      this.pending = [index];
      this.history = [];
      this.period = refPeriod;
      this.travellerDistribution[0] = 0;
      this.travellerDistribution[1] = 0;
      this.travellerDistribution[2] = 0;
      this.travellerDistribution[index] = 1;
    };

    /**
     * GAME-2159 Distribute first three color of trains.  Test case:  2015-02-26 Tutor.  Expect at least one of each 3 colors.  Got missing middle color.
     */
    BoardModel.prototype.spawnMore = function(count) {
      modelMaxSpawn = this.spawnCount + count;
      this.travelTime = WEE_INT;
      this.setDestinationFraction(1.0);
      this.travellerDistribution[0] = 1;
      this.travellerDistribution[1] = 1;
      this.travellerDistribution[2] = 1;
    };

    return BoardModel;

  })();

  refMaxLevel = tileTreeMaxLevel * 10 + 9;

  refMinLevel = tileTreeMinLevel * 10;

  refMaxDifficulty = 20;

  refPeriod = 2000;

  refSpawnPeriod = 2250;

  refBasePoints = 100;

  refEquilibriumAccuracy = 0.87;

  refWrongLevelUp = 3;

  refLevelChanges = [[3, 10], [4, 0], [BIG_INT, 0]];

  refInit = function(level) {
    if (level === 0) {
      level = refMinLevel + 10;
    }
    level = Math.max(refMinLevel, level);
    refWrongLevelUp = refSetWrongLevelUp(level);
    return level;
  };


  /**
  * Approximately halfway between a triangle wave and sawtooth wave.
  * Gradual ascent, through a root power curve.
  * @return [0.25..1.0]
   */

  refDifficultyDensity = function(difficulty, destinationCount) {
    var density, destinationFraction, max, min, normal, power, start;
    if (destinationCount == null) {
      destinationCount = 5;
    }
    power = 0.25;
    start = 5 - destinationCount * 0.5;
    normal = Math.pow(Math.max(0, difficulty - start), power) / Math.pow(refMaxDifficulty - start, power);
    destinationFraction = (destinationCount - tileTreeMinLevel) / (tileTreeMaxLevel - tileTreeMinLevel);
    normal = 0.5 * normal + 0.5 * destinationFraction;
    min = 0.3;
    max = 1.0;
    density = min + (max - min) * normal;
    return Math.max(min, Math.min(max, density));
  };

  /**
   * GAME-845 GAME-2160 If fit test, then quarter the rate of introducing additional train destinations.  Test case:  2014-05-05 Fit test for a beginner at 7 stations.  Ethan expects slow rate of introducing new trains.
   */
  refDifficultyVariety = function(difficulty) {
    var scale = 0.2;
    if (LumosityUtils.isFitTestModeOn()) {
      scale = 0.05;
    }
    var min = 0.2;
    var max = 1.0;
    var product = scale * difficulty;
    return Math.max(min, Math.min(max, product + min));
  };

  refStartDifficulty = function(level) {
    var perLevel;
    perLevel = 1.0;
    return (level % 10) * perLevel + 1;
  };

  /**
   * Scale between about 50 and 90 trains by interpolating period between about 2.25 and 1.25 seconds by level.
   * 
   * 2013-11-26 Because Train of Thought is x10 more popular than the second most popular beta, and there is a maximum difficulty level, Aaron observed that at least 3% of users have scored perfectly at 14 stations at least one session.  He says that for other games, the top score is more like 0.3% of users or less.  The scores at lower levels (and non-perfect performance at level 14) are fine for BPI, but Train of Thought perfect score is too popular.  
   * 
   * Early in the prototyping, we witnessed that if the frequency is quicker, any coworker can be challenged.
   * 
   * At 14 stations only, if a user makes 1 or fewer mistakes, next session increases the traveler frequency; if a user makes 4 or more mistakes, next session decreases the traveler frequency. The step size is 100 ms, which is about 2 more trains possible.  The max frequency might be about 48 trains per minute (about 90 trains altogether), or whatever value such that train graphics and clicking switches is not ambiguous to an expert player. Internally, the level parameter tracks these steps.  To test, a developer text box may set level above 140..  Example of steps of frequency in milliseconds at each level: 140: 2250, 150: 2150, 160: 2050, 240: 1250.
   * Example @see TestReferee
   * 2013-11-26 Aaron expects to discriminate no more than 0.1% of users achieving the highest score in one week.
   * https://trello.com/c/9zleac1g/775-13-train-of-thought-launch-w-bpi
   */
  refScaleSpawnPeriodSuper = function(level) {
    var step;
    step = -100;
    return Math.max(1250, step * Math.floor((level - 140) / 10) + refSpawnPeriod);
  };

  /**
   * Interpolate spawn period between 2.6 and 2.25 seconds based on selected level.
   *
   * Change log:
   * 2013-07-29 Select spawn rate "201308". Level 3. Play. Ben expects spawn about 2.7 seconds.
   * https://trello.com/c/T0PHBPc3/709-train-of-thought-post-beta-cleanup-fixes
   * 2013-09-25 Only in group "201309" of "flash_tot_spawn_rate2", min level 50.  Max 2.60 seconds. Min 2.25 seconds. Max level 140.  Interpolate spawn period between min level and max level.  Control group has identical results as "flash_spawn_rate" experiment group.
   * https://trello.com/c/lvaFMCk9/769-5-train-of-thought-run-v2-of-the-spawn-rate-split-test
   * 2013-10-29 Aaron and Ben expect experimental spawn rate group wins.
   * https://trello.com/c/9zleac1g/775-train-of-thought-launch-w-bpi
   */
  refScaleSpawnPeriod = function(level) {
    var factor, maxScaleLevel, minScaleLevel, minSpawnPeriod;
    if (refMaxLevel < level) {
      return refScaleSpawnPeriodSuper(level);
    }
    minScaleLevel = 50;
    maxScaleLevel = 140;
    minSpawnPeriod = 2600;
    factor = (level - minScaleLevel) / (maxScaleLevel - minScaleLevel);
    factor = Math.max(0, Math.min(1, factor));
    return Math.round(factor * refSpawnPeriod + (1 - factor) * minSpawnPeriod);
  };

  /**
   * @return Estimated number of travelers, based on fact that current max is frequently 50 at 14 stations. Actual results vary depending on performance and a little some random variance.
   * Decreasing the frequency is more coarse than increasing the frequency every turn.
   * The additional number of trains at each step grows with each step.
   * Example @see TestReferee
   */
  var refEstimateMaxTrials = function(level)
  {
    var period = refScaleSpawnPeriod(level);
    var periodAtMaxLevel = refScaleSpawnPeriod(refMaxLevel);
    var trialsAtMaxLevel = 50;
    var maxDensity = refDifficultyDensity(refMaxDifficulty,
        clampDestinationCount(level / 10));
    var max = maxDensity * trialsAtMaxLevel * periodAtMaxLevel / period;
    return max;
  };

  /**
   * Max at level 140 is 50 trains, increases with frequency.
   *
   * @param level       Selected level.
   * @param num_trials  Maximum number of trials in any case.  Usually very big.  A small number may be used in testing (such as short games mode) to truncate a session.
   * 2013/12/03 Aaron expects maximum spawn consistent at highest level (13/12/4 +Samantha Yang).
   * https://trello.com/c/9zleac1g/775-13-train-of-thought-launch-w-bpi
   */
  var refMaxTrials = function(level, num_trials) {
    var max;
    if (refMaxLevel < level + 10) {
      max = Math.min(num_trials, refEstimateMaxTrials(level));
    }
    else {
      max = num_trials;
    }
    ll.log("TrainOfThoughtModel.refMaxTrials: level: " + level + " max: " + max );
    return max;
  };

  /**
  * If meter is full.  Will progress to next destination.
  * If make a mistake.  Meter regresses.
  * Each correct.  Meter progresses proportional to progress. 
  * Ease early inaccuracy and weight late inaccuracy.
  * @param   timePassedRatio     Includes time for last train to reach correct destination.
  * @return  range [0.0..1.0]
   */

  refProgress = function(accuracy, timePassedRatio, nextLevelAccuracy) {
    return Math.min(timePassedRatio, Math.min(1.0, timePassedRatio * Math.pow(accuracy / nextLevelAccuracy, 1 + timePassedRatio)));
  };


  /**
  * Adjust next session level.
   */

  refNextLevel = function(startLevel, level, num_correct, num_tries) {
    var i, next, offset, pair, superLevel, superWrong, wrong, _i, _len;
    if (level < startLevel) {
      return startLevel;
    }
    next = level;
    if (1 <= num_tries) {
      offset = refLevelChanges[0][0] - refSetWrongLevelUp(level);
      wrong = num_tries - num_correct;
      for (i = _i = 0, _len = refLevelChanges.length; _i < _len; i = ++_i) {
        pair = refLevelChanges[i];
        if (wrong <= pair[0] - offset) {
          next += pair[1];
          break;
        }
      }
      if (refLevelChanges[0][0] - offset < wrong) {
        next = Math.floor(startLevel / 10 + 1) * 10 - 1 < next ? Math.floor(startLevel / 10) * 10 : next;
        next = Math.max(next, (Math.floor(startLevel / 10) - 1) * 10);
      }
      if (refMaxLevel < level) {
        superWrong = 3;
        if (superWrong < wrong) {
          next -= 10;
        }
      }
    }
    superLevel = 240;
    next = Math.max(refMinLevel, Math.min(superLevel, next));
    if (level <= refMaxLevel) {
      next = Math.max(Math.floor(startLevel / 10) * 10, next);
    }
    return next;
  };

  refSetWrongLevelUp = function(level) {
    if (refMaxLevel - 20 < level) {
      return refLevelChanges[0][0] - 2;
    } else if (refMaxLevel - 40 < level) {
      return refLevelChanges[0][0] - 1;
    } else {
      return refLevelChanges[0][0];
    }
  };


  /**
  * If not a full level, show needed more correct to advance or maintain.
   */

  refSetEndMessage = function(startLevel, level, user_level, wrong) {
    var className, destinationCount, nextDestinationCount, nextString, up, up10;
    if (wrong == null) {
      wrong = -1;
    }
    className = "";
    nextString = "";
    up = user_level - startLevel;
    up10 = Math.floor(user_level / 10) - Math.floor(startLevel / 10);
    if (refMaxLevel - 9 <= user_level && refMaxLevel - 9 <= startLevel) {
      className = "EndMessageMax";
      nextString = "";
    } else if (level < startLevel) {
      className = "EndMessageLowerLevel";
      nextString = "" + (Math.floor(startLevel / 10));
    } else if (0 <= up10) {
      if (1 <= up10) {
        nextString = "" + (Math.floor(user_level / 10));
        className = 20 + refMinLevel <= user_level ? "EndMessageUp" : "EndMessageUpLong";
      } else {
        nextString = "" + (refSetWrongLevelUp(startLevel));
        className = "EndMessageProgress";
      }
    }
    destinationCount = clampDestinationCount(startLevel / 10);
    nextDestinationCount = clampDestinationCount(user_level / 10);
    return [className, nextString, destinationCount, nextDestinationCount];
  };


  /**
  * Helper function, same as Cg's "saturate"
  * @function
  * @name clamp01
   */

  clamp01 = function(u) {
    return Math.min(Math.max(u, 0), 1);
  };


  /**
  * The traveller model converts the obtuse structure-of-arrays
  * source implementation of logical travellers into a grokkable
  * reference object.
  * The backing-index for each traveller model is updated when 
  * the array gets spliced and everything gets downshifted.
  * @class
  * @name TravellerModel
   */

  TravellerModel = (function() {

    /**
    	* @constructor
     */
    function TravellerModel(board, index, now, count) {
      this.board = board;
      this.index = index;
      this.count = count; // unlike index, this wont change
      this.board.times[this.index] = now;
      this.arrivalStatus = 0;
      this.finished = false;
      this.address = this.board.waypoints[this.index];
      this.committed = false;
    }


    /**
    	* Numeric identifiers of the current waypoints "exit" direction
    	* @method
    	* @name vector
    	* @returns direction ID [0,4)
     */

    TravellerModel.prototype.vector = function() {
      return this.board.vectors[this.index];
    };


    /**
    	* @method
    	* @name vectorOffssetR
    	* @returns the "row offset" for the current waypoint
     */

    TravellerModel.prototype.vectorOffsetR = function() {
      var vec;
      vec = this.vector();
      if (vec < 0) {
        return 0;
      } else {
        return modelOffR[vec];
      }
    };


    /**
    	* @method
    	* @name vectorOffsetC
    	* @returns The Column-Offset of the current waypoint vector
     */

    TravellerModel.prototype.vectorOffsetC = function() {
      var vec;
      vec = this.vector();
      if (vec < 0) {
        return 0;
      } else {
        return modelOffC[vec];
      }
    };


    /**
    	* @method
    	* @name destination
    	* @returns The desired destination for this traveller
     */

    TravellerModel.prototype.destination = function() {
      return this.board.traveling[this.index];
    };


    /**
    	* @method
    	* @name timeLastMove
    	* @returns The last time that we logically "moved" i.e. changed waypoints
     */

    TravellerModel.prototype.timeLastMove = function() {
      return this.board.times[this.index];
    };

    TravellerModel.prototype.waypointProgressRaw = function() {
      return (this.board.model.now - this.timeLastMove()) / this.board.period;
    };


    /**
    	* @method
    	* @name waypointProgress
    	* @returns Out percentage progress through the current waypoint
     */

    TravellerModel.prototype.waypointProgress = function() {
      var result;
      result = this.timeLastMove < 0 ? 0 : clamp01((this.board.model.now - this.timeLastMove()) / this.board.period);
      if (this.arrivalStatus === 0) {
        return result;
      } else {
        return Math.min(0.5, result);
      }
    };


    /**
    	* @method
    	* @name timeLastMove
    	* @returns {Boolean} Are we more than half way through the current waypoint?
     */

    TravellerModel.prototype.halfway = function() {
      return this.waypointProgress(this.board.model.now) > 0.49;
    };


    /**
    	* @method
    	* @name waypointRowColumn
    	* @return tuple of unpack waypoint (row, column)
     */

    TravellerModel.prototype.waypointRowColumn = function() {
      var val;
      val = rc(this.address);
      return [Math.floor(val / 10), val % 10];
    };


    /**
    	* When the model removes a traveller we may need to downshift our array index
    	* @method
    	* @name willRemoveTraveller
     */

    TravellerModel.prototype.willRemoveTraveller = function(index) {
      if (this.index > index) {
        return this.index -= 1;
      }
    };


    /**
    	* Update
    	* @method
    	* @name step
    	* @returns {Boolean} whether our state changed this frame
     */

    TravellerModel.prototype.step = function() {
      if (this.halfway() && !this.committed) {
        this.board.step(this.index);
        this.committed = true;
      }
      if (this.waypointProgressRaw() > 1.0) {
        this.finished = this.arrivalStatus !== 0;
        this.address = this.board.waypoints[this.index];
        this.board.times[this.index] = this.board.model.now;
        this.arrivalStatus = this.board.mayArrive(this.index);
        this.committed = false;
        return true;
      } else if (this.halfway()) {
        this.finished = this.arrivalStatus !== 0;
        return this.finished;
      } else {
        return false;
      }
    };

    return TravellerModel;

  })();

  this.require("src/Game/Base/GameModel.js");

  boardUntilTurn = 32 / 60;


  /**
  * Logically transpose a diagram before we pass it to the
  * data model so that mobile levels are vertically oriented
  * without modifying any of the underlying representations.
  * @function
  * @name tranpose
  * @param diagram - string pictogram of the level
  * @returns - modified pictogram
   */

  transpose = function(diagram) {
    var c, cell, dstPictograms, i, ncols, r, resultRows, row, rows, src, srcPictograms, srcToDst, _i, _j, _k, _len, _len1, _len2;
    srcPictograms = ">v<^DUCApqdb";
    dstPictograms = "v>^<UDACpbdq";
    srcToDst = {};
    for (i = _i = 0, _len = srcPictograms.length; _i < _len; i = ++_i) {
      src = srcPictograms[i];
      srcToDst[src] = dstPictograms[i];
    }
    rows = textToTable(diagram);
    ncols = rows[0].length;
    resultRows = (function() {
      var _j, _results;
      _results = [];
      for (i = _j = 1; 1 <= ncols ? _j <= ncols : _j >= ncols; i = 1 <= ncols ? ++_j : --_j) {
        _results.push([]);
      }
      return _results;
    })();
    for (r = _j = 0, _len1 = rows.length; _j < _len1; r = ++_j) {
      row = rows[r];
      for (c = _k = 0, _len2 = row.length; _k < _len2; c = ++_k) {
        cell = row[c];
        resultRows[c][r] = cell in srcToDst ? srcToDst[cell] : cell;
      }
    }
    return tableToText(resultRows);
  };


  /**
  * The Train of Thought @board.  This is the only exported interface.
  * @class
  * @name TrainOfThoughtModel
   */

  this.TrainOfThoughtModel = (function(_super) {
    __extends(TrainOfThoughtModel, _super);


    /**
     * Backward compatible:  If does not find user level, load from game stats level.
     * @constructor
     */
    function TrainOfThoughtModel(settings) {
      FitTest.init();
      var modelPeriod;
      this.settings = settings;
      TrainOfThoughtModel.__super__.constructor.call(this);
      modelPeriod = refPeriod;
      modelSpawnPeriod = refSpawnPeriod;
      this.isomorph = null;
      this.now = 0.0;
      this.spawnCount = 0;
      this.travellers = [];
      this.numTrials = 0;
      this.numCorrect = 0;
      this.numTries = 0;
      if (this.nextDiagram == null) {
        this.nextDiagram = null;
      }
      this.minDifficulty = 1;
      this.trials = [];
      this.spawns = [];
      this.switchMetadata = [];
      this.minDifficulty = 1;
      this.wrongTime = 0;
      this.invincibleMode = false;
      this.onBegin();
      this.minGameLevel = this.minDifficulty;
      this.maxGameLevel = tileTreeMaxLevel;      
    }

    TrainOfThoughtModel.prototype.onBegin = function() {
      this.difficulty = refStartDifficulty(0);
      this.level = this.loadSessionLevel(30);
      this.level = this.sessionLevel;
      this.numStationsUnlocked = Math.floor(this.level / 10);
      this.numStationsSelected = Math.min(14, this.numStationsUnlocked);
    };

    /**
     * Save session in persistent data.
     * Test case:  2015-06-04 Restart.  Expect session level remains.
     */
    TrainOfThoughtModel.prototype.loadSessionLevel = function(defaultLevel) {
      if (undefined === this.persistentData.session_level) {
        this.originalSessionLevel = this.loadFromLastResults(defaultLevel, "level");
        this.persistentData.session_level = this.originalSessionLevel;
      }
      ll.log("loadSessionLevel: " + this.persistentData.session_level);
      this.originalSessionLevel = this.persistentData.session_level;
      this.sessionLevel = this.originalSessionLevel;
      return this.sessionLevel;
    };

    TrainOfThoughtModel.prototype.getNextLevel = function(startLevel, level, num_correct, num_tries) {
        return refNextLevel(startLevel, level, num_correct, num_tries);
    };

    TrainOfThoughtModel.prototype.initSpawn = function() {
      modelSpawnPeriod = refScaleSpawnPeriod(this.level);
      ll.log("TrainOfThoughtModel.initSpawn: " + modelSpawnPeriod + " milliseconds");
      modelMaxSpawn = refMaxTrials(this.level, BIG_INT);
    };

    /**
    	* @returns whether a new level was unlocked
     */

    TrainOfThoughtModel.prototype.completeLevel = function() {
      var newStationsUnlocked;
      this.level = refNextLevel(this.level, this.level, this.numCorrect, this.numTries);
      if(this.level > this.sessionLevel)
      {
          this.sessionLevel = this.level;
      }
      newStationsUnlocked = Math.floor(this.level / 10);
      if (newStationsUnlocked > this.numStationsUnlocked) {
        this.numStationsUnlocked = newStationsUnlocked;
        return true;
      } else {
        return false;
      }

    };

    /**
     * So all teams can find the next session level, save it in the conventional key that the website and database established years ago, called "user_level".
     * Also save "session_level".  The conventional term "session_level" is ambiguous, and so in this program got repurposed by a different programmer. 
     * To clarify, the session level is the level at the start of the session.
     * Test case:  2015-05-29 Zassmin Montes de Oca expects to find user_level key.
     */
    TrainOfThoughtModel.prototype.exportResults = function() {
      var stats;
      stats = {
        trains: this.numCorrect
      };
      this.metadata.user_level = this.sessionLevel;
      this.metadata.session_level = this.originalSessionLevel;
      return TrainOfThoughtModel.__super__.exportResults.call(this, stats, "trains");
    };




      /**
    	* Append metadata into array of objects, called "trials".
    	* @method
    	* @name recordTrial
    	* @param correct - was the trial finished correctly?
    	* @param points - how many points were scored
    	* @param i - the identifier of the traveller for this trial
     */

    TrainOfThoughtModel.prototype.recordTrial = function(correct, points, i, arrivalTime) {
      var destination, metadata;
      destination = this.board.traveling[i];
      metadata = this.board.spawns[i];
      var trialLevel = this.level;
      if(trialLevel < this.sessionLevel)
      {
          trialLevel = this.sessionLevel;
      }
      metadata["level"] = trialLevel;
      metadata["destination"] = destination;
      metadata["destination_switch_count"] = this.board.switchCounts[destination];
      metadata["correct"] = correct;
      metadata["density"] = this.board.getDensity(100);
      metadata["difficulty"] = this.difficulty;
      metadata["points"] = points;
      metadata["variety"] = this.board.maxDestinationCount;
      //metadata["arrival_time"] = arrivalTime;
      this.trials.push(metadata);
      return metadata;
    };


    /**
    	* Progress player level and trash recorded trials
    	* @method
    	* @name endGame
     */

    TrainOfThoughtModel.prototype.endGame = function() {
      if (this.trials === null) {
        return;
      }
      this.userLevel = refNextLevel(this.sessionLevel, this.level, this.numCorrect, this.numTries);
      return this.trials = null;
    };


    /**
    	* Indicate that no more travellers should spawn from the source
    	* (I think this method is deprecated in the original source)
    	* @method
    	* @name stopSpawn
     */

    TrainOfThoughtModel.prototype.stopSpawn = function() {
      return modelMaxSpawn = this.board.spawnCount;
    };

    /**
    	* Update the data model to reflect the player's choice on the level
    	* select dialog.
    	* @method
    	* @name selectNumStations
    	* @param numStations    the index of the selected level. 
        * @param isomorph       If undefined, then select randomly.
        * Test case:  TODO:  Test fit test of level 10, not becoming level 100.
     */

    TrainOfThoughtModel.prototype.selectNumStations = function(numStations, isomorph) {

      if(numStations === 14 && this.level > 140)
      {
         // leave the level at previously store 140+ value
      }
      else {
          this.level = numStations * 10;
      }
      if (isomorph) {
          this.isomorph = isomorph;
      }
      else {
          this.isomorph = selectIsomorph(this.level, this.settings.boards);
          this.isomorph = transpose(vary(this.isomorph));
      }
      this.isBlockingInput = false;
      this.board = new BoardModel(this.isomorph);
      this.board.model = this;
      this.board.pending.push(this.board.totalDestinationCount - 1);
      this.board.begin = 0.0;
      this.board.shufflePending();
      return modelMaxSpawn = BIG_INT;
    };


    /**
    	* Logical size of the board
    	* @method
    	* @name boardSize
    	* @returns a tuple of the number of rows and columns in the currently loaded board.
     */

    TrainOfThoughtModel.prototype.boardSize = function() {
      return [this.board.table.length, this.board.table[0].length];
    };


    /**
    	* Lookup the orientation of the source
    	* @method
    	* @name getSourceOrientation
    	* @returns the direction ID [0,4) of the board's source.
     */

    TrainOfThoughtModel.prototype.getSourceOrientation = function() {
      return modelSources.indexOf(this.board.table[this.board.spawnRow][this.board.spawnColumn]);
    };


    /**
    	* Which image should be used at the given logical coordinate?
    	* Thought decorative, included in the model because it depends on
    	* line-ported model implementation goo.
    	* @method
    	* @name classify
    	* @param row - the row (y-position) of the tile
    	* @param col - the column (x-position) of the tile
    	* @returns the name of the tile, which is mapped to an asset path
     */

    TrainOfThoughtModel.prototype.classify = function(row, col) {
      return doClassify(this.board.table, row, col);
    };


    /**
    	* Retrieve a list of destinations
    	* @method
    	* @name getDestinations
    	* @returns a list of destination-tuples (row, column, id, and direction)
     */

    TrainOfThoughtModel.prototype.getDestinations = function() {
      var c, cell, fromVec, idx, r, result, row, _i, _j, _len, _len1, _ref;
      result = [];
      _ref = this.board.table;
      for (r = _i = 0, _len = _ref.length; _i < _len; r = ++_i) {
        row = _ref[r];
        for (c = _j = 0, _len1 = row.length; _j < _len1; c = ++_j) {
          cell = row[c];
          idx = modelDestinations.indexOf(cell);
          if (idx !== -1) {
            fromVec = outgoing(this.board.table, r, c);
            result.push({
              row: r,
              column: c,
              id: idx,
              fromVec: fromVec
            });
          }
        }
      }
      return result;
    };


    /**
    	* Lookup the direction of the destination at the given address
    	* @name getDestinationDirection
    	* @returns a direction ID [0,4)
     */

    TrainOfThoughtModel.prototype.getDestinationDirection = function(address) {
      return outgoing(this.board.table, Math.floor(address / 10), address % 10);
    };


    /**
    	* Returns normalized-coordiantes for decorations, which are inferred
    	* from map data rather than placed explicitly
    	* @method
    	* @name getDecorations
    	* @returns a list of decoration tuples (row, column, top, flip)
     */

    TrainOfThoughtModel.prototype.getDecorations = function() {
      var address, result, v, _i, _len, _ref;
      result = [];
      _ref = vacantCorners(this.board.diagram);
      for (v = _i = 0, _len = _ref.length; _i < _len; v = ++_i) {
        address = _ref[v];
        if (address !== modelOccupied) {
          result.push({
            row: Math.floor(address / 10),
            column: address % 10,
            top: Math.floor(v / 2) === 0,
            flip: v % 2 !== 0
          });
        }
      }
      return result;
    };


    TrainOfThoughtModel.prototype.getVariance = function() {
        return Math.round(variance(this.board.switchCounts) * 100.0);
    };

    /**
    	* retrieve the current level, monkeypatching with referee if necessary
    	* @method
    	* @name getLevel
    	* @return the current level [0,100]
     */

    TrainOfThoughtModel.prototype.getLevel = function() {
      if (1 <= this.level && this.level < refMinLevel) {
        this.level *= 10;
      }
      return this.level;
    };


    /**
    	* set the difficulty level, monkeypatching with referee if necessary
    	* @method
    	* @name setLevel
    	* @param val the current level
     */

    TrainOfThoughtModel.prototype.setLevel = function(val) {
      var clampedLevel;
      if (1 <= val && val < refMinLevel) {
        val *= 10;
      }
      this.level = val;
      return clampedLevel = Math.max(minDestinationCount, Math.floor(this.level / 10));
    };


    /**
    	* At 0 seconds stops spawning.  
    	* When doing well, expand which destinations travellers may be spawned for.
    	* Instead of absolute sawtooth over time, offset sawtooths over time since last mistake.
    	* @method
    	* @name update
    	* @param   difficulty  if -1, get from this. Does not use NaN, because that evaluated to 0.  Synonymous with "difficulty".  May rename to difficulty if no longer multiplying.
    	* @return  density.
     */

    TrainOfThoughtModel.prototype.update = function(countdownTime) {
      var stopSpawnTime;
      stopSpawnTime = 0;
      if (countdownTime < stopSpawnTime) {
        this.stopSpawn();
      }
      this.seeCorrectProgressesDifficulty(this.board.seeCorrects, countdownTime);
      this.difficulty = Math.max(this.minDifficulty, this.difficulty);
      this.board.density = refDifficultyDensity(this.difficulty, this.board.totalDestinationCount, countdownTime);
      this.board.setTravellerDistribution(this.board.density);
      this.board.regulateDensity();
      return this.board.setDestinationFraction(refDifficultyVariety(this.difficulty));
    };


    /** 
    	* Update difficulty based on the number of travellers that are on the 
    	* "right course" to their destination
    	* @private
    	* @method
    	* @name setCorrectProgressesDifficulty
    	* @param seeCorrects - database of traveller courses, from impl
    	* @param countdownTIme - time left in level
     */

    TrainOfThoughtModel.prototype.seeCorrectProgressesDifficulty = function(seeCorrects, countdownTime) {
      var s, sc, _i, _len;
      for (s = _i = 0, _len = seeCorrects.length; _i < _len; s = ++_i) {
        sc = seeCorrects[s];
        if (1 === sc) {
          this.difficulty = Math.min(this.difficulty + 1, refMaxDifficulty);
          seeCorrects[s] += 1;
        } else if (-1 === sc) {
          this.wrongTime = this.time - countdownTime;
          this.difficulty = Math.max(this.difficulty - 4, this.minDifficulty);
          seeCorrects[s] -= 1;
        }
      }
      return this.difficulty;
    };


    /**
    	* Check if it's time to spawn a new traveller.
    	* @method
    	* @name checkSpawn
    	* @return new traveller, if spawned, else null
     */

    TrainOfThoughtModel.prototype.checkSpawn = function(currentTimeOffset) {
      var index, result;
      if (this.board.travelTime + modelSpawnPeriod <= this.now || this.board.travelTime <= -1) {
        this.board.travelTime = this.now;
        index = this.board.spawn(this.board.spawnCount <= 0 ? modelRestIndex : WEE_INT, currentTimeOffset);
        if (index >= 0) {
          result = new TravellerModel(this.board, this.travellers.length, this.now, this.spawnCount);
          this.spawnCount = this.spawnCount + 1;
          this.travellers.push(result);
          return result;
        }
      }
      return null;
    };


    /**
    	* Toggle the switch at the given address, unless a 
    	* traveller is blocking this interaction.
    	* @method
    	* @name checkToggle
    	* @param address - the address (10*r+c) of the toggle switch
    	* @return whether the toggle was successfully committed.  If input or address is blocked, return false.
        * If this.onToggleDelegate defined, call its method "onToggle".
     */

    TrainOfThoughtModel.prototype.checkToggle = function(address, timeOffset, toggled) {
        if (this.isBlockingInput 
        || (this.isBlockingInputExceptAddress && address != this.isBlockingInputExceptAddress)) {
          ll.log("TrainOfThoughtModel.checkToggle: blocked. allowed address " +  this.isBlockingInputExceptAddress);
          return false;
        }
        // this straightorcurved logic is insane
        // because of the way things are set up
        var c, r;
        r = Math.floor(address / 10);
        c = address % 10;

        var swName = doClassify(this.board.table, r, c);
        var idx = switchNames.indexOf(swName)
        var straightOrCurved = 0;
        if(idx % 2 === 0) {
            straightOrCurved = 1;
            if (toggled) {
                straightOrCurved = 0;
            }
        }
        else
        {
            straightOrCurved = 0;
            if (toggled) {
                straightOrCurved = 1;
            }
        }
        // end insane logic

        var togVal = this.board.toggle(address);

        /*
        address - ID of switch
        time_offset - Time after beginning of game that switch is clicked, in milliseconds
        state - Straight or curved? [0/1]
        active_trains - Number of active trains at time of click
        */
        this.switchMetadata.push({
            "address" : address,
            "time_offset": timeOffset,
            "state" : straightOrCurved,
            "active_trains": this.travellers.length
        });
        if (this.onToggleDelegate) {
          this.onToggleDelegate.onToggle();
        }
      return true;
    };


    /**
     * GAME-845 GAME-2160 [~aaron] In fit test, upon arrival multiply points by maximum destinations currently spawning.  This probably doubles the discrimination of median scores, because frequent mistakes will diminish the point value of subsequent arrivals, especially in the "less_variety" split test group.  Test case:  2014-05-09 Expect resolution of median scores to be more discrete than 10 percentiles.  
     */
    TrainOfThoughtModel.prototype.point = function() {
        var destinationCount = LumosityUtils.isFitTestModeOn() 
           ? this.board.maxDestinationCount 
           : this.board.totalDestinationCount;
        var points = refBasePoints * destinationCount;
        // ll.log("TrainOfThought.processArrival: points " + points);
        return points;
    };

    /**
    	* Process an arriving traveller, updating score
    	* depending on whether the destination was correct 	
    	* or not.
    	* @method
    	* @name processArrival
    	* @param traveller - a travellerModel that's currently active
     */

    TrainOfThoughtModel.prototype.processArrival = function(traveller, arrivalTime) {
      this.numTries += 1;
      if (traveller.arrivalStatus === 1 || LumosityUtils.isDebugModeOn() || this.invincibleMode === true) {
        this.numCorrect += 1;
        this.score += this.point();
        return this.recordTrial("T", refBasePoints, traveller.index, arrivalTime);
      } else {
        if (this.onAnswerWrongDelegate) {
            this.onAnswerWrongDelegate.onAnswerWrong();
        }
        return this.recordTrial("F", 0, traveller.index, arrivalTime);
      }
    };


    /**
    	* Dispose of the given traveller, and downshift
    	* the impl indices for all the travellers after
    	* this one
    	* @method
    	* @name disposeOf
    	* @param traveller - a travellerModel that's currently arriving
     */

    TrainOfThoughtModel.prototype.disposeOf = function(traveller) {
      var t, _i, _len, _ref;
      _ref = this.travellers;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        t = _ref[_i];
        t.willRemoveTraveller(traveller.index);
      }
      this.board.splice(traveller.index);
      return this.travellers.splice(traveller.index, 1);
    };

    TrainOfThoughtModel.prototype.estimateMaxTrials = function() {
      return refEstimateMaxTrials(this.level);
    };

    return TrainOfThoughtModel;

  })(this.GameModel);

}).call(this);

