// Generated by CoffeeScript 1.7.1
(function() {
  var FRAME_COUNT, FlipbookView, SwitchView, TravellerView, cc, fromLabels, fromNames, idxToFrame, modHelper, offsetPoint, sourceNames, switchNames, switchTimeout, toLabels, topLeft, tt, zero,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  this.require("src/Game/TrainOfThoughtUtil.js");

  this.require("src/Game/Base/GameView.js");

  this.require("src/Game/Widget/GameWidgetFeedback.js");

  this.require("src/Game/Widget/GameWidgetHud.js");

  cc = this.cc;

  tt = this.TrainOfThoughtUtil;

  topLeft = cc.p(0, 1);

  zero = cc.p(0, 0);


  /**
  * helper method for updating a cc.Point inline without allocating
  * a new instance.
  * @function
  * @name offsetPoint
  * @param {cc.Point} p
  * @param {Number} x
  * @param {Number} y
   */

  offsetPoint = function(p, x, y) {
    p.x += x;
    p.y -= y;
    return p;
  };


  /**
  * The number of frames that were exported from blender
  * @constant
   */

  FRAME_COUNT = 84;


  /**
  * Names of switch assets, sorted in ID-order
  * @constant
   */

  switchNames = ["EastNorth", "EastWest", "NorthSouth", "NorthWest", "SouthEast", "SouthNorth", "WestEast", "WestNorth"];


  /**
  * Names of source assets, sorted in ID-order
  * @constant
   */

  sourceNames = ["SourceEast", "SourceSouth", "SourceEast", "SourceNorth"];


  /**
  * Names of directions, used as suffixes, in ID-order
  * @constant
   */

  fromNames = ["East", "South", "West", "North"];


  /**
  * Map an index to a blender frame-name suffix with
  * padded zeroes
  * @function
   */

  idxToFrame = function(i) {
    if (i < 10) {
      return "000" + i;
    } else if (i < 100) {
      return "00" + i;
    } else if (i < 1000) {
      return "0" + i;
    } else {
      return "" + i;
    }
  };


  /**
   * The Train of Thought view
   * @class
   * @name TrainOfThoughtView
   */

  this.TrainOfThoughtView = (function(_super) {
    __extends(TrainOfThoughtView, _super);


    /**
    	 * The Train of Thought view.
    	 * @cnostructor
    	 * @param {TrainOfThought} game This view's game. Note that this is temporary to have this passed in!
    	 * @param {cc.Size} screenSize The size of the screen. Use it to create stuff intelligently.
     */

    function TrainOfThoughtView(screenSize, settings, model, interactionResponder, preload) {
      this.screenSize = screenSize;
      this.settings = settings;
      this.model = model;
      TrainOfThoughtView.__super__.constructor.call(this, this.screenSize, this.settings, this.model, interactionResponder, preload);
      this.flipbookCount = 0;
      this.touchCount = 0;
      this.pivots = this.settings.pivots;
      this.tw = 0.995 * this.settings.tileWidth;
      this.height = this.screenSize.height;

        var hudWidget = this.getHudWidget();
        if ( hudWidget )
        {
            var hudScoreText = "0" + localize( "Of_Separator", "of" ) + "0";
            hudWidget.addCustomComponent( "CORRECT", GameWidgetHudElementType.HudElementLabelValue, localize( "HUD_ScoreText", "CORRECT" ), hudScoreText );
            this.correctAdded = true;
            hudWidget.setCustomComponentValue( "CORRECT", hudScoreText, true );
            hudWidget.setComponentOrder ( "CORRECT", 0 );
        }
        this.preloadArrivalEffects();
    };

    TrainOfThoughtView.prototype.createAnimFrame = function (path, w, h)
    {
        var animFrame, frame, tex;
        tex = cc.TextureCache.getInstance().addImage(path);
        frame = cc.SpriteFrame.createWithTexture(tex, cc.rect(0, 0, w, h));
        animFrame = new cc.AnimationFrame();
        animFrame.initWithSpriteFrame(frame, 1, null);
        return animFrame;
     };

    /**
    	* Release all the assets "retain()ed" by the view
    	* @method
    	* @name dispose
     */

    TrainOfThoughtView.prototype.dispose = function() {
      ll.log("TrainOfThoughtView.dispose");
      var shader, switcher, tv, _i, _j, _k, _len, _len1, _len2, _ref, _ref1, _ref2, _results;
      _ref = this.shaders;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        shader = _ref[_i];
        if (shader)
          shader.release();
      }
      _ref1 = this.switches;
      for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
        switcher = _ref1[_j];
        if (switcher)
          switcher.releaseSprites();
      }
      _ref2 = this.travellers;
      _results = [];
      for (_k = 0, _len2 = _ref2.length; _k < _len2; _k++) {
        tv = _ref2[_k];
        if (tv)
          _results.push(tv.dispose);
      }
      if (this.boardView && this.boardView.getChildrenCount && 1 <= this.boardView.getChildrenCount())
        this.boardView.removeAllChildren();
      if (this.trialRoot && this.trialRoot.getChildrenCount && 1 <= this.trialRoot.getChildrenCount())
        this.trialRoot.removeAllChildren();
      return _results;
    };


    /**
    	* helper function to create a sprite with the anchor set
    	* explicitly
    	* @method
    	* @name createSprite
    	* @param {string} name - filename, without extension
     */

    TrainOfThoughtView.prototype.createSprite = function(name) {
      var anchorPoint, piv, result, sz;
      result = cc.Sprite.create("" + name + ".png");
      piv = this.pivots[name];
      if (piv !== void 0) {
        sz = result.getContentSize();
        anchorPoint = cc.p(piv[0] / sz.width, 1 - (piv[1] / sz.height));
        result.setAnchorPoint(anchorPoint);
      } else {
        result.setAnchorPoint(topLeft);
      }
      return result;
    };

    TrainOfThoughtView.prototype.anyTouches = function() {
      return this.touchCount > 0;
    };

    /**
     * Estimate trains from a public model function.  Test case:  2015-03-05 Short game.  Already above level 14.  Unlock next level.  Expect to see unlock more trains.  Got crash.
     *
     * GAME-2835 Program replaces "[unlocked suffix]" with emptiness.  I deleted the solo pilcrow from the English suffix, "UnlockLevel_Line3".  Now the text is:  "[unlocked suffix]".  Text was "¶".  Logcat that Rob Ivey sent has backtrace with "CCLabelTTF::setString".  Is this Cocos2d-x error related to the bug 624 listed in the Cocos2d-x changelog?  'CCLabelTTF::setString("\n\n\n") crashes on android'  http://www.cocos2d-x.org/issues/624   I updated and published translations in Smartling.com.  Smartling.com expects a string that has non-whitespace to translate and expects the string does not begin and end with XML tag markers, such as  "<unlock suffix>".  It is more clear to replace the unintelligible English "¶" with a precise identifier that translates into German.  The Test case: 2015-03-03 Android. Select Train of Thought. Rob Ivey expects to see train tracks enter. Got application crashes. iOS has no problem.  Android:  Select language Japanese (which also had text "¶").  Start.  Expect can start.  Got crash.  In Japanese, level  up.  Expect no text.  Android:  Select language German.  Level up.  Expect text translated.
     *
     * GAME-3648 When unlocking more trains, set the level up suffix to be blank. Test case:  2015-06-14  Open live version of iOS or Android app (or any version since live German translation of Train of Thought).  Set short games mode and debug mode to test quicker.  Start Train of Thought at Level 14.  Play and make 1 or fewer mistakes total.  See how many trains next time.  Samantha Yang expects to not read after that text "[unlocked suffix]".
     */
    TrainOfThoughtView.prototype._loadLayout = function() {
        var levelPrefix;

        // load train frames
        var cache, frame, i, tw, _i;
        cache = cc.SpriteFrameCache.getInstance();
        tw = this.settings.tileWidth;
        for (i = _i = 1; 1 <= FRAME_COUNT ? _i <= FRAME_COUNT : _i >= FRAME_COUNT; i = 1 <= FRAME_COUNT ? ++_i : --_i) {
            frame = cc.SpriteFrame.create("TrainFrames/TrainFrame" + (idxToFrame(i)) + ".png", cc.rect(0, 0, tw, tw));
            cache.addSpriteFrame(frame, "Frame" + (i - 1));
        }

        this.selectedButton = null;
        this.nameToButton = {};


        TrainOfThoughtView.__super__._loadLayout.call(this);
        levelPrefix = localize("UnlockLevelPrefix", "LEVEL");
        // old Max code, before Torri's anim changes
        //this.newLevelSubtitle.setString("" + levelPrefix + " " + (this.model.numStationsUnlocked + 1));
        //this.header.runAction(cc.Hide.create());
        // return this.levelSelectText.setOpacity(0.75 * 255);

        // New Shalin code for Torri's anims and artwork
        // load this and store away the station details
        LayoutLoader.loadLayoutFile("stationLayout1.json", null, this, this);
        // Then load this and create the appropriate buttons
        LayoutLoader.loadLayoutFile("menuLayout.json", this.menuContainer, this, this);
        // undo the scale and offset from parenting underneath menuContainer
        this.menu.setScaleX(1 / this.menuContainer.getScaleX());
        this.menu.setScaleY(1 / this.menuContainer.getScaleY());
        var halfW = this.menu.getContentSize().width * 0.5;
        var halfH = this.menu.getContentSize().height * 0.5;
        //ll.verbose("HALF " + halfW + " " + halfH);
        this.menu.setPosition(cc.p(halfW, halfH));

        // Load the level unlocked and replace the text with current level count + 1
        LayoutLoader.loadLayoutFile("levelUnlockedLayout.json", this, this, this);
        if (this.model.numStationsUnlocked >= 14)
        {
            var nextTime = localize("UnlockAllLevels1", "Next time you may manage" );
            var trainsSuffix = localize("UnlockAllLevels2", "TRAINS");
            var numTrains = Math.floor(this.model.estimateMaxTrials());
            this.titleLabel.setString("" + nextTime);
            this.levelLabel.setString("" + numTrains + " " + trainsSuffix);
            this.levelSuffixLabel.setString( " " );
        }
        else {
            var unlockedLevels = this.model.numStationsUnlocked + 1;
            this.levelLabel.setString("" + levelPrefix + " " + unlockedLevels);

            if (lumosity.game.locale === "de") {
              this.levelSuffixLabel.setString( localize( "UnlockLevel_Line3", " " ).replace("[unlocked suffix]", " ") );
            } else {
              this.levelSuffixLabel.setString(" ");
            }
        }

        // If you have a better way to do this? Please tell me :)
        // I need to toggle these from the controller on button press
        this.stationOutlines = [this.stationSelect_3, this.stationSelect_4, this.stationSelect_5,
                                this.stationSelect_6, this.stationSelect_7, this.stationSelect_8,
                                this.stationSelect_9, this.stationSelect_10, this.stationSelect_11,
                                this.stationSelect_12, this.stationSelect_13, this.stationSelect_14];

        for(var i = 0; i < this.stationOutlines.length; i++)
        {
            // uncomment this if a station should be shown selected on startup
            var fadeIn = cc.FadeIn.create(1);
            var fadeOut = cc.FadeOut.create(1);
            var pulseSeq = cc.Sequence.create(fadeIn, fadeOut);
            var pulse = cc.RepeatForever.create(pulseSeq);
            this.stationOutlines[i].runAction(pulse);

            if (i + 3 === this.model.numStationsSelected)
            {

                continue;
            }
           this.stationOutlines[i].setVisible(false);
        }

        this.levelUnlocked.setVisible(false);
        this._localizeAnimatedText();
        //cc.TextureCache.getInstance().dumpCachedTextureInfo();
        //cc.TextureCache.getInstance().removeUnusedTextures();
        //cc.Director.getInstance().purgeCachedData();

    };

    /**
     * GAME-2159 After loading loayout, translate level select text and correct HUD "of" separator.  Test case:  2015-02-24 Select German.  Start Train of Thought.  Read level select text.  [~mtorii] expects German.  Got English.  Select level.  Look at HUD.  Expect translation of "of" into German.  Got English.
     */
    TrainOfThoughtView.prototype._localizeAnimatedText = function() {
        this.levelSelectTextBottom.setString(localize("LevelSelect_Footer"));
        this.levelSelectTextTop.setString(localize("LevelSelect_Header"));
        this.showScore(0, 0);
    };

    TrainOfThoughtView.prototype.layout_readingData = function(name, data) {
      switch (name) {
        case "btnNext":
          this.btnFace = data.textData.face;
          this.btnSize = data.textData.size;
          return this.btnColor = cc.c4b(data.textData.color.Red, data.textData.color.Green, data.textData.color.Blue, data.textData.color.Alpha);
          break;
        case "blackNumber":
          this.stationButtonTextFace = LumosityUtils.getFontLocation(data.textData.face);
          this.stationButtonTextSize = data.textData.size;
          this.stationButtonTextColor = cc.c4b(255, 0, 0, data.textData.color.Alpha);
          break;
        default:
          break;
      }
    };

    TrainOfThoughtView.prototype.layout_creatingNode = function(name, data) {

      var result;
      result = TrainOfThoughtView.__super__.layout_creatingNode.call(this, name, data);
      if (result !== null) {
        return result;
      }
      //tt.log("Creating Node: " + name);
      this.layout_readingData(name, data);

      if (name.substring(0, "station_".length) === "station_") {
          // replace the image, add a pip and level text
          // also add selection states
          return this.createLevelButton(name, data);
      }

      var unlockedStations = this.model.numStationsUnlocked;
      if(unlockedStations > 14)
      {
          unlockedStations  = 14; // for loading train image
      }
      switch (name) {
        //case "next":
          //return this._createButton("buttons/game_buttonNextUp.png", "buttons/game_buttonNextDown.png", "btnNext", localize("nextButtonLabel", "NEXT"), this.btnFace, this.btnSize, this.btnColor, cc.c4b(255, 255, 255, 255), data.size);
        case "station": // new
            return this._createSprite("" + (this.settings.getDestinationName(unlockedStations)) + "_Big.png");
        case "newLevelIcon":  // old
          return this._createSprite("" + (this.settings.getDestinationName(unlockedStations)) + "_Big.png");
        case "newLevelAccent":
          return this._createSprite('levelUpAccent.png');
        case "decorations1":
          return this._createSprite('miscTrees1.png');
        case "decorations2":
          return this._createSprite('miscTrees2.png');
        case "decorations3":
          return this._createSprite('miscTrees3.png');
        case "decorations4":
          return this._createSprite('miscTrees1.png');
        default:
          return null;
      }
    };


    TrainOfThoughtView.prototype.createLevelButton = function(name, data) {
      var btn, i, idx;
      i = parseInt(name.substring("station_".length));
      var imgI = i - 1; // to index the station images
      idx = "" + i; // to index button clicks for gameplay

      var szSprite; // a sprite to use for sizing

      if (i > this.model.numStationsUnlocked  || i < 3) {

         szSprite =  cc.Sprite.create("Destination10_duo_East.png");
         var tColor = cc.c4b(0,0,0,255);
         if(i > this.model.numStationsUnlocked)
         {
             tColor = cc.c4b(52,107,67,255);
         }
         btn = this._createButton("DestinationEmpty.png", "DestinationEmpty.png", "lockedBtn", idx, this.stationButtonTextFace, this.stationButtonTextSize, tColor, tColor, szSprite.getContentSize());
      }
      else {
          var stationImageName = this.settings.getDestinationName(imgI) + "_East.png";
          szSprite =   cc.Sprite.create(stationImageName);
          btn = this._createButton(stationImageName, stationImageName, name, idx, this.stationButtonTextFace, this.stationButtonTextSize, cc.c4b(0, 0, 0, 255), cc.c4b(0, 0, 0, 255), szSprite.getContentSize());
      }

      var pipSprite;
      // if i < 3 white pip else black pip
      if (i > this.model.numStationsUnlocked)
      {
          pipSprite = this._createSprite("blackPip.png");
      }
      else
      {
          pipSprite = this._createSprite("stationPip.png");
      }

      // position the pip by hand
      var scaleFactor = 1;
      var frameWidth = cc.Director.getInstance().getWinSize().width;
      if( frameWidth > 1200 )
      {
          scaleFactor = ( 1600 / 640 );
      }
      else if( frameWidth > 800 )
      {
          scaleFactor = ( 1200 / 640 );
      }
      else if( frameWidth > 640 )
      {
          scaleFactor = ( 800 / 640 );
      }
      else if( frameWidth > 400)
      {
          scaleFactor = ( 640 / 640);
      }
      else
      {
         scaleFactor = ( 400 / 640 );
      }
      var ctPt = cc.p( 47.5 * this.centeredContent.getContentSize().width/640, 40 * this.centeredContent.getContentSize().height/960);
      btn.titleLabel.setPosition(ctPt)
      btn.addChild(pipSprite,0);
      pipSprite.setPosition(ctPt);
      pipSprite.setScaleX(1 / scaleFactor);
      pipSprite.setScaleY(1 / scaleFactor);

      this.nameToButton[name] = btn;
      btn.numStations = i;//this.model.numStationsUnlocked;

      if (i === this.model.numStationsSelected) {
        this.selectedButton = btn;
      }
      return btn;
    };


    /**
     * GAME-2159 At end of tutorial or restart, remove any previous board.  Unfortunately, Cocos2d-x v2.2.2 crashes if removing children when new board is to be created.  Test case:  2015-02-25 Tutor.  Play.  Expect one board.  Got another board behind it.  
     *
     * GAME-2160 At restart, hide previous board children.  Does not remove children:  If remove, then cocos2d-x crashes with log "invalid native object".  Test case:  2015-03-04 Fit test.  Short game.  Train of Thought.  After tutorial.  Play.  3-2-1 countdown.  Pause.  Restart.  [~wouter] expects 3-stations enter.  Got 3-stations and 7-stations overlay enters.  
     */
    TrainOfThoughtView.prototype.removeBoard = function() {
      var boardView = this.boardView;
      if (boardView && boardView instanceof cc.Node 
      && 1 <= boardView.getChildrenCount()) 
      {
        ll.log("TrainOfThoughtView.removeBoard: Hiding children of board view.");
        var children = boardView.getChildren();
        for (var c = 0; c < children; c++)
        {
          children[c].setVisible(false);
        }
        this.boardView.release();
        this.boardView = undefined;
      }
    };

    /**
   	 * Method for the root controller to populate the display tree with the 
     * dynamic game elements, after the level is selected.
     * @method
     * @name showBoard
     */
    TrainOfThoughtView.prototype.showBoard = function() {
      ll.log("TrainOfThoughtView.showBoard");
      this.boardView = cc.Node.create();
      this.trialRoot.addChild(this.boardView);
      var backgroundLayer, col, corner, dest, halfTileWidth, i, innerHeight, innerWidth, logicalWidth, name, numCols, numDestinations, numRows, orientation, paddingX, paddingY, physicalWidth, pos, row, scale, shader, size, sprite, switchView, tree, _i, _j, _k, _l, _len, _len1, _m, _ref, _ref1, _ref2, _ref3, _ref4;
      numDestinations = this.model.board.totalDestinationCount;
      this.shaders = [];
      for (i = _i = 0, _ref = numDestinations - 1; 0 <= _ref ? _i <= _ref : _i >= _ref; i = 0 <= _ref ? ++_i : --_i) {
        shader = cc.GLProgram.create('src/Shaders/traveller.vsh', "src/Shaders/traveller" + i + ".fsh");
        shader.retain();
        shader.addAttribute(cc.ATTRIBUTE_NAME_POSITION, cc.VERTEX_ATTRIB_POSITION);
        shader.addAttribute(cc.ATTRIBUTE_NAME_COLOR, cc.VERTEX_ATTRIB_COLOR);
        shader.addAttribute(cc.ATTRIBUTE_NAME_TEX_COORD, cc.VERTEX_ATTRIB_TEX_COORDS);
        shader.link();
        shader.updateUniforms();
        this.shaders.push(shader);
      }
      this.travellers = [];
      _ref1 = this.model.boardSize(), numRows = _ref1[0], numCols = _ref1[1];
      paddingX = 0.5 * (this.screenSize.width - this.tw * numCols);
      paddingY = 0.5 * (this.screenSize.height - this.tw * numRows);
      tt.log("TILE WIDTH : " + this.tw);
      // this tree was totally random and overlapping stuff
      // so I got rid of it - Shalin
      //tree = this._createSprite('miscTrees1.png');
      //this.trialRoot.addChild(tree);
      size = this.trialRoot.getContentSize();
      logicalWidth = 5 * this.tw;
      physicalWidth = this.trialRoot.getContentSize().width;


      var offset = 0;
      //ll.verbose("SOURCE " + this.model.board.spawnRow);
      if(this.model.board.spawnRow < 2)
      {
          // top source
          // solve HUD overlap
          // only an issue in full size grids (7 rows)
          if(numRows > 6) {
              //ll.verbose("TOP OFFSET");
              offset = -size.height * 0.030;
          }
    }
      else
      {
          // bottom source
          // solve centering problem
          // only an issue in smaller grids (< 7 rows)
          if(numRows < 7)
          {
              //ll.verbose("BOTTOM OFFSET");
              offset = size.height * 0.06;
          }

      }

      scale = physicalWidth / logicalWidth;
      this.boardView.setScale(scale, scale);
      innerWidth = (numCols - 1) * this.tw * scale;
      innerHeight = (numRows - 1) * this.tw * scale;
      this.boardView.setPosition(cc.p(0.5 * (size.width - innerWidth), size.height - 0.5 * (size.height - innerHeight) + offset));
      _ref2 = this.model.getDecorations();
      for (_j = 0, _len = _ref2.length; _j < _len; _j++) {
        corner = _ref2[_j];
        //ll.verbose("CORNER TOP " + corner.top + " FLIP " + corner.fip + " X " + corner.row + " Y " + corner.column  );
        name = corner.top ? "DecorationTop" : "DecorationBottom";
        sprite = this.createSprite(name);
        sprite.setPosition(this.tilePoint(corner.row, corner.column));
        this.boardView.addChild(sprite);
      }
      // this bottom decorator was totally random
      // and overlapping stuff, so I got rid of it - Shalin
      //sprite = this.createSprite("DecorationBottom");
      //sprite.setPosition(this.tilePoint(numRows, numCols));
      //this.boardView.addChild(sprite);
      this.switches = [];
      backgroundLayer = cc.Node.create();
      this.boardView.addChild(backgroundLayer);
      halfTileWidth = this.settings.tileWidth;
      for (row = _k = 0, _ref3 = numRows - 1; 0 <= _ref3 ? _k <= _ref3 : _k >= _ref3; row = 0 <= _ref3 ? ++_k : --_k) {
        for (col = _l = 0, _ref4 = numCols - 1; 0 <= _ref4 ? _l <= _ref4 : _l >= _ref4; col = 0 <= _ref4 ? ++_l : --_l) {
          name = this.model.classify(row, col);
          if (name) {
            pos = this.tilePoint(row, col);
            if (name.indexOf('Switch') === 0) {
              switchView = new SwitchView(this, name, 10 * row + col);
              switchView.setPosition(pos);
              this.switches.push(switchView);
              backgroundLayer.addChild(switchView);
            } else {
              sprite = this.createSprite(name);
              sprite.setPosition(pos);
              backgroundLayer.addChild(sprite);
            }
          }
        }
      }
      this.travellerContainer = cc.Node.create();
      this.boardView.addChild(this.travellerContainer);
      orientation = this.model.getSourceOrientation();
      sprite = this.createSprite(sourceNames[orientation]);
      sprite.setPosition(this.tilePoint(this.model.board.spawnRow, this.model.board.spawnColumn));
      if (orientation === 2) {
        sprite.setScaleX(-1);
      }
      this.boardView.addChild(sprite);
      return this.addDestinations(this.model.getDestinations());
    };

    /**
     * GAME-2949 To reduce texture memory about 1%, flip east destination to reuse as the west destination.  On iPad this saves about 2 MB from a total of about 270 MB of texture memory.  iPad 2 and iPad Mini only has 512 MB RAM, so reducing 30 MB slightly reduces the probability of running out of memory.  Test case:  2015-01-12 on iPad, subscriber expects to load Train of Thought.  Got app crashes.  Shalin suspects app is out of memory.
     */
    TrainOfThoughtView.prototype.addDestinations = function(destinations) {
      this.addressToDestination = {};
      var _results = [];
      for (_m = 0, _len1 = destinations.length; _m < _len1; _m++) {
        dest = destinations[_m];
        var scaleX = 1.0;
        var dir = dest.fromVec;
        if (2 == dir) {
          scaleX = -1.0;
          dir = 0;
        }
        name = "" + (this.settings.getDestinationName(dest.id)) + "_" + fromNames[dir];
        sprite = this.createSprite(name);
        sprite.setPosition(this.tilePoint(dest.row, dest.column));
        sprite.setScaleX(scaleX);
        this.boardView.addChild(sprite);
        _results.push(this.addressToDestination[10 * dest.row + dest.column] = sprite);
      }
      return _results;
    };

    /**
    	* Show the given score in the hud
    	* @method
    	* @name showScore
    	* @param {Number} curr - trials completed successfully
    	* @param {Number} total - number of trials initiated in total
     */

    TrainOfThoughtView.prototype.showScore = function(curr, total) {
      var ofSep;
      ofSep = localize("Of_Separator", " of ");
      var hudWidget = this.getHudWidget();
      if ( hudWidget )
      {
          var autoResize = false;
          // 100 is for that one crazy person who might get to 100 trains
          if(curr === 10 || curr === 100)
          {
              autoResize = true;
          }
          hudWidget.setCustomComponentValue( "CORRECT", "" + curr + ofSep + total, autoResize );
      }
      //return this.setScore("" + curr + ofSep + total);
    };


    /**
    	* Map a logical row/col pair to a screen position relative to the boardView node
    	* @method
    	* @name tilePoint
    	* @param {Number} row
    	* @param {Number} column
     */

    TrainOfThoughtView.prototype.tilePoint = function(row, column) {
      return cc.p(this.tw * column, -this.tw * row);
    };


    /**
    	* Toggle the switch view
    	* @method
    	* @name toggle
    	* @param {SwitchView} switchView
     */

    TrainOfThoughtView.prototype.toggle = function(switchView) {
      var t, _i, _len, _ref, _results;
      switchView.toggle();
      _ref = this.travellers;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        t = _ref[_i];
        if (t.traveller.address === switchView.address && !t.traveller.committed) {
          _results.push(t.updateLabel());
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };


    /**
    	* Method to create a new traveller view from the given model
    	* @method
    	* @name createTravellerView
    	* @param {TravellerModel} traveller
     */

    TrainOfThoughtView.prototype.createTravellerView = function(traveller) {
      var result;
      result = new TravellerView(this, traveller);
      this.travellers.push(result);
      result.retain();
      this.travellerContainer.addChild(result);
      return result;
    };

    /**
     * preload smoke anims.  
     *
     * GAME-2949 Do not load "East".
     */
    TrainOfThoughtView.prototype.preloadArrivalEffects = function() {
        for(var d =1; d < 4; d++)
        {
            var name ="Puff" + fromNames[d];
            var ref = this.settings.animBounds[name];
            var n = ref[2];
            frames = (function() {
                var _i, _ref2, _results;
                _results = [];
                for (i = _i = 0, _ref2 = n - 1; 0 <= _ref2 ? _i <= _ref2 : _i >= _ref2; i = 0 <= _ref2 ? ++_i : --_i) {
                    _results.push(this.createAnimFrame("" + name + "/" + name + "_" + i + ".png", ref[0], ref[1]));
                }
                return _results;
            }).call(this);
            anim = cc.Animation.create(frames, 0.033, 1);
            cc.AnimationCache.getInstance().addAnimation(anim, name);
        }
    };

    /**
     * Show a little puff of smoke animation for this traveller.
   	 * @param {TravellerModel} traveller
     *
     * GAME-2949 To reduce texture memory about 10%, flip west train arrival puff animation to reuse as the east train arrival puff animation.  On iPad this saves about 30 MB from a total of about 270 MB of texture memory.  iPad 2 and iPad Mini only has 512 MB RAM, so reducing 30 MB slightly reduces the probability of running out of memory.  Test case:  2015-01-12 on iPad, subscriber expects to load Train of Thought.  Got app crashes.  Shalin suspects app is out of memory.
     */
    TrainOfThoughtView.prototype.showArrivalEffectsFor = function(traveller) {
      var addr, dir, puff;
      if (traveller.arrivalStatus === 0) {
        return;
      }
      addr = traveller.address;
      if (!(addr in this.addressToDestination)) {
        return;
      }
      dir = this.model.getDestinationDirection(addr);
      var scaleX = 1.0;
      if (0 == dir) {
        scaleX = -1.0;
        dir = 2;
      }
      puff = new FlipbookView(this, "Puff" + fromNames[dir]);
      puff.setPosition(this.tilePoint(Math.floor(addr / 10), addr % 10));
      puff.setScaleX(scaleX);
      return this.boardView.addChild(puff);
    };


    /**
    	* Update animations that interpolate between
    	* waypoint periods
    	* @method
    	* @name update
    	* @param {Number} now - current game time in milliseconds
    	* @param {Number} dt - delta time (in seconds)
     */

    TrainOfThoughtView.prototype.update = function(dt) {
      var switcher, traveller, _i, _j, _len, _len1, _ref, _ref1, _results;
      _ref = this.switches;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        switcher = _ref[_i];
        switcher.update(dt);
      }
      _ref1 = this.travellers;
      _results = [];
      for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
        traveller = _ref1[_j];
        _results.push(traveller.update());
      }
      return _results;
    };


    /**
    	* dispose of the traveller and replace it with a feedback widget
    	* @method
    	* @name disposeOf
    	* @param {TravellerView} traveller
     */

    TrainOfThoughtView.prototype.disposeOf = function(traveller) {
      var worldPos;
      if (traveller.traveller.arrivalStatus !== 1) {
        this.showFeedback(false, 0, false);
      }
      worldPos = this.boardView.convertToWorldSpace;
        this.feedback= this.getFeedbackWidget();
      this.feedback.setPosition(LumosityUtils.convertPointSpace(traveller.getPosition(), traveller.getParent(), this.feedback.getParent()));
      this.travellers.remove(traveller);
      this.travellerContainer.removeChild(traveller);
      traveller.dispose();
      return traveller.release();
    };

    return TrainOfThoughtView;

  })(this.GameView);


  /**
  * List of from-prefixes, in ID-order
  * @constant
   */

  fromLabels = ["east", "south", "east", "north"];


  /**
  * List of to-suffices, in ID-order
  * @constant
   */

  toLabels = ["East", "South", "East", "North"];


  /**
  * Helper to do mod-math without worrying about negative results
  * @function
   */

  modHelper = function(dividend, divisor) {
    var remainder;
    remainder = dividend % divisor;
    if (remainder < 0) {
      remainder += divisor;
    }
    return remainder;
  };


  /**
  * Subview specifically for handling Travellers (the little trains)
  * @name TravellerView
  * @class
   */

  TravellerView = cc.Node.extend({

    /**
    	* Constructor
    	* @param parent - the root train of thought view
    	* @param traveller - submodel for the logical traveller
     */
    ctor: function(parent, traveller) {
      this.parent = parent;
      this.traveller = traveller;
      this._super();
      this.init();
      this.innerSprite = cc.Sprite.createWithSpriteFrame(cc.SpriteFrameCache.getInstance().getSpriteFrame("Frame0"));
      this.innerSprite.setShaderProgram(this.parent.shaders[this.traveller.destination()]);
      this.addChild(this.innerSprite);
      this.innerSprite.retain();
      this.fromIndex = -1;
      this.toIndex = -1;
      this.label = null;
      this.move();
      return this.traveller.view = this;
    },

    /**
    	* Advance the traveller to the next waypoint
    	* @method
    	* @name move
     */
    move: function() {
      var _ref;
      _ref = this.traveller.waypointRowColumn(), this.row = _ref[0], this.column = _ref[1];
      this.fromIndex = this.toIndex;
      this.updateLabel();
      return this.setPosition(this.parent.tilePoint(this.row, this.column));
    },

    /**
    	* Sample the current waypoint and pick the current animation path "label"
    	* @method
    	* @name updateLabel
     */
    updateLabel: function() {
      var vi;
      vi = this.traveller.board.vectors[this.row][this.column];
      if (vi === this.fromIndex || this.fromIndex === -1) {
        this.label = fromLabels[vi];
      } else if (vi === -1) {
        this.label = fromLabels[this.fromIndex];
      } else if (fromLabels[this.fromIndex] === fromLabels[vi]) {
        this.label = fromLabels[vi];
      } else {
        this.label = fromLabels[this.fromIndex] + toLabels[vi];
      }
      if ((vi % 2) === 0 && this.fromIndex !== vi) {
        this.setScaleX(vi === 0 ? 1 : -1);
      }
      return this.toIndex = vi;
    },

    /**
    	* Advance the traveller along the animation path for the current label
    	* @method
    	* @name update
     */
    update: function() {
      var frame, pr, progress, rot;
      if (this.innerSprite === null) {
        return;
      }
      if (this.label !== null) {
        progress = this.traveller.waypointProgress(this.parent.model.now);
        pr = this.parent.settings.getTravellerPositionRotation(this.label, progress);
        this.innerSprite.setPosition(cc.p(pr.x, -pr.y));
        rot = pr.r;
        if (this.label === "southEast" || this.label === "eastNorth") {
          rot = 180 - rot;
          this.innerSprite.setScaleX(-1);
        } else {
          this.innerSprite.setScaleX(1);
        }
        frame = Math.round(FRAME_COUNT * modHelper(rot, 360) / 360) % FRAME_COUNT;
        return this.innerSprite.setDisplayFrame(cc.SpriteFrameCache.getInstance().getSpriteFrame("Frame" + frame));
      }
    },

    /**
    	* Release resource for this subview
    	* @method
    	* @name dispose
     */
    dispose: function() {
      this.innerSprite.release();
      return this.innerSprite = null;
    }
  });


  /**
  * The minimum number of seconds that the "highlight" state is kept on the stage,
  * to prevent flicker.
  * @constant
   */

  switchTimeout = 0.1;


  /**
  * Subview specifically for handling Switch Waypoints (which the player taps)
  * @class
  * @name SwitchView
   */

  SwitchView = cc.Node.extend({

    reflections: {
      "SwitchNorthSouth_0": ["SwitchEastNorth_1", 1],
      "SwitchSouthEast_0": ["SwitchEastWest_1", -1],
      "SwitchSouthEast_1": ["SwitchEastWest_0", -1],
      "SwitchSouthNorth_0": ["SwitchEastNorth_1", -1],
      "SwitchSouthNorth_1": ["SwitchEastNorth_0", -1],
      "SwitchWestEast_0": ["SwitchNorthWest_1", -1],
      "SwitchWestEast_1": ["SwitchNorthWest_0", -1],
      "SwitchWestNorth_1": ["SwitchEastNorth_1", -1],
      "SwitchWestNorth_0": ["SwitchNorthSouth_1", -1]
    },

    /**
    	* Constructor
    	* @constructor
    	* @param parent - the root train of thought view
    	* @param name - the name of the art asset (without the prefix)
    	* @param address - the logical address of the switch
     * Reflections:
     *  EastNorth, NorthSouth (1 only) : SouthNorth, WestNorth (1 only)
     *  EastWest 0 : SouthEast 1
     */
    ctor: function(parent, name, address) {
      this.parent = parent;
      this.address = address;
      this._super();
      this.init();
      this.toggled = false;
      this.timeout = -1;
      for (var d = 0; d < 2; d++) {
        this.createDirection(name, d);
      }
      return this.addChild(this.sprite0);
    },
    
    /**
     * GAME-2949 To save about 1% texture memory, reuse mirror images of switches.  Delete redundant images.  On iPad Retina this is about 2 MB of 270 MB.  Test case:  2015-03-22 Level 14.  Play several times and toggle switches.  Expect switches connect without a seam.  Observe train go over switch.
     */
    createDirection: function(name, d) {
      d = d.toString();
      var name = name + "_" + d;
      var imageName = name;
      var scaleX = 1.0;
      if (name in this.reflections) {
        imageName = this.reflections[name][0];
        scaleX = this.reflections[name][1];
      }
      var spriteName = "sprite" + d;
      this[spriteName] = this.parent.createSprite(imageName);
      this[spriteName].setScaleX(scaleX);
      this[spriteName].retain();
      var spriteNameA = spriteName + "a";
      this[spriteNameA] = this.parent.createSprite(imageName + "a");
      this[spriteNameA].setScaleX(scaleX);
      this[spriteNameA].retain();
    },

    /**
    	* Toggle this switch view by shuffling which sprites are attached
    	* to the view's root node.
    	* @method
    	* @name toggle
     */
    toggle: function() {
      this.toggled = !this.toggled;
      this.timeout = switchTimeout;
      if (this.toggled) {
        this.sprite0.removeFromParent();
        this.sprite0a.removeFromParent();
        return this.addChild(this.sprite1a);
      } else {
        this.sprite1.removeFromParent();
        this.sprite1a.removeFromParent();
        return this.addChild(this.sprite0a);
      }
    },

    /**
    	* Update tick, to be called every frame
    	* @method
    	* @name update
    	* @param dt delta time (in seconds)
     */
    update: function(dt) {
      if (this.timeout < 0) { // || this.parent.anyTouches()) {
        return;
      }
      this.timeout -= dt;
      if (this.timeout < 0) {
        if (this.toggled) {
          this.sprite1a.removeFromParent();
          return this.addChild(this.sprite1);
        } else {
          this.sprite0a.removeFromParent();
          return this.addChild(this.sprite0);
        }
      }
    },

    /**
    	* Sprites are retained so they're not reclaimed when they're off-screen.  Use this
    	* method to release them explicitly.
    	* @method
    	* @name releaseSprites
     */
    releaseSprites: function() {
      this.sprite0.release();
      this.sprite1.release();
      this.sprite0a.release();
      return this.sprite1a.release();
    }
  });


  /**
  * Helper object for doing flipbook animations based on a naming
  * convention for assets that's consistent with the vector exporter.
  * @name FlipbookView
   */

  FlipbookView = cc.Node.extend({

    /**
    	* Flipbook constructor
    	* @constructor
    	* @param parent - reference to the TrainOfThoughtView
    	* @param name - the directory name of the animation (also used as a key
    	*               into game settings)
     */
    ctor: function(parent, name) {
      var anim, animateAction, callFuncAction, frames, i, _ref, _ref1;
      this.parent = parent;
      this._super();
      this.init();
      _ref = this.parent.settings.animBounds[name], this.w = _ref[0], this.h = _ref[1], this.n = _ref[2];
      _ref1 = this.parent.settings.pivots[name], this.px = _ref1[0], this.py = _ref1[1];
      anim = cc.AnimationCache.getInstance().getAnimation(name);
      if (anim === null) {
        frames = (function() {
          var _i, _ref2, _results;
          _results = [];
          for (i = _i = 0, _ref2 = this.n - 1; 0 <= _ref2 ? _i <= _ref2 : _i >= _ref2; i = 0 <= _ref2 ? ++_i : --_i) {
            _results.push(this.createAnimFrame("" + name + "/" + name + "_" + i + ".png"));
          }
          return _results;
        }).call(this);
        anim = cc.Animation.create(frames, 0.033, 1);
        cc.AnimationCache.getInstance().addAnimation(anim, name);
      }
      this.sprite = cc.Sprite.create();
      animateAction = cc.Animate.create(anim);
      callFuncAction = cc.CallFunc.create(this.onFinished, this);
      this.sprite.runAction(cc.Sequence.create(animateAction, callFuncAction));
      this.sprite.setAnchorPoint(cc.p(this.px / this.w, 1 - (this.py / this.h)));
      this.addChild(this.sprite);
      return this.parent.flipbookCount += 1;
    },

    /**
    	* Cocos action callback to automatically clean ourselves up
    	* @method
    	* @name onFinished
     */
    onFinished: function() {
      this.parent.flipbookCount -= 1;
      return this.removeFromParent(true);
    },

    /**
    	* Helper method to map a file-path to a sprite frame based
    	* on geometry written to the settings configurations by the
    	* vector exporter
    	* @mthod
    	* @name createAnimFrame
    	* @param {string} path - relative path to the image for this frame
     */
    createAnimFrame: function(path) {
      var animFrame, frame, tex;
      tex = cc.TextureCache.getInstance().addImage(path);
      frame = cc.SpriteFrame.createWithTexture(tex, cc.rect(0, 0, this.w, this.h));
      animFrame = new cc.AnimationFrame();
      animFrame.initWithSpriteFrame(frame, 1, null);
      return animFrame;
    }
  });

}).call(this);
